﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Quadrics;
using GMath;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.BaseClasses;
using Tao.Platform.Windows;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.AddForms;
using Geometry.GGraphic.Controls;

namespace Geometry
{
    public class AddManager
    {
        private ManipulationManager _manipulationManager;
        private Form1 _form;

        public AddManager(Form1 form, ManipulationManager manipulationManager) 
        {
            _manipulationManager = manipulationManager;
            _form = form;
        }

        internal void AddQuadricInvoked()
        {
            QuadricEquation equation = new QuadricEquation(7, 7, 10, -2, -4, 4, -12, 12, 60, 42);
            QuadricCommonKoefsControl koefs = new QuadricCommonKoefsControl(equation);
            CommonEquationFormatString formatString = new CommonEquationFormatString(Quadric._commonEquationInitialFormatString);
            formatString.SubstituteToFormatString(equation);
            CommonEquationGR equationGRControl = new CommonEquationGR();
            equationGRControl.SetContents(null, formatString.GetGeneralFormatString());
            AddQuadricDialog addQuadricDialog = new AddQuadricDialog(_manipulationManager, formatString, equationGRControl, koefs);
            addQuadricDialog.ShowDialog();
        }
        internal void AddConicSectionInvoked()
        {
            ConicEquation equation = new ConicEquation(1, 1, 1, 1, 1, -1);
            ConicCommonKoefsControl koefs = new ConicCommonKoefsControl(equation);
            CommonEquationFormatString equationFormatString = new CommonEquationFormatString(ConicSection._conicCommonFormatString);
            equationFormatString.SubstituteToFormatString(equation);
            CommonEquationGR equationGRControl = new CommonEquationGR();
            equationGRControl.SetContents(null, equationFormatString.GetGeneralFormatString());
            AddConicDialog addConicDialog = new AddConicDialog(_manipulationManager, equationFormatString, equationGRControl, koefs);
            addConicDialog.ShowDialog();
        }
        #region<Pridat konkretni kvadriku>

        private void ShowAddSpecificQuadricDialog(Quadric quadric, ParametricEquation canonicEquation,
            CenterParametrization center)
        {
            CanonicEquationGR equationGRControl = new CanonicEquationGR();
            equationGRControl.SetContents(quadric, quadric.GetCanonicEquationFormatString().GetGeneralFormatString());

            QuadricCanonicKoefsControl koefsControl = new QuadricCanonicKoefsControl(canonicEquation);
            CenterKoeficientsControl centerControl =
                new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount());
            KoefsAndCenterControl pair = new KoefsAndCenterControl(koefsControl, centerControl);

            AddSpecificQuadricDialog addForm = new AddSpecificQuadricDialog(_manipulationManager, quadric, equationGRControl, pair);
            addForm.ShowDialog();
        }

        internal void AddElipsoidRealInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 1, 0, 0, 0, 0, 0, 0, -1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Elipsoid quadric = new Elipsoid(commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddElipsoidImaginarInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(-1, -1, -1, 0, 0, 0, 0, 0, 0, 1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarElipsoid quadric = new ImaginarElipsoid(commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddHyperboloidTwoSheetInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, -1, 0, 0, 0, 0, 0, 0, 1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Hyperboloid2 quadric = new Hyperboloid2(commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddHyperboloidOneSheetInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, -1, 0, 0, 0, 0, 0, 0, -1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Hyperboloid1 quadric = new Hyperboloid1
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddHyperbolicParaboloidInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, -1, 0, 0, 0, 0, 0, 0, 2, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 1);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            HyperbolicParaboloid quadric = new HyperbolicParaboloid
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddElipticParaboloidInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 0, 0, 0, 0, 0, 0, 2, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 1);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ElipticParaboloid quadric = new ElipticParaboloid
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddConeRealInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, -1, 0, 0, 0, 0, 0, 0, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Cone quadric = new Cone
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddConeImaginarInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 1, 0, 0, 0, 0, 0, 0, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarCone quadric = new ImaginarCone
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddElipticCylinderRealInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 0, 0, 0, 0, 0, 0, 0, -1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ElipticCylinder quadric = new ElipticCylinder
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddElipticCylinderImaginarInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 0, 0, 0, 0, 0, 0, 0, 1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 0);
            Matrix toCanonicRotaion = new Matrix(4);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarElipticCylinder quadric = new ImaginarElipticCylinder
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddHyperbolicCylinderInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, -1, 0, 0, 0, 0, 0, 0, 0, -1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            HyperbolicCylinder quadric = new HyperbolicCylinder
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddParabolicCylinderInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 0, 0, 0, 0, 0, 0, 0, 2, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 0, 0, 1);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ParabolicCylinder quadric = new ParabolicCylinder
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddDoublePlanInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 0, 0, 0, 0, 0, 0, 0, 0, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 0, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            DoublePlan quadric = new DoublePlan
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddParalelPlansRealInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 0, 0, 0, 0, 0, 0, 0, 0, -1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 0, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ParallelPlans quadric = new ParallelPlans
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddParalelPlansImaginarInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 0, 0, 0, 0, 0, 0, 0, 0, 1);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 0, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarParallelPlans quadric = new ImaginarParallelPlans
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddCrossingPlansRealInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, -1, 0, 0, 0, 0, 0, 0, 0, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            CrossingPlans quadric = new CrossingPlans
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }
        internal void AddCrossingPlansImaginarInvoked()
        {
            CommonEquation commonEquation = new QuadricEquation(1, 1, 0, 0, 0, 0, 0, 0, 0, 0);
            ParametricEquation canonicEquation = new QuadricParametricEquation(1, 1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewQuadricDefaulColor;
            string name = Configuration.NewQuadricDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarCrossingPlans quadric = new ImaginarCrossingPlans
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            CenterParametrization center = new CenterParametrization();
            ShowAddSpecificQuadricDialog(quadric, canonicEquation, center);
        }

        #endregion
        #region<Pridat konkretni kuzelosecku>

        private void ShowAddSpecificConicDialog(ConicSection conic, ParametricEquation canonicEquation,
            CenterParametrization center)
        {
            CanonicEquationGR requlationControl = new CanonicEquationGR();
            requlationControl.SetContents(conic, conic.GetCanonicEquationFormatString().GetGeneralFormatString());

            ConicCanonicKoefsControl koefsControl =
                new ConicCanonicKoefsControl((ConicParametricEquation)canonicEquation);
            CenterKoeficientsControl centerControl =
                new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount());
            KoefsAndCenterControl pair = new KoefsAndCenterControl(koefsControl, centerControl);

            AddSpecificConicDialog addForm = new AddSpecificConicDialog(_manipulationManager, conic, requlationControl, pair);
            addForm.ShowDialog();
        }

        internal void AddElipseImaginarInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(-1, -1, 0, 0, 0, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarElipse conic = new ImaginarElipse
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddElipseRealInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, 1, 0, 0, 0, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Elipse conic = new Elipse
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddHyperbolaInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, -1, 0, 0, 0, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Hyperbola conic = new Hyperbola
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddParabolaInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, 0, 0, 0, 2, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 0, 1);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Parabola conic = new Parabola
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddDoubleLineInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, 0, 0, 0, 0, 0);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            DoubleLine conic = new DoubleLine
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddParalelLinesRealInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, 0, 0, 0, 0, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ParallelLines conic = new ParallelLines
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddParalelLinesImaginarInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(-1, 0, 0, 0, 0, -1);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 0, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarParallelLines conic = new ImaginarParallelLines
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddCrossingLinesRealInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, -1, 0, 0, 0, 0);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            CrossingLines conic = new CrossingLines
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }
        internal void AddCrossingLinesImaginarInvoked()
        {
            ConicEquation commonEquation = new ConicEquation(1, 1, 0, 0, 0, 0);
            CenterParametrization center = new CenterParametrization();
            ParametricEquation canonicEquation = new ConicParametricEquation(1, 1, 0);
            TransformationMatrix userRotation = new TransformationMatrix(4);
            GColor color = Configuration.NewConicDefaultColor;
            string name = Configuration.NewConicDefaultName;
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ImaginarCrossingLines conic = new ImaginarCrossingLines
                (commonEquation, canonicEquation, userRotation, color, name, ID);
            ShowAddSpecificConicDialog(conic, canonicEquation, center);
        }

        #endregion
        internal void AddPlanInvoked()
        {
            PlanEquation equation = new PlanEquation(1, 1, 1, 0);
            PlanCommonKoefsControl koefs = new PlanCommonKoefsControl(equation);
            CommonEquationFormatString formatString = new CommonEquationFormatString(Plan._planCommonFormatString);
            formatString.SubstituteToFormatString(equation);
            CommonEquationGR equationGRControl = new CommonEquationGR();
            equationGRControl.SetContents(null, formatString.GetGeneralFormatString());
            AddPlanDialog addPlanDialog = new AddPlanDialog(_manipulationManager, formatString, equationGRControl, koefs);
            addPlanDialog.ShowDialog();
        }
        internal void AddPlanParametricalyInvoked()
        {
            PlanParametricEquation equation = new PlanParametricEquation(0, 1, 0, 1, 0, 0);
            CenterParametrization center = new CenterParametrization();
            PlanParametricKoeficientsControl koefsControl = new PlanParametricKoeficientsControl(equation);
            CenterKoeficientsControl centerControl =
                new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount());
            ParametricEquationsFormatString formatString = new ParametricEquationsFormatString(
                Plan._parametricXFormatString,
                Plan._parametricYFormatString,
                Plan._parametricZFormatString);
            formatString.SubstituteToFormatStrings(equation, center);
            KoefsAndCenterControl centerAndKoefsControl = new KoefsAndCenterControl(koefsControl, centerControl);
            ParametricEquationsGR equationGRControl = new ParametricEquationsGR();
            equationGRControl.SetContents(null, formatString.GetXGeneralFormatString(), formatString.GetYGeneralFormatString(),
                formatString.GetZGeneralFormatString());
            Plan plan = new Plan(equation, center, Configuration.NewPlanDefaultColor, Configuration.NewPlanDefaultName, _manipulationManager.GetNewGraphicObjectID());
            AddPlanParametricDialog addPlanDialog =
                new AddPlanParametricDialog(_manipulationManager, plan, equationGRControl, centerAndKoefsControl);
            addPlanDialog.ShowDialog();
        }
        internal void AddLineInvoked()
        {
            LineParametricEquation equation = new LineParametricEquation(1, 1, 1);
            CenterParametrization center = new CenterParametrization();
            LineParametricKoeficientsControl koefsControl = new LineParametricKoeficientsControl(equation);
            CenterKoeficientsControl centerControl =
                new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount());
            ParametricEquationsFormatString formatString = new ParametricEquationsFormatString(
                Line._parametricXFormatString,
                Line._parametricYFormatString,
                Line._parametricZFormatString);
            formatString.SubstituteToFormatStrings(equation, center);
            KoefsAndCenterControl centerAndKoefsControl = new KoefsAndCenterControl(koefsControl, centerControl);
            ParametricEquationsGR equationGRControl = new ParametricEquationsGR();
            equationGRControl.SetContents(null, formatString.GetXGeneralFormatString(), formatString.GetYGeneralFormatString(),
                formatString.GetZGeneralFormatString());
            Line line = new Line(equation, center, Configuration.NewLineDefaultColor, Configuration.NewLineDefaultName, _manipulationManager.GetNewGraphicObjectID());
            AddLineDialog addPlanDialog = new AddLineDialog(_manipulationManager, line, equationGRControl, centerAndKoefsControl);
            addPlanDialog.ShowDialog();
        }
        internal void AddPointInvoked()
        {
            CenterParametrization center = new CenterParametrization();
            PointParametricKoeficientsControl koefsControl = new PointParametricKoeficientsControl();
            CenterKoeficientsControl centerControl =
                new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount());
            ParametricEquationsFormatString formatString = new ParametricEquationsFormatString(
                GPoint._parametricXFormatString,
                GPoint._parametricYFormatString,
                GPoint._parametricZFormatString);
            formatString.SubstituteToFormatStrings(null, center);
            KoefsAndCenterControl centerAndKoefsControl = new KoefsAndCenterControl(koefsControl, centerControl);
            ParametricEquationsGR equationGRControl = new ParametricEquationsGR();
            equationGRControl.SetContents(null, formatString.GetXGeneralFormatString(), formatString.GetYGeneralFormatString(),
                formatString.GetZGeneralFormatString());
            GPoint point = new GPoint(center.GetReprezentant(), Configuration.NewPointDefaultColor, Configuration.NewPlanDefaultName, _manipulationManager.GetNewGraphicObjectID());
            AddPointDialog addPointDialog = new AddPointDialog(_manipulationManager, point, equationGRControl, centerAndKoefsControl);
            addPointDialog.ShowDialog();
        }

        internal void AddIntersectionInvoked()
        {
            IntPtr form1hRC = _form.GetCurrenthRC();
            IntPtr form1hDC = _form.GetCurrentDC();
            Gdi.PIXELFORMATDESCRIPTOR form1pfd = _form.GetCurrentPixelFormat();
            AddIntersection qid =
                new AddIntersection(_form, form1hDC, form1hRC, form1pfd,
                    Configuration.NewQuadricsIntersectionColor, Configuration.NewQuadricsIntersectionName);
            qid.ShowDialog();
        }


        internal void AddQuadricDialogClosed(AddQuadricDialog addQuadricDialog)
        {
            if (addQuadricDialog.DialogResult != DialogResult.OK)
                return;
            CommonEquation equation = addQuadricDialog.GetCommonEquation();
            string name = addQuadricDialog.GetName();
            GColor color = addQuadricDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            _form.SetProgressBarText("Probíhá rozpoznávání kvadriky.");
            QuadricRecognizer qrecognizer = new QuadricRecognizer((QuadricEquation)equation, color, name, ID);
            Quadric quadric = qrecognizer.RecognizeQuadric();
            _form.SetProgressBarText("");
            _manipulationManager.AddObjectInvoked(quadric);
        }
        internal void AddConicDialogClosed(AddConicDialog addConicDialog)
        {
            if (addConicDialog.DialogResult != DialogResult.OK)
                return;
            CommonEquation equation = addConicDialog.GetCommonEquation();
            string name = addConicDialog.GetName();
            GColor color = addConicDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            _form.SetProgressBarText("Probíhá rozpoznávání kuželosečky.");
            ConicRecognizer crecognizer = new ConicRecognizer((ConicEquation)equation, color, name, ID);
            ConicSection conic = crecognizer.Recognize();
            _form.SetProgressBarText("");
            _manipulationManager.AddObjectInvoked(conic);
        }
        internal void AddPlanDialogClosed(AddPlanDialog addPlanDialog)
        {
            if (addPlanDialog.DialogResult != DialogResult.OK)
                return;
            CommonEquation commonEquation = addPlanDialog.GetCommonEquation();
            string name = addPlanDialog.GetName();
            GColor color = addPlanDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Plan plan = new Plan((PlanEquation)commonEquation, color, name, ID);
            _manipulationManager.AddObjectInvoked(plan);
        }

        internal void AddQuadricDialogClosed(AddSpecificQuadricDialog addSpecificQuadricDialog)
        {
            if (addSpecificQuadricDialog.DialogResult != DialogResult.OK)
                return;
            KoefsAndCenterPair newPair = addSpecificQuadricDialog.GetKoefsAndCenterPair();
            string name = addSpecificQuadricDialog.GetName();
            GColor color = addSpecificQuadricDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Quadric newQuadric = (Quadric)((Quadric)addSpecificQuadricDialog.GetGraphicObject()).MakeCopyOfObject(newPair, ID);
            newQuadric.ChangeColor(color);
            newQuadric.ChangeName(name);
            _manipulationManager.AddObjectInvoked(newQuadric);
        }
        internal void AddConicDialogClosed(AddSpecificConicDialog addSpecificConicDialog)
        {
            if (addSpecificConicDialog.DialogResult != DialogResult.OK)
                return;
            KoefsAndCenterPair newPair = addSpecificConicDialog.GetKoefsAndCenterPair();
            string name = addSpecificConicDialog.GetName();
            GColor color = addSpecificConicDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            ConicSection conic = (ConicSection)((ConicSection)addSpecificConicDialog.GetGraphicObject()).MakeCopyOfObject(newPair, ID);
            conic.ChangeColor(color);
            conic.ChangeName(name);
            _manipulationManager.AddObjectInvoked(conic);
        }
        internal void AddPlanDialogClosed(AddPlanParametricDialog addPlanParametricDialog)
        {
            if (addPlanParametricDialog.DialogResult != DialogResult.OK)
                return;
            KoefsAndCenterPair newPair = addPlanParametricDialog.GetKoefsAndCenterPair();
            string name = addPlanParametricDialog.GetName();
            GColor color = addPlanParametricDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Plan newPlan = (Plan)((Plan)addPlanParametricDialog.GetGraphicObject()).MakeCopyOfObject(newPair, ID);
            newPlan.ChangeColor(color);
            newPlan.ChangeName(name);
            _manipulationManager.AddObjectInvoked(newPlan);
        }
        internal void AddLineDialogClosed(AddLineDialog addLineDialog)
        {
            if (addLineDialog.DialogResult != DialogResult.OK)
                return;
            KoefsAndCenterPair newPair = addLineDialog.GetKoefsAndCenterPair();
            string name = addLineDialog.GetName();
            GColor color = addLineDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            Line line = (Line)((Line)addLineDialog.GetGraphicObject()).MakeCopyOfObject(newPair, ID);
            line.ChangeColor(color);
            line.ChangeName(name);
            _manipulationManager.AddObjectInvoked(line);
        }
        internal void AddPointDialogClosed(AddPointDialog addPointDialog)
        {
            if (addPointDialog.DialogResult != DialogResult.OK)
                return;
            KoefsAndCenterPair newPair = addPointDialog.GetKoefsAndCenterPair();
            string name = addPointDialog.GetName();
            GColor color = addPointDialog.GetColor();
            int ID = _manipulationManager.GetNewGraphicObjectID();
            GPoint point = (GPoint)((GPoint)addPointDialog.GetGraphicObject()).MakeCopyOfObject(newPair, ID);
            point.ChangeColor(color);
            point.ChangeName(name);
            _manipulationManager.AddObjectInvoked(point);
        }
    }
}
