﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Tao.OpenGl;
using Geometry.BaseClasses;

namespace Geometry.GraphicObjects.Planar
{
    public class Axes:GraphicObject
    {
        public static GColor _xColor = new GColor(1, 0, 0);
        public static GColor _yColor = new GColor(0, 0.7, 0);
        public static GColor _zColor = new GColor(0, 0, 1);

        private int _xLineWithArrowDisplayListIdentificator;
        private int _yLineWithArrowDisplayListIdentificator;
        private int _zLineWithArrowDisplayListIdentificator;

        private int _xDisplayListIdentificator;
        private int _yDisplayListIdentificator;
        private int _zDisplayListIdentificator;
        private GColor _characterBorderColor = new GColor(0, 0, 0);

        private double _textLineWidth = 0.4;
        private double _width = 1.5;
        private double _height = 2;
        private double _arrowHeight = 5;
        private double _arrowRadius = 5;

        public Axes(int ID) 
        {
            base._ID = ID;
            base._color = new GColor(0, 0, 0);
            base.SetWordDescriptionFirstCase("Souřadné osy");
            base.SetWordDescriptionSecondCase("Souřadných os");
            base.SetWordDescriptionFourthCase("Souřadné osy");
        }

        public override void Draw(Scene scene)
        {
            int clippingCubeWidth = Scene.GetClippingCubeWidth();
            if (_visible)
            {
                Gl.glDisable(Gl.GL_LIGHTING);
                Gl.glPushMatrix();
                Gl.glColor3dv(_xColor);
                Gl.glCallList(_xLineWithArrowDisplayListIdentificator);
                Gl.glTranslated(-clippingCubeWidth + 1.5, 0, 0);
                Gl.glRotated(scene.GetSceneAngles().GetYAngle(), 0, 1, 0);
                Gl.glRotated(scene.GetSceneAngles().GetXAngle(), 1, 0, 0);
                Gl.glCallList(_xDisplayListIdentificator);
                Gl.glPopMatrix();

                Gl.glPushMatrix();
                Gl.glColor3dv(_yColor);
                Gl.glCallList(_yLineWithArrowDisplayListIdentificator);
                Gl.glTranslated(0, -clippingCubeWidth + _height, 0);
                Gl.glRotated(scene.GetSceneAngles().GetYAngle(), 0, 1, 0);
                Gl.glRotated(scene.GetSceneAngles().GetXAngle(), 1, 0, 0);   
                Gl.glCallList(_yDisplayListIdentificator);
                Gl.glPopMatrix();

                Gl.glPushMatrix();
                Gl.glColor3dv(_zColor);
                Gl.glCallList(_zLineWithArrowDisplayListIdentificator);
                Gl.glTranslated(0, 0, -clippingCubeWidth + 1.5);
                Gl.glRotated(scene.GetSceneAngles().GetYAngle(), 0, 1, 0);
                Gl.glRotated(scene.GetSceneAngles().GetXAngle(), 1, 0, 0);   
                Gl.glCallList(_zDisplayListIdentificator);
                Gl.glPopMatrix();
                Gl.glEnable(Gl.GL_LIGHTING);
            }
        }

        public override void DrawIgnoreVisibilityAndAxes(Scene scene)
        {
            Draw(scene);
        }

        internal override void DeleteDisplayList()
        {
            Gl.glDeleteLists(_xLineWithArrowDisplayListIdentificator, 1);
            Gl.glDeleteLists(_yLineWithArrowDisplayListIdentificator, 1);
            Gl.glDeleteLists(_zLineWithArrowDisplayListIdentificator, 1);
            Gl.glDeleteLists(_xDisplayListIdentificator, 1);
            Gl.glDeleteLists(_yDisplayListIdentificator, 1);
            Gl.glDeleteLists(_zDisplayListIdentificator, 1);
            _xDisplayListIdentificator = -1;
            _yDisplayListIdentificator = -1;
            _zDisplayListIdentificator = -1;
            _xLineWithArrowDisplayListIdentificator = -1;
            _yLineWithArrowDisplayListIdentificator = -1;
            _zLineWithArrowDisplayListIdentificator = -1;
        }



        public override void BuildDisplayListIfNecessary(Scene scene, RelativeQuality relativeQuality)
        {
            int clippingCubeWidth = Scene.GetClippingCubeWidth();
            double left = -clippingCubeWidth;
            double right = clippingCubeWidth;
            double front = clippingCubeWidth;
            double back = -clippingCubeWidth;
            double top = clippingCubeWidth;
            double bottom = -clippingCubeWidth;

            _xLineWithArrowDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_xLineWithArrowDisplayListIdentificator, Gl.GL_COMPILE);
            Gl.glPushMatrix();
            Gl.glRotated(90, 0, 1, 0);
            BuildLineWithArrow(left, right, _arrowHeight, _arrowRadius);
            Gl.glPopMatrix();
            Gl.glEndList();

            _yLineWithArrowDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_yLineWithArrowDisplayListIdentificator, Gl.GL_COMPILE);
            Gl.glPushMatrix();            
            Gl.glRotated(-90, 1, 0, 0);
            BuildLineWithArrow(bottom, top, _arrowHeight, _arrowRadius);
            Gl.glPopMatrix();
            Gl.glEndList();

            _zLineWithArrowDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_zLineWithArrowDisplayListIdentificator, Gl.GL_COMPILE);
            BuildLineWithArrow(back, front, _arrowHeight, _arrowRadius);
            Gl.glEndList();

            BuildX(_textLineWidth, _width, _height);
            BuildY(_textLineWidth, _width, _height);
            BuildZ(_textLineWidth, _width, _height);
        }

        public override void BuildDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            BuildDisplayListIfNecessary(scene, relativeQuality);
        }


        public void BuildLineWithArrow(double back, double front, double arrowHeight, double arrowRadius) 
        {
            Gl.glPushMatrix();
            Gl.glBegin(Gl.GL_LINES);
            Gl.glVertex3d(0, 0, back);
            Gl.glVertex3d(0, 0, front);
            Gl.glEnd();
            Gl.glTranslated(0,0, front - arrowHeight);
            BuildArrow(arrowHeight, arrowRadius);
            Gl.glPopMatrix();
        }

        // kresli spicku sipky smerem z obrazovky 
        public void BuildArrow(double arrowHeight, double arrowRadius)
        {
            int slices = 20;
            double x, y;
            Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
            Gl.glBegin(Gl.GL_TRIANGLE_FAN);
            Gl.glVertex3d(0, 0, arrowHeight);
            for (int i = 0; i <= slices; i++)
            {
                x = Math.Cos(i * Math.PI * 2 / slices);
                y = Math.Sin(i * Math.PI * 2 / slices);
                Gl.glVertex3d(x, y, 0);
            }
            Gl.glEnd();
            Gl.glBegin(Gl.GL_TRIANGLE_FAN);
            Gl.glVertex3d(0, 0, 0);
            for (int i = 0; i <= slices; i++)
            {
                x = Math.Cos(i * Math.PI * 2 / slices);
                y = Math.Sin(i * Math.PI * 2 / slices);
                Gl.glVertex3d(x, y, 0);
            }
            Gl.glEnd();
        }

        public void BuildX(double textLineWidth, double width, double height)
        {
            _xDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_xDisplayListIdentificator, Gl.GL_COMPILE);

            Gl.glPushMatrix();
            Gl.glTranslated(-width / 2.0, height / 2.0, 0);
            
            Gl.glBegin(Gl.GL_QUADS);
            // shora dolu predek
            //Gl.glNormal3d(0, 0, -1);
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width - textLineWidth, -height, 0);
            // zespoda nahoru predek
            //Gl.glNormal3d(0, 0, 1);
            Gl.glVertex3d(0, -height, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glEnd();

            //Gl.glEnable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(2.0f);
            Gl.glColor3dv(_characterBorderColor);
            // obrysy
            Gl.glBegin(Gl.GL_LINES);
            // shora dolu predek
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width - textLineWidth, -height, 0);
            Gl.glVertex3d(width - textLineWidth, -height, 0);
            Gl.glVertex3d(0, 0, 0);
            // zespoda nahoru predek
            Gl.glVertex3d(0, -height, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(0, -height, 0);
            Gl.glEnd();
            //Gl.glDisable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(1.0f);
            Gl.glPopMatrix();

            Gl.glEndList();
        }

        public void BuildY(double textLineWidth, double width, double height)
        {
            _yDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_yDisplayListIdentificator, Gl.GL_COMPILE);

            Gl.glPushMatrix();
            Gl.glTranslated(-width / 2.0, height / 2.0, 0);

            double h2 = height*1.3/ 2.0;
            double w2 = width / 2.0;
            Gl.glBegin(Gl.GL_QUADS);
            // vrch
            Gl.glNormal3d(0, 0, 1);
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(w2, -h2, 0);
            Gl.glVertex3d(w2, -h2+textLineWidth*1.5, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            // vrch
            Gl.glNormal3d(0, 0, -1);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(w2, -h2, 0);
            Gl.glVertex3d(w2, -h2 + textLineWidth*1.5, 0);
            Gl.glVertex3d(width- textLineWidth, 0, 0);
            // spodek
            Gl.glNormal3d(0, 0, -1);
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -height, 0);
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -height, 0);
            Gl.glEnd();

            //Gl.glEnable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(2.0f);
            Gl.glColor3dv(_characterBorderColor);
            Gl.glBegin(Gl.GL_LINES);
            // vrch
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(w2, -h2, 0);
            Gl.glVertex3d(w2, -h2, 0);            
            Gl.glVertex3d(w2, -h2 + textLineWidth * 1.5, 0);
            Gl.glVertex3d(w2, -h2 + textLineWidth * 1.5, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            Gl.glVertex3d(textLineWidth, 0, 0);
            Gl.glVertex3d(0, 0, 0);
            // vrch
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(w2, -h2, 0);
            Gl.glVertex3d(w2, -h2, 0);
            Gl.glVertex3d(w2, -h2 + textLineWidth * 1.5, 0);
            Gl.glVertex3d(w2, -h2 + textLineWidth * 1.5, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(width, 0, 0);
            // spodek
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -height, 0);            
            Gl.glVertex3d(w2 + textLineWidth / 2.0, -height, 0);
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -height, 0);
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -height, 0);            
            Gl.glVertex3d(w2 - textLineWidth / 2.0, -h2 + textLineWidth, 0);
            Gl.glEnd();
            //Gl.glDisable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(1.0f);
            Gl.glPopMatrix();
            Gl.glEndList();
        }

        public void BuildZ(double textLineWidth, double width, double height)
        {
            _zDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_zDisplayListIdentificator, Gl.GL_COMPILE);

            Gl.glPushMatrix();
            Gl.glTranslated(-width / 2.0, height / 2.0, 0);

            Gl.glBegin(Gl.GL_QUADS);
            // vrch
            //Gl.glNormal3d(0, 0, 1);
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(0, -textLineWidth, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width, 0, 0);
            // zespoda nahoru predek
            //Gl.glNormal3d(0, 0, 1);
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            // spodek
            //Gl.glNormal3d(0, 0, 1);
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            Gl.glVertex3d(0, -height, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width, -height + textLineWidth, 0);
            Gl.glEnd();

            //Gl.glEnable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(2.0f);
            Gl.glColor3dv(_characterBorderColor);
            Gl.glBegin(Gl.GL_LINES);
            // vrch
            Gl.glVertex3d(0, 0, 0);
            Gl.glVertex3d(0, -textLineWidth, 0);
            Gl.glVertex3d(0, -textLineWidth, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(width, 0, 0);
            Gl.glVertex3d(0, 0, 0);
            // zespoda nahoru predek
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(textLineWidth, -height, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width, -textLineWidth, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(width - textLineWidth, 0, 0);
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            // spodek
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            Gl.glVertex3d(0, -height, 0);
            Gl.glVertex3d(0, -height, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width, -height, 0);
            Gl.glVertex3d(width, -height + textLineWidth, 0);
            Gl.glVertex3d(width, -height + textLineWidth, 0);
            Gl.glVertex3d(0, -height + textLineWidth, 0);
            Gl.glEnd();
            //Gl.glDisable(Gl.GL_LINE_WIDTH);
            Gl.glLineWidth(1.0f);
            Gl.glPopMatrix();
            Gl.glEndList();
        }
    }
}
