﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;

namespace Geometry.GGraphic.Controls.ElementaryControls
{
    public partial class CleverLabel : Label, IFakeable
    {
        // Summary:
        // Zamýšleno jen pro použití v CurveTabPage
        public int DynamicWidth { get; set; }
        public int DynamicHeight { get; set; }
        public Point DynamicLocation { get; set; }

        public CleverLabel()
            : base()
        {
            this.Font = U.UnicodeFont1;
            this.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
            this.CausesValidation = false;
        }

        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                SuspendLayout();
                base.Text = FormatText(value);
                this.Width = this.PreferredWidth;
                this.Height = this.PreferredHeight;
                this.DynamicWidth = this.Width;
                this.DynamicHeight = this.Height;
                ResumeLayout();
            }
        }


        private string FormatText(string text)
        {
            string t1 = EliminateSpaces(text);
            string t10 = EliminateZeroMembers(t1);
            string t2 = EliminatePlusPlus(t10);
            string t3 = EliminatePlusMinus(t2);
            string t4 = EliminateMinusMinus(t3);
            string t5 = EliminateMinusPlus(t4);
            string t6 = EliminatePlusOnStart(t5);
            string t7 = EliminatePlusAfterBracket(t6);
            string t8 = EliminatePM(t7);
            string t9 = EliminateSignAfterTimesWithoutBracket(t8);
            string retval = AddSpacesExceptStartEndAndAfterBracket(t9);
            return EliminateQuestionMark(retval);
        }

        private string EliminateQuestionMark(string text)
        {
            if (text.Length > 0 && text[0] == '?') // naleznu-li '+', pak jej odstraním
                return text.Substring(1);
            return text;
        }

        private string EliminateZeroMembers(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (i > 0 && !(text[i - 1] >= '0' && text[i - 1] <= '9') && text[i - 1] != ',' &&
                    i < text.Length - 1 && text[i] == '0' &&
                    (text[i + 1] == 'x' || text[i + 1] == 'y' || text[i + 1] == 'z'|| text[i + 1] == 'u'|| text[i + 1] == 'v'))
                {
                    retval = retval.Substring(0, retval.Length - 1); // odstraneni znamenka pred nulou
                    i = Math.Max(text.IndexOf('+', i), i + 1);  // preskocim promennou za nulou, scitance jsou oddeleny +.
                }
                else { retval += text[i]; }
            }
            return retval;
        }

        //private string EliminateZeroMembers(string text)
        //{
        //    string retval = "";
        //    for (int i = 0; i < text.Length; i++)
        //    {
        //        if (i > 0 && !(text[i - 1] >= '0' && text[i - 1] <= '9') && text[i - 1] != ',' &&
        //            i < text.Length - 1 && text[i] == '0' &&
        //            (text[i + 1] == 'x' || text[i + 1] == 'y' || text[i + 1] == 'z'))
        //        {
        //            if (retval.Length > 0 && (retval[retval.Length - 1] == '-' || retval[retval.Length - 1] == '+'))
        //                retval = retval.Substring(0, retval.Length - 1); // odstraneni znamenka pred nulou
        //            i = Math.Max(text.IndexOf('+', i), i + 1);  // preskocim promennou za nulou, scitance jsou oddeleny +.
        //        }
        //        else { retval += text[i]; }
        //    }
        //    return retval;
        //}

        private string EliminateSpaces(string text)
        {
            return text.Replace(" ", "");
        }

        private string EliminatePlusOnStart(string text)
        {
            if (text.Length == 1 && text[0] == '+')
                return text;
            if (text.Length > 0 && text[0] == '+' && text.Length > 1 && text[1] != '(') // naleznu-li '+', pak jej odstraním
                return text.Substring(1);
            return text;
        }

        private string EliminatePlusAfterBracket(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == '(' && i + 1 < text.Length && text[i + 1] == '+')
                {
                    retval += "(";
                    i++; // preskocim plus.
                }
                else retval += text[i];
            }
            return retval;
        }

        private string AddSpacesExceptStartEndAndAfterBracket(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if ((text[i] == '+' || text[i] == '-') // je plus nebo minus, nebo rovna se nebo plusMinus
                    || text[i] == '=' || text[i] == U.PM[0] || text[i] == U.Belong[0])
                {
                    if (i > 0 && (text[i - 1] == 'E' || text[i - 1] == 'e'))
                        retval += text[i];
                    else
                    if (i > 0 && text[i - 1] != '(') // ale ne po zavorce
                        retval += " " + text[i] + " ";
                    else retval += text[i] + " ";
                }
                else retval += text[i];
            }
            if (retval.Length > 0 && retval[0] == ' ')
                retval = retval.Substring(1);
            if (retval.Length > 0 && retval[retval.Length - 1] == ' ')
                retval = retval.Substring(0, retval.Length - 1);
            return retval;
        }

        private string EliminatePM(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == U.PM[0] && i + 1 < text.Length && (text[i + 1] == '-' || text[i + 1] == '+'))
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += U.PM;
                }
                else retval += text[i];
            }
            return retval;
        }

        private string EliminatePlusMinus(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == '+' && i + 1 < text.Length && text[i + 1] == '-')
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += "-";
                }
                else retval += text[i];
            }
            return retval;
        }

        private string EliminatePlusPlus(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == '+' && i + 1 < text.Length && text[i + 1] == '+')
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += "+";
                }
                else retval += text[i];
            }
            return retval;
        }

        private string EliminateMinusMinus(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == '-' && i + 1 < text.Length && text[i + 1] == '-')
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += "+";
                }
                else retval += text[i];
            }
            return retval;
        }

        private string EliminateMinusPlus(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == '-' && i + 1 < text.Length && text[i + 1] == '+')
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += "-";
                }
                else retval += text[i];
            }
            return retval;
        }

        private string EliminateSignAfterTimesWithoutBracket(string text)
        {
            string retval = "";
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == U.Times[0] && i + 1 < text.Length && (text[i + 1] == '+'))
                {
                    i++; // preskocim minus (cily to druhe znamenko)
                    retval += U.Times;
                }
                else retval += text[i];
            }
            return retval;
        }


        /// <summary>
        /// Nefunguje, spatne pracuje se zbytkem, jinak OK.
        /// </summary>
        private static List<CleverLabel> FormatOnSpecificLineWidth(string text, int lineWidth)
        {
            CleverLabel temp = new CleverLabel();
            temp.Text = text;

            List<CleverLabel> retval = new List<CleverLabel>();
            text = temp.Text;
            string line = "";
            string[] words = text.Split(' ');
            Label helpLabel = new Label();
            helpLabel.AutoSize = true;

            foreach (string word in words)
            {
                line += word + " ";
                helpLabel.Text = line;
                if (helpLabel.PreferredWidth <= lineWidth)
                    continue;
                else
                {
                    line = line.Substring(0, line.Length - word.Length);
                    CleverLabel label = new CleverLabel();
                    label.Text = line;
                    line = word;
                    retval.Add(label);
                }
            }
            return retval;
        }



        // vraci bod udavajici sirku a vysku ExpMemberu v graphics jednotkach
        // tzn. pri kresleni stejnym fontem je text vetsi.
        public Size DrawToToolTip(Graphics g, System.Drawing.PointF location)
        {
            Brush brush = Brushes.Black;
            SizeF textSize = g.MeasureString(this.Text, this.Font);
            g.DrawString(this.Text, this.Font, brush, location);
            return new Size((int)textSize.Width, (int)textSize.Height);
        }


        public Size GetGraphicsSize(Graphics g)
        {
            SizeF textSize = g.MeasureString(this.Text, this.Font);
            return new Size((int)textSize.Width, (int)textSize.Height);
        }

        public void SetRealDynamicSize()
        {
            this.Location = DynamicLocation;
        }
    }
}
