﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Planar;
using System;
using Geometry.GraphicObjects.Intersections;
using System.Drawing;

namespace Geometry.GGraphic.Controls.ElementaryControls
{
    public partial class ListViewContextMenu : ContextMenuStrip
    {
        private ToolStripMenuItem _hide = new ToolStripMenuItem("Skrýt výběr");
        private ToolStripMenuItem _show = new ToolStripMenuItem("Zobrazit výběr");
        private ToolStripMenuItem _delete = new ToolStripMenuItem("Smazat výběr");
        private ToolStripMenuItem _name = new ToolStripMenuItem("Přejmenovat výběr");        
        private ToolStripMenuItem _color = new ToolStripMenuItem("Přebarvit výběr");
        private ToolStripMenuItem _intersection = new ToolStripMenuItem("Přidat průnik výběru");
        private ToolStripMenuItem _hideAxes = new ToolStripMenuItem("Skrýt osy výběru");
        private ToolStripMenuItem _showAxes = new ToolStripMenuItem("Zobrazit osy výběru");
        private Form1 _form;
        List<GraphicObject> _selectedGraphicObjects;

        public ListViewContextMenu(Form1 form)
        {
            InitializeComponent();
            _form = form;
            this.Items.Add(_hide);
            this.Items.Add(_show);
            this.Items.Add(new ToolStripSeparator());
            this.Items.Add(_delete);
            this.Items.Add(new ToolStripSeparator());
            this.Items.Add(_name);            
            this.Items.Add(_color);
            this.Items.Add(new ToolStripSeparator());
            this.Items.Add(_intersection);
            this.Items.Add(new ToolStripSeparator());
            this.Items.Add(_hideAxes);
            this.Items.Add(_showAxes);            
            _hide.Click += new EventHandler(HideClick);
            _show.Click += new EventHandler(ShowClick);
            _delete.Click += new EventHandler(DeleteClick);
            _name.Click += new EventHandler(NameClick);
            _color.Click += new EventHandler(ColorClick);
            _intersection.Click += new EventHandler(IntersectionClick);
            _hideAxes.Click += new EventHandler(HideAxesClick);
            _showAxes.Click += new EventHandler(ShowAxesClick);
        }

        public void EnableValidOperations(List<GraphicObject> selectedGraphicObjects) 
        {
            _selectedGraphicObjects = selectedGraphicObjects;

            _hide.Enabled = AreObjectsHideAble(selectedGraphicObjects);
            _show.Enabled = _hide.Enabled;
            _delete.Enabled = AreObjectsDeleteAble(selectedGraphicObjects);
            _name.Enabled = AreObjectsNameAble(selectedGraphicObjects);
            _color.Enabled = AreObjectsColorAble(selectedGraphicObjects);
            _intersection.Enabled = AreObjectsIntersectionAble(selectedGraphicObjects);
            _hideAxes.Enabled = AreObjectsAxesHideAble(selectedGraphicObjects);
            _showAxes.Enabled = _hideAxes.Enabled;
        }

        private bool AreObjectsHideAble(List<GraphicObject> graphicObjects)
        {
            return true;
        }
        private bool AreObjectsDeleteAble(List<GraphicObject> graphicObjects)
        {
            foreach (GraphicObject graphicObject in graphicObjects)
                if (graphicObject is Axes)
                    return false;
            return true;
        }
        private bool AreObjectsNameAble(List<GraphicObject> graphicObjects)
        {
            return graphicObjects != null && graphicObjects.Count == 1 && !(graphicObjects[0] is Axes);
        }
        private bool AreObjectsColorAble(List<GraphicObject> graphicObjects)
        {
            foreach (GraphicObject graphicObject in graphicObjects)
                if (graphicObject is Axes)
                    return false;
            return true;
        }
        private bool AreObjectsIntersectionAble(List<GraphicObject> graphicObjects)
        {
            if (graphicObjects.Count != 2)
                return false;
            foreach (GraphicObject graphicObject in graphicObjects)
                if (graphicObject is Axes || graphicObject is IIntersection)
                    return false;
            return true;
        }
        private bool AreObjectsAxesHideAble(List<GraphicObject> graphicObjects)
        {
            foreach (GraphicObject graphicObject in graphicObjects)
                if (!(graphicObject is WithParametricEquationObject))
                    return false;
            return true;
        }

        private void HideClick(object sender, EventArgs e)
        {
            _form.GetManipulation().HideObjectInvoked(_selectedGraphicObjects, true);
        }
        private void DeleteClick(object sender, EventArgs e)
        {
            _form.GetManipulation().DeleteObjectInvoked(_selectedGraphicObjects);
        }
        private void ShowClick(object sender, EventArgs e) 
        {
            _form.GetManipulation().HideObjectInvoked(_selectedGraphicObjects, false);
        }
        private void NameClick(object sender, EventArgs e)
        {
            if (_selectedGraphicObjects != null && _selectedGraphicObjects.Count == 1)
                _form.GetManipulation().ChangeNameInvoked(_selectedGraphicObjects[0]);
        }
        private void ColorClick(object sender, EventArgs e)
        {
            _form.GetManipulation().ChangeColorInvoked(_selectedGraphicObjects);
        }
        private void IntersectionClick(object sender, EventArgs e)
        {
            _form.GetManipulation().AddIntersectionInvoked(_selectedGraphicObjects);
        }
        private void HideAxesClick(object sender, EventArgs e)
        {
            _form.GetManipulation().HideObjectAxesInvoked(_selectedGraphicObjects, true);
        }
        private void ShowAxesClick(object sender, EventArgs e)
        {
            _form.GetManipulation().HideObjectAxesInvoked(_selectedGraphicObjects, false);
        }
    }
}
