﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using Geometry.GraphicObjects;

namespace Geometry.GGraphic.Forms.Detail
{
    public class EquationLineControl : PictureBox
    {
        GraphicObject _graphicObject = null;
        CombinedControl _combinedControl;
        public int FakeWidth { get; set; }
        public int FakeHeight { get; set; }
        public Point FakeLocation { get; set; }

        /// <summary>
        /// Vytvoří jednotlive prvky rovnice a ty vloží do seznamu objektů.
        /// Vytváří obecnou rovnici tzn. neobsahuje konkretni hodnoty.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="eqExpr"></param>
        public EquationLineControl()
        {
            _combinedControl = new CombinedControl();
            //this.BackColor = System.Drawing.Color.Red;
        }

        public void SetContents(GraphicObject graphicObject, string expression)
        {
            if (expression == null)
            {
                this.FakeWidth = 0;
                this.FakeHeight = 0;
                return;
            }
            _graphicObject = graphicObject;
            List<MemberControl> memberControls = MakeNewControls(expression);
            _combinedControl.SetContents(memberControls);
            this.FakeWidth = _combinedControl.DynamicWidth;
            this.FakeHeight = _combinedControl.DynamicHeight;
        }
        protected List<MemberControl> MakeNewControls(string expression)
        {
            ParserPair pair = new ParserPair(expression);
            while (pair.GetExpression() != "")
            {
                pair = AddNextMemberControl(pair);
            }
            return pair.GetMemberControls();
        }
        private ParserPair AddNextMemberControl(ParserPair pair) 
        {
            string expression = SeparateMemberControlExpression(pair.GetExpression());
            pair.SetExpression(pair.GetExpression().Substring(expression.Length));
            MemberControl memberControl = ConstructNewMemberControl(expression);
            pair.AddMemberControl(memberControl);
            return pair;
        }

        private string SeparateMemberControlExpression(string expression)
        {
            string retval = "";
            int i = 0;
            if (expression.Length < 1)
                return "";
            if (expression[0] == ';')
                return ";";
            if (expression[0] != 'L' && expression[0] != 'F' && 
                 expression[0] != 'E' && expression[0] != 'S' && expression[0] != 'P')
            {
                return "";
            }
            int openingBrackets = 0;
            int closingBrackets = 0;
            while (true)
            {
                if (expression[i] == '[')
                    openingBrackets++;
                if (expression[i] == ']')
                    closingBrackets++;
                retval += expression[i++];
                if (openingBrackets != 0 && openingBrackets - closingBrackets == 0)
                    break;
            }
            return retval;
        }
        private MemberControl ConstructNewMemberControl(string expression)
        {
            switch (expression[0]) 
            {
                case 'L': return ConstructNewLabelMember(expression);
                case 'F': return ConstructNewFracMember(expression);
                case 'P': return ConstructNewParaboloidMember(expression);
                case 'S': return ConstructNewSqrtMember(expression);
                case 'E': return ConstructNewExpMember(expression);
                default: throw new Exception("Syntaktická chyba předpisu matematického výrazu.");
            }
        }
        private LabelControl ConstructNewLabelMember(string expression)
        {
            expression = expression.Substring(2); // odstraneni L[
            string text = SeparateMemberControlArgument(expression);
            return new LabelControl(text);
        }
        private FracControl ConstructNewFracMember(string expression)
        {
            expression = expression.Substring(2, expression.Length - 3); // odstranim S[ a ]
            List<MemberControl> numeratorControls = new List<MemberControl>();            
            List<MemberControl> denominatorControls = new List<MemberControl>();
            string subExpression;
            while (true)
            {
                subExpression = SeparateMemberControlExpression(expression);
                expression = expression.Substring(subExpression.Length);
                if (subExpression == ";")
                    break;
                numeratorControls.Add(ConstructNewMemberControl(subExpression));
            }
            while (true)
            {
                subExpression = SeparateMemberControlExpression(expression);
                expression = expression.Substring(subExpression.Length);
                if (subExpression == "")
                    break;
                denominatorControls.Add(ConstructNewMemberControl(subExpression));
            }
            CombinedControl numeratorControl = new CombinedControl(numeratorControls);
            CombinedControl denominatorControl = new CombinedControl(denominatorControls);
            return new FracControl(numeratorControl, denominatorControl);
        }
        private ExpControl ConstructNewExpMember(string expression)
        {
            expression = expression.Substring(2); // odstraneni E[
            string numeratorText = SeparateMemberControlArgument(expression);
            expression = expression.Substring(numeratorText.Length + 1); // odstanim ;
            string denominatorText = SeparateMemberControlArgument(expression);

            return new ExpControl(numeratorText, denominatorText);
        }
        private SqrtControl ConstructNewSqrtMember(string expression)
        {
            expression = expression.Substring(2, expression.Length - 3); // odstranim S[ a ]
            List<MemberControl> underSqrtControls = new List<MemberControl>();
            string subExpression;
            while (true)
            {
                subExpression = SeparateMemberControlExpression(expression);
                expression = expression.Substring(subExpression.Length);
                if (subExpression == "")
                    break;
                underSqrtControls.Add(ConstructNewMemberControl(subExpression));
            }
            CombinedControl underSqrtControl = new CombinedControl(underSqrtControls);
            return new SqrtControl(underSqrtControl);
        }
        private ParaboloidControl ConstructNewParaboloidMember(string expression) 
        {
            expression = expression.Substring(2); // odstraneni P[
            string u = SeparateMemberControlArgument(expression);
            expression = expression.Substring(u.Length + 1); // odstanim ;
            string a = SeparateMemberControlArgument(expression);
            expression = expression.Substring(a.Length + 1); // odstranim ;
            string p = SeparateMemberControlArgument(expression);
            return new ParaboloidControl(u, a, p);
        }

        /// <summary>
        /// Z vyrazu vyextrahuje clen, ktery zacina na from a konci bud zavorkou nebo carkou.
        /// </summary>
        /// <returns></returns>
        private string SeparateMemberControlArgument(string expression)
        {
            string retval = "";
            int i = 0;
            while (expression[i] != ';' && expression[i] != ']')
            {
                retval += expression[i++];
            }
            return retval;
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // EquationLineControl
            // 
            this.BackColor = System.Drawing.SystemColors.ButtonShadow;
            this.Name = "EquationLineControl";
            //this.Paint += new System.Windows.Forms.PaintEventHandler(this.EquationLineControl_Paint);
            this.ResumeLayout(false);

        }


        public Size GetToolTipSize(Graphics graphics) 
        {
            return _combinedControl.GetGraphicsSize(graphics);
        }
        
        public Size DrawToToolTip(Graphics graphics) 
        {
            return _combinedControl.Draw(graphics, new Point(12 + FakeLocation.X, 12 + FakeLocation.Y));
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            //base.OnPaint(e);
            _combinedControl.Draw(e.Graphics, new Point(0,0));
        }

        public void SetSize()
        {
            this.Bounds = new Rectangle(FakeLocation.X, FakeLocation.Y, FakeWidth, FakeHeight);            
        }
    }
}
