﻿using System;
using Geometry.GraphicObjects.Equations;
using GMath; 

namespace Geometry.GGraphic.Forms.Detail.FormatString
{
    public class ParametricEquationsFormatString:IFormatString
    {
        private string _xEquationInitialFormatString;
        private string _yEquationInitialFormatString;
        private string _zEquationInitialFormatString;

        private string _xEquationGeneralFormatString;
        private string _yEquationGeneralFormatString;
        private string _zEquationGeneralFormatString;

        private string _xEquationSubstitutedFormatString;
        private string _yEquationSubstitutedFormatString;
        private string _zEquationSubstitutedFormatString;

        public ParametricEquationsFormatString(string xEquationInitialFormatString, 
                                                string yEquationInitialFormatString, 
                                                string zEquationInitialFormatString) 
        {
            _xEquationInitialFormatString = xEquationInitialFormatString;
            _yEquationInitialFormatString = yEquationInitialFormatString;
            _zEquationInitialFormatString = zEquationInitialFormatString;
            _xEquationGeneralFormatString = base.IgnoreSpecialCharacters(xEquationInitialFormatString);
            _yEquationGeneralFormatString = base.IgnoreSpecialCharacters(yEquationInitialFormatString);
            _zEquationGeneralFormatString = base.IgnoreSpecialCharacters(zEquationInitialFormatString);        
        }

        public ParametricEquationsFormatString(string xEquationInitialFormatString,
                                                string yEquationInitialFormatString)
        {
            _xEquationInitialFormatString = xEquationInitialFormatString;
            _yEquationInitialFormatString = yEquationInitialFormatString;
            _xEquationGeneralFormatString = base.IgnoreSpecialCharacters(xEquationInitialFormatString);
            _yEquationGeneralFormatString = base.IgnoreSpecialCharacters(yEquationInitialFormatString);
        }



        public void SubstituteToFormatStrings(ParametricEquation parametricEquation, CenterParametrization center)
        {
            _xEquationSubstitutedFormatString = SubstituteToFormatString(_xEquationInitialFormatString, parametricEquation, center);
            _yEquationSubstitutedFormatString = SubstituteToFormatString(_yEquationInitialFormatString, parametricEquation, center);
            if (_zEquationGeneralFormatString != null)
                _zEquationSubstitutedFormatString = SubstituteToFormatString(_zEquationInitialFormatString, parametricEquation, center);
        }

        // vykricnik znamena substituci
        protected string SubstituteToFormatString
            (string expression, ParametricEquation parametricEquation, CenterParametrization center)
        {
            string retval = "";
            double temp = 0;
            for (int i = 0; i < expression.Length; i++)
            {
                if (expression[i] == '!')
                {
                    switch (expression[++i])
                    {
                        case 'a': temp = parametricEquation.GetA(); break;
                        case 'b': temp = parametricEquation.GetB(); break;
                        case 'c': temp = parametricEquation.GetC(); break;
                        case 'p': temp = parametricEquation.GetP(); break;
                        case 'm': temp = center.GetX().GetConstant(); break;
                        case 'n': temp = center.GetY().GetConstant(); break;
                        case 'o': temp = center.GetZ().GetConstant(); break;
                        case 'u': temp = GetU(expression[++i], parametricEquation); break;
                        case 'v': temp = GetV(expression[++i], parametricEquation); break;
                        default: throw new Exception("Je pozadovana substituce za neznamy znak.");
                    }
                    retval += Program.GetOutputFormatedForCleverLabel(temp);
                }
                else retval += expression[i];
            }
            return retval;
        }

        private double GetU(char index, ParametricEquation parametricEquation)
        {
            switch (index)
            {
                case U.Index1Char: return parametricEquation.GetUx();
                case U.Index2Char: return parametricEquation.GetUy();
                case U.Index3Char: return parametricEquation.GetUz();
                default: throw new Exception("Je pozadovana substituce za neznamou sekvenci znaků.");
            }
        }
        private double GetV(char index, ParametricEquation parametricEquation)
        {
            switch (index)
            {
                case U.Index1Char: return parametricEquation.GetVx();
                case U.Index2Char: return parametricEquation.GetVy();
                case U.Index3Char: return parametricEquation.GetVz();
                default: throw new Exception("Je pozadovana substituce za neznamou sekvenci znaků.");
            }
        }




        public string GetXGeneralFormatString() { return _xEquationGeneralFormatString; }
        public string GetYGeneralFormatString() { return _yEquationGeneralFormatString; }
        public string GetZGeneralFormatString() { return _zEquationGeneralFormatString; }

        public string GetXSubstitutedFormatString() { return _xEquationSubstitutedFormatString; }
        public string GetYSubstitutedFormatString() { return _yEquationSubstitutedFormatString; }
        public string GetZSubstitutedFormatString() { return _zEquationSubstitutedFormatString; }
    }
}
