﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class CombinedControl : MemberControl
    {
        private List<MemberControl> _memberControls;

        public CombinedControl() 
        {
            _memberControls = new List<MemberControl>();
            DynamicWidth = 0;
            DynamicHeight = 0;
        }
        public CombinedControl(List<MemberControl> memberControls) 
        {
            SetContents(memberControls);
        }

        /// <summary>
        /// Prvky jsou zarovnány na střed. 
        /// </summary>
        public void SetContents(List<MemberControl> memberControls)
        {
            if (memberControls == null)
            {
                _memberControls = new List<MemberControl>();
                DynamicWidth = 0;
                DynamicHeight = 0;
                return;
            }
            _memberControls = memberControls;
            int maxHeight = 0;

            foreach (MemberControl memberControl in memberControls) 
            {
                maxHeight = Math.Max(maxHeight, memberControl.DynamicHeight); // najdu nejvyssi prvek
            }
            int nextMemberControlXLocation = 0;
            foreach(MemberControl memberControl in memberControls)
            {
                memberControl.DynamicLocation = 
                    new Point(nextMemberControlXLocation, (maxHeight - memberControl.DynamicHeight) / 2); // zarovnano na stred
                nextMemberControlXLocation += memberControl.DynamicWidth;
            }
            this.DynamicWidth = nextMemberControlXLocation;
            this.DynamicHeight = maxHeight;
        }



        public override Size Draw(Graphics g, Point parentLocation)
        {
            foreach (MemberControl memberControl in _memberControls)
            {
                memberControl.Draw(g, new Point(parentLocation.X + DynamicLocation.X, parentLocation.Y + DynamicLocation.Y));
            }
            return new Size(DynamicWidth, DynamicHeight);
        }
    }
}
