﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using Geometry.GGraphic.Controls.ElementaryControls;

namespace Geometry.GGraphic.Forms.Detail
{
    public class SqrtControl : MemberControl
    {
        private Label _sqrtChar;
        private Size _sqrtCharSize;
        private Font _sqrtCharFont;
        private MemberControl _underSqrtControl;

        private int _sqrtLineOverSize = 2;
        //private int _sqrtLineHeight = 1;       
        private Color _sqrtLineColor = Color.Black;

        /// <summary>
        /// Vytvorí obecny clen u^2*b^2/a^2
        /// 
        /// </summary>
        /// <param name="x"> [x,y,z] </param>
        public SqrtControl(MemberControl underSqrtControl)
        {
            SetContents(underSqrtControl);
        }


        public void SetContents(MemberControl underSqrtControl)
        {
            _underSqrtControl = underSqrtControl;                        
            
            if (underSqrtControl.DynamicWidth * underSqrtControl.DynamicHeight == 0)
            {
                this.DynamicWidth = 0;
                this.DynamicHeight = 0;
                return;
            }

            _sqrtChar = new Label();
            _sqrtChar.Font =
                new Font(FontFamily.GenericSansSerif,
                        underSqrtControl.DynamicHeight , FontStyle.Regular, GraphicsUnit.Pixel);
            _sqrtChar.TextAlign = ContentAlignment.TopLeft;
            _sqrtChar.Text = U.Sqrt;
            _sqrtChar.AutoSize = true;
            _sqrtChar.Width = (int)(_sqrtChar.PreferredWidth);
            _sqrtChar.Height = (int)(_sqrtChar.PreferredHeight);
            _sqrtChar.Location = new Point(0, -2);
            _sqrtCharFont = _sqrtChar.Font;
            _sqrtCharSize = new Size(_sqrtChar.Width, _sqrtChar.Height);

            Point sqrtLineLocation = new Point(_sqrtChar.Location.X + _sqrtChar.Width * 2/3, 0);
            int sqrtLineWidth = underSqrtControl.DynamicWidth + 2;

            underSqrtControl.DynamicLocation =
                new Point(sqrtLineLocation.X, (int)(_sqrtChar.Height - _underSqrtControl.DynamicHeight)/2 + 2);

            this.DynamicWidth = sqrtLineLocation.X + sqrtLineWidth + _sqrtLineOverSize;
            this.DynamicHeight = _sqrtChar.Location.Y + _sqrtChar.Height + 4;
        }

        public override Size Draw(Graphics g, Point parentLocation)
        {
            Brush brush = Brushes.Black;

            int x = parentLocation.X + DynamicLocation.X;
            int y = parentLocation.Y + DynamicLocation.Y;
            _underSqrtControl.Draw(g, new Point(x, y));
            g.DrawString(_sqrtChar.Text, _sqrtChar.Font, brush, x + _sqrtChar.Location.X , y + _sqrtChar.Location.Y);
            g.DrawLine(new Pen(brush),
                new PointF(x + (int)(_sqrtCharSize.Width * 2.0 / 3.0 - 1), y),
                new PointF(x + (int)(_sqrtCharSize.Width * 2.0 / 3.0) + _underSqrtControl.DynamicWidth, y));
            return new Size(DynamicWidth, DynamicHeight);
        }
    }
}
