﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Drawing;

namespace Geometry.GGraphic.Controls.GroupBoxes
{
    public partial class ConstantGroupBox : GroupBox
    {
        private GroupBoxLayoutStyle _layoutStyle;
        private Size _minimumSize;

        /// <summary>
        /// ConstantTabPage nastaví všem svým groupBoxům a jejim kontrolům velikosti 
        /// a pozice hned po prvním vytvoření a pak už je nemění.
        /// 
        /// Až se přidají všechny groupboxi s controly, zavolat SetConstantSize.
        /// - Tato metoda projde každý groupBox a zavolá na něm SetMinimumConstantSize, zárověň jim nastaví pozici.
        /// - Potom nastaví všem groupBoxům stejnou velikost zavoláním SetConstantSize, tím se posunou i jejich controly.
        /// Nakonec si nastaví Width a Height, které čte detail.
        /// </summary>
        public ConstantGroupBox()
        {
            _minimumSize = new Size(300, 100);
        }

        /// <summary>
        /// ConstantTabPage nastaví všem svým groupBoxům a jejim kontrolům velikosti 
        /// a pozice hned po prvním vytvoření a pak už je nemění
        /// 
        /// Až se přidají všechny groupboxi s controly, zavolat SetConstantSize.
        /// - Tato metoda projde každý groupBox a zavolá na něm SetMinimumConstantSize, zárověň jim nastaví pozici.
        /// - Potom nastaví všem groupBoxům stejnou velikost zavoláním SetConstantSize, tím se posunou i jejich controly.
        /// Nakonec si nastaví Width a Height, které čte detail.
        /// </summary>
        public ConstantGroupBox(Size minimumSize)
            : base()
        {
            this._minimumSize = minimumSize;
        }

        public void SetContents(List<Control> contents, GroupBoxLayoutStyle layoutStyle)
        {
            this._layoutStyle = layoutStyle;
            foreach (Control c in contents)
                this.Controls.Add(c);
            SetConstantMimimumSize();
        }

        /// <summary>
        /// Předpokládá uložení prvků jako při psaní!
        /// </summary>
        public void SetConstantMimimumSize()
        {
            if (this._layoutStyle == GroupBoxLayoutStyle.Equations)
                SetControlsEquationMinimumSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.TwoControlsInRowFar)
                SetTwoControlsInRowFarMinimumSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.ControlOnLeft)
                SetControlOnLeftMinimumSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.ControlInCenter)
                SetControlInCenterMinimumSize();
        }
        public void SetControlsEquationMinimumSize()
        {
            if (this.Controls.Count != 3)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control A = this.Controls[0];
            Control B = this.Controls[1];
            Control C = this.Controls[2];
            this.Width =
                LayoutProperties.GroupBoxSideSpace * 2 +
                A.Width + B.Width + C.Width +
                LayoutProperties.ControlsSideSpace * 2;
            this.Height = Math.Max(Math.Max(A.Height, B.Height), C.Height) +
                LayoutProperties.GroupBoxTopSpace + LayoutProperties.GroupBoxBottomSpace;
            this.Width = Math.Max(_minimumSize.Width, this.Width);
        }

        private void SetControlInCenterMinimumSize()
        {
            if (this.Controls.Count != 1)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control control = this.Controls[0];
            int height = control.Height;
            int width = LayoutProperties.GroupBoxSideSpace * 2 + control.Width;
            this.Height = LayoutProperties.GroupBoxTopSpace + LayoutProperties.GroupBoxBottomSpace + height;
            this.Width = Math.Max(_minimumSize.Width, width);
        }
        private void SetControlOnLeftMinimumSize()
        {
            SetControlInCenterMinimumSize();
        }
        private void SetTwoControlsInRowFarMinimumSize()
        {
            if (this.Controls.Count != 2)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control A = this.Controls[0];
            Control B = this.Controls[1];

            int width =
                 LayoutProperties.GroupBoxSideSpace * 2 + LayoutProperties.ControlsSideSpace + A.Width + B.Width;
            int height =
                 Math.Max(A.Height, B.Height) +
                 LayoutProperties.GroupBoxTopSpace + LayoutProperties.GroupBoxBottomSpace;
            this.Width = Math.Max(_minimumSize.Width, width);
            this.Height = height;
        }


        public void SetConstantWidth(int width)
        {
            SuspendLayout();
            this.Width = width;
            if (this._layoutStyle == GroupBoxLayoutStyle.Equations)
                SetControlsEquationConstantSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.TwoControlsInRowFar)
                SetControlsInRowFarConstantSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.ControlOnLeft)
                SetControlOnLeftConstantSize();
            if (this._layoutStyle == GroupBoxLayoutStyle.ControlInCenter)
                SetControlInCenterConstantSize();
            ResumeLayout(false);
        }
        private void SetControlsEquationConstantSize()
        {
            if (this.Controls.Count != 3)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control A = this.Controls[0];
            Control B = this.Controls[1];
            Control C = this.Controls[2];

            A.Location = new Point(LayoutProperties.GroupBoxSideSpace, ComputeYCoordinateToBeInCenter(A));
            C.Location = new Point(this.Width - C.Width - LayoutProperties.GroupBoxSideSpace, ComputeYCoordinateToBeInCenter(C));
            B.Location = new Point(C.Location.X - B.Width - LayoutProperties.ControlsSideSpace, ComputeYCoordinateToBeInCenter(B));
        }
        private void SetControlInCenterConstantSize()
        {
            if (this.Controls.Count != 1)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control control = this.Controls[0];
            int x = (this.Width - control.Width) / 2;
            control.Location = new Point(x, LayoutProperties.GroupBoxTopSpace);
        }
        private void SetControlOnLeftConstantSize()
        {
            if (this.Controls.Count != 1)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control control = this.Controls[0];
            int x = (this.Width - control.Width) / 2;
            control.Location = new Point(LayoutProperties.GroupBoxSideSpace, LayoutProperties.GroupBoxTopSpace);
        }
        private void SetControlsInRowFarConstantSize()
        {
            if (this.Controls.Count != 2)
                throw new Exception("Špatný počet prvků v groupBoxu.");

            Control A = this.Controls[0];
            Control B = this.Controls[1];

            A.Location =
                new Point(LayoutProperties.GroupBoxSideSpace, ComputeYCoordinateToBeInCenter(A));
            B.Location =
                new Point(this.Width - B.Width - LayoutProperties.GroupBoxSideSpace, ComputeYCoordinateToBeInCenter(B));
        }


        private int ComputeYCoordinateToBeInCenter(Control control)
        {
            int height = control.Height;
            return (this.Height
                - LayoutProperties.GroupBoxTopSpace
                - LayoutProperties.GroupBoxBottomSpace) / 2
                - height / 2
                + LayoutProperties.GroupBoxTopSpace;
        }
        private int ComputeXCoordinateToBeInCenter(Control control)
        {
            return (this.Width - control.Width) / 2;
        }
    }
}
