﻿using System;
using System.Windows.Forms;
using System.Drawing;
using Geometry.GGraphic.Forms.Detail.Layout; 
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class CenterKoeficientsControl : IKoeficientsControl
    {

        CenterParametrization _centerParamerization;

        public CenterKoeficientsControl(Dimension dimension) 
        { 
            InitializeComponent();
            labelAndTextBoxControlM.SetLabelText("m:");
            labelAndTextBoxControlN.SetLabelText("n:");
            labelAndTextBoxControlO.SetLabelText("o:");
            SetLayout(dimension);
        }

        public CenterKoeficientsControl(CenterParametrization center, Dimension dimension)
        {
            InitializeComponent();
            labelAndTextBoxControlM.SetLabelText("m:");
            labelAndTextBoxControlN.SetLabelText("n:");
            labelAndTextBoxControlO.SetLabelText("o:");
            SetLayout(dimension);         
            SetContents(center);
        }

        private void SetLayout(Dimension dimension)
        {
            SuspendLayout();

            labelAndTextBoxControlM.Location = new Point(0, 0);
            labelAndTextBoxControlN.Location =
                new Point(labelAndTextBoxControlM.Width + LayoutProperties.TextSideSpace, 0);
            labelAndTextBoxControlO.Location =
                new Point(labelAndTextBoxControlN.Location.X + labelAndTextBoxControlN.Width +
                    LayoutProperties.TextSideSpace, 0);

            NumericalLabelAndTextBoxControl temp = labelAndTextBoxControlM;
            labelAndTextBoxControlN.Enabled = false;
            labelAndTextBoxControlO.Enabled = false;
            if (dimension == Dimension.TWO)
            {
                temp = labelAndTextBoxControlN;
                labelAndTextBoxControlN.Enabled = true;
            }
            if (dimension == Dimension.THREE)
            {
                temp = labelAndTextBoxControlO;
                labelAndTextBoxControlN.Enabled = true;
                labelAndTextBoxControlO.Enabled = true;
            }

            ResumeLayout();
            this.Width = temp.Location.X + temp.Width;
            this.Height = temp.Location.Y + temp.Height;
        }

        public void SetContents(CenterParametrization center)
        {
            _centerParamerization = center;
            SuspendLayout();
            labelAndTextBoxControlM.SetTextBoxText(
                Program.GetOutputFormatedForTextBox(center.GetX().GetConstant()));
            labelAndTextBoxControlN.SetTextBoxText(
                Program.GetOutputFormatedForTextBox(center.GetY().GetConstant()));
            labelAndTextBoxControlO.SetTextBoxText(
                Program.GetOutputFormatedForTextBox(center.GetZ().GetConstant()));
            ResumeLayout();
        }

        public CenterParametrization ParseCenter()
        {
            try
            {
                double m, n, o;
                m = labelAndTextBoxControlM.GetTextBoxValueThrowError();
                n = labelAndTextBoxControlN.GetTextBoxValueThrowError();
                o = labelAndTextBoxControlO.GetTextBoxValueThrowError();
                CenterParametrization newCenter = new CenterParametrization(_centerParamerization);
                newCenter.GetX().SetConstant(m);
                newCenter.GetY().SetConstant(n);
                newCenter.GetZ().SetConstant(o);
                return newCenter;
            }
            catch (Exception)
            {
                MessageBox.Show("Zadané hodnoty nejsou v požadovaném tvaru.");
                return null;
            }
        }
    }
}
