﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Text.RegularExpressions;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    /// <summary>
    /// TextBox si hlídá maximální číselnou hodnotu, kterou má program specifikovanou.
    /// </summary>
    public partial class NumericalLabelAndTextBoxControl : UserControl
    {

        // proto abych mohl rici AddObjectDetailu, ze se zmenila hodnota taxtBoxu.
        IEditEquation _addObjectDialog;
        private string _previousString;

        public NumericalLabelAndTextBoxControl()
        {
            InitializeComponent();
            SuspendLayout();

            label1.Location = new System.Drawing.Point(0, (textBox1.Height - label1.PreferredHeight) / 2);
            //label1.Font = U.UnicodeFont1;
            textBox1.Location = new System.Drawing.Point(LayoutProperties.LabelLetterWidth, 0);
            toolTip1.SetToolTip(textBox1, 
                "Zadané hodnoty musí být z intervalů (-1000; -0,001), [0], (0,001; 1000)\n"+
                "Pokud je hodnota v textovém poli mimo tento rozsah, lze tuto hod-\n" +
                "notu pouze přiblížit k uvedenému rozsahu.");
            toolTip1.AutomaticDelay = 30;
            toolTip1.AutoPopDelay = 10000;
            toolTip1.IsBalloon = true;
            
            ResumeLayout();
            this.Width = textBox1.Location.X + textBox1.Width;
            this.Height = textBox1.Height;
        }



        public void SetAddObjectDialog(IEditEquation addObjectDialog) { _addObjectDialog = addObjectDialog; }




        public void DisableTextBox() { textBox1.Enabled = false; }

        public void EnableTextBox() { textBox1.Enabled = false; }



        public bool GetTextBoxEnabled() { return textBox1.Enabled; }



        //public string GetTextBoxText() { return textBox1.Text; }

        /// <summary>
        /// Pokud zadany text neodpovídá typu double, vrátí 0.
        /// </summary>
        /// <returns></returns>
        public double GetTextBoxValueIgnoreError() 
        {
            try
            {
                return double.Parse(textBox1.Text);
            }
            catch (Exception) { return 0; }
        }

        /// <summary>
        /// Pokud zadany text neodpovídá typu double, vyhodí výjimku.
        /// </summary>
        /// <returns></returns>
        public double GetTextBoxValueThrowError()
        {
            return double.Parse(textBox1.Text);
        }


        public void SetTextBoxText(string textBoxText) 
        {
            _previousString = textBoxText;
            textBox1.Text = textBoxText; 
        }




        public string GetLabelStrig() { return label1.Text; }

        public void SetLabelText(string labelText) 
        {
            // proto ze dialogy meni velikost po vytvoreni, takze hodnoty nastavene v kostruktoru jsou prepsany.
            this.Width = textBox1.Location.X + textBox1.Width;
            this.Height = textBox1.Height;

            label1.Text = labelText; 
        }


        public void SetTextBoxWidth(int width) 
        {
            textBox1.Width = width;

            this.Width = textBox1.Location.X + textBox1.Width;
            this.Height = textBox1.Height;
        }



        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            string text = textBox1.Text;
            bool error = false;
            double value = 0;
            try
            {
                value = double.Parse(text);
            }
            catch (Exception) 
            {
                if (!(textBox1.Text == "" || textBox1.Text == "-" || textBox1.Text == "+"))
                    error = true;
            }
            double oldValue = 0;
            try {
                oldValue = double.Parse(_previousString);
            }
            catch(Exception){}

            if (!Program.CheckInputNumberConstraints(value, oldValue))
                error = true;
            if (error)
            {
                System.Media.SystemSounds.Beep.Play();
                textBox1.Text = _previousString;
                textBox1.SelectionStart = textBox1.Text.Length;
                textBox1.SelectionLength = 0;
                return;
            }
            _previousString = textBox1.Text;
            if (_addObjectDialog != null)
                _addObjectDialog.TextBoxTextChanged();
        }
    }
}