﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GGraphic.Forms.AddForms;
using Geometry.GraphicObjects.ConicSections; 

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class ConicCommonKoefsControl : ICommonKoeficientsControl
    {

        public ConicCommonKoefsControl() { InitializeComponent(); }

        public ConicCommonKoefsControl(CommonEquation commonEquation)
            : base()
        {
            InitializeComponent();
            SuspendLayout();
            InitializeContents();
            SetKoeficients(commonEquation);
            SetLayout();
            ResumeLayout();
            Refresh();
        }


        private void InitializeContents()
        {
            labelAndTextBoxControlA.SetLabelText("A:");
            labelAndTextBoxControlB.SetLabelText("B:");
            labelAndTextBoxControlD.SetLabelText("D:");
            labelAndTextBoxControlG.SetLabelText("G:");
            labelAndTextBoxControlH.SetLabelText("H:");
            labelAndTextBoxControlL.SetLabelText("L:");
        }


        /// <summary>
        /// Nastaví každému LabelAndTextBox páru dialog, kterému se bude přeposílat událost změny textu.
        /// </summary>
        public override void SetAddObjectDialog(IEditEquation addObjectEquation)
        {
            labelAndTextBoxControlA.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlB.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlD.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlG.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlH.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlL.SetAddObjectDialog(addObjectEquation);
        }



        public void SetEquation(ConicEquation equation)
        {
            SetKoeficients(equation);
        }


        private void SetKoeficients(CommonEquation commonEquation)
        {
            commonEquation = (ConicEquation)commonEquation;
            labelAndTextBoxControlA.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetA()));
            labelAndTextBoxControlB.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetB()));
            labelAndTextBoxControlD.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetD()));
            labelAndTextBoxControlG.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetG()));
            labelAndTextBoxControlH.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetH()));
            labelAndTextBoxControlL.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetL()));
        }

        public override CommonEquation ParseCommonEquationIgnoreError()
        {
            double A, B,D, G, H, L;
            A = labelAndTextBoxControlA.GetTextBoxValueIgnoreError();
            B = labelAndTextBoxControlB.GetTextBoxValueIgnoreError();
            D = labelAndTextBoxControlD.GetTextBoxValueIgnoreError();
            G = labelAndTextBoxControlG.GetTextBoxValueIgnoreError();
            H = labelAndTextBoxControlH.GetTextBoxValueIgnoreError();
            L = labelAndTextBoxControlL.GetTextBoxValueIgnoreError();
            return new ConicEquation(A, B, D, G, H, L);
        }

        public override CommonEquation ParseCommonEquationThrowError()
        {
            double A, B, D, G, H, L;
            string errorKoef = "A";
            try
            {
                errorKoef = "A";
                A = labelAndTextBoxControlA.GetTextBoxValueThrowError();
                errorKoef = "B";
                B = labelAndTextBoxControlB.GetTextBoxValueThrowError();
                errorKoef = "D";
                D = labelAndTextBoxControlD.GetTextBoxValueThrowError();
                errorKoef = "G";
                G = labelAndTextBoxControlG.GetTextBoxValueThrowError();
                errorKoef = "H";
                H = labelAndTextBoxControlH.GetTextBoxValueThrowError();
                errorKoef = "L";
                L = labelAndTextBoxControlL.GetTextBoxValueThrowError();

                if (Program.IsEqualInTolerance(A, 0) &&
                    Program.IsEqualInTolerance(B, 0) &&
                    Program.IsEqualInTolerance(D, 0)) 
                {
                    ShowWarningMessage("Alespoň jeden z koeficientů A, B, C musí být nenulový.");
                    return null;
                }
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
            return new ConicEquation(A, B, D, G, H, L);
        }

        private void SetLayout()
        {
            int x = 0;
            int y = 0;
            int sideSpace = LayoutProperties.TextSideSpace;
            int topSpace = LayoutProperties.TextTopSpace;

            labelAndTextBoxControlA.Location = new Point(x, y);
            labelAndTextBoxControlB.Location =
                new Point(x += labelAndTextBoxControlA.Width + sideSpace, y);
            labelAndTextBoxControlD.Location =
                new Point(x += labelAndTextBoxControlB.Width + sideSpace, y);

            x = 0;
            y = labelAndTextBoxControlA.Location.Y + labelAndTextBoxControlA.Height + topSpace;

            labelAndTextBoxControlG.Location =
                new Point(x, y);
            labelAndTextBoxControlH.Location =
                new Point(x += labelAndTextBoxControlG.Width + sideSpace, y);
            labelAndTextBoxControlL.Location =
                new Point(x += labelAndTextBoxControlH.Width + sideSpace, y);


            this.Width = labelAndTextBoxControlD.Location.X + labelAndTextBoxControlD.Width;
            this.Height = labelAndTextBoxControlL.Location.Y + labelAndTextBoxControlL.Height;
        }
    }
}

