﻿using System;
using System.Drawing;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.Layout;
using GMath;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class PlanCommonKoefsControl : ICommonKoeficientsControl
    {

        public PlanCommonKoefsControl() { InitializeComponent(); }

        public PlanCommonKoefsControl(CommonEquation commonEquation)
            : base()
        {
            InitializeComponent();
            SuspendLayout();
            InitializeContents();
            SetKoeficients(commonEquation);
            SetLayout();
            ResumeLayout();
            Refresh();
        }


        private void InitializeContents()
        {
            labelAndTextBoxControlA.SetLabelText("A:");
            labelAndTextBoxControlB.SetLabelText("B:");
            labelAndTextBoxControlC.SetLabelText("C:");
            labelAndTextBoxControlD.SetLabelText("D:");
        }


        /// <summary>
        /// Nastaví každému LabelAndTextBox páru dialog, kterému se bude přeposílat událost změny textu.
        /// </summary>
        public override void SetAddObjectDialog(IEditEquation addObjectEquation)
        {
            labelAndTextBoxControlA.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlB.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlC.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlD.SetAddObjectDialog(addObjectEquation);
        }



        public void SetEquation(PlanEquation equation)
        {
            SetKoeficients(equation);
        }


        private void SetKoeficients(CommonEquation commonEquation)
        {
            commonEquation = (PlanEquation)commonEquation;
            labelAndTextBoxControlA.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetA()));
            labelAndTextBoxControlB.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetB()));
            labelAndTextBoxControlC.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetC()));
            labelAndTextBoxControlD.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetD()));
        }



        public override CommonEquation ParseCommonEquationIgnoreError()
        {
            double A, B, C, D;
            A = labelAndTextBoxControlA.GetTextBoxValueIgnoreError();
            B = labelAndTextBoxControlB.GetTextBoxValueIgnoreError();
            C = labelAndTextBoxControlC.GetTextBoxValueIgnoreError();
            D = labelAndTextBoxControlD.GetTextBoxValueIgnoreError();
            return new PlanEquation(A, B, C, D);
        }

        public override CommonEquation ParseCommonEquationThrowError()
        {
            double A, B, C, D;
            string errorKoef = "A";
            try
            {
                errorKoef = "A";
                A = labelAndTextBoxControlA.GetTextBoxValueThrowError();
                errorKoef = "B";
                B = labelAndTextBoxControlB.GetTextBoxValueThrowError();
                errorKoef = "C";
                C = labelAndTextBoxControlC.GetTextBoxValueThrowError();
                errorKoef = "D";
                D = labelAndTextBoxControlD.GetTextBoxValueThrowError();
                Vector u = new Vector(A, B, C, D);
                if (u.IsZero())
                {
                    ShowWarningMessage("Alepoň jedna složka normálového vektoru n = (A, B, C) musí být nenulová.");
                    return null;
                }
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
            return new PlanEquation(A, B, C, D);
        }


        private void SetLayout()
        {
            int x = 0;
            int y = 0;
            int sideSpace = LayoutProperties.TextSideSpace;

            labelAndTextBoxControlA.Location = new Point(x, y);
            labelAndTextBoxControlB.Location =
                new Point(x += labelAndTextBoxControlA.Width + sideSpace, y);
            labelAndTextBoxControlC.Location =
                new Point(x += labelAndTextBoxControlB.Width + sideSpace, y);
            labelAndTextBoxControlD.Location =
                new Point(x += labelAndTextBoxControlC.Width + sideSpace, y);

            this.Width = labelAndTextBoxControlD.Location.X + labelAndTextBoxControlD.Width;
            this.Height = labelAndTextBoxControlD.Location.Y + labelAndTextBoxControlD.Height;
        }
    }
}
