﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GGraphic.Forms.Detail.Layout; 
using Geometry.GGraphic.Forms.AddForms;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class QuadricCommonKoefsControl : ICommonKoeficientsControl
    { 

        public QuadricCommonKoefsControl() { InitializeComponent(); }

        public QuadricCommonKoefsControl(CommonEquation commonEquation):base()
        {
            InitializeComponent();
            SuspendLayout();
            InitializeContents();
            SetKoeficients(commonEquation);
            SetLayout();
            ResumeLayout();
            Refresh();
        }


        private void InitializeContents() 
        {
            labelAndTextBoxControlA.SetLabelText("A:");
            labelAndTextBoxControlB.SetLabelText("B:");
            labelAndTextBoxControlC.SetLabelText("C:");
            labelAndTextBoxControlD.SetLabelText("D:");
            labelAndTextBoxControlE.SetLabelText("E:");
            labelAndTextBoxControlF.SetLabelText("F:");
            labelAndTextBoxControlG.SetLabelText("G:");
            labelAndTextBoxControlH.SetLabelText("H:");
            labelAndTextBoxControlK.SetLabelText("K:");
            labelAndTextBoxControlL.SetLabelText("L:");
        }


        /// <summary>
        /// Nastaví každému LabelAndTextBox páru dialog, kterému se bude přeposílat událost změny textu.
        /// </summary>
        public override void SetAddObjectDialog(IEditEquation addObjectEquation) 
        {
            labelAndTextBoxControlA.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlB.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlC.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlD.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlE.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlF.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlG.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlH.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlK.SetAddObjectDialog(addObjectEquation);
            labelAndTextBoxControlL.SetAddObjectDialog(addObjectEquation); 
        }



        public void SetEquation(QuadricEquation equation) 
        {
            SetKoeficients(equation);
        }


        private void SetKoeficients(CommonEquation commonEquation)
        {
            commonEquation = (QuadricEquation)commonEquation;
            labelAndTextBoxControlA.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetA()));
            labelAndTextBoxControlB.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetB()));
            labelAndTextBoxControlC.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetC()));
            labelAndTextBoxControlD.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetD()));
            labelAndTextBoxControlE.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetE()));
            labelAndTextBoxControlF.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetF()));
            labelAndTextBoxControlG.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetG()));
            labelAndTextBoxControlH.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetH()));
            labelAndTextBoxControlK.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetK()));
            labelAndTextBoxControlL.SetTextBoxText(Program.GetOutputFormatedForTextBox(commonEquation.GetL()));       
        }



        public override CommonEquation ParseCommonEquationIgnoreError()
        {
            double A, B, C, D, E, F, G, H, K, L;
            A = labelAndTextBoxControlA.GetTextBoxValueIgnoreError();
            B = labelAndTextBoxControlB.GetTextBoxValueIgnoreError();
            C = labelAndTextBoxControlC.GetTextBoxValueIgnoreError();
            D = labelAndTextBoxControlD.GetTextBoxValueIgnoreError();
            E = labelAndTextBoxControlE.GetTextBoxValueIgnoreError();
            F = labelAndTextBoxControlF.GetTextBoxValueIgnoreError();
            G = labelAndTextBoxControlG.GetTextBoxValueIgnoreError();
            H = labelAndTextBoxControlH.GetTextBoxValueIgnoreError();
            K = labelAndTextBoxControlK.GetTextBoxValueIgnoreError();
            L = labelAndTextBoxControlL.GetTextBoxValueIgnoreError();
            return new QuadricEquation(A, B, C, D, E, F, G, H, K, L);
        }

        public override CommonEquation ParseCommonEquationThrowError()
        {
            double A, B, C, D, E, F, G, H, K, L;
            string errorKoef = "A";
            try
            {
                errorKoef = "A";
                A = labelAndTextBoxControlA.GetTextBoxValueThrowError();
                errorKoef = "B";                
                B = labelAndTextBoxControlB.GetTextBoxValueThrowError();
                errorKoef = "C";
                C = labelAndTextBoxControlC.GetTextBoxValueThrowError();
                errorKoef = "D"; 
                D = labelAndTextBoxControlD.GetTextBoxValueThrowError();
                errorKoef = "E"; 
                E = labelAndTextBoxControlE.GetTextBoxValueThrowError();
                errorKoef = "F"; 
                F = labelAndTextBoxControlF.GetTextBoxValueThrowError();
                errorKoef = "G"; 
                G = labelAndTextBoxControlG.GetTextBoxValueThrowError();
                errorKoef = "H"; 
                H = labelAndTextBoxControlH.GetTextBoxValueThrowError();
                errorKoef = "K";
                K = labelAndTextBoxControlK.GetTextBoxValueThrowError();
                errorKoef = "L";
                L = labelAndTextBoxControlL.GetTextBoxValueThrowError();

                if (Program.IsEqualInTolerance(A, 0) &&
                    Program.IsEqualInTolerance(B, 0) &&
                    Program.IsEqualInTolerance(C, 0) &&
                    Program.IsEqualInTolerance(D, 0) &&
                    Program.IsEqualInTolerance(E, 0) &&
                    Program.IsEqualInTolerance(F, 0))
                {
                    ShowWarningMessage("Alespoň jeden z koeficientů A, B, C, D, E, F musí být nenulový.");
                    return null;
                }
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
            return new QuadricEquation(A, B, C, D, E, F, G, H, K, L);
        }




        private void SetLayout()
        {
            int x = 0;
            int y = 0;
            int sideSpace = LayoutProperties.TextSideSpace;
            int topSpace = LayoutProperties.TextTopSpace;

            labelAndTextBoxControlA.Location = new Point(x, y);
            labelAndTextBoxControlB.Location =
                new Point(x += labelAndTextBoxControlA.Width + sideSpace, y);
            labelAndTextBoxControlC.Location =
                new Point(x += labelAndTextBoxControlB.Width + sideSpace, y);

            x = 0;
            y = labelAndTextBoxControlA.Location.Y + labelAndTextBoxControlA.Height + topSpace;

            labelAndTextBoxControlD.Location =
                new Point(x, y);
            labelAndTextBoxControlE.Location =
                new Point(x += labelAndTextBoxControlD.Width + sideSpace, y);
            labelAndTextBoxControlF.Location =
                new Point(x += labelAndTextBoxControlE.Width + sideSpace, y);

            x = 0;
            y = labelAndTextBoxControlD.Location.Y + labelAndTextBoxControlD.Height + topSpace;

            labelAndTextBoxControlG.Location =
                new Point(x, y);
            labelAndTextBoxControlH.Location =
                new Point(x += labelAndTextBoxControlG.Width + sideSpace, y);
            labelAndTextBoxControlK.Location =
                new Point(x += labelAndTextBoxControlH.Width + sideSpace, y);

            x = labelAndTextBoxControlK.Location.X;
            y = labelAndTextBoxControlK.Location.Y + labelAndTextBoxControlK.Height + topSpace;

            labelAndTextBoxControlL.Location =
                new Point(x, y);

            this.Width = labelAndTextBoxControlC.Location.X + labelAndTextBoxControlC.Width;
            this.Height = labelAndTextBoxControlL.Location.Y + labelAndTextBoxControlL.Height;
        }
    }
}
