﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.GraphicObjects.Equations;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Layout; 

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class ConicCanonicKoefsControl : IParametricKoeficientsControl
    {
        private ConicParametricEquation _conicParametricEquation;

        public ConicCanonicKoefsControl()
        {
            InitializeComponent();
            SetLayout();
        }

        public ConicCanonicKoefsControl(ParametricEquation conicParametricEquation)
            :base()
        {
            InitializeComponent();
            SetKoeficients((ConicParametricEquation)conicParametricEquation);
            SetLayout();
        }

        private void SetLayout()
        {
            labelAndTextBoxControlA.Location = new Point(0, 0);
            labelAndTextBoxControlBP.Location = 
                new Point(labelAndTextBoxControlA.Width + LayoutProperties.TextSideSpace, 0);

            NumericalLabelAndTextBoxControl temp = labelAndTextBoxControlA;
            if (labelAndTextBoxControlBP.Visible)
                temp = labelAndTextBoxControlBP;
            this.Width = temp.Location.X + temp.Width;
            this.Height = temp.Location.Y + temp.Height;
        }

        private void SetKoeficients(ConicParametricEquation canonicEquation)
        {
            _conicParametricEquation = canonicEquation;

            double a = canonicEquation.GetA();
            double b = canonicEquation.GetB();
            double p = canonicEquation.GetP();

            if (!Program.IsEqualInTolerance(a, 0))
            {
                // A je vzdy nenulove, krome prazdne
                labelAndTextBoxControlA.SetTextBoxText(Program.GetOutputFormatedForTextBox(a));
                labelAndTextBoxControlA.SetLabelText("a:");
            }
            else
            {
                labelAndTextBoxControlA.Visible = false;
            }
            if (!Program.IsEqualInTolerance(b, 0))
            {
                labelAndTextBoxControlBP.SetTextBoxText(Program.GetOutputFormatedForTextBox(b));
                labelAndTextBoxControlBP.SetLabelText("b:");
            }
            else if (!Program.IsEqualInTolerance(p, 0))
            {
                labelAndTextBoxControlBP.SetTextBoxText(Program.GetOutputFormatedForTextBox(p));
                labelAndTextBoxControlBP.SetLabelText("p:");
            }
            else
            {
                labelAndTextBoxControlBP.Visible = false;
            }
        }

        public override ParametricEquation ParseParametricEquation()
        {
            string errorKoef = "";
            try
            {
                double a2 = _conicParametricEquation.GetA();
                double b2 = _conicParametricEquation.GetB();
                double p2 = _conicParametricEquation.GetP();

                double a, b, p;
                a = 0;
                b = 0;
                p = 0;

                if (!Program.IsEqualInTolerance(a2, 0))
                {
                    errorKoef = "A";
                    a = labelAndTextBoxControlA.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(a, 0) || a < 0)
                    {
                        ShowWarningMessage("Hodnota koeficientu \"a\" musí být kladná.");
                        return null;
                    }
                }

                if (!Program.IsEqualInTolerance(b2, 0))
                {
                    errorKoef = "B";
                    b = labelAndTextBoxControlBP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(b, 0) || b < 0)
                    {
                        ShowWarningMessage("Hodnota koeficientu \"b\" musí být kladná.");
                        return null;
                    }
                }
                else if (!Program.IsEqualInTolerance(p2, 0))
                {
                    errorKoef = "P";
                    p = labelAndTextBoxControlBP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(p, 0))
                    {
                        ShowWarningMessage("Hodnota koeficientu \"p\" musí být nenulová.");
                        return null;
                    }
                }
                return new ConicParametricEquation(a, b, p);
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
        }

        public override int GetParametersCount()
        {
            int i = 0;
            if (labelAndTextBoxControlA.Visible)
                i++;
            if (labelAndTextBoxControlBP.Visible)
                i++;
            return i;
        }
    }
}