﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.Layout;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class LineParametricKoeficientsControl : IParametricKoeficientsControl
    {
        LineParametricEquation _lineParametricEquation;

        public LineParametricKoeficientsControl() { InitializeComponent(); }

        public LineParametricKoeficientsControl(ParametricEquation parametricEquation)
            : base()
        {
            InitializeComponent();
            SetKoeficients((LineParametricEquation)parametricEquation);
            SetLayout();
        }

        private void SetLayout()
        {
            labelAndTextBoxControlUx.Location = new Point(0, 0);
            labelAndTextBoxControlUy.Location =
                new Point(labelAndTextBoxControlUx.Width + LayoutProperties.TextSideSpace, 0);
            labelAndTextBoxControlUz.Location =
                new Point(labelAndTextBoxControlUy.Location.X + labelAndTextBoxControlUy.Width +
                    LayoutProperties.TextSideSpace, 0);

            this.Width = labelAndTextBoxControlUz.Location.X + labelAndTextBoxControlUz.Width;
            this.Height = labelAndTextBoxControlUx.Location.Y + labelAndTextBoxControlUx.Height;
        }

        private void SetKoeficients(LineParametricEquation parametricEquation)
        {
            _lineParametricEquation = parametricEquation;

            double ux = parametricEquation.GetUx();
            double uy = parametricEquation.GetUy();
            double uz = parametricEquation.GetUz();

            labelAndTextBoxControlUx.SetTextBoxText(Program.GetOutputFormatedForTextBox(parametricEquation.GetUx()));
            labelAndTextBoxControlUx.SetLabelText("u" + U.Index1Char + ":");

            labelAndTextBoxControlUy.SetTextBoxText(Program.GetOutputFormatedForTextBox(parametricEquation.GetUy()));
            labelAndTextBoxControlUy.SetLabelText("u" + U.Index2Char + ":");

            labelAndTextBoxControlUz.SetTextBoxText(Program.GetOutputFormatedForTextBox(parametricEquation.GetUz()));
            labelAndTextBoxControlUz.SetLabelText("u" + U.Index3Char + ":");
        }

        public override ParametricEquation ParseParametricEquation()
        {
            string errorKoef = "";
            try
            {
                double ux, uy, uz;
                errorKoef = "u" + U.Index1Char;                
                ux = labelAndTextBoxControlUx.GetTextBoxValueThrowError();
                errorKoef = "u" + U.Index2Char;
                uy = labelAndTextBoxControlUy.GetTextBoxValueThrowError();
                errorKoef = "u" + U.Index3Char;
                uz = labelAndTextBoxControlUz.GetTextBoxValueThrowError();
                if (Program.IsEqualInTolerance(ux, 0) &&
                    Program.IsEqualInTolerance(uy, 0) &&
                    Program.IsEqualInTolerance(uz, 0))
                {
                    ShowWarningMessage("Alespoň jedna složka směrového vektoru u = (u" + U.Index1Char
                        +", u" + U.Index2Char+", u" +U.Index3Char+") přímky musí být nenulová.");
                    return null;
                }
                return new LineParametricEquation(ux, uy, uz);
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
        }

        public override int GetParametersCount()
        {
            return 3;
        }
    }
}