﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GraphicObjects.Equations;
using GMath;

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class PlanParametricKoeficientsControl : IParametricKoeficientsControl
    {
        PlanParametricEquation _planParametricEquation;

        public PlanParametricKoeficientsControl() { InitializeComponent(); }

        public PlanParametricKoeficientsControl(ParametricEquation parametricEquation)
            : base()
        {
            InitializeComponent();
            SetKoeficients((PlanParametricEquation)parametricEquation);
            SetLayout();
        }

        private void SetLayout()
        {
            labelAndTextBoxControlUx.Location = new Point(0, 0);
            labelAndTextBoxControlUy.Location =
                new Point(labelAndTextBoxControlUx.Width + LayoutProperties.TextSideSpace, 0);
            labelAndTextBoxControlUz.Location =
                new Point(labelAndTextBoxControlUy.Location.X + labelAndTextBoxControlUy.Width +
                    LayoutProperties.TextSideSpace, 0);

            int y = labelAndTextBoxControlUx.Location.Y + labelAndTextBoxControlUx.Height + LayoutProperties.TextTopSpace;

            labelAndTextBoxControlVx.Location =
                new Point(labelAndTextBoxControlUx.Location.X, y);
            labelAndTextBoxControlVy.Location =
                new Point(labelAndTextBoxControlVx.Width + LayoutProperties.TextSideSpace, y);
            labelAndTextBoxControlVz.Location =
                new Point(labelAndTextBoxControlVy.Location.X + labelAndTextBoxControlVy.Width +
                    LayoutProperties.TextSideSpace, y);

            this.Width = labelAndTextBoxControlUz.Location.X + labelAndTextBoxControlUz.Width;
            this.Height = labelAndTextBoxControlVx.Location.Y + labelAndTextBoxControlVx.Height;
        }

        private void SetKoeficients(PlanParametricEquation parametricEquation)
        {
            _planParametricEquation = parametricEquation;

            double ux = parametricEquation.GetUx();
            double uy = parametricEquation.GetUy();
            double uz = parametricEquation.GetUz();

            double vx = parametricEquation.GetVx();
            double vy = parametricEquation.GetVy();
            double vz = parametricEquation.GetVz();

            labelAndTextBoxControlUx.SetTextBoxText(Program.GetOutputFormatedForTextBox(ux));
            labelAndTextBoxControlUx.SetLabelText("u" + U.Index1Char + ":");
            labelAndTextBoxControlUy.SetTextBoxText(Program.GetOutputFormatedForTextBox(uy));
            labelAndTextBoxControlUy.SetLabelText("u" + U.Index2Char + ":");
            labelAndTextBoxControlUz.SetTextBoxText(Program.GetOutputFormatedForTextBox(uz));
            labelAndTextBoxControlUz.SetLabelText("u" + U.Index3Char + ":");

            labelAndTextBoxControlVx.SetTextBoxText(Program.GetOutputFormatedForTextBox(vx));
            labelAndTextBoxControlVx.SetLabelText("v" + U.Index1Char + ":");
            labelAndTextBoxControlVy.SetTextBoxText(Program.GetOutputFormatedForTextBox(vy));
            labelAndTextBoxControlVy.SetLabelText("v" + U.Index2Char + ":");
            labelAndTextBoxControlVz.SetTextBoxText(Program.GetOutputFormatedForTextBox(vz));
            labelAndTextBoxControlVz.SetLabelText("v" + U.Index3Char + ":");
        }

        public override ParametricEquation ParseParametricEquation()
        {
            string errorKoef = "";
            try
            {
                double ux, uy, uz, vx, vy, vz;
                errorKoef = "u" + U.Index1Char;
                ux = labelAndTextBoxControlUx.GetTextBoxValueThrowError();
                errorKoef = "u" + U.Index2Char;                
                uy = labelAndTextBoxControlUy.GetTextBoxValueThrowError();
                errorKoef = "u" + U.Index3Char;                
                uz = labelAndTextBoxControlUz.GetTextBoxValueThrowError();

                errorKoef = "v" + U.Index1Char;
                vx = labelAndTextBoxControlVx.GetTextBoxValueThrowError();
                errorKoef = "v" + U.Index2Char;
                vy = labelAndTextBoxControlVy.GetTextBoxValueThrowError();
                errorKoef = "v" + U.Index3Char;
                vz = labelAndTextBoxControlVz.GetTextBoxValueThrowError();
                Vector u = new Vector(ux, uy, uz);
                Vector v = new Vector(vx, vy, vz);
                string warningMessage = "";
                if (u.IsZero())
                {
                    warningMessage += "Alespoň jedna složka směrového vektoru u = (u" + U.Index1Char;
                    warningMessage += ", u" + U.Index2Char + ", u" + U.Index3Char + " ) musí být nenulová.";
                    ShowWarningMessage(warningMessage);
                    return null;
                } 
                if (v.IsZero())
                {
                    warningMessage += "Alespoň jedna složka směrového vektoru v = (v" + U.Index1Char;
                    warningMessage += ", v" + U.Index2Char + ", v" + U.Index3Char +" ) musí být nenulová.";
                    ShowWarningMessage(warningMessage);
                    return null;
                }
                if (Vector.AreLinearyBounded(u, new List<Vector>(){v}))
                {
                    warningMessage += "Směrové vektory u = (u" + U.Index1Char;
                    warningMessage += ", u" + U.Index2Char + ", u" + U.Index3Char + " )  a v = (v" + 
                        U.Index1Char + ", v" + U.Index2Char + ", v"+ U.Index3Char+") nesmí být lineárně závislé.";
                    ShowWarningMessage(warningMessage);
                    return null;
                }
                return new PlanParametricEquation(ux, uy, uz, vx, vy, vz);
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
        }

        public override int GetParametersCount()
        {
            return 3;
        }
    }
}
