﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.Layout; 

namespace Geometry.GGraphic.Forms.Detail.Controls
{
    public partial class QuadricCanonicKoefsControl : IParametricKoeficientsControl
    {

        QuadricParametricEquation _quadricParametricEquation;

        public QuadricCanonicKoefsControl() { InitializeComponent(); }

        public QuadricCanonicKoefsControl(ParametricEquation canonicEquation)
            : base()
        {
            InitializeComponent();
            SetKoeficients((QuadricParametricEquation)canonicEquation);
            SetLayout();
        }

        private void SetLayout()
        {
            labelAndTextBoxControlA.Location = new Point(0, 0);
            labelAndTextBoxControlBP.Location = 
                new Point(labelAndTextBoxControlA.Width + LayoutProperties.TextSideSpace, 0);
            labelAndTextBoxControlCP.Location =
                new Point(labelAndTextBoxControlBP.Location.X + labelAndTextBoxControlBP.Width + 
                    LayoutProperties.TextSideSpace, 0);

            NumericalLabelAndTextBoxControl temp = labelAndTextBoxControlA;
            if (labelAndTextBoxControlBP.Visible)
                temp = labelAndTextBoxControlBP;
            if (labelAndTextBoxControlCP.Visible)
                temp = labelAndTextBoxControlCP;
            this.Width = temp.Location.X + temp.Width;
            this.Height = temp.Location.Y + temp.Height;
        }

        private void SetKoeficients(QuadricParametricEquation canonicEquation)
        {
            _quadricParametricEquation = canonicEquation;

            double a = canonicEquation.GetA();
            double b = canonicEquation.GetB();
            double c = canonicEquation.GetC();
            double p = canonicEquation.GetP();


            if (!Program.IsEqualInTolerance(a, 0))
            {
                // A je vzdy nenulove, krome prazdne
                labelAndTextBoxControlA.SetTextBoxText(Program.GetOutputFormatedForTextBox(a));
                labelAndTextBoxControlA.SetLabelText("a:");
            }
            else labelAndTextBoxControlA.Visible = false;

            if (!Program.IsEqualInTolerance(b, 0))
            {
                labelAndTextBoxControlBP.SetTextBoxText(Program.GetOutputFormatedForTextBox(b));
                labelAndTextBoxControlBP.SetLabelText("b:");
            }
            else if (!Program.IsEqualInTolerance(p, 0))
            {
                labelAndTextBoxControlBP.SetTextBoxText(Program.GetOutputFormatedForTextBox(p));
                labelAndTextBoxControlBP.SetLabelText("p:");
            }
            else labelAndTextBoxControlBP.Visible = false;

            if (!Program.IsEqualInTolerance(c, 0))
            {
                labelAndTextBoxControlCP.SetTextBoxText(Program.GetOutputFormatedForTextBox(c));
                labelAndTextBoxControlCP.SetLabelText("c:");
            }
            else if (!Program.IsEqualInTolerance(p, 0) && !Program.IsEqualInTolerance(b, 0))
            {
                labelAndTextBoxControlCP.SetTextBoxText(Program.GetOutputFormatedForTextBox(p));
                labelAndTextBoxControlCP.SetLabelText("p:");
            }
            else labelAndTextBoxControlCP.Visible = false;
        }

        public override ParametricEquation ParseParametricEquation()
        {
            string errorKoef = "";
            try
            {
                double a2 = _quadricParametricEquation.GetA();
                double b2 = _quadricParametricEquation.GetB();
                double c2 = _quadricParametricEquation.GetC();
                double p2 = _quadricParametricEquation.GetP();

                double a, b, c, p;
                a = 0;
                b = 0;
                c = 0;
                p = 0;

                if (!Program.IsEqualInTolerance(a2, 0))
                {
                    errorKoef = "A";
                    a = labelAndTextBoxControlA.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(a, 0) || a < 0)
                    {
                        ShowWarningMessage("Hodnota koeficientu \"a\" musí být kladná.");
                        return null;
                    }
                }

                if (!Program.IsEqualInTolerance(b2, 0))
                {
                    errorKoef = "B";
                    b = labelAndTextBoxControlBP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(b, 0) || b < 0)
                    {
                        ShowWarningMessage("Hodnota koeficientu \"b\" musí být kladná.");
                        return null;
                    }
                }
                else if (!Program.IsEqualInTolerance(p2, 0))
                {
                    errorKoef = "P";
                    p = labelAndTextBoxControlBP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(p, 0))
                    {
                        ShowWarningMessage("Hodnota koeficientu \"p\" musí být nenulová.");
                        return null;
                    }
                }

                if (!Program.IsEqualInTolerance(c2, 0))
                {
                    errorKoef = "C";
                    c = labelAndTextBoxControlCP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(c, 0) || c < 0)
                    {
                        ShowWarningMessage("Hodnota koeficientu \"c\" musí být kladná.");
                        return null;
                    }
                }
                else if (!Program.IsEqualInTolerance(p2, 0) && !Program.IsEqualInTolerance(b2, 0))
                {
                    errorKoef = "P";
                    p = labelAndTextBoxControlCP.GetTextBoxValueThrowError();
                    if (Program.IsEqualInTolerance(p, 0))
                    {
                        ShowWarningMessage("Hodnota koeficientu \"p\" musí být nenulová.");
                        return null;
                    }
                }
                return new QuadricParametricEquation(a, b, c, p);
            }
            catch (Exception)
            {
                ShowBadInputFormatWarningMessage(errorKoef);
                return null;
            }
        }

        public override int GetParametersCount() 
        {
            int i = 0;
            if (labelAndTextBoxControlA.Visible)
                i++;
            if (labelAndTextBoxControlBP.Visible)
                i++;
            if (labelAndTextBoxControlCP.Visible)
                i++;
            return i;
        }
    }
}