﻿using System;
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GraphicObjects;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.BaseClasses;
using GMath;
using System.Collections.Generic;
 
using Geometry.GGraphic.Controls;
using Geometry.GGraphic.Forms.Detail;
using Geometry.GGraphic.Controls.GroupBoxes;

namespace Geometry.GGraphic.Controls.TabPages
{
    public partial class CenterTabPage : ConstantTabPage
    {
        private ConstantGroupBox groupBoxCenterInCanonicBasis;
        private CenterGR controlCenterInCanonicBasis;
        private Button buttonCenterInCanonicBasis;

        private ConstantGroupBox groupBoxCenterInStandardBasis;
        private CenterGR controlCenterInStandardBasis;
        private Button buttonCenterInStandardBasis;

        private Form1 _form;
        private GraphicObjectDetail _graphicObjectDetail;

        public CenterTabPage(Form1 form, GraphicObjectDetail detail,
            Size minimumSize)
            : base(minimumSize)
        {
            _form = form;
            _graphicObjectDetail = detail;
            SuspendLayout();
            InitializeComponents();
            ResumeLayout();
            this.Text = "Střed";
        }

        /// <summary>
        /// Pridam vsechen obsah tabPage. Tedy i tlacitko zmeny kanonicke i standardni rotace. 
        /// </summary>
        private void InitializeComponents()
        {
            controlCenterInStandardBasis = new CenterGR();

            buttonCenterInStandardBasis = new Button();
            buttonCenterInStandardBasis.Text = "Změnit";
            buttonCenterInStandardBasis.Location = new Point(10, 0);
            buttonCenterInStandardBasis.UseVisualStyleBackColor = true;
            buttonCenterInStandardBasis.Click += new EventHandler(buttonCenterInStandardBasisClick);

            groupBoxCenterInStandardBasis =
                new ConstantGroupBox(_minimumSize);
            groupBoxCenterInStandardBasis.Text = "Střed vzhledem k bázi okna:";
            groupBoxCenterInStandardBasis.Location = new Point(0, 10);
            controlCenterInCanonicBasis = new CenterGR();

            buttonCenterInCanonicBasis = new Button();
            buttonCenterInCanonicBasis.Text = "Změnit";
            buttonCenterInCanonicBasis.Location = new Point(10, 0);
            buttonCenterInCanonicBasis.UseVisualStyleBackColor = true;
            buttonCenterInCanonicBasis.Click += new EventHandler(buttonCenterInCanonicBasisClick);

            groupBoxCenterInCanonicBasis =
                new ConstantGroupBox(_minimumSize);
            groupBoxCenterInCanonicBasis.Text = "Střed vzhledem ke kanonické bázi:";
            groupBoxCenterInCanonicBasis.Location = new Point(0, 0);

            groupBoxCenterInCanonicBasis.SetContents(
                new List<Control>() { controlCenterInCanonicBasis, buttonCenterInCanonicBasis }, GroupBoxLayoutStyle.TwoControlsInRowFar);

            groupBoxCenterInStandardBasis.SetContents(
                new List<Control>() { controlCenterInStandardBasis, buttonCenterInStandardBasis }, GroupBoxLayoutStyle.TwoControlsInRowFar);

            this.Controls.Add(groupBoxCenterInStandardBasis);  // pridam vsechny mozne groupBoxi.
            groupBoxCenterInStandardBasis.Tag = true;
            this.Controls.Add(groupBoxCenterInCanonicBasis);
            groupBoxCenterInCanonicBasis.Tag = true;
            base.SetConstantSize();             // nastavím jim konstatní velikosti a pozice.
            // pri kazdem odebrani/pridani groupBoxu si zmenit vysku.
        }


        public override void DisableButtons()
        {
            if (buttonCenterInCanonicBasis != null)
                buttonCenterInCanonicBasis.Enabled = false;
            if (buttonCenterInStandardBasis != null)
                buttonCenterInStandardBasis.Enabled = false;
        }
        public override void EnableButtons()
        {
            if (buttonCenterInCanonicBasis != null)
                buttonCenterInCanonicBasis.Enabled = true;
            if (buttonCenterInStandardBasis != null)
                buttonCenterInStandardBasis.Enabled = true;
        }

        /// <summary>
        /// Podle toho zda je graphicObject withCanonicEquation se 
        /// prida/odebere tlacitko editace kanonickeho centru a zmeni groupBoxLayoutStyle.
        /// </summary>
        /// <param name="graphicObject"></param>
        public override void GenerateContents(GraphicObject graphicObject)
        {
            GenerateStandardContents(graphicObject);
            if (graphicObject is WithCanonicEquationObject)
            {
                if (!IsInControls(groupBoxCenterInCanonicBasis)) // groupbox má být přidán.
                {
                    AddControl(groupBoxCenterInCanonicBasis);
                    SetNeededHeight(groupBoxCenterInCanonicBasis);
                }
                GenerateCanonicContents(graphicObject);
            }
            else
            {
                if (IsInControls(groupBoxCenterInCanonicBasis)) // groupbox nemá být přidán
                {
                    DeleteControl(groupBoxCenterInCanonicBasis);
                    SetNeededHeight(groupBoxCenterInStandardBasis);
                }
            }
        }

        /// <summary>
        /// Tyto metody slouži jen pro práci s Groupboxem centerInCanonicBasis.
        /// Tento totiž není dostupný pro všechny objekty.
        /// </summary>
        private bool IsInControls(Control control)
        {
            try
            {
                return (bool)control.Tag;
            }
            catch (Exception) { return false; }
        }
        private void AddControl(Control control)
        {
            this.Controls.Add(control);
            control.Tag = true;
        }
        private void DeleteControl(Control control)
        {
            this.Controls.Remove(control);
            control.Tag = false;
        }

        private void GenerateStandardContents(GraphicObject graphicObject)
        {
            CenterFormatString sceneCenterRegulation = new CenterFormatString(
                ((WithParametricEquationObject)graphicObject).GetCenterInStandardBasis());
            controlCenterInStandardBasis.SetContents(
                graphicObject,
                sceneCenterRegulation.GetXSubstitutedFormatString(),
                sceneCenterRegulation.GetYSubstitutedFormatString(),
                sceneCenterRegulation.GetZSubstitutedFormatString());
        }
        private void GenerateCanonicContents(GraphicObject graphicObject)
        {
            CenterFormatString canonicCenterRegulation =
                new CenterFormatString(((WithCanonicEquationObject)graphicObject).GetCenterInCanonicBasis());
            controlCenterInCanonicBasis.SetContents(
                graphicObject,
                canonicCenterRegulation.GetXSubstitutedFormatString(),
                canonicCenterRegulation.GetYSubstitutedFormatString(),
                canonicCenterRegulation.GetZSubstitutedFormatString());
        }


        private void buttonCenterInStandardBasisClick(object sender, EventArgs e)
        {
            GraphicObject graphicObject = _graphicObjectDetail.GetGraphicObject();
            WithParametricEquationObject withParametricEquationObject = (WithParametricEquationObject)graphicObject;
            _form.GetManipulation().ChangeStandardCenterInvoked(withParametricEquationObject);
        }
        private void buttonCenterInCanonicBasisClick(object sender, EventArgs e)
        {
            GraphicObject graphicObject = _graphicObjectDetail.GetGraphicObject();
            WithCanonicEquationObject withCanoicEquationObject = (WithCanonicEquationObject)graphicObject;
            _form.GetManipulation().ChangeCanonicCenterInvoked(withCanoicEquationObject);
        }
    }
}
