﻿using System;
using System.Drawing; 
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GGraphic.Controls.GroupBoxes;

namespace Geometry.GGraphic.Controls.TabPages
{
    public partial class ConstantTabPage : ITabPage
    {
        /// <summary>
        /// Potomci této třídy nastaví všem svým groupBoxům a jejim kontrolům velikosti 
        /// a pozice hned po prvním vytvoření a pak už je nemění.
        /// 
        /// Až se přidají všechny groupboxi s controly, zavolat SetConstantSize.
        /// - Tato metoda projde každý groupBox a zavolá na něm SetMinimumSize, zárověň jim nastaví pozici.
        /// - Potom nastaví všem groupBoxům stejnou velikost zavoláním SetConstantSize.
        /// Nakonec si nastaví Width a Height, které čte detail.
        /// </summary>
        public ConstantTabPage()
        {

        }

        /// <summary>
        /// Potomci této třídy nastaví všem svým groupBoxům a jejim kontrolům velikosti 
        /// a pozice hned po prvním vytvoření a pak už je nemění.
        /// 
        /// Až se přidají všechny groupboxi s controly, zavolat SetConstantSize.
        /// - Tato metoda projde každý groupBox a zavolá na něm SetMinimumSize, zárověň jim nastaví pozici.
        /// - Potom nastaví všem groupBoxům stejnou velikost zavoláním SetConstantSize.
        /// Nakonec si nastaví Width a Height, které čte detail.
        /// </summary>
        public ConstantTabPage(Size minimumSize)
            : base(minimumSize)
        {
        }


        public void SetConstantSize()
        {
            int width = 0;
            int height = LayoutProperties.TabPageTopSpace;
            foreach (ConstantGroupBox groupBox in this.Controls)
            {
                groupBox.SetConstantMimimumSize();
                width = Math.Max(width, groupBox.Width); // Find the biggest groupBoxLayouted
                groupBox.Location = new Point(LayoutProperties.TabPageSideSpace, height); // Set groupBox Location
                height += groupBox.Height + LayoutProperties.ControlsTopSpace; // Increment next groupBox Location
            }
            height -= LayoutProperties.ControlsTopSpace;
            foreach (ConstantGroupBox groupBox in this.Controls)
            {
                groupBox.SetConstantWidth(width); // Set all groupBoxes the maximum width
            }
            base._width = width + LayoutProperties.TabPageSideSpace * 2;
            base._height = height + LayoutProperties.TabPageTopSpace;
        }

        protected void SetNeededHeight(ConstantGroupBox bottomMostGroupBox)
        {
            this._height = bottomMostGroupBox.Location.Y + bottomMostGroupBox.Height + LayoutProperties.TabPageTopSpace;
        }
    }
}
