﻿
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Drawing;
using Geometry.GraphicObjects;
using GMath;
using Geometry.GraphicObjects.Intersections;
using System.Windows.Forms;
using System.Collections.Generic;
using Geometry.GGraphic.Forms.Detail.FormatString;
 
using Geometry.GGraphic.Controls.ElementaryControls;
using Geometry.GGraphic.Controls.GroupBoxes;
using Geometry.GGraphic.Controls.IntersectionControls;


namespace Geometry.GGraphic.Controls.TabPages
{
    public partial class CurveTabPage : DynamicTabPage
    {
        protected IntersectionParametrizationControl _parametrizationControl;
        protected IntersectionVParametrizationControl _vParametrizationControl;

        protected DynamicGroupBox _groupBoxParametrization;
        protected DynamicGroupBox _groupBoxVvalue;
        protected DynamicGroupBox _groupBoxIntervals;
        protected CleverLabel _labelInterval;


        public CurveTabPage(Size minimumSize)
            : base(minimumSize)
        {

            _minimumSize = minimumSize;
            InitializeComponent(minimumSize);
            this.Text = "Křivka";
        }

        public override void DisableButtons()
        {
        }
        public override void EnableButtons()
        {
        }

        public override void GenerateContents(GraphicObject graphicObject)
        {
            CurveAsQuadricsIntersection section = (CurveAsQuadricsIntersection)graphicObject;

            _parametrizationControl.SetContents(section);

            if (!(((CurveAsQuadricsIntersection)section).GetVParametrization() is ConstantU ||
                  ((CurveAsQuadricsIntersection)section).GetVParametrization() is ConstantV))
            {
                _vParametrizationControl.SetContents(section);
                SetDefinitionIntervals((CurveAsQuadricsIntersection)section);
                if (!(bool)_groupBoxVvalue.Tag)
                {
                    _groupBoxVvalue.Tag = true;
                    this.Controls.Add(_groupBoxVvalue);
                    _groupBoxIntervals.Tag = true;
                    this.Controls.Add(_groupBoxIntervals);
                }
            }
            else
            {
                if ((bool)_groupBoxVvalue.Tag)
                {
                    _groupBoxVvalue.Tag = false;
                    this.Controls.Remove(_groupBoxVvalue);
                    _groupBoxIntervals.Tag = false;
                    this.Controls.Remove(_groupBoxIntervals);
                }
            }
            base.SetDynamicFakeBounds();
        }
        private void InitializeComponent(Size minimumSize)
        {
            _parametrizationControl = new IntersectionParametrizationControl();
            _vParametrizationControl = new IntersectionVParametrizationControl();
            //_parametrizationControl = new ParametricEquationsGR();
            //_vParametrizationControl = new CommonEquationGR();

            _groupBoxParametrization = new DynamicGroupBox(minimumSize);
            _groupBoxParametrization.Text = "Parametrické vyjádření:";
            _groupBoxParametrization.SetContents(new List<Control>() { _parametrizationControl }, GroupBoxLayoutStyle.ControlOnLeft);


            _groupBoxIntervals = new DynamicGroupBox(minimumSize);
            _groupBoxIntervals.Text = "Definiční obor";
            _labelInterval = new CleverLabel();
            _groupBoxIntervals.SetContents(new List<Control>() { _labelInterval }, GroupBoxLayoutStyle.ControlOnLeft);

            _groupBoxVvalue = new DynamicGroupBox(minimumSize);
            _groupBoxVvalue.Text = "Vyjádření parametru v:";
            _groupBoxVvalue.SetContents(new List<Control> { _vParametrizationControl }, GroupBoxLayoutStyle.ControlOnLeft);

            this.Controls.Add(_groupBoxParametrization);

            _groupBoxParametrization.Tag = true;
            _groupBoxVvalue.Tag = false;
            _groupBoxIntervals.Tag = false;
        }
        private void SetDefinitionIntervals(CurveAsQuadricsIntersection section)
        {
            _labelInterval.Text = "u" + U.Belong;
            int originalLenght = _labelInterval.Text.Length;
            foreach (Interval interval in section.GetDefinitionIntervals().GetIntervals())
            {
                _labelInterval.Text += "[" + Program.GetOutputFormatedForCleverLabel(interval.GetA()) + ";"
                    + Program.GetOutputFormatedForCleverLabel(interval.GetB()) + "]" + U.Union;
            }
            if (_labelInterval.Text.Length > originalLenght) // pokud neni definicni interval prazdna mnozina 
            {
                _labelInterval.Text = _labelInterval.Text.Substring(0, _labelInterval.Text.Length - 1);// oddelam posledni union
            }
            else
            {
                _labelInterval.Text += U.EmptySet;
            }
            if (section.GetDefinitionIntervals().GetPlusValues().Count > 0)
            {
                _labelInterval.Text += U.Union + "{";
                foreach (double d in section.GetDefinitionIntervals().GetPlusValues())
                    _labelInterval.Text += Program.GetOutputFormatedForCleverLabel(d) + "; ";
                _labelInterval.Text = _labelInterval.Text.Substring(0, _labelInterval.Text.Length - 1);
                // odstranim posledni strednik
                _labelInterval.Text += "}";
            }
            if (section.GetDefinitionIntervals().GetMinusValues().Count > 0)
            {
                _labelInterval.Text += "-{";
                foreach (double d in section.GetDefinitionIntervals().GetMinusValues())
                    _labelInterval.Text += Program.GetOutputFormatedForCleverLabel(d) + "; ";
                _labelInterval.Text = _labelInterval.Text.Substring(0, _labelInterval.Text.Length - 1);
                // odstranim posledni strednik
                _labelInterval.Text += "}";
            }
            //_groupBoxIntervals.DynamicWidth = 2 * _labelInterval.DynamicLocation.X + _labelInterval.DynamicWidth;
            //_groupBoxIntervals.DynamicHeight = _labelInterval.DynamicLocation.Y + _labelInterval.DynamicHeight + 12;
        }
    }
}
