﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.GraphicObjects;
 
using Geometry.GGraphic.Forms.Detail;
using Geometry.GGraphic.Controls.GroupBoxes;

namespace Geometry.GGraphic.Controls.TabPages
{
    public partial class DrawableTabPage : ConstantTabPage
    {
        private ConstantGroupBox groupBoxName;
        private Label nameLabel;
        private Button nameButton;

        private ConstantGroupBox groupBoxColor;
        private Panel colorPanel;
        private Button colorButton;

        private ConstantGroupBox groupBoxOthers;
        private CheckBoxesComponent checkBoxesComponent;

        private const int colorPanelWidth = 75;
        private const int colorPanelHeight = 25;

        private GraphicObjectDetail _graphicObjectDetail;
        private Form1 _form;

        public DrawableTabPage(Form1 form, GraphicObjectDetail detail,
            Size minimumSize)
            : base(minimumSize)
        {
            _form = form;
            _graphicObjectDetail = detail;
            SuspendLayout();
            InitializeComponents();
            base.SetConstantSize();
            nameLabel.AutoEllipsis = true;
            nameLabel.MaximumSize = new Size(nameButton.Location.X - nameLabel.Location.X - LayoutProperties.ControlsSideSpace, nameLabel.Height);
            ResumeLayout();
            this.Text = "Obecné";
        }

        public override void DisableButtons()
        {
            nameButton.Enabled = false;
            colorButton.Enabled = false;
            checkBoxesComponent.Enabled = false;
        }
        public override void EnableButtons()
        {
            nameButton.Enabled = true;
            colorButton.Enabled = true;
            checkBoxesComponent.Enabled = true;
        }

        private void InitializeComponents()
        {
            InitializeNameComponents();
            InitializeColorComponents();
            InitializeOthersComponents();
            ConnectEvents();
        }
        private void InitializeNameComponents()
        {
            nameLabel = new Label();
            nameLabel.Text = "";
            nameLabel.AutoSize = true;
            nameLabel.Location = new Point(0, 0);

            nameButton = new Button();
            nameButton.Text = "Změnit";
            nameButton.Location = new Point(10, 0);
            nameButton.UseVisualStyleBackColor = true;

            groupBoxName = new ConstantGroupBox(_minimumSize);
            groupBoxName.Text = "Jméno:";
            groupBoxName.Location = new Point(0, 0);
            groupBoxName.SetContents(new List<Control>() { nameLabel, nameButton }, GroupBoxLayoutStyle.TwoControlsInRowFar);

            this.Controls.Add(groupBoxName);
        }
        private void InitializeColorComponents()
        {
            colorPanel = new Panel();
            colorPanel.BorderStyle = BorderStyle.Fixed3D;
            colorPanel.Width = colorPanelWidth;
            colorPanel.Height = colorPanelHeight;
            colorPanel.Location = new Point(0, 0);

            colorButton = new Button();
            colorButton.Text = "Změnit";
            colorButton.Location = new Point(10, 0);
            colorButton.UseVisualStyleBackColor = true;


            groupBoxColor = new ConstantGroupBox(_minimumSize);
            groupBoxColor.Text = "Barva:";
            groupBoxColor.Location = new Point(0, 10);
            groupBoxColor.SetContents(new List<Control>() { colorPanel, colorButton }, GroupBoxLayoutStyle.TwoControlsInRowFar);

            this.Controls.Add(groupBoxColor);

        }
        private void InitializeOthersComponents()
        {
            checkBoxesComponent = new CheckBoxesComponent(this);

            groupBoxOthers = new ConstantGroupBox(_minimumSize);
            groupBoxOthers.Text = "Ostatní:";
            groupBoxOthers.Location = new Point(0, 20);
            groupBoxOthers.SetContents(new List<Control>() { checkBoxesComponent }, GroupBoxLayoutStyle.ControlOnLeft);

            this.Controls.Add(groupBoxOthers);
        }

        public override void GenerateContents(GraphicObject graphicObject)
        {
            nameLabel.Text = graphicObject.GetName();
            colorPanel.BackColor = Program.ColorToSystemColor(graphicObject.GetColor());
            if (graphicObject is WithParametricEquationObject)
                checkBoxesComponent.GenerateContents(!graphicObject.GetVisible(),
                    ((WithParametricEquationObject)graphicObject).GetAxesVisible());
            else
            {
                checkBoxesComponent.GenerateContents(!graphicObject.GetVisible(), false);
                checkBoxesComponent.DisableObjectsAxesCheckBox();
            }
        }

        private void ConnectEvents()
        {
            this.nameButton.Click += new EventHandler(NameButtonClick);
            this.colorButton.Click += new EventHandler(ColorButtonClick);
        }

        private void NameButtonClick(object sender, EventArgs e)
        {
            GraphicObject graphicObject = _graphicObjectDetail.GetGraphicObject();
            _form.GetManipulation().ChangeNameInvoked(graphicObject);
        }
        private void ColorButtonClick(object sender, EventArgs e)
        {
            List<GraphicObject> graphicObjects = new List<GraphicObject>() { _graphicObjectDetail.GetGraphicObject() };
            _form.GetManipulation().ChangeColorInvoked(graphicObjects);
        }
        public void ShowAxesChanged(bool showAxes)
        {
            GraphicObject graphicObject = _graphicObjectDetail.GetGraphicObject();
            _form.GetManipulation().HideObjectsAxesInvoked(graphicObject, !showAxes);
        }
        public void HideObjectChanged(bool hideObject)
        {
            GraphicObject graphicObject = _graphicObjectDetail.GetGraphicObject();
            _form.GetManipulation().HideObjectInvoked(graphicObject, hideObject);
        }
    }
}
