﻿using System;
using System.Drawing;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GGraphic.Controls.GroupBoxes;
 

namespace Geometry.GGraphic.Controls.TabPages
{
    public partial class DynamicTabPage : ITabPage
    {

        public DynamicTabPage() { }

        /// <summary>
        /// When all controls are added call perform layout to scale tabPage.
        /// Restriction is that tabPageLayouted excepts groupBoxes is controls only.
        /// It scales all groupBoxes on same width, layouts them horizontaly and sets width itseft.
        /// Then layouts groupBoxes verticaly and scales itself.
        /// </summary>
        public DynamicTabPage(Size minimumSize)
            : base(minimumSize)
        {
        }

        public void SetDynamicFakeBounds()
        {
            int width = 0;
            int height = LayoutProperties.TabPageTopSpace;
            foreach (DynamicGroupBox groupBox in this.Controls)
            {
                groupBox.SetDynamicMinimumSize();
                //groupBox.SetMinimumBounds(_minimumSize); // Set minimum size to groupBox (all controls are in)
                width = Math.Max(width, groupBox.DynamicWidth); // Find the biggest groupBoxLayouted
                groupBox.DynamicLocation = new Point(LayoutProperties.TabPageSideSpace, height); // Set groupBox Location
                height += groupBox.DynamicHeight + LayoutProperties.ControlsTopSpace; // Increment next groupBox Location
            }
            height -= LayoutProperties.ControlsTopSpace;
            foreach (DynamicGroupBox groupBox in this.Controls)
            {
                groupBox.SetDynamicWidth(width);        // Set all groupBoxes the maximum width
            }
            base._width = width + LayoutProperties.TabPageSideSpace * 2;
            base._height = height + LayoutProperties.TabPageTopSpace;
        }

        public void SetRealSize()
        {
            foreach (DynamicGroupBox groupBox in this.Controls)
            {
                groupBox.SetRealDynamicSize();
            }
        }
    }
}
