﻿using System;
using System.Windows.Forms;
using System.Drawing;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Equations;
using System.Collections.Generic;
 
using Geometry.GGraphic.Forms.Detail;
using Geometry.GGraphic.Controls.GroupBoxes;

namespace Geometry.GGraphic.Controls.TabPages
{
    /// <summary>
    /// Uložím si matici rotace objektu vždy před otevřením angleDialogu.
    /// Po obdržení zprávy od angleDialogu aktualizuji novou matici rotace.
    /// Pokud nakonec dialog pošle zprávu o svém uzavření, pak zkontroluji, 
    /// zda se změnila matice rotace a pošlu Formu zprávu, aby přidal item do historie,
    /// dále požádám objekt, aby aktualizoval svou obecnou rovnici. Nakonec pošlu zprávu 
    /// graphicObjectDetailu, aby se aktualizoval po rotaci.
    /// Behem otaceni se muze zmenit object editací v jene tabPage.
    /// Proto tyto tabPage nejdrive pred potvrzenim zmen poslou zpravu, aby se toto osetrilo.
    /// Ošetřím to tak, že uložím aktuální rotaci jako by se potvrdila a nastavim příznak, že došlo k této události.
    /// Potom v případě ok/storno dojde k jinemu chovani.
    /// </summary>
    public partial class RotationTabPage : ConstantTabPage
    {
        private ConstantGroupBox groupBoxUserStandardRotation;
        private Rotation3DControl controlUserStandardRotation;
        private Button buttonEditRotation;

        private Form1 _form;
        private GraphicObjectDetail _graphicObjectDetail;

        private enum ApplyOrSet
        {
            Apply,
            Set
        }

        private RotationTabPage()
        {
        }

        public RotationTabPage(Form1 form, GraphicObjectDetail detail,
            Size minimumSize)
            : base(minimumSize)
        {
            _form = form;
            _graphicObjectDetail = detail;
            InitializeComponents();
            this.Text = "Rotace";
        }

        private void InitializeComponents()
        {
            groupBoxUserStandardRotation =
                    new ConstantGroupBox(_minimumSize);
            groupBoxUserStandardRotation.Text = "Rotace vzhledem ke standardní bázi:";
            groupBoxUserStandardRotation.Location = new Point(0, 0);

            buttonEditRotation = new Button();
            buttonEditRotation.Text = "Editovat";
            buttonEditRotation.Location = new Point(10, 0);
            buttonEditRotation.UseVisualStyleBackColor = true;
            buttonEditRotation.Click += new EventHandler(buttonEditRotationClick);


            controlUserStandardRotation = new Rotation3DControl();
            controlUserStandardRotation.Location = new Point(0, 0);

            groupBoxUserStandardRotation.SetContents(
                new List<Control>() { controlUserStandardRotation, buttonEditRotation }, GroupBoxLayoutStyle.TwoControlsInRowFar);
            this.Controls.Add(groupBoxUserStandardRotation);
            base.SetConstantSize();
        }


        public override void DisableButtons()
        {
            if (buttonEditRotation != null)
                buttonEditRotation.Enabled = false;
        }
        public override void EnableButtons()
        {
            if (buttonEditRotation != null)
                buttonEditRotation.Enabled = true;
        }

        public override void GenerateContents(GraphicObject graphicObject)
        {
            controlUserStandardRotation.SetContents(
                ((WithParametricEquationObject)graphicObject).GetRotationAnglesInStandardBasis());// aktualizuji svuj obsah
        }

        private void buttonEditRotationClick(object sender, EventArgs e)
        {
            _form.GetManipulation().ChangeObjectsRotationInvoked(
                (WithParametricEquationObject)_graphicObjectDetail.GetGraphicObject());
        }


        private void Test(GraphicObject graphicObject)
        {
            if (graphicObject is GraphicObjects.Quadrics.Quadric)
            {
                GraphicObjects.Quadrics.Quadric temp =
                   new Geometry.GraphicObjects.Quadrics.QuadricRecognizer(
                        ((GraphicObjects.Quadrics.Quadric)graphicObject).GetCommonEquation(),
                        graphicObject.GetColor(), graphicObject.GetName(), graphicObject.GetID())
                        .RecognizeQuadric();
                QuadricParametricEquation p1 = (QuadricParametricEquation)temp.GetParametricEquation();
                QuadricParametricEquation p2 = (QuadricParametricEquation)((GraphicObjects.Quadrics.Quadric)graphicObject).GetParametricEquation();
                if (!QuadricParametricEquation.AreSame(p1, p2))
                {
                    MessageBox.Show("kurnik");
                }
            }
        }
    }
}
