﻿using System.Collections.Generic;
using System.Drawing;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Intersections;
using Geometry.GGraphic.Forms.Detail.Layout;
using System.Windows.Forms;
using Geometry.GGraphic.Controls.GroupBoxes;
using Geometry.GGraphic.Controls.IntersectionControls;

namespace Geometry.GGraphic.Controls.TabPages
{
    // tyto křivky mohou být maximálně 4, protože jejich počet je závislý na počtu MinusHodnot definicniho oboru,
    // Což je maximálně 2 u kvadratického v*M*v (maximálně kvadratický člen u V2) 
    // a maximálně 4 u lineárního v*M*v (maximálně kvartický u V).
    // Krát počet řešení V pro danou minus Hodnotu, kterých je max. 2, resp 1.
    public partial class SimpleCurvesTabPage : DynamicTabPage
    {
        protected IntersectionParametrizationControl _parametrizationControl1;
        protected IntersectionParametrizationControl _parametrizationControl2;
        protected IntersectionParametrizationControl _parametrizationControl3;
        protected IntersectionParametrizationControl _parametrizationControl4;

        protected DynamicGroupBox _groupBoxParametrization1;
        protected DynamicGroupBox _groupBoxParametrization2;
        protected DynamicGroupBox _groupBoxParametrization3;
        protected DynamicGroupBox _groupBoxParametrization4;

        protected List<IntersectionParametrizationControl> _parametrizationControls;
        protected List<DynamicGroupBox> _groupBoxes;

        // tyto křivky mohou být maximálně 4, protože jejich počet je závislý na počtu MinusHodnot definicniho oboru,
        // Což je maximálně 2 u kvadratického v*M*v (maximálně kvadratický člen u V2) 
        // a maximálně 4 u lineárního v*M*v (maximálně kvartický u V).
        // Krát počet řešení V pro danou minus Hodnotu, kterých je max. 2, resp 1.
        public SimpleCurvesTabPage(Size minimumSize)
            : base(minimumSize)
        {

            _minimumSize = minimumSize;
            InitializeComponent(minimumSize);
            this.Text = "Křivky";
        }

        private void InitializeComponent(Size minimumSize)
        {
            _parametrizationControl1 = new IntersectionParametrizationControl();
            _parametrizationControl2 = new IntersectionParametrizationControl();
            _parametrizationControl3 = new IntersectionParametrizationControl();
            _parametrizationControl4 = new IntersectionParametrizationControl();
            //_parametrizationControl = new ParametricEquationsGR();
            //_vParametrizationControl = new CommonEquationGR();

            _groupBoxParametrization1 = new DynamicGroupBox(minimumSize);
            _groupBoxParametrization1.Text = "Parametrické vyjádření:";
            _groupBoxParametrization1.SetContents(new List<Control>() { _parametrizationControl1 }, GroupBoxLayoutStyle.ControlOnLeft);


            _groupBoxParametrization2 = new DynamicGroupBox(minimumSize);
            _groupBoxParametrization2.Text = "Parametrické vyjádření:";
            _groupBoxParametrization2.SetContents(new List<Control>() { _parametrizationControl2 }, GroupBoxLayoutStyle.ControlOnLeft);

            _groupBoxParametrization3 = new DynamicGroupBox(minimumSize);
            _groupBoxParametrization3.Text = "Parametrické vyjádření:";
            _groupBoxParametrization3.SetContents(new List<Control>() { _parametrizationControl3 }, GroupBoxLayoutStyle.ControlOnLeft);

            _groupBoxParametrization4 = new DynamicGroupBox(minimumSize);
            _groupBoxParametrization4.Text = "Parametrické vyjádření:";
            _groupBoxParametrization4.SetContents(new List<Control>() { _parametrizationControl4 }, GroupBoxLayoutStyle.ControlOnLeft);

            this.Controls.Add(_groupBoxParametrization1);

            _groupBoxParametrization1.Tag = true;
            _groupBoxParametrization2.Tag = false;
            _groupBoxParametrization3.Tag = false;
            _groupBoxParametrization4.Tag = false;

            _parametrizationControls = new List<IntersectionParametrizationControl>() { _parametrizationControl1, _parametrizationControl2, _parametrizationControl3, _parametrizationControl4 };
            _groupBoxes = new List<DynamicGroupBox>() { _groupBoxParametrization1, _groupBoxParametrization2, _groupBoxParametrization3, _groupBoxParametrization4 };
        }

        public override void DisableButtons()
        {
        }
        public override void EnableButtons()
        {
        }

        private void AddControls(int countOfCurves)
        {
            int countOfGroupBoxesBefore = 0;
            if ((bool)_groupBoxParametrization1.Tag) countOfGroupBoxesBefore++;
            if ((bool)_groupBoxParametrization2.Tag) countOfGroupBoxesBefore++;
            if ((bool)_groupBoxParametrization3.Tag) countOfGroupBoxesBefore++;
            if ((bool)_groupBoxParametrization4.Tag) countOfGroupBoxesBefore++;

            if (countOfGroupBoxesBefore >= countOfCurves)
            {
                for (int i = countOfCurves; i < countOfGroupBoxesBefore; i++)
                {
                    DynamicGroupBox groupBox = this._groupBoxes[i];
                    groupBox.Tag = false;
                    this.Controls.Remove(groupBox);
                }
            }
            else
            {
                for (int i = countOfGroupBoxesBefore; i < countOfCurves; i++)
                {
                    DynamicGroupBox groupBox = this._groupBoxes[i];
                    groupBox.Tag = true;
                    this.Controls.Add(groupBox);
                }
            }
        }
        public override void GenerateContents(GraphicObject graphicObject)
        {
            SuspendLayout();

            IIntersection section = (IIntersection)graphicObject;

            List<CurveAsQuadricsIntersection> simpleCurves = new List<CurveAsQuadricsIntersection>();
            foreach (GraphicObject g in section.GetIntersections())
                if (g is CurveAsQuadricsIntersection)
                {
                    IParameterVParametrization vParametrization = ((CurveAsQuadricsIntersection)g).GetVParametrization();
                    if (vParametrization is ConstantU || vParametrization is ConstantV)
                        simpleCurves.Add((CurveAsQuadricsIntersection)g);
                }
            int countOfCurves = System.Math.Min(simpleCurves.Count, 4);

            AddControls(countOfCurves);
            for (int i = 0; i < countOfCurves; i++)
                _parametrizationControls[i].SetContents(simpleCurves[i]);

            if (countOfCurves > 1)
                this.Text = "Křivky";
            else this.Text = "Křivka";
            ResumeLayout(false);
            base.SetDynamicFakeBounds();
        }
    }
}
