﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection;

namespace Geometry.BaseClasses.CustomCursor
{
    public class RotationCursor
    {
        private struct IconInfo
        {    // http://msdn.microsoft.com/en-us/library/ms648052(VS.85).aspx
            public bool fIcon;      // Icon or cursor. True = Icon, False = Cursor
            public int xHotspot;
            public int yHotspot;
            public IntPtr hbmMask;  // Specifies the icon bitmask bitmap. If this structure defines a black and white icon, 
            // this bitmask is formatted so that the upper half is the icon AND bitmask and the lower 
            // half is the icon XOR bitmask. Under this condition, the height should be an even multiple of two. 
            // If this structure defines a color icon, this mask only defines the AND bitmask of the icon.
            public IntPtr hbmColor; // Handle to the icon color bitmap. This member can be optional if this structure defines a black 
            // and white icon. The AND bitmask of hbmMask is applied with the SRCAND flag to the destination; 
            // subsequently, the color bitmap is applied (using XOR) to the destination by using the SRCINVERT flag. 

        }

        [DllImport("user32.dll", EntryPoint = "CreateIconIndirect")]
        private static extern IntPtr CreateIconIndirect(ref IconInfo icon);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool GetIconInfo(IntPtr hIcon, ref IconInfo pIconInfo);

        private static Cursor CreateCursor(Bitmap bmp, int xHotSpot, int yHotSpot)
        {
            IconInfo iconInfo = new IconInfo();
            GetIconInfo(bmp.GetHicon(), ref iconInfo);
            iconInfo.hbmColor = (IntPtr)0;
            iconInfo.hbmMask = bmp.GetHbitmap();
            iconInfo.xHotspot = xHotSpot;
            iconInfo.yHotspot = yHotSpot;
            iconInfo.fIcon = false;

            return new Cursor(CreateIconIndirect(ref iconInfo));
        }

        public static Cursor GetRotationCursor()
        {
            try
            {
                string path = Path.GetFullPath("rotationCursor1.bmp");
                Bitmap cursorBitmap = new Bitmap(path);
                return CreateCursor(cursorBitmap, 0,0);
            }
            catch (Exception e) { MessageBox.Show(e.Message); return Cursors.Default; }
        }

        public static class NativeMethods
        {
            public static Cursor LoadCustomCursor(string path)
            {
                IntPtr hCurs = LoadCursorFromFile(path);
                if (hCurs == IntPtr.Zero) throw new Exception();
                var curs = new Cursor(hCurs);
                // Note: force the cursor to own the handle so it gets released properly
                var fi = typeof(Cursor).GetField("ownHandle", BindingFlags.NonPublic | BindingFlags.Instance);
                fi.SetValue(curs, true);
                return curs;
            }
            [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
            private static extern IntPtr LoadCursorFromFile(string path);
        }
    }
}
