﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Geometry.GraphicObjects;
using Tao.Platform.Windows;
using Geometry.GraphicObjects.Intersections;
using System.IO;
using System.Threading;
using GMath;
using Tao.OpenGl;
using System.Drawing;
using Geometry.BaseClasses.CustomCursor;


namespace Geometry.GGraphic.Forms.AddForms
{
    public partial class AddIntersection : Form
    {
        private List<GraphicObject> _graphicObjects;
        private List<GraphicObject> _choosenObjects;
        private AddIntersectionScene  _currentScene;
        protected ManipulationManager _manipulationManager;

        private double _wheelZoomStep = 0.5;
        private bool _rightButtonDown = false;
        private bool _leftButtonDown = false;
        private bool _middleButtonDown = false;
        private System.Drawing.Point oldMouseLocation = new System.Drawing.Point();
        private System.Drawing.Point newMouseLocation = new System.Drawing.Point();
        private bool _trackBarWarningLock = false;

        private Cursor _rotationCursor;
        private Cursor _scaleCursor;

        private IntPtr _formhRC;
        private IntPtr _formhDC;
        private int _formPixelFormatID;
        private Gdi.PIXELFORMATDESCRIPTOR _formpfd;

        private IntPtr hRC;
        private IntPtr hDC;
        private Gdi.PIXELFORMATDESCRIPTOR pfd;
        private bool doubleBuffer;

        bool _shareEnabled = false;

        public AddIntersection() 
        {
            InitializeComponent();
        }

        public AddIntersection(Form1 form, IntPtr form1hDC, IntPtr form1hRC, Gdi.PIXELFORMATDESCRIPTOR form1FixelFormat,
            GColor color, string name)
        {
            _formhDC = form1hDC;
            _formhRC = form1hRC;
            _formPixelFormatID = form.GetPixelFormatID();
            _formpfd = form1FixelFormat;
            string Version = Gl.glGetString(Gl.GL_VERSION);
            try
            {
                if ( Version[0] > '1' ||  // verze je vetsi rovno 2.
                    (Version[0] == '1' && Version[2] > '0') || // nebo vetsi rovno 1.1 
                    (Version[0] == '1' && Version[3] > '0')) // nebo je vetsi rovno 1.x1.
                {
                    _shareEnabled = true;
                }
            }
            catch (Exception) {} // retezec je kratsi nez jsem si myslel, pak neni podporovan
            InitializeComponent();
            LoadCursors();
            _manipulationManager = form.GetManipulation();
            _currentScene = new AddIntersectionScene();
            this.panel1.SetScene(_currentScene);
            NormalyInitialize();
            if (_shareEnabled)
                Wgl.wglShareLists(_formhRC, hRC);
            _choosenObjects = new List<GraphicObject>();
            _graphicObjects = BaseWantsGetGraphicObjectsContents(form.GetScene());
            AddObjectsToListBox(_graphicObjects);
            nameAndColorControlTwoRows1.SetName(name);
            nameAndColorControlTwoRows1.SetColor(Program.ColorToSystemColor(color));
            this.MouseWheel += new MouseEventHandler(mouseWheel);
            SetZoomTrackBarValue(_currentScene.GetSceneZoom());            
        }

        protected void LoadCursors()
        {
            try
            {
                _rotationCursor = RotationCursor.NativeMethods.LoadCustomCursor(
                    Directory.GetCurrentDirectory() + "\\Depencies\\RotationCursor.cur");
                _scaleCursor = RotationCursor.NativeMethods.LoadCustomCursor(
                    Directory.GetCurrentDirectory() + "\\Depencies\\ScaleCursor.cur");
            }
            catch (Exception)
            {
                _rotationCursor = System.Windows.Forms.Cursors.Default;
                _scaleCursor = System.Windows.Forms.Cursors.Default;
            }
        }

        private void NormalyInitialize()
        {

            this.pfd = _formpfd;
            this.hDC = User.GetDC(this.panel1.Handle);

            //int pixFormat = Wgl.wglChoosePixelFormatARB(this.hDC, this.pfd);
            //int pixFormat = Gdi.ChoosePixelFormat(this.hDC, ref this.pfd);
            //if (pixFormat <= 0) throw new Exception("ChoosePixelFormat failed.");

            //Wgl.wglDescribePixelFormat(this.hDC, pixFormat, 40, ref this.pfd);
            //Wgl.wglSwapBuffers(this.hDC);
            int err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
                MessageBox.Show("Nejaka chyba jeste pred setPixelFormat.");
            Gdi.SetPixelFormat(this.hDC, _formPixelFormatID, ref this.pfd);
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Nepodarilo se nastavit pixel format kontextu, errorCode =." + err.ToString());
            }
            this.hRC = Wgl.wglCreateContext(this.hDC);
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR) 
            {
                MessageBox.Show("Nepodarilo se vytvorit kontext.");
            }

            if (this.hRC == IntPtr.Zero)
            {
                throw new Exception("Creating dummy context failed.");
            }
            Wgl.wglMakeCurrent(this.hDC, this.hRC);
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Nepodarilo se nastavit aktualni kontext.");
            }

            Gl.ReloadFunctions();
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Nepodarilo se Gl.ReloadFunctions.");
            }
            Wgl.ReloadFunctions();
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Nepodarilo se Wgl.ReloadFuntions kontext.");
            }
            //Gl.glEnable(Gl.GL_MULTISAMPLE);
            err = Gl.glGetError();
            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Nejaky error Code, errorCode =" + err.ToString());
                //throw new Exception("Error code = " + err.ToString());
            }
            //MessageBox.Show("form pixelFormatId =" + _formPixelFormatID.ToString());
            //MessageBox.Show("form hdc =" + _formhDC.ToString());
            //MessageBox.Show("form hrc =" + _formhRC.ToString());

            //MessageBox.Show("this hdc =" + hDC.ToString());
            //MessageBox.Show("thic hrc =" + hRC.ToString());


            this.doubleBuffer = ((this.pfd.dwFlags & Gdi.PFD_DOUBLEBUFFER) == Gdi.PFD_DOUBLEBUFFER);
            _currentScene.GLInit(GetOpenglControlWidth(), GetOpenglControlHeight(), this.hDC, this.doubleBuffer);
            Render();
        }

        private void DestroyContext()
        {
            Wgl.wglDeleteContext(this.hRC);
        }


        public void SetSelectedObjects(GraphicObject a, GraphicObject b) 
        {
            listView2.Items.Add(a.GetWordDescriptionFirstCase());
            _choosenObjects.Add(a);
            _currentScene.AddObject(a);

            listView2.Items.Add(b.GetWordDescriptionFirstCase());
            _choosenObjects.Add(b);
            _currentScene.AddObject(b);
            Render();
        }
        private void Render() 
        {
            this.panel1.RenderScene();
        }

        protected List<GraphicObject> BaseWantsGetGraphicObjectsContents(Scene formsScene)
        {
            List<GraphicObject> retval = new List<GraphicObject>();
            if (!_shareEnabled)
                for (int i = 0; i < formsScene.GetObjectsCount(); i++)
                {
                    if (formsScene.ObjectAt(i) is WithParametricEquationObject)
                    {
                        WithParametricEquationObject go = ((WithParametricEquationObject)formsScene.ObjectAt(i)).MakeCopyOfObject(formsScene.ObjectAt(i).GetID());
                        go.BuildDisplayList(_currentScene, new Geometry.BaseClasses.RelativeQuality(Geometry.BaseClasses.RelativeQuality.RelativeEnum.NORMAL));
                        retval.Add(go);
                    }
                }
            else
                for (int i = 0; i < formsScene.GetObjectsCount(); i++)
                {
                    if (formsScene.ObjectAt(i) is WithParametricEquationObject)
                        retval.Add(formsScene.ObjectAt(i));
                }
            return retval;
        }

        private void AddObjectsToListBox(List<GraphicObject> graphicObjects) 
        {
            for (int i = 0; i < graphicObjects.Count; i++)
            {
                this.listView1.Items.Add(graphicObjects[i].GetWordDescriptionFirstCase());
            }
            listView1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
        }


        //////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////

        protected override void OnResize(EventArgs e)
        {
            this.panel1.DrawDisabled = true;
            base.OnResize(e);
            if (_currentScene != null)
                _currentScene.SetProjectiveView(GetOpenglControlWidth(), GetOpenglControlHeight());
            this.panel1.DrawDisabled = false;
            RenderScene();
        }
        public int GetOpenglControlWidth()
        {
            return this.panel1.ClientRectangle.Width;
        }
        public int GetOpenglControlHeight()
        {
            return this.panel1.ClientRectangle.Height;
        }

        private void SetScaleCursor(object sender, EventArgs e)
        {
            this.Cursor = _scaleCursor;
        }
        private void SetDefaultCursor(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Default;
        }
        private void ShowZoomCursor()
        {
            EventHandler e = new EventHandler(SetScaleCursor);
            this.Invoke(e);
            Thread.Sleep(200);
            e = new EventHandler(SetDefaultCursor);
            this.Invoke(e);
        }
        private void ShowWarningOnZoomTrackBar()
        {
            if (!_trackBarWarningLock)
            {
                _trackBarWarningLock = true;
                SetTrackBarRedBackColor(null, null);
                //EventHandler e = new EventHandler(SetTrackBarRedBackColor);
                //this.Invoke(e);
                Thread.Sleep(100);
                SetTrackBarDefaultBackColor(null, null);
                //e = new EventHandler(SetTrackBarDefaultBackColor);
                //this.Invoke(e);
                _trackBarWarningLock = false;
            }
        }
        private void SetTrackBarRedBackColor(object sender, EventArgs e)
        {
            zoomTrackBar.BackColor = Color.Red;
        }
        private void SetTrackBarDefaultBackColor(object sender, EventArgs e)
        {
            zoomTrackBar.BackColor = System.Drawing.SystemColors.Control;
        }

        private void mouseWheel(object sender, MouseEventArgs e)
        {
            Thread t = new Thread(new ThreadStart(ShowZoomCursor));
            t.Start();
            if (e.Delta > 0 && _currentScene.GetSceneZoom() - _wheelZoomStep >= 0)
            {
                _currentScene.IncSceneZoom(-_wheelZoomStep);
                SetZoomTrackBarValue(_currentScene.GetSceneZoom());
            }
            else if (e.Delta < 0 && _currentScene.GetSceneZoom() + _wheelZoomStep <= zoomTrackBar.Maximum)
            {
                _currentScene.IncSceneZoom(_wheelZoomStep);
                SetZoomTrackBarValue(_currentScene.GetSceneZoom());
            }
            else
            {
                ShowWarningOnZoomTrackBar();
            }
            Render();
        }
        private void simpleOpenGlControl1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left && !_rightButtonDown && !_middleButtonDown)
            {
                _leftButtonDown = true;
                this.Cursor = Cursors.Hand;
                oldMouseLocation = e.Location;
            }
            if (e.Button == MouseButtons.Right && !_leftButtonDown && !_middleButtonDown)
            {
                _rightButtonDown = true;
                this.Cursor = _rotationCursor;
                oldMouseLocation = e.Location;
            }
            if (e.Button == MouseButtons.Middle && !_rightButtonDown && !_leftButtonDown)
            {
                _middleButtonDown = true;
                this.Cursor = _scaleCursor;
                oldMouseLocation = e.Location;
            }
        }
        private void simpleOpenGlControl1_MouseUP(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left && _leftButtonDown)
            {
                _leftButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (e.Button == MouseButtons.Right && _rightButtonDown)
            {
                _rightButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (e.Button == MouseButtons.Middle && _middleButtonDown)
            {
                _middleButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (_leftButtonDown)
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.LeftMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
            if (_rightButtonDown) 
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.RightMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
            if (_middleButtonDown)
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.MiddleMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
        }
        private void simpleOpenGlControl1_MouseMove(object sender, MouseEventArgs e)
        {

            if (_rightButtonDown)
            {
                newMouseLocation = e.Location;
                double xDelta = (oldMouseLocation.Y - newMouseLocation.Y);
                double yDelta = (oldMouseLocation.X - newMouseLocation.X);
                _currentScene.IncSceneAngles(xDelta / 3.0, yDelta / 3.0);
                oldMouseLocation = e.Location;
                Render();
            }
            if (_leftButtonDown)
            {
                newMouseLocation = e.Location;
                double xDelta = (oldMouseLocation.X - newMouseLocation.X);
                double yDelta = -(oldMouseLocation.Y - newMouseLocation.Y);
                _currentScene.MoveSceneCenter(new Vector(xDelta / 10.0, yDelta / 10.0, 0));
                oldMouseLocation = e.Location;
                Render();
            }
            if (_middleButtonDown)
            {
                newMouseLocation = e.Location;
                double yDelta = (oldMouseLocation.Y - newMouseLocation.Y);
                if ((yDelta > 0 && _currentScene.GetSceneZoom() + yDelta * _wheelZoomStep > zoomTrackBar.Maximum)
                    || (yDelta < 0 && _currentScene.GetSceneZoom() + yDelta * _wheelZoomStep < zoomTrackBar.Minimum))
                {
                    ShowWarningOnZoomTrackBar();
                    return;
                }
                _currentScene.IncSceneZoom(yDelta * _wheelZoomStep);
                SetZoomTrackBarValue(_currentScene.GetSceneZoom());
                oldMouseLocation = e.Location;
                Render();
            }
        }
        private void zoomTrackBar_Scroll(object sender, EventArgs e)
        {
            _currentScene.IncSceneZoom((zoomTrackBar.Maximum - zoomTrackBar.Value) - _currentScene.GetSceneZoom());
            Render();
        }
        private void SetZoomTrackBarValue(double zoom)
        {
            zoomTrackBar.Value = zoomTrackBar.Maximum - (int)zoom;
        }
        protected void RenderScene() 
        {
            this.panel1.RenderScene();
        }

        //////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////

        protected void ShowWarningMessageBox(string text)
        {
            MessageBox.Show(text, "Upozornění", 
                MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);
        }

        protected bool AddToChoosenObjectsP(List<GraphicObject> choosenObjects, GraphicObject graphicObject)
        {
            return !(graphicObject is GraphicObjects.Planar.Axes);
        }


        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            int listView1SelectedIndex = 0;
            if (listView1.SelectedItems.Count == 1)
                listView1SelectedIndex = listView1.SelectedItems[0].Index;
            else return;
            GraphicObject temp = null;
            if (listView2.Items.Count < 2)
            {
                try
                {
                    temp = this._graphicObjects[listView1SelectedIndex];
                }
                catch (Exception) { return; }// jde aby byl selected index -1 a proslo to sem.
                if (AddToChoosenObjectsP(_choosenObjects, temp))
                {
                    listView2.Items.Add(temp.GetWordDescriptionFirstCase());
                    listView2.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
                    _choosenObjects.Add(temp);
                    _currentScene.AddObject(temp);
                    Render();
                }
            }
        }
        private void listBox2_DoubleClick(object sender, EventArgs e)
        {
            int listView2SelectedIndex = 0;
            if (listView2.SelectedItems.Count == 1)
                listView2SelectedIndex = listView2.SelectedItems[0].Index;
            else return;
            try
            {
                GraphicObject q = this._choosenObjects[listView2SelectedIndex];
                this._choosenObjects.RemoveAt(listView2SelectedIndex);
                listView2.Items.RemoveAt(listView2SelectedIndex);
                listView1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
                this._currentScene.DeleteObject(q);
                Render();
            }
            catch (Exception) { } // jde aby byl selected index -1 a proslo to sem.
        }

        
        private void buttonStornoClick(object sender, EventArgs e)
        {
            Close();
        }
        private void buttonOkClick(object sender, EventArgs e)
        {
            if (_choosenObjects.Count != 2)
            {
                ShowWarningMessageBox("Musí být vybrány dva objekty.");
                return;
            }
            Close();
            GColor color = Program.SystemColorToColor(nameAndColorControlTwoRows1.GetColor());
            string name = nameAndColorControlTwoRows1.GetName();
            OkButtonPressedAndIntersectionWanted(_choosenObjects, color, name);
        }


        protected void OkButtonPressedAndIntersectionWanted(
            List<GraphicObject> choosenObjects, GColor color, string name)
        {
            _manipulationManager.AddIntersection(choosenObjects[0], choosenObjects[1], color, name);
        }

        private void AddIntersection_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!_shareEnabled)
                for (int i = 0; i < _graphicObjects.Count; i++)
                {
                    // nutno smazat ještě v kontextu AddIntersectionu, jinak uvolní displayListy v hlavním okně
                    _graphicObjects[i].DeleteDisplayList();
                }
            Wgl.wglMakeCurrent(_formhDC, _formhRC);
            DestroyContext();
        }

        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            //if (e.KeyCode == Keys.Enter)
            //    listBox1_DoubleClick(sender, EventArgs.Empty);
        }

        private void listView2_KeyDown(object sender, KeyEventArgs e)
        {
            //if (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Delete)
            //    listBox2_DoubleClick(sender, EventArgs.Empty);
        }
    }
}
