﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GGraphic.Forms.Detail;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GraphicObjects;
using Geometry.GGraphic;
using Geometry.GGraphic.Controls.ElementaryControls;
using Geometry.GGraphic.Controls; 

namespace Geometry.GGraphic.Forms.AddForms
{
    public partial class AddGraphicObjectCommonlyDialog : IEditEquation
    {
        protected CommonEquationFormatString _commonEquationFormatString;
        protected CleverLabel _resultEquationLabel;
        protected CommonEquation _actualEquation;

        public AddGraphicObjectCommonlyDialog() 
        {
            InitializeComponent();
        }

        public AddGraphicObjectCommonlyDialog(ManipulationManager manipulationManager, 
            CommonEquationFormatString commonEquationFormatString, GColor color, string name,
            IEquationGR equationGRControl, IKoeficientsControl koeficientsControl)
            : base(manipulationManager, null, equationGRControl, koeficientsControl)
        {
            InitializeComponent();

            _commonEquationFormatString = new CommonEquationFormatString(commonEquationFormatString);
            _groupBox.Text = "Obecná rovnice:";
            _resultEquationLabel = new CleverLabel();
            this.Controls.Add(_resultEquationLabel);
            ((ICommonKoeficientsControl)_koeficientsControl).SetAddObjectDialog(this);
            this._commonEquationFormatString = commonEquationFormatString;
            this.nameAndColorControl.SetColor(Program.ColorToSystemColor(color));
            this.nameAndColorControl.SetName(name);
            koeficientsControl.TabIndex = 0;
            nameAndColorControl.TabIndex = 1;

            equationGRControl.SetRealDynamicSize();
            _groupBox.SetConstantMimimumSize();
            UpdateLabel();
        }

        public CommonEquation GetCommonEquation()
        {
            return _actualEquation;
        }
        public GColor GetColor()
        {
            return Program.SystemColorToColor(this.nameAndColorControl.GetColor());
        }
        public string GetName()
        {
            return this.nameAndColorControl.GetName();
        }

        private List<Control> GetListOfControls() 
        {
            List<Control> list = new List<Control>();
            list.Add(_resultEquationLabel);
            list.Add(base._groupBox);
            list.Add(base._koeficientsControl);
            list.Add(nameAndColorControl);
            return list;
        }
        private void UpdateLabel()
        {
            CommonEquation equation = ((ICommonKoeficientsControl)base._koeficientsControl).ParseCommonEquationIgnoreError();
            _commonEquationFormatString.SubstituteToFormatString(equation);
            string temp = _commonEquationFormatString.GetSubstitutedFormatString();
            temp = temp.Substring(2, temp.Length - 3);
            _resultEquationLabel.Text = temp;
            base.PerformCustomLayout(GetListOfControls());
        }


        public override void TextBoxTextChanged()
        {
            UpdateLabel();
        }
        protected override void OkButtonClick(object sender, EventArgs e)
        {
            ICommonKoeficientsControl control = (ICommonKoeficientsControl)_koeficientsControl;
            _actualEquation = control.ParseCommonEquationThrowError();
            if (_actualEquation != null) // null znamená chybu, informace o ní však již byla sdělena.
            {
                DialogResult = DialogResult.OK; // pri pridani obecnou rovnici nevadi, ze se nezmenila rovnice
                Close();
            }
        }
    }
}


