﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Intersections;
using Geometry.GGraphic.Controls.TabPages;

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class GraphicObjectDetail : Form
    {
        public int _maxLabelLength = 10;
        protected GraphicObject _graphicObject;
        protected static Size _minimumGroupBoxSize = new Size(250, 0); // height ignored
        protected Form1 _form;
        //private SuspendRedrawManager _suspendDrawManager;
        int WindowWidthDelta = 0;
        int WindowHeightDelta = 0;
        int TabControlWidthDelta = 0;
        int TabControlHeightDelta = 0;
        protected bool DrawDisabled = false;


        public GraphicObjectDetail() { }
        public GraphicObjectDetail(Form1 form, GraphicObject graphicObject)
            : base()
        {
            _form = form;
            //_suspendDrawManager = new SuspendRedrawManager(this);
            _graphicObject = graphicObject;
            InitializeComponent();
            WindowWidthDelta = this.ClientSize.Width - tabControl1.SelectedTab.Width;
            WindowHeightDelta = this.ClientSize.Height - tabControl1.SelectedTab.Height;
            TabControlWidthDelta = tabControl1.Width - tabControl1.SelectedTab.Width;
            TabControlHeightDelta = tabControl1.Height - tabControl1.SelectedTab.Height;
            tabControl1.Controls.Clear();
            tabControl1.Dock = DockStyle.None;
            //tabControl1.Anchor = AnchorStyles.None;
        }

        public GraphicObject GetGraphicObject()
        {
            return _graphicObject;
        }

        protected void GenerateContentsOfTabPage(int tabPageIndex, string text, GraphicObject graphicObject)
        {
            ((ITabPage)tabControl1.TabPages[tabPageIndex]).GenerateContents(graphicObject);
            tabControl1.TabPages[tabPageIndex].Text = text;
        }
        public void GenerateContents(GraphicObject graphicObject)
        {
            DrawDisabled = true;
            if (graphicObject is IIntersection)
            {
                GenerateIntersectionContents((IIntersection)graphicObject);
            }
            else
            {
                _graphicObject = graphicObject;
                for (int i = 0; i < tabControl1.TabCount; i++)
                    if (tabControl1.TabPages[i] is ITabPage)
                    {
                        ((ITabPage)tabControl1.TabPages[i]).GenerateContents(graphicObject);
                    }
            }
            tabControl1_SelectedIndexChanged(null, EventArgs.Empty);
            DrawDisabled = false;
            Refresh();
        }
        public void GenerateDrawableContents(GraphicObject graphicObject)
        {
            for (int i = 0; i < tabControl1.TabCount; i++)
                if (tabControl1.TabPages[i] is DrawableTabPage)
                    ((DrawableTabPage)tabControl1.TabPages[i]).GenerateContents(graphicObject);
            Rescale();
        }
        protected virtual void GenerateIntersectionContents(IIntersection intersection) { }

        protected int GetTabPagesCount() { return tabControl1.TabCount; }
        protected void InsertToTabControl(int position, ITabPage tabPage)
        {
            IntPtr h = this.tabControl1.Handle;
            tabControl1.TabPages.Insert(position, tabPage);
        }
        protected void AddToTabControl(ITabPage tabPage)
        {
            tabControl1.TabPages.Add(tabPage);
        }
        protected void ClearTabPages()
        {
            this.tabControl1.TabPages.Clear();
        }
        protected void RemoveTabPage(ITabPage tabPage)
        {
            this.tabControl1.TabPages.Remove(tabPage);
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.TabCount == 0)
                return;

            this.MaximumSize = new Size(int.MaxValue, int.MaxValue);

            ITabPage temp = (ITabPage)tabControl1.SelectedTab;
            int width = temp._width + WindowWidthDelta;
            int height = temp._height + WindowHeightDelta;
            
            Size oldClientSize = this.ClientSize;
            Size newClientSize = new Size(width, height);
            this.ClientSize = newClientSize;
            this.MaximumSize = new Size(this.Width, this.Height);
            // kdyz se nezmeni velikost, OnResize se nezavola a tim padem bude mit stranka spravne jen Dynamic promenne.
            if (oldClientSize == newClientSize) 
                if (tabControl1.SelectedTab != null && tabControl1.SelectedTab is DynamicTabPage)
                {
                    DynamicTabPage tabPage = (DynamicTabPage)tabControl1.SelectedTab;
                    tabPage.SetRealSize();
                }
        }
        public void SetSelectedTabPage(int index)
        {
            if (index >= 0 && index < tabControl1.TabCount)
            {
                tabControl1.SelectedIndex = index;
            }
        }
        public int GetSelectedTabIndex() { return tabControl1.SelectedIndex; }

        protected void SetWindowProperties()
        {
            this.ShowIcon = false;
            this.ShowInTaskbar = false;
            this.MinimizeBox = false;
            this.MaximizeBox = false;
            this.StartPosition = FormStartPosition.Manual;
        }

        /// <summary>
        /// Všechny detaily, kromě IntersectionDetailu nepotřebují tuto metodu přepisovat.
        /// Povolí akce tlačítek ve stránkách. 
        /// </summary>
        public virtual void EnableEditButtons()
        {
            for (int i = 0; i < tabControl1.TabCount; i++)
                ((ITabPage)tabControl1.TabPages[i]).EnableButtons();
        }
        public virtual void DisableEditButtons()
        {
            for (int i = 0; i < tabControl1.TabCount; i++)
                ((ITabPage)tabControl1.TabPages[i]).DisableButtons();
        }


        public void NameChanged()
        {
            this.tabControl1_SelectedIndexChanged(null, EventArgs.Empty);
        }
        public void ColorChanged()
        {
        }

        public void Rescale()
        {
            tabControl1_SelectedIndexChanged(tabControl1, EventArgs.Empty);
            //Refresh(); 
        }

        private void CloseButton_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void DeleteButton_Click(object sender, EventArgs e)
        {
            _form.GetManipulation().DeleteObjectInvoked(_graphicObject);
        }

        private void GraphicObjectDetail_FormClosing(object sender, FormClosingEventArgs e)
        {
            _form.GetManipulation().DetailClosed(this);
        }

        //protected override void OnSizeChanged(EventArgs e)
        //{
        //    base.OnSizeChanged(e);
        //    //base.OnResize(e);
        //    try
        //    {
        //        ITabPage temp = (ITabPage)tabControl1.SelectedTab;
        //        int width = temp._width + TabControlWidthDelta;
        //        int height = temp._height + TabControlHeightDelta;
        //        tabControl1.Size = new Size(width, height);
        //        CloseButton.Location = new Point(this.tabControl1.Width - LayoutProperties.WindowSideSpace - CloseButton.Width, 
        //            this.tabControl1.Height + LayoutProperties.ControlsTopSpace);
        //        DeleteButton.Location = new Point(CloseButton.Location.X - LayoutProperties.ControlsSideSpace - DeleteButton.Width,
        //            CloseButton.Location.Y);
        //        if (tabControl1.SelectedTab != null && tabControl1.SelectedTab is DynamicTabPage)
        //        {
        //            DynamicTabPage tabPage = (DynamicTabPage)tabControl1.SelectedTab;
        //            tabPage.SetRealSize();
        //        }
        //    }
        //    catch (Exception) { }
        //}

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            try
            {
                ITabPage temp = (ITabPage)tabControl1.SelectedTab;
                int width = temp._width + TabControlWidthDelta;
                int height = temp._height + TabControlHeightDelta;
                tabControl1.Size = new Size(width, height);
                CloseButton.Location = new Point(this.tabControl1.Width - LayoutProperties.WindowSideSpace - CloseButton.Width,
                    this.tabControl1.Height + LayoutProperties.TextTopSpace);
                DeleteButton.Location = new Point(CloseButton.Location.X - LayoutProperties.ControlsSideSpace - DeleteButton.Width,
                    CloseButton.Location.Y);
                if (tabControl1.SelectedTab != null && tabControl1.SelectedTab is DynamicTabPage)
                {
                    DynamicTabPage tabPage = (DynamicTabPage)tabControl1.SelectedTab;
                    tabPage.SetRealSize();
                }
            }
            catch (Exception) { }
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            if (!DrawDisabled)
                base.OnPaint(e);
        }
    }
}
