﻿
using Geometry.GraphicObjects.Intersections;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using System.Collections.Generic;
using Geometry.GGraphic.Controls.TabPages;

namespace Geometry.GGraphic.Forms.Detail.QQIntersection
{
    public partial class IntersectionDetail : GraphicObjectDetail
    {
        private CurveTabPage _curveTabPage1;
        private CurveTabPage _curveTabPage2;
        private SimpleCurvesTabPage _simpleCurveTabPage;
        private PointsTabPage _pointsTabPage;
        private EquationsTabPage _equationsTabPage;
        private PlanEquationsTabPage _planTabPage;
        private CenterTabPage _centerTabPage;
        private RotationTabPage _rotationTabPage;
        private DrawableTabPage _drawableTabPage;

        public IntersectionDetail(Form1 form, IIntersection intersection)
            : base(form, intersection)
        {
            base.SetWindowProperties();
            _curveTabPage1 = new CurveTabPage(_minimumGroupBoxSize);
            _curveTabPage2 = new CurveTabPage(_minimumGroupBoxSize);
            _simpleCurveTabPage = new SimpleCurvesTabPage(_minimumGroupBoxSize);
            _pointsTabPage = new PointsTabPage(_form, this, _minimumGroupBoxSize);
            _equationsTabPage = new EquationsTabPage(_form, this, _minimumGroupBoxSize);
            _planTabPage = new PlanEquationsTabPage(_form, this, _minimumGroupBoxSize);
            _centerTabPage = new CenterTabPage(_form, this, _minimumGroupBoxSize);
            _rotationTabPage = new RotationTabPage(_form, this, _minimumGroupBoxSize);
            _drawableTabPage = new DrawableTabPage(_form, this, _minimumGroupBoxSize);
            _curveTabPage1.Tag = false;
            _curveTabPage2.Tag = false;
            _simpleCurveTabPage.Tag = false;
            _pointsTabPage.Tag = false;
            _equationsTabPage.Tag = false;
            _planTabPage.Tag = false;
            _centerTabPage.Tag = false;
            _rotationTabPage.Tag = false;
            _drawableTabPage.Tag = false;

            _curveTabPage1.DisableButtons();
            _curveTabPage2.DisableButtons();
            _simpleCurveTabPage.DisableButtons();
            _pointsTabPage.DisableButtons();
            _equationsTabPage.DisableButtons();
            _planTabPage.DisableButtons();
            _centerTabPage.DisableButtons();
            _rotationTabPage.DisableButtons();

            this.Text = "Detail průniku";
            int minimumWidth = _minimumGroupBoxSize.Width;
            GenerateContents(intersection);
        }

        private void AddQuadricContents(Quadric quadric)
        {
            _equationsTabPage.GenerateContents(quadric);
            _centerTabPage.GenerateContents(quadric);
            _rotationTabPage.GenerateContents(quadric);

            base.InsertToTabControl(0, _equationsTabPage);
            base.InsertToTabControl(1, _centerTabPage);
            base.InsertToTabControl(2, _rotationTabPage);
        }
        private void AddConicContents(ConicSection conic)
        {
            _equationsTabPage.GenerateContents(conic);
            _planTabPage.GenerateContents(conic);
            _centerTabPage.GenerateContents(conic);
            _rotationTabPage.GenerateContents(conic);

            base.InsertToTabControl(0, _equationsTabPage);
            base.InsertToTabControl(1, _planTabPage); 
            base.InsertToTabControl(2, _centerTabPage);
            base.InsertToTabControl(3, _rotationTabPage);
        }
        private void AddPlanContents(Plan plan)
        {
            _equationsTabPage.GenerateContents(plan);
            _centerTabPage.GenerateContents(plan);
            _rotationTabPage.GenerateContents(plan);

            base.InsertToTabControl(0, _equationsTabPage);
            base.InsertToTabControl(1, _centerTabPage);
            base.InsertToTabControl(2, _rotationTabPage);
        }
        private void AddLineContents(Line line)
        {
            _equationsTabPage.GenerateContents(line);
            _centerTabPage.GenerateContents(line);
            _rotationTabPage.GenerateContents(line);

            base.InsertToTabControl(0, _equationsTabPage);
            base.InsertToTabControl(1, _centerTabPage);
            base.InsertToTabControl(2, _rotationTabPage);
        }
        private void AddPointsContents(List<GPoint> points, IIntersection intersection)
        {
            _pointsTabPage.GenerateContents(intersection);
            base.AddToTabControl(_pointsTabPage);
        }
        private void AddComplexCurvesContents(List<CurveAsQuadricsIntersection> curves)
        {
            if (curves.Count == 0)
                return;
            if (curves.Count == 1)
            {
                _curveTabPage1.GenerateContents(curves[0]);
                base.AddToTabControl(_curveTabPage1);
            }
            if (curves.Count == 2)
            {
                _curveTabPage1.GenerateContents(curves[0]);
                _curveTabPage2.GenerateContents(curves[1]);
                base.AddToTabControl(_curveTabPage2);
                base.AddToTabControl(_curveTabPage1);
            }
        }
        private void AddSimpleCurvesContents(List<CurveAsQuadricsIntersection> curves, IIntersection intersection) 
        {
            _simpleCurveTabPage.GenerateContents(intersection);
            base.AddToTabControl(_simpleCurveTabPage);
        }
        private void AddEmptyContents(EmptyIntersection empty)
        {
        }


        /// <summary>
        /// U libovolného průniku nechceme povolit editaci něčeho jiného než Drawable vlastností.
        /// </summary>
        public override void EnableEditButtons()
        {
            _drawableTabPage.EnableButtons();
        }

        protected override void GenerateIntersectionContents(IIntersection intersection)
        {
            _graphicObject = intersection;
            SuspendLayout();
            base.ClearTabPages();

            List<CurveAsQuadricsIntersection> complexCurves = new List<CurveAsQuadricsIntersection>();
            List<CurveAsQuadricsIntersection> simpleCurves = new List<CurveAsQuadricsIntersection>();
            List<GPoint> points = new List<GPoint>();
            foreach (GraphicObject o in intersection.GetIntersections())
            {
                if (o is GPoint)
                    points.Add((GPoint)o);
                if (o is CurveAsQuadricsIntersection)
                    if (((CurveAsQuadricsIntersection)o).GetVParametrization() is ConstantU ||
                        ((CurveAsQuadricsIntersection)o).GetVParametrization() is ConstantV)
                        simpleCurves.Add((CurveAsQuadricsIntersection)o);
                    else complexCurves.Add((CurveAsQuadricsIntersection)o);
            }
            foreach (GraphicObject go in intersection.GetIntersections())
            {
                if (go is GPoint || go is CurveAsQuadricsIntersection)
                    continue;
                if (go is Quadric)
                    AddQuadricContents((Quadric)go);
                if (go is ConicSection)
                    AddConicContents((ConicSection)go);
                if (go is Plan)
                    AddPlanContents((Plan)go);
                if (go is Line)
                    AddLineContents((Line)go);
                if (go is EmptyIntersection)
                    AddEmptyContents((EmptyIntersection)go);
            }
            if (complexCurves.Count > 0)
                AddComplexCurvesContents(complexCurves);
            if (simpleCurves.Count > 0)
                AddSimpleCurvesContents(simpleCurves, intersection);
            if (points.Count > 0)
                AddPointsContents(points, intersection);

            _drawableTabPage.GenerateContents(intersection);
            base.AddToTabControl(_drawableTabPage);
            SetSelectedTabPage(0); // base hlida hranice
            ResumeLayout();
        }

        protected void GenerateQuadricsIntersectionContents(QuadricAndQuadricIntersection intersection)
        {
            if ((bool)_pointsTabPage.Tag) { base.RemoveTabPage(_pointsTabPage); _pointsTabPage.Tag = false; }
            if ((bool)_planTabPage.Tag) { base.RemoveTabPage(_planTabPage); _planTabPage.Tag = false; }
            if ((bool)_equationsTabPage.Tag) { base.RemoveTabPage(_equationsTabPage); _equationsTabPage.Tag = false; }
            if ((bool)_centerTabPage.Tag) { base.RemoveTabPage(_centerTabPage); _centerTabPage.Tag = false; }
            if ((bool)_rotationTabPage.Tag) { base.RemoveTabPage(_rotationTabPage); _rotationTabPage.Tag = false; }

            if (!(bool)_drawableTabPage.Tag) { base.InsertToTabControl(GetTabPagesCount(), _drawableTabPage); _drawableTabPage.Tag = true; }

            bool containsSimpleCurve = false;
            Line line = null;
            bool containsPoint = false;
            List<CurveAsQuadricsIntersection> complexCurves = new List<CurveAsQuadricsIntersection>();
            foreach (GraphicObject o in intersection.GetIntersections())
            {
                if (o is GPoint)
                    containsPoint = true;
                if (o is Line)
                    line = (Line)o;
                if (o is CurveAsQuadricsIntersection)
                    if (((CurveAsQuadricsIntersection)o).GetVParametrization() is ConstantU ||
                        ((CurveAsQuadricsIntersection)o).GetVParametrization() is ConstantV)
                        containsSimpleCurve = true;
                    else complexCurves.Add((CurveAsQuadricsIntersection)o);
            }
            if (containsPoint)
            {
                _pointsTabPage.GenerateContents(intersection);
                if (!(bool)_pointsTabPage.Tag) { base.InsertToTabControl(0, _pointsTabPage); _pointsTabPage.Tag = true; }
            }
            else if ((bool)_pointsTabPage.Tag) { base.RemoveTabPage(_pointsTabPage); _pointsTabPage.Tag = false; }

            if (containsSimpleCurve)
            {
                _simpleCurveTabPage.GenerateContents(intersection);
                if (!(bool)_simpleCurveTabPage.Tag) { base.InsertToTabControl(0, _simpleCurveTabPage); _simpleCurveTabPage.Tag = true; }
            }
            else if ((bool)_simpleCurveTabPage.Tag) { base.RemoveTabPage(_simpleCurveTabPage); _simpleCurveTabPage.Tag = false; }

            if (complexCurves.Count == 1)
            {
                _curveTabPage1.GenerateContents(complexCurves[0]);
                if (!(bool)_curveTabPage1.Tag) { base.InsertToTabControl(0, _curveTabPage1); _curveTabPage1.Tag = true; }
                if ((bool)_curveTabPage2.Tag) { base.RemoveTabPage(_curveTabPage2); _curveTabPage2.Tag = false; }
            }
            else if (complexCurves.Count == 2)
            {
                _curveTabPage1.GenerateContents(complexCurves[0]);
                if (!(bool)_curveTabPage1.Tag) { base.InsertToTabControl(0, _curveTabPage1); _curveTabPage1.Tag = true; }
                _curveTabPage2.GenerateContents(complexCurves[1]);
                if (!(bool)_curveTabPage2.Tag) { base.InsertToTabControl(1, _curveTabPage2); _curveTabPage2.Tag = true; }
            }
            else
            {
                if ((bool)_curveTabPage1.Tag) { base.RemoveTabPage(_curveTabPage1); _curveTabPage1.Tag = false; }
                if ((bool)_curveTabPage2.Tag) { base.RemoveTabPage(_curveTabPage2); _curveTabPage2.Tag = false; }
            }

            _drawableTabPage.GenerateContents(intersection);
            _curveTabPage1.DisableButtons();
            _curveTabPage2.DisableButtons();
            _pointsTabPage.DisableButtons();
            _simpleCurveTabPage.DisableButtons();
        }
        protected void GenerateConicsIntersectionContents(ConicAndConicIntersection intersection) 
        {

        }
    }
}
