﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GraphicObjects;

using Geometry.GGraphic.Forms.Detail.Layout;
using GMath;
using Geometry.GGraphic.Forms.Detail;
using Geometry.GGraphic;

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class AngleDialog : IDialog, IInterActive
    {
        private TransformationMatrix _initialRotation;

        /// <summary>
        /// Tyto promenne nesou informaci o hodnotach před posunutím slidebaru.
        /// </summary>
        private RotationAngles _dynamicPolarValues;
        private RotationAngles _dynamicStandardValues;


        public AngleDialog(GraphicObject graphicObject, ManipulationManager manipulationManager,
            bool includeCanonicRotationTabPage)
            :base(manipulationManager, graphicObject)
        {
            InitializeComponent(); 
            if (!includeCanonicRotationTabPage)
                this.tabControl1.TabPages.RemoveAt(0);
            this.Text = "Editace rotace objektu";
            SetBounds();
            _dynamicPolarValues = new RotationAngles(0, 0, 0);
            _dynamicStandardValues = new RotationAngles(0, 0, 0);
            GenerateContents(graphicObject);
            //testFunction();
        }

        public override void GenerateContents(GraphicObject graphicObject)
        {
            _graphicObject = graphicObject;
            _initialRotation = ((WithParametricEquationObject)graphicObject).GetRotationInStandardBasis();
            RotationAngles rotationAngles = ((WithParametricEquationObject)graphicObject).GetRotationAnglesInStandardBasis();
            SetStaticAngleValues(rotationAngles);
            SetStaticSlideBarsValues(rotationAngles);
        }

        public TransformationMatrix GetInitialRotation() 
        {
            return _initialRotation;
        }
        public bool GetUseLowerQuality() 
        {
            return checkBoxQuality.Checked;
        }
        
        private void SetBounds() 
        {
            int width = tabControl1.Width - 2;
            int height = checkBoxQuality.Location.Y + checkBoxQuality.Height + 
                LayoutProperties.ControlsTopSpace * 2 + buttonOK.Height;
            this.ClientSize = new Size(width, height);
        }

        public void SetStaticSlideBarsValues(RotationAngles angles) 
        {
            UnConnectStaticEvents();
            try{
                this.xScrollStaticStandard.Value = (int)angles.GetXAngle() + 180;
                this.yScrollStaticStandard.Value = (int)angles.GetYAngle() + 180;
                this.zScrollStaticStandard.Value = (int)angles.GetZAngle() + 180;
            }catch(Exception)
            {
                System.Media.SystemSounds.Asterisk.Play();
                ((WithParametricEquationObject)_graphicObject).GetRotationAnglesInStandardBasis();
            }
            ConnectStaticEvents();
        }
        public void SetStaticAngleValues(RotationAngles rotationAngles)
        {
            int xAngle = (int)rotationAngles.GetXAngle();
            int yAngle = (int)rotationAngles.GetYAngle();
            int zAngle = (int)rotationAngles.GetZAngle();

            xLabelValueStatic.Text = xAngle.ToString() + "°";
            yLabelValueStatic.Text = yAngle.ToString() + "°";
            zLabelValueStatic.Text = zAngle.ToString() + "°";
            xLabelValueStatic.Refresh();
            yLabelValueStatic.Refresh();
            zLabelValueStatic.Refresh();
        }


        private int ParseXValueDynamicPolar() { return (int)xScrollDynamicPolar.Value - 180; }
        private int ParseYValueDynamicPolar() { return (int)yScrollDynamicPolar.Value - 180; }
        private int ParseZValueDynamicPolar() { return (int)zScrollDynamicPolar.Value - 180; }

        private int ParseXValueDynamicStandard() { return (int)xScrollDynamicStandard.Value - 180; }
        private int ParseYValueDynamicStandard() { return (int)yScrollDynamicStandard.Value - 180; }
        private int ParseZValueDynamicStandard() { return (int)zScrollDynamicStandard.Value - 180; }

        private int ParseXValueStatic() { return (int)xScrollStaticStandard.Value - 180; }
        private int ParseYValueStatic() { return (int)yScrollStaticStandard.Value - 180; }
        private int ParseZValueStatic() { return (int)zScrollStaticStandard.Value - 180; }

        private RotationAngles ParseAnglesDynamicPolar()
        {
            double xAngle = ParseXValueDynamicPolar();
            double yAngle = ParseYValueDynamicPolar();
            double zAngle = ParseZValueDynamicPolar();
            return new RotationAngles(xAngle, yAngle, zAngle);
        }
        private RotationAngles ParseAnglesDynamicStandard()
        {
            double xAngle = ParseXValueDynamicStandard();
            double yAngle = ParseYValueDynamicStandard();
            double zAngle = ParseZValueDynamicStandard();
            return new RotationAngles(xAngle, yAngle, zAngle);
        }
        private RotationAngles ParseAnglesStatic()
        {
            double xAngle = ParseXValueStatic();
            double yAngle = ParseYValueStatic();
            double zAngle = ParseZValueStatic();
            return new RotationAngles(xAngle, yAngle, zAngle);
        }




        private void xScrollDynamicPolar_ValueChanged(object sender, EventArgs e)
        {
            int x = ParseXValueDynamicPolar();
            RotationAngles anglesDelta = new RotationAngles(x - _dynamicPolarValues.GetXAngle(), 0, 0);
            _dynamicPolarValues.SetXAngle(x);
            DynamicPolarRotationAnglesChanged(anglesDelta);
        }
        private void yScrollDynamicPolar_ValueChanged(object sender, EventArgs e)
        {
            int y = ParseYValueDynamicPolar();
            RotationAngles anglesDelta = new RotationAngles(0, y - _dynamicPolarValues.GetYAngle(), 0);
            _dynamicPolarValues.SetYAngle(y);
            DynamicPolarRotationAnglesChanged(anglesDelta);
        }
        private void zScrollDynamicPolar_ValueChanged(object sender, EventArgs e)
        {
            int z = ParseZValueDynamicPolar();
            RotationAngles anglesDelta = new RotationAngles(0, 0, z - _dynamicPolarValues.GetZAngle());
            _dynamicPolarValues.SetZAngle(z);
            DynamicPolarRotationAnglesChanged(anglesDelta);
        }

        private void xScrollDynamicStandard_ValueChanged(object sender, EventArgs e)
        {
            int x = ParseXValueDynamicStandard();
            RotationAngles anglesDelta = new RotationAngles(x - _dynamicStandardValues.GetXAngle(), 0, 0);
            _dynamicStandardValues.SetXAngle(x);
            DynamicStandardRotationAnglesChanged(anglesDelta);
        }
        private void yScrollDynamicStandard_ValueChanged(object sender, EventArgs e)
        {
            int y = ParseYValueDynamicStandard();
            RotationAngles anglesDelta = new RotationAngles(0, y - _dynamicStandardValues.GetYAngle(), 0);
            _dynamicStandardValues.SetYAngle(y);
            DynamicStandardRotationAnglesChanged(anglesDelta);
        }
        private void zScrollDynamicStandard_ValueChanged(object sender, EventArgs e)
        {
            int z = ParseZValueDynamicStandard();
            RotationAngles anglesDelta = new RotationAngles(0, 0, z - _dynamicStandardValues.GetZAngle());
            _dynamicStandardValues.SetZAngle(z);
            DynamicStandardRotationAnglesChanged(anglesDelta);
        }

        private void xScrollStatic_Scroll(object sender, EventArgs e)
        {
            StaticRotationAnglesChanged();
        }
        private void yScrollStatic_Scroll(object sender, EventArgs e)
        {
            StaticRotationAnglesChanged();
        }
        private void zScrollStatic_Scroll(object sender, EventArgs e)
        {
            StaticRotationAnglesChanged();
        }

        private void DynamicStandardRotationAnglesChanged(RotationAngles anglesDelta)
        {
            _manipulationManager.ApplyObjectsStandardRotation(this, anglesDelta);
            RotationAngles rotationAngles = ((WithParametricEquationObject)_graphicObject).GetRotationAnglesInStandardBasis();
            SetStaticAngleValues(rotationAngles);
            SetStaticSlideBarsValues(rotationAngles);
        }
        private void DynamicPolarRotationAnglesChanged(RotationAngles anglesDelta)
        {
            _manipulationManager.ApplyObjectsCanonicRotation(this, anglesDelta);
            RotationAngles rotationAngles = ((WithParametricEquationObject)_graphicObject).GetRotationAnglesInStandardBasis();
            SetStaticAngleValues(rotationAngles);
            SetStaticSlideBarsValues(rotationAngles);
        }
        private void StaticRotationAnglesChanged()
        {
            _manipulationManager.SetObjectsStandardRotation(this, ParseAnglesStatic());
            RotationAngles rotationAngles = ((WithParametricEquationObject)_graphicObject).GetRotationAnglesInStandardBasis();
            SetStaticAngleValues(ParseAnglesStatic());
        }


        private void ConnectStaticEvents() 
        {
            xScrollStaticStandard.ValueChanged += new EventHandler(xScrollStatic_Scroll);
            yScrollStaticStandard.ValueChanged += new EventHandler(yScrollStatic_Scroll);
            zScrollStaticStandard.ValueChanged += new EventHandler(zScrollStatic_Scroll);
        }
        private void UnConnectStaticEvents() 
        {
            xScrollStaticStandard.ValueChanged -= new EventHandler(xScrollStatic_Scroll);
            yScrollStaticStandard.ValueChanged -= new EventHandler(yScrollStatic_Scroll);
            zScrollStaticStandard.ValueChanged -= new EventHandler(zScrollStatic_Scroll);
        }


        private void buttonOK_Click(object sender, EventArgs e)
        {
            TransformationMatrix actualRotation = ((WithParametricEquationObject)_graphicObject).GetRotationInStandardBasis();
            if (!TransformationMatrix.AreMatrixesEqualInTolerance(_initialRotation, actualRotation))
                DialogResult = DialogResult.OK;
            Close();
        }
        private void buttonStorno_Click(object sender, EventArgs e)
        {
            Close();
        }
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            _manipulationManager.EditObjectsRotationClosed(this);
        }


        public void testFunction() 
        {
            int x = 0;
            int y = 0;
            int z = 0;
            while (z < 360)
            {
                //z++;
                //if (z > 360)
                //    z = 0;
                //this.zScrollDynamicStandard.Value = z;

                //break;
                //x+=2;
                if (x < 360)
                {
                    x+=3;
                    this.xScrollDynamicStandard.Value = x;
                    //break;
                }
                else
                    if (y < 360)
                    {
                        x = 0;
                        y+=3;
                        this.yScrollDynamicStandard.Value = y;
                    }
                    else if (z < 360)
                    {
                        y = 0;
                        z+=3;
                        this.zScrollDynamicStandard.Value = z;
                    }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            testFunction();
        }

        private void buttonStorno_KeyDown(object sender, KeyEventArgs e)
        {
            //Form1.form.MSAATEST();
        }
    }
}
