﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.GraphicObjects;
using Geometry.GGraphic;
using Geometry.BaseClasses;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GGraphic.Controls;  

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class EditCanonicEquationDialog : IEditEquation
    {
        protected KoefsAndCenterPair _actualKoefsAndCenterPair;
        protected KoefsAndCenterPair _initialKoefsAndCenterPair;

        public EditCanonicEquationDialog() { }
        public EditCanonicEquationDialog(ManipulationManager manipulationManager, WithCanonicEquationObject graphicObject)
            : base(manipulationManager, graphicObject, new CanonicEquationGR(), 
            new KoefsAndCenterControl(GetParametricKoeficientsControl(graphicObject), GetCenterKoeficientsControl(graphicObject)))
        {
            this.Text = "Editace kanonické rovnice";
            base._groupBox.Text = "Kanonická rovnice:";
            GenerateContents(graphicObject);
        }
        
        private static IParametricKoeficientsControl GetParametricKoeficientsControl(WithParametricEquationObject graphicObject)
        {
            if (graphicObject is Quadric)
                return new QuadricCanonicKoefsControl(graphicObject.GetParametricEquation());
            if (graphicObject is ConicSection)
                return new ConicCanonicKoefsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Plan)
                return new PlanParametricKoeficientsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Line)
                return new LineParametricKoeficientsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Geometry.GraphicObjects.Planar.GPoint)
                return new PointParametricKoeficientsControl();
            throw new Exception("Nelze sestavit parametricKoeficientsControl daného objektu.");
        }
        private static CenterKoeficientsControl GetCenterKoeficientsControl(WithParametricEquationObject graphicObject) 
        {
            WithCanonicEquationObject wcego = (WithCanonicEquationObject)graphicObject;
            IParametricKoeficientsControl koefsControl = GetParametricKoeficientsControl(graphicObject);
            return new CenterKoeficientsControl(wcego.GetCenterInCanonicBasis(), (Dimension)koefsControl.GetParametersCount());
        }
        public override void GenerateContents(GraphicObject graphicObject) 
        {
            WithCanonicEquationObject wcego = (WithCanonicEquationObject)graphicObject;
            _graphicObject = graphicObject;;
            _equationGRControl.SetContents(wcego, wcego.GetCanonicEquationFormatString().GetGeneralFormatString());
            _equationGRControl.SetRealDynamicSize();

            _initialKoefsAndCenterPair = ((KoefsAndCenterControl)_koeficientsControl).ParseKoefsAndCenterPair();
            PerformCustomLayout(new List<Control>() { _groupBox, _koeficientsControl });
        }

        public KoefsAndCenterPair GetKoefsAndCenterPair() 
        {
            return _actualKoefsAndCenterPair;
        }
        
        protected override void OkButtonClick(object sender, EventArgs e)
        {
            KoefsAndCenterControl koefsAndCenterControl = (KoefsAndCenterControl)_koeficientsControl;
            _actualKoefsAndCenterPair = koefsAndCenterControl.ParseKoefsAndCenterPair();
            // null znamená chybu, informace o ní však již byla sdělena.
            if (_actualKoefsAndCenterPair.GetCenterParametrization() != null &&
                _actualKoefsAndCenterPair.GetParametricEquation() != null) 
            {
                if (!KoefsAndCenterPair.AreEqualInTolerance(_initialKoefsAndCenterPair, _actualKoefsAndCenterPair))
                    DialogResult = DialogResult.OK;
                Close();
            }
        }
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            _manipulationManager.EditCanonicEquationClosed(this);
        }
    }
}
