﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GGraphic.Forms.Detail.Controls; 
using Geometry.GraphicObjects.Equations;

using Geometry.GGraphic;
using Geometry.GraphicObjects;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GGraphic.Controls.ElementaryControls;
using Geometry.GGraphic.Controls;  

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class EditCommonEquationDialog : IEditEquation
    {
        protected CommonEquationFormatString _commonEquationFormatString;
        protected CleverLabel _resultEquationLabel;
        protected CommonEquation _initialEquation;
        protected CommonEquation _actualEquation;

        public EditCommonEquationDialog()
        {
        }

        public EditCommonEquationDialog(ManipulationManager manipulationManager, WithCommonEquationObject graphicObject)
            : base(manipulationManager, graphicObject, new CommonEquationGR(), GetCommonKoeficientsControl(graphicObject))
        {
            this.Text = "Editace obecné rovnice";
            base._groupBox.Text = "Obecná rovnice:";

            _resultEquationLabel = new CleverLabel();
            this.Controls.Add(_resultEquationLabel);
            ((ICommonKoeficientsControl)_koeficientsControl).SetAddObjectDialog(this);
            GenerateContents(graphicObject);
        }

        private static ICommonKoeficientsControl GetCommonKoeficientsControl(WithCommonEquationObject graphicObject)
        {
            if (graphicObject is Quadric)
                return new QuadricCommonKoefsControl(graphicObject.GetCommonEquation());
            if (graphicObject is ConicSection)
                return new ConicCommonKoefsControl(graphicObject.GetCommonEquation());
            if (graphicObject is Plan)
                return new PlanCommonKoefsControl(graphicObject.GetCommonEquation());
            throw new Exception("Nelze sestavit commonKoeficientsControl daného objektu.");
        }
        
        public override void GenerateContents(GraphicObject graphicObject) 
        {
            _graphicObject = graphicObject;
            _commonEquationFormatString = new CommonEquationFormatString(((WithCommonEquationObject)graphicObject).GetCommonEquationFormatString());
            _equationGRControl.SetContents(graphicObject, _commonEquationFormatString.GetGeneralFormatString());
            _equationGRControl.SetRealDynamicSize();            

            _initialEquation = ((ICommonKoeficientsControl)_koeficientsControl).ParseCommonEquationThrowError();
            UpdateLabel();
        }

        public CommonEquation GetCommonEquation() 
        {
            return _actualEquation;
        }

        private List<Control> GetListOfControls()
        {
            List<Control> list = new List<Control>();
            list.Add(_resultEquationLabel);
            list.Add(base._groupBox);
            list.Add(base._koeficientsControl);
            return list;
        }
        private void UpdateLabel()
        {
            CommonEquation equation = ((ICommonKoeficientsControl)base._koeficientsControl).ParseCommonEquationIgnoreError();
            this._commonEquationFormatString.SubstituteToFormatString(equation);
            string temp = _commonEquationFormatString.GetSubstitutedFormatString();
            temp = temp.Substring(2, temp.Length - 3);
            this._resultEquationLabel.Text = temp;
            base.PerformCustomLayout(GetListOfControls());
        }
        public override void TextBoxTextChanged()
        {
            UpdateLabel();
        }


        protected override void OkButtonClick(object sender, EventArgs e)
        {
            ICommonKoeficientsControl control = (ICommonKoeficientsControl)_koeficientsControl;
            _actualEquation = control.ParseCommonEquationThrowError();
            if (_actualEquation != null) // null znamená chybu, informace o ní však již byla sdělena.
            {
                if (!CommonEquation.AreEqualInTolerance(_initialEquation, _actualEquation))
                    DialogResult = DialogResult.OK; // pri editaci obecne rovnice ignoruji potvrzeni nezmenene rovnice
                Close();
            }
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            _manipulationManager.EditCommonEquationClosed(this);
        }
    }
}
