﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GGraphic.Forms.Detail.Controls;

using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic;
using GMath;
using Geometry.GraphicObjects;
using Geometry.BaseClasses;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GGraphic.Controls; 

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class EditParametricEquationDialog : IEditEquation
    {
        protected KoefsAndCenterPair _actualKoefsAndCenterPair;
        protected KoefsAndCenterPair _initialKoefsAndCenterPair;

        public EditParametricEquationDialog() 
        {
        }

        public EditParametricEquationDialog(ManipulationManager manipulationManager, WithParametricEquationObject graphicObject)
            : base(manipulationManager, graphicObject, new ParametricEquationsGR(),
            new KoefsAndCenterControl(GetParametricKoeficientsControl(graphicObject), GetCenterKoeficientsControl(graphicObject)))
        {
            this.Text = "Editace parametrické rovnice";
            base._groupBox.Text = "Parametrická rovnice:";
            GenerateContents(graphicObject);
        }

        private static IParametricKoeficientsControl GetParametricKoeficientsControl(WithParametricEquationObject graphicObject)
        {
            if (graphicObject is Quadric)
                return new QuadricCanonicKoefsControl(graphicObject.GetParametricEquation());
            if (graphicObject is ConicSection)
                return new ConicCanonicKoefsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Plan)
                return new PlanParametricKoeficientsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Line)
                return new LineParametricKoeficientsControl(graphicObject.GetParametricEquation());
            if (graphicObject is Geometry.GraphicObjects.Planar.GPoint)
                return new PointParametricKoeficientsControl();
            throw new Exception("Nelze sestavit parametricKoeficientsControl daného objektu.");
        }
        private static CenterKoeficientsControl GetCenterKoeficientsControl(WithParametricEquationObject graphicObject)
        {
            CenterParametrization center = graphicObject.GetCenterInStandardBasis();
            if (graphicObject is WithCanonicEquationObject)
                center = ((WithCanonicEquationObject)graphicObject).GetCenterInCanonicBasis();
            IParametricKoeficientsControl koefsControl = GetParametricKoeficientsControl(graphicObject);
            return new CenterKoeficientsControl(center, (Dimension)koefsControl.GetParametersCount()); 
        }

        public override void GenerateContents(GraphicObject graphicObject) 
        {
            WithParametricEquationObject wpego = (WithParametricEquationObject)graphicObject;
            _graphicObject = graphicObject;
            _equationGRControl.SetContents(
                wpego,
                wpego.GetParametricEquationsFormatString().GetXGeneralFormatString(),
                wpego.GetParametricEquationsFormatString().GetYGeneralFormatString(),
                wpego.GetParametricEquationsFormatString().GetZGeneralFormatString());
            _equationGRControl.SetRealDynamicSize();

            _initialKoefsAndCenterPair = ((KoefsAndCenterControl)_koeficientsControl).ParseKoefsAndCenterPair();
            PerformCustomLayout(new List<Control>() { _groupBox, _koeficientsControl });
        }

        public KoefsAndCenterPair GetKoefsAndCenterPair() 
        {
            return _actualKoefsAndCenterPair;
        }
        
        protected override void OkButtonClick(object sender, EventArgs e)
        {
            KoefsAndCenterControl koefsAndCenterControl = (KoefsAndCenterControl)_koeficientsControl;
            _actualKoefsAndCenterPair = koefsAndCenterControl.ParseKoefsAndCenterPair();
            // null znamená chybu, informace o ní však již byla sdělena.
            if (_actualKoefsAndCenterPair.GetCenterParametrization() != null &&
                _actualKoefsAndCenterPair.GetParametricEquation() != null) 
            {
                if (!KoefsAndCenterPair.AreEqualInTolerance(_initialKoefsAndCenterPair, _actualKoefsAndCenterPair))
                    DialogResult = DialogResult.OK;
                Close();
            }
        }
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            _manipulationManager.EditParametricEquationsClosed(this);
        }
    }
}
