﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Geometry.GraphicObjects;
using GMath;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic;

namespace Geometry.GGraphic.Forms.Detail
{
    /// <summary>
    /// Dialog ma nastaven DialogResult na Cancel.
    /// V případě potvrzení nějaké hodnoty se DialogResult nastaví na OK.
    /// Při zavírání Dialogu se zavolá metoda _onCloseMethod.
    /// </summary>
    public partial class IDialog : Form
    {
        protected ManipulationManager _manipulationManager;
        protected GraphicObject _graphicObject;

        public IDialog() { }
        public IDialog(ManipulationManager manipulationManager, GraphicObject graphicObject)
        {
            InitializeComponent();
            _manipulationManager = manipulationManager;
            _graphicObject = graphicObject;
            DialogResult = DialogResult.Cancel;
        }

        public virtual void GenerateContents(GraphicObject graphicObject) 
        {
            _graphicObject = graphicObject;
        }
        public GraphicObject GetGraphicObject() 
        {
            return _graphicObject;
        }


        public CenterParametrization MakeTextBoxVersionOfCenter(CenterParametrization center)
        {
            Vector c = center.GetReprezentant();
            return new CenterParametrization(
                double.Parse(Program.GetOutputFormatedForTextBox(c.GetX())),
                double.Parse(Program.GetOutputFormatedForTextBox(c.GetY())),
                double.Parse(Program.GetOutputFormatedForTextBox(c.GetZ()))
                );
        }
        public ParametricEquation MakeTextBoxVersionOfEquation(ParametricEquation equation)
        {
            List<double> values = new List<double>();
            for (int i = 0; i < equation.GetMembersCount(); i++)
            {
                values.Add(double.Parse(Program.GetOutputFormatedForTextBox(equation.Get(i))));
            }
            return new ParametricEquation(values);
        }
        public CommonEquation MakeTextBoxVersionOfEquation(CommonEquation equation)
        {
            List<double> values = new List<double>();
            for (int i = 0; i < equation.GetMembersCount(); i++)
            {
                values.Add(double.Parse(Program.GetOutputFormatedForTextBox(equation.Get(i))));
            }
            return new CommonEquation(values);
        }
        public KoefsAndCenterPair MakeTextBoxVersionOfPair(KoefsAndCenterPair pair) 
        {
            return new KoefsAndCenterPair(
                MakeTextBoxVersionOfEquation(pair.GetParametricEquation()),
                MakeTextBoxVersionOfCenter(pair.GetCenterParametrization()));
        }
    }
}
