﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Geometry.GGraphic.Forms.Detail.Controls;
using Geometry.GGraphic.Forms.Detail.Layout;
using Geometry.GraphicObjects;
using Geometry.GGraphic.Controls;
using Geometry.GGraphic.Controls.GroupBoxes;

namespace Geometry.GGraphic.Forms.Detail
{
    public partial class IEditEquation : IDialog
    {
        protected IEquationGR _equationGRControl;
        protected IKoeficientsControl _koeficientsControl;
        protected ConstantGroupBox _groupBox;

        public IEditEquation() 
        {
        }

        public IEditEquation(ManipulationManager manipulationManager, GraphicObject graphicObject, 
                            IEquationGR equationGR, IKoeficientsControl koeficientsControl)
            : base(manipulationManager, graphicObject)
        {
            InitializeComponent();
            _equationGRControl = equationGR;
            _koeficientsControl = koeficientsControl;
            _groupBox = new ConstantGroupBox(new Size(0, 0));            
            _groupBox.SetContents(new List<Control>() { _equationGRControl }, GroupBoxLayoutStyle.ControlInCenter);
            this.Controls.Add(_groupBox);
            this.Controls.Add(_koeficientsControl);  
            okButton.TabIndex = 10;
            stornoButton.TabIndex = 11;
        }


        protected void SetTabOrder(int okOrder) 
        {
            okButton.TabIndex = okOrder;
            stornoButton.TabIndex = okOrder + 1;
        }




        protected void PerformCustomLayout(List<Control> controls)
        {
            _groupBox.SuspendLayout();
            _equationGRControl.SuspendLayout();
            _koeficientsControl.SuspendLayout();

            _groupBox.SetConstantMimimumSize();

            PerformXCenterLayoutX(controls);
            PerformYRowsLayoutY(controls);
            SetSize(controls);
            SetButtonsPosition();

            _groupBox.ResumeLayout();
            _equationGRControl.ResumeLayout();
            _koeficientsControl.ResumeLayout();
        }
        private void PerformXCenterLayoutX(List<Control> controls) 
        {
            int width = LayoutProperties.ControlsSideSpace + okButton.Width + stornoButton.Width;
            foreach (Control control in controls)
            {
                width = Math.Max(width, control.Width);
            }
            _groupBox.SetConstantWidth(width);
            foreach (Control control in controls)
            {
                control.Location =
                    new Point((width - control.Width) / 2 + LayoutProperties.WindowSideSpace, control.Location.Y);
            }
        }
        private void PerformYRowsLayoutY(List<Control> controls)
        {
            int y = LayoutProperties.WindowTopSpace;
            foreach (Control control in controls)
            {
                control.Location =
                    new Point(control.Location.X, y);
                y += control.Height + LayoutProperties.ControlsTopSpace;
            }
        }

        private void SetSize(List<Control> controls) 
        {
            int width = 0;
            int height = 0;
            foreach (Control control in controls)
            {
                width = Math.Max(width, control.Width);
                height = Math.Max(height, control.Location.Y + control.Height);
            }
            this.ClientSize = new Size(
                width + LayoutProperties.WindowSideSpace * 2,
                height + LayoutProperties.WindowSideSpace * 2 + stornoButton.Height);
        }
        private void SetButtonsPosition() 
        {
            int x = this.ClientSize.Width;
            int y = this.ClientSize.Height;


            this.stornoButton.Location =
                new Point(
                    x - stornoButton.Width - LayoutProperties.WindowSideSpace,
                    y - stornoButton.Height - LayoutProperties.WindowSideSpace);
            this.okButton.Location =
                new Point(
                    stornoButton.Location.X - okButton.Width - LayoutProperties.TextSideSpace,
                    stornoButton.Location.Y); 
        }


        public virtual void TextBoxTextChanged() { }



        protected virtual void OkButtonClick(object sender, EventArgs e) { }
        protected void cancelButton_Click(object sender, EventArgs e) 
        {
            Close();
        }
    }
}
