﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Tao.OpenGl;
using System.IO;
using Geometry.GraphicObjects;
using System.Threading;
using GMath;
using Geometry.LoadSave;
using Tao.Platform.Windows;
using Geometry.BaseClasses.CustomCursor;
using Geometry.GGraphic.Controls.ElementaryControls;
using System.Drawing;
using Geometry.GGraphic.Forms;
using System.Runtime.InteropServices;


namespace Geometry 
{
    public partial class Form1 : Form
    {
        private Scene _scene;
        private Color _enabledListViewItemColor = SystemColors.WindowText;
        private Color _disabledListViewItemColor = SystemColors.GrayText;
        private Cursor _rotationCursor;
        private Cursor _scaleCursor;
        private ListViewContextMenu _listViewContextMenu;
        private ManipulationManager _manipulationManager;

        private bool _trackBarWarningLock = false;
        private double _wheelZoomStep = 0.5;
        private bool _rightButtonDown = false;
        private bool _leftButtonDown = false;
        private bool _middleButtonDown = false;
        private Point oldMouseLocation = new Point();
        private Point newMouseLocation = new Point();
        private bool _errorDuringSave;
        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();

        public static Form1 form;
        public Form1()
        {
            form = this;
            InitializeComponent();
            _scene = new Scene();         // musí existovat před inicializací openGl, protože vznikne nový kontext, tak se zavolá přebuildování.
            this.panel1.SetScene(_scene); // panelu se musí nastavit scéna ať ví, co má kreslit.
            InitializeMultisampling();
            timer.Tick += new EventHandler(ClearProgressBarText);
            //this.simpleOpenGlControl1.InitializeContexts();
            LoadCursors();
            _listViewContextMenu = new ListViewContextMenu(this);
            _manipulationManager = new ManipulationManager(this); // musí být vytvořen po Inicializaci openGl, protože Builduje osy.
            this.MouseWheel += new MouseEventHandler(mouseWheel);          
            //this.OnResize(new EventArgs());
            this.toolStripStatusLabel1.Text = "";
            SetZoomTrackBarValue(_scene.GetSceneZoom());
            vyplňitPlochyToolStripMenuItem.Checked = true;
            //OpenGlMemmoryFallTest();
        }
        protected void LoadCursors()
        {
            try
            {
                _rotationCursor = RotationCursor.NativeMethods.LoadCustomCursor(
                    Directory.GetCurrentDirectory() + "\\Depencies\\RotationCursor.cur");
                _scaleCursor = RotationCursor.NativeMethods.LoadCustomCursor(
                    Directory.GetCurrentDirectory() + "\\Depencies\\ScaleCursor.cur");
            }
            catch (Exception)
            {
                _rotationCursor = System.Windows.Forms.Cursors.Default;
                _scaleCursor = System.Windows.Forms.Cursors.Default;
                ShowWarning("Nebyly nalezeny soubory kurzorů, místo nich bude použit defaultní kurzor.");
            }
        }


        public Point SumPoints(Point a, Point b)
        {
            return new Point(a.X + b.X, a.Y + b.Y);
        }
        public Scene GetScene()
        {
            return _scene;
        }
        public ManipulationManager GetManipulation()
        {
            return _manipulationManager;
        }
        public int GetOpenglControlWidth()
        {
            return this.panel1.ClientRectangle.Width;
        }
        public int GetOpenglControlHeight()
        {
            return this.panel1.ClientRectangle.Height;
        }

        public static void ShowWarning(string text)
        {
            MessageBox.Show(
                text,
                "Upozornění",
                MessageBoxButtons.OK,
                MessageBoxIcon.Warning);
        }
        public static void ShowError(string text) 
        {
            MessageBox.Show(
                text,
                "Chyba",
                MessageBoxButtons.OK,
                MessageBoxIcon.Error);
        }
        public static DialogResult ShowQuestion(string text) 
        {
            return MessageBox.Show(text, "Dotaz",
                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);
        }
        public static void ShowAnounce(string text)
        {
            MessageBox.Show(
                text,
                "Oznámení",
                MessageBoxButtons.OK,
                MessageBoxIcon.Information);
        }


        public void SetProgressBarText(string text) 
        { 
            toolStripStatusLabel1.Text = text;
            statusStrip1.Refresh();
        }
        public void SetProgressBarText(string text, int timeout)
        {
            timer.Stop();
            toolStripStatusLabel1.Text = text;
            statusStrip1.Refresh();
            timer.Interval = timeout * 1000;
            timer.Start();
        }
        private void ClearProgressBarText(object sender, EventArgs e) 
        {
            timer.Stop();
            toolStripStatusLabel1.Text ="";
            statusStrip1.Refresh();
        } 

        

        ///////////////////////////////////////////////////////////////////////////////////////
        //                              Udalosti spojene s paintControlem
        ///////////////////////////////////////////////////////////////////////////////////////

        private void SetScaleCursor(object sender, EventArgs e)
        {
            this.Cursor = _scaleCursor;
        }
        private void SetDefaultCursor(object sender, EventArgs e) 
        {
            this.Cursor = Cursors.Default;
        }
        private void ShowZoomCursor() 
        {
            EventHandler e = new EventHandler(SetScaleCursor);
            this.Invoke(e);
            Thread.Sleep(200);
            e = new EventHandler(SetDefaultCursor);
            this.Invoke(e);
        }
        private void ShowWarningOnZoomTrackBar()
        {
            if (!_trackBarWarningLock)
            {
                _trackBarWarningLock = true;
                SetTrackBarRedBackColor(null, null);
                //EventHandler e = new EventHandler(SetTrackBarRedBackColor);
                //this.Invoke(e);
                Thread.Sleep(100);
                SetTrackBarDefaultBackColor(null, null);
                //e = new EventHandler(SetTrackBarDefaultBackColor);
                //this.Invoke(e);
                _trackBarWarningLock = false;
            }
        }
        private void SetTrackBarRedBackColor(object sender, EventArgs e) 
        {
            zoomTrackBar.BackColor = Color.Red;
        }
        private void SetTrackBarDefaultBackColor(object sender, EventArgs e)
        {
            zoomTrackBar.BackColor = System.Drawing.SystemColors.Control;
        }

        private void mouseWheel(object sender, MouseEventArgs e)
        {
            Thread t = new Thread(new ThreadStart(ShowZoomCursor));
            t.Start();
            if (e.Delta == 0)
                return;
            if (e.Delta > 0 && _scene.GetSceneZoom() - _wheelZoomStep >= 0)
            {
                _scene.IncSceneZoom(-_wheelZoomStep);
                SetZoomTrackBarValue(_scene.GetSceneZoom());
            }
            else if (e.Delta < 0 && _scene.GetSceneZoom() + _wheelZoomStep <= zoomTrackBar.Maximum)
            {
                _scene.IncSceneZoom(_wheelZoomStep);
                SetZoomTrackBarValue(_scene.GetSceneZoom());
            }
            else 
            {
                ShowWarningOnZoomTrackBar();
            }
            RenderScene();
        }
        private void simpleOpenGlControl1_MouseDown(object sender, MouseEventArgs e)
        {
            this.panel1.Focus();
            if (e.Button == MouseButtons.Left && !_rightButtonDown && !_middleButtonDown)
            {
                _leftButtonDown = true;
                this.Cursor = Cursors.Hand;
                oldMouseLocation = e.Location;
            }
            if (e.Button == MouseButtons.Right && !_leftButtonDown && !_middleButtonDown)
            {
                _rightButtonDown = true;
                this.Cursor = _rotationCursor;
                oldMouseLocation = e.Location;
            }
            if (e.Button == MouseButtons.Middle && !_rightButtonDown && !_leftButtonDown)
            {
                _middleButtonDown = true;
                this.Cursor = _scaleCursor;
                oldMouseLocation = e.Location;
            }
        }
        private void simpleOpenGlControl1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left && _leftButtonDown)
            {
                _leftButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (e.Button == MouseButtons.Right && _rightButtonDown)
            {
                _rightButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (e.Button == MouseButtons.Middle && _middleButtonDown)
            {
                _middleButtonDown = false;
                this.Cursor = Cursors.Default;
            }
            if (_leftButtonDown)
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.LeftMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
            if (_rightButtonDown)
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.RightMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
            if (_middleButtonDown)
            {
                Point panel1Center = new Point(panel1.Location.X + panel1.Width / 2, panel1.Location.Y + panel1.Height / 2);
                Point panel1CenterScreen = PointToScreen(panel1Center);
                Point oldMousePosition = Cursor.Position;
                Cursor.Position = panel1CenterScreen;
                MyMouse.MiddleMouseDown(panel1CenterScreen.X, panel1CenterScreen.Y);
                Cursor.Position = oldMousePosition;
            }
        }
        private void simpleOpenGlControl1_MouseMove(object sender, MouseEventArgs e)
        {
            if (_rightButtonDown)
            {
                newMouseLocation = e.Location;
                double xDelta = (oldMouseLocation.Y - newMouseLocation.Y);
                double yDelta = (oldMouseLocation.X - newMouseLocation.X);
                _scene.IncSceneAngles(xDelta / 3.0, yDelta / 3.0);
                oldMouseLocation = e.Location;
                RenderScene();
            }
            if (_leftButtonDown)
            {
                newMouseLocation = e.Location;
                double xDelta = (oldMouseLocation.X - newMouseLocation.X);
                double yDelta = -(oldMouseLocation.Y - newMouseLocation.Y);
                _scene.MoveSceneCenter(new Vector(xDelta / 10.0, yDelta / 10.0, 0));
                oldMouseLocation = e.Location;
                RenderScene();
            }
            if (_middleButtonDown) 
            {
                newMouseLocation = e.Location;
                double yDelta = (oldMouseLocation.Y - newMouseLocation.Y);
                if ((yDelta > 0 && _scene.GetSceneZoom() + yDelta * _wheelZoomStep > zoomTrackBar.Maximum)
                    || (yDelta < 0 && _scene.GetSceneZoom() + yDelta * _wheelZoomStep < zoomTrackBar.Minimum))
                {
                    ShowWarningOnZoomTrackBar();
                    return;
                }
                _scene.IncSceneZoom(yDelta * _wheelZoomStep);
                SetZoomTrackBarValue(_scene.GetSceneZoom());
                oldMouseLocation = e.Location;
                RenderScene();
            }
        }
        private void zoomTrackBar_Scroll(object sender, EventArgs e)
        {
            _scene.IncSceneZoom((zoomTrackBar.Maximum - zoomTrackBar.Value) - _scene.GetSceneZoom());
            RenderScene();
        }
        private void SetZoomTrackBarValue(double zoom) 
        {
            zoomTrackBar.Value = zoomTrackBar.Maximum - (int)zoom;
        }

        ///////////////////////////////////////////////////////////////////////////////////////
        //                              Historie
        ///////////////////////////////////////////////////////////////////////////////////////


        private void zpětToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (vpředToolStripMenuItem.ForeColor != Color.Red)
                _manipulationManager.UndoClick();
        }
        private void vpředToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (vpředToolStripMenuItem.ForeColor != Color.Red)
                _manipulationManager.RedoClick();
        }
        
        public void EnableUndo()
        {
            zpětToolStripMenuItem.Enabled = true;
        }
        public void DisableUndo()
        {
            zpětToolStripMenuItem.Enabled = false;
        }
        public void EnableRedo() 
        {
            vpředToolStripMenuItem.Enabled = true;
        }
        public void DisableRedo()
        {
            vpředToolStripMenuItem.Enabled = false;
        }

        public void DisableInterActiveColisingActions() 
        {
            _manipulationManager.ControlUndoRedoButtons();
            //vpředToolStripMenuItem.ForeColor = Color.Red;
            //zpětToolStripMenuItem.ForeColor = Color.Red;
        }
        public void EnableInterActiveColisingActions()
        {
            vpředToolStripMenuItem.Enabled = true;
            zpětToolStripMenuItem.Enabled = true;
            //vpředToolStripMenuItem.ForeColor = Color.Black;
            //zpětToolStripMenuItem.ForeColor = Color.Black;
        }


        ///////////////////////////////////////////////////////////////////////////////////////
        //                              Pohyb splitteru (mezi paint contolem a listboxem)
        ///////////////////////////////////////////////////////////////////////////////////////

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            //simpleOpenGlControl1.Width = e.SplitX;
            _scene.SetProjectiveView(GetOpenglControlWidth(), GetOpenglControlHeight());
            RenderScene(); 
        }
        private void splitContainer1_SplitterMoving(object sender, SplitterCancelEventArgs e)
        {
            //_scene.SetProjectiveView(simpleOpenGlControl1.Height, simpleOpenGlControl1.Width);
            RenderScene();
            //Refresh();
        }

        ///////////////////////////////////////////////////////////////////////////////////////
        //                              ListBox
        ///////////////////////////////////////////////////////////////////////////////////////


        public void RenderScene()
        {
            this.panel1.RenderScene();
        }        

        private void listView1_MouseClick(object sender, MouseEventArgs e)
        {
            Point mousePosition =  new Point(MousePosition.X, MousePosition.Y);
            if (e.Button == MouseButtons.Right)
            {
                if (listView1.SelectedItems.Count < 1)
                    return;
                List<GraphicObject> selectedGraphicObjects = new List<GraphicObject>();
                for (int i = 0; i < listView1.SelectedItems.Count; i ++)
                {
                    selectedGraphicObjects.Add((GraphicObject)listView1.SelectedItems[i].Tag);
                }
                _listViewContextMenu.EnableValidOperations(selectedGraphicObjects);
                _listViewContextMenu.Show(mousePosition);
            }
        }
        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            ListViewItem selectedItem = null; ;
            if (listView1.SelectedItems.Count == 1)
                selectedItem = listView1.SelectedItems[0];
            else return; // neni vybran zadny prvek listview.
            _manipulationManager.ListViewItemDoubleClick(selectedItem);
        }
        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                _manipulationManager.ListViewItemDeleteDown(listView1);
            }
            if (e.KeyCode == Keys.Enter)
            {
                listView1_DoubleClick(sender, EventArgs.Empty);
            }
        }



        /// <summary>
        /// Smazu z listview item, pokud ma tag stejne ID jako graphicObject. 
        /// </summary>
        /// <param name="graphicObject"></param>
        public void DeleteItemFromListBox(GraphicObject graphicObject)
        {
            foreach (ListViewItem item in listView1.Items)
                if (((GraphicObject)item.Tag).GetID() == graphicObject.GetID())
                {
                    listView1.Items.Remove(item);
                    break;
                }
            listView1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
        }
        /// <summary>
        /// Pridam do listView item tak, aby jeho tag byl mezi tagy jeho sousedu. 
        /// </summary>
        /// <param name="graphicObject"></param>
        public void AddItemToListBox(GraphicObject graphicObject) 
        {
            int i = 0;
            foreach (ListViewItem item in listView1.Items)
            {
                if (((GraphicObject)item.Tag).GetID() > graphicObject.GetID())
                    break;
                i++;
            }
            listView1.Items.Insert(i, graphicObject.GetWordDescriptionFirstCase());
            listView1.Items[i].Tag = graphicObject;
            listView1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
        }
        public void ChangeItemFromListBox(GraphicObject oldGraphicObject, GraphicObject newGraphicObject) 
        {
            foreach (ListViewItem item in listView1.Items)
                if (((GraphicObject)item.Tag).GetID() == oldGraphicObject.GetID())
                {
                    if (item.Text != newGraphicObject.GetWordDescriptionFirstCase())
                        item.Text = newGraphicObject.GetWordDescriptionFirstCase();
                    item.Tag = newGraphicObject;
                    break;
                }
            listView1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
        }

        public void DisableItemFromListBox(GraphicObject graphicObject) 
        {
            foreach (ListViewItem item in listView1.Items)
                if (graphicObject.GetID() == ((GraphicObject)item.Tag).GetID())
                    item.ForeColor = _disabledListViewItemColor;
        }
        public void EnableItemFromListBox(GraphicObject graphicObject) 
        {
            foreach (ListViewItem item in listView1.Items)
                if (graphicObject.GetID() == ((GraphicObject)item.Tag).GetID())
                    item.ForeColor = _enabledListViewItemColor;
        }

        public int GetListBoxLocationX()
        {
            return this.splitContainer1.SplitterDistance;
        }
        public int GetListBoxLocationY() 
        {
            return this.splitContainer1.Location.Y;
        }


        ///////////////////////////////////////////////////////////////////////////////////////
        //                       Manipulace s objekty a Prace s historii
        ///////////////////////////////////////////////////////////////////////////////////////


        #region <AddRegion>
        private void přidatObecnouRovnicíToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddQuadricInvoked();
        }        
        private void přidatObecnouRovnicíToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddConicSectionInvoked();
        }
        #region<Pridat konkretni kvadriku>
        private void přidatReálnýElipsoidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipsoidRealInvoked();
        }
        private void přidatImaginárníElipsoidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipsoidImaginarInvoked();
        }
        private void přidatDvoudílnýHyperboloidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddHyperboloidTwoSheetInvoked();
        }
        private void přidatJednodílnýHyperboloidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddHyperboloidOneSheetInvoked();
        }
        private void přidatHyperbolickýParaboloidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddHyperbolicParaboloidInvoked();
        }
        private void přidatEliptickýParaboloidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipticParaboloidInvoked();
        }
        private void přidatReálnouKuželovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddConeRealInvoked();
        }
        private void přidatImaginárníKuželovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddConeImaginarInvoked();
        }
        private void přidatEliptickouVálcovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipticCylinderRealInvoked();
        }
        private void přidatImaginárníEliptickouVálcovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipticCylinderImaginarInvoked();
        }
        private void přidatHyperbolickouVálcovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddHyperbolicCylinderInvoked();
        }
        private void přidatParabolickouVálcovouPlochuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParabolicCylinderInvoked();
        }
        private void přidatDvojnásobnouRovinuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddDoublePlanInvoked();
        }
        private void přidatRovnoběžnéRovinyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParalelPlansRealInvoked();
        }
        private void přidatImaginárníRovnoběžnéRovinyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParalelPlansImaginarInvoked();
        }
        private void přidatRůznoběžnéRovinyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddCrossingPlansRealInvoked();
        }
        private void přidatImaginárníRůznoběžnéRovinyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddCrossingPlansImaginarInvoked();
        }
        #endregion
        #region<Pridat konkretni kuzelosecku>

        private void přidatImaginárníElipsuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipseImaginarInvoked();
        }
        private void přidatReálnouElipsuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddElipseRealInvoked();
        }
        private void přidatHyperboluToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddHyperbolaInvoked();
        }
        private void přidatParaboluToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParabolaInvoked();
        }
        private void přidatDvojnásobnouPřímkuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddDoubleLineInvoked();
        }
        private void přidatRovnoběžnéPřímkyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParalelLinesRealInvoked();
        }
        private void přidatImaginárníRovnoběžnéPřímkyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddParalelLinesImaginarInvoked();
        }
        private void přidatRůznoběžnéPřímkyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddCrossingLinesRealInvoked();
        }
        private void přiidatImaginárníRůznoběžnéPřímkyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddCrossingLinesImaginarInvoked();
        }

        #endregion
        private void přidatRovinuObecnouRovnicí_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddPlanInvoked();
        }
        private void přidatRovinuParametrickouRovnicí_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddPlanParametricalyInvoked();
        }
        private void přidatPřímku_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddLineInvoked();
        }
        private void přidatBod_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddPointInvoked();
        }

        private void přidatPrůnikToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _manipulationManager.GetAddManager().AddIntersectionInvoked();
        }
        #endregion

        private void kvalitaObjektůToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Settings settings = new Settings(this);
            settings.ShowDialog();
        }

        protected override void OnResize(EventArgs e)
        {
            // nutno zakazat kresleni, jinak se kresli scena s nastavenym viewPortem, do zvetseneho panelu. - spatne 
            // pak se prekresli s novym viewPortem do zvetseneho panelu.
            this.panel1.DrawDisabled = true;           
            base.OnResize(e);
            if (_scene != null)
                _scene.SetProjectiveView(GetOpenglControlWidth(), GetOpenglControlHeight());
            this.panel1.DrawDisabled = false;
            RenderScene();
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = CheckSaveBeforeExit(this);
            if (!e.Cancel)
                _manipulationManager.DeleteAllDisplayLists();
        }

        private void uložitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();
                saveFileDialog.DefaultExt = "xml";
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string path = saveFileDialog.FileName;
                    SaveManager saveManager = new SaveManager(path);
                    saveManager.Save(_manipulationManager.GetGraphicObjectsToSave(), _manipulationManager.GetHistory());
                    if (!(sender is Form1)) // Sender je form pokud ukladame pred ukoncenim
                        SetProgressBarText("Uložení bylo úspěšné.", 3);
                    //ShowAnounce("Uložení bylo úspěšné.");
                }
                _errorDuringSave = false;
                _manipulationManager.SetSaveOcurred();
            }
            catch (Exception)
            {
                ShowError("Při ukládání souboru došlo k chybě.");
                _errorDuringSave = true;
            }
        }
        private void načístToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CheckSaveBeforeExit(null))
                return;
            List<GraphicObject> originalGraphicObjects = _manipulationManager.GetGraphicObjects();
            // objekty se jakoze nacítají z souboru, proto musí mít stejné hodnoty jako nově vytvořené objekty.
            // tady je to důležité jen u isInScene. jinak oba objekty pruniku pridaji prunik.
            foreach (GraphicObject o in originalGraphicObjects)
                o.SetInScene(false); 
            List<IHistoryItem> originalHistoryItems = _manipulationManager.GetHistory().GetHistoryItems();
            int originalHistoryPosition = _manipulationManager.GetHistory().GetHistoryPosition();
            try
            {
                OpenFileDialog openFileDialog = new OpenFileDialog();
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string path = openFileDialog.FileName;
                    LoadManager loadManager = new LoadManager(path);
                    List<GraphicObject> loadGraphicObjects = loadManager.LoadObjects();
                    _manipulationManager.LoadNewScene(loadGraphicObjects, 
                        loadManager.LoadHistoryItems(loadGraphicObjects), loadManager.LoadHistoryPosition());
                    _manipulationManager.SetSaveOcurred();
                    SetProgressBarText("Načtení bylo úspěšné.", 3);
                    Refresh();
                }
            }
            catch (Exception)
            {
                ShowError("Při načítání souboru došlo k chybě.");
                _manipulationManager.ClearScene();
                _manipulationManager.LoadNewScene(originalGraphicObjects, originalHistoryItems, originalHistoryPosition);
            }
        }

        /// <summary>
        /// Návratová hodnota udává jestli nebylo stisknuto cancel, tedy zruseni operace (nacitani, ukoncovani)
        /// </summary>
        /// <param name="sender"></param>
        /// <returns></returns>
        private bool CheckSaveBeforeExit(object sender)
        {
            if (!_manipulationManager.SaveNeeded())
                return false;
            DialogResult result = ShowQuestion("Uložit obsah scény?");
            if (result == DialogResult.Yes)
            {
                uložitToolStripMenuItem_Click(sender, EventArgs.Empty);
                if (_errorDuringSave)
                    return true;
            }
            if (result == DialogResult.Cancel)
                return true;
            return false;
        }
        private void uložitImageScényToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();
                saveFileDialog.Title = "Export do obrázku";
                saveFileDialog.Filter = "Bmp (*.bmp)|*.bmp|JPEG (*.jpg*)|*.jpg|PNG (*.png*)|*.png|GIF (*.gif*)|*.gif";
                saveFileDialog.FilterIndex = 0;
                saveFileDialog.AddExtension = true;
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string path = saveFileDialog.FileName;
                    System.Drawing.Imaging.ImageFormat imageFormat = System.Drawing.Imaging.ImageFormat.Bmp;
                    if (saveFileDialog.FilterIndex == 2)
                        imageFormat = System.Drawing.Imaging.ImageFormat.Jpeg;
                    if (saveFileDialog.FilterIndex == 3)
                        imageFormat = System.Drawing.Imaging.ImageFormat.Png;
                    if (saveFileDialog.FilterIndex == 4)
                        imageFormat = System.Drawing.Imaging.ImageFormat.Gif;
                    SaveImageFast(path, imageFormat);
                    SetProgressBarText("Uložení obrázku bylo úspěšné.", 3);
                    //ShowAnounce("Uložení bylo úspěšné.");
                }
            }
            catch (Exception)
            {
                ShowError("Při ukládání obrázku došlo k chybě.");
            }
        }
        private void SaveImage(string path, System.Drawing.Imaging.ImageFormat imageFormat) 
        {
            // vytvorim bitmapu stejne velikosti jako ma openGL platno.
            int bitmapWidth = GetOpenglControlWidth();
            int bitmapHeight = GetOpenglControlHeight();
            Bitmap bitmap = new Bitmap(bitmapWidth, bitmapHeight);         
            // do bufferu nactu ctverice hodnot pro vsechny pixely openGL platna.
            // pixely jsou invertovany vzhledem k ose Y.
            byte[] buffer = new byte[bitmap.Width * bitmap.Height * 4];
            Gl.glPixelStorei(Gl.GL_PACK_ALIGNMENT, 1); // urcuje zpusob ulozeni dat do klientske pameti (radky zarovnany na x bytu)
            Gl.glPixelStorei(Gl.GL_UNPACK_ALIGNMENT, 1); // urcuje zpusob nacteni dat z klientske pameti 
            RenderScene();
            RenderScene();
            Gl.glReadPixels(0, 0, bitmap.Width, bitmap.Height, Gl.GL_RGBA, Gl.GL_UNSIGNED_BYTE, buffer);            
            // z hodnot v bufferu vytvorim pro kazdy pixel jeho barvu a zapisu pixeli do bitmapy.
            for (int y = bitmap.Height - 1; y >= 0; y--)
                for (int x = 0; x < bitmap.Width; x++)
                    bitmap.SetPixel(x, bitmap.Height - y - 1, Color.FromArgb(
                        buffer[(y * bitmap.Width + x) * 4],
                        buffer[(y * bitmap.Width + x) * 4 + 1],
                        buffer[(y * bitmap.Width + x) * 4 + 2]));
            // bitmapu zapisu do souboru.
            bitmap.Save(path, imageFormat);
        }
        /// <summary>
        /// Nevím proč, ale png obrázky nemají pozadí. 
        /// </summary>
        private void SaveImageFast(string path, System.Drawing.Imaging.ImageFormat imageFormat)
        {
            try
            {
                // vytvorim bitmapu stejne velikosti jako ma openGL platno.
                int bitmapWidth = GetOpenglControlWidth();
                int bitmapHeight = GetOpenglControlHeight();
                // do bufferu nactu ctverice hodnot pro vsechny pixely openGL platna.
                // pixely jsou invertovany vzhledem k ose Y.
                byte[] buffer = new byte[bitmapWidth * bitmapHeight * 4];
                Gl.glPixelStorei(Gl.GL_PACK_ALIGNMENT, 1); // urcuje zpusob ulozeni dat do klientske pameti (radky zarovnany na x bytu)
                Gl.glPixelStorei(Gl.GL_UNPACK_ALIGNMENT, 1); // urcuje zpusob nacteni dat z klientske pameti 

                GCHandle pinnedArray = GCHandle.Alloc(buffer, GCHandleType.Pinned);
                IntPtr pointer = pinnedArray.AddrOfPinnedObject();
                //do your stuff
                Gl.glFrontFace(Gl.GL_CW);
                _scene.RenderSceneInvertedY();
                _scene.RenderSceneInvertedY();                
                Gl.glReadPixels(0, 0, bitmapWidth, bitmapHeight, Gl.GL_BGRA, Gl.GL_UNSIGNED_BYTE, pointer);
                Gl.glFrontFace(Gl.GL_CCW);
                RenderScene();
                Bitmap bitmap = new Bitmap(bitmapWidth, bitmapHeight, bitmapWidth * 4, System.Drawing.Imaging.PixelFormat.Format32bppArgb, pointer);
                bitmap.Save(path, imageFormat);
                pinnedArray.Free();

            }
            catch (Exception)
            {
            }
            finally 
            {
                Gl.glFrontFace(Gl.GL_CCW);
                RenderScene();
            }
        }

        private void novýToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CheckSaveBeforeExit(null))
                return;
            _manipulationManager.ClearScene();
            _manipulationManager.SetSaveOcurred();
            SetProgressBarText("Nová úspěšně scéna načtena." ,3);
            Refresh();
        }


        public void ClearListViewItems()
        {
            this.listView1.Items.Clear();
        }

        private void barvaPozadíToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ColorDialog d = new ColorDialog();
            if (d.ShowDialog() == DialogResult.OK) 
            {
                _scene.SetBackColor(Program.SystemColorToColor(d.Color));
                RenderScene();
            }
        }

        private void zobrazitNápověduToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Geometry.GGraphic.Forms.HelpForm hf = new Geometry.GGraphic.Forms.HelpForm();
            hf.ShowDialog();
        }

        private void splitContainer1_MouseDown(object sender, MouseEventArgs e)
        {
            this.Cursor = Cursors.VSplit;
        }
        private void splitContainer1_MouseUp(object sender, MouseEventArgs e)
        {
            this.Cursor = Cursors.Default;
        }

        private void vyplňitPlochyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _scene.SetFillPolygons(true);
            trojúhelníkováSítToolStripMenuItem.Checked = false;
            vyplňitPlochyToolStripMenuItem.Checked = true;
            RenderScene();
        }
        private void trojúhelníkováSítToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _scene.SetFillPolygons(false);
            trojúhelníkováSítToolStripMenuItem.Checked = true;
            vyplňitPlochyToolStripMenuItem.Checked = false;
            RenderScene();
        }

        private void vycentrovatScénuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _scene.ResetSceneCenter();
            RenderScene();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //base.OnPaintBackground(e);
        }




        protected IntPtr hDC;
        protected IntPtr hRC;
        protected int pixelFormatID;
        protected Gdi.PIXELFORMATDESCRIPTOR pfd;
        protected bool doubleBuffer;

        /// <summary>
        /// Každá chyba která může nastat je fatátní a vede k výjimce.
        /// </summary>
        protected IntPtr InitializeDummyContext(IntPtr dummyHDC) 
        {
            Gdi.PIXELFORMATDESCRIPTOR dummyPfd = new Gdi.PIXELFORMATDESCRIPTOR();
            dummyPfd.dwFlags = Gdi.PFD_SUPPORT_OPENGL | Gdi.PFD_DRAW_TO_WINDOW;
            dummyPfd.iPixelType = Gdi.PFD_TYPE_RGBA;
            dummyPfd.cColorBits = 24;
            dummyPfd.cColorBits = 0;

            int dummyPixFormat = Gdi.ChoosePixelFormat(dummyHDC, ref dummyPfd);

            if (dummyPixFormat <= 0)
            {
                MessageBox.Show("ChoosePixelFormat failed.");
                throw new Exception("ChoosePixelFormat failed.");
            }
            Wgl.wglDescribePixelFormat(dummyHDC, dummyPixFormat, 40, ref dummyPfd);
            Wgl.wglSwapBuffers(dummyHDC);
            Gdi.SetPixelFormat(dummyHDC, dummyPixFormat, ref dummyPfd);
            IntPtr dummyHRC = Wgl.wglCreateContext(dummyHDC);

            if (dummyHRC == IntPtr.Zero)
            {
                MessageBox.Show("Creating dummy context failed.");
                throw new Exception("Creating dummy context failed.");
            }
            return dummyHRC;
        }
        protected bool IsAllAboutMsaaSupported(IntPtr dummyHDC, IntPtr dummyHRC)
        {
            Wgl.wglMakeCurrent(dummyHDC, dummyHRC);            
            Wgl.ReloadFunctions();
            bool supportWglExtString = Wgl.IsExtensionSupported("WGL_EXT_extensions_string");

            if (!supportWglExtString)
                return false;

            IntPtr ptr = Wgl.wglGetProcAddress("wglGetExtensionsStringEXT");

            List<string> wglExtensions = new List<string>(Wgl.wglGetExtensionsStringEXT().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries));
            bool supportWglArbPixFormat = wglExtensions.Contains("WGL_ARB_pixel_format");
            if (!supportWglArbPixFormat)
                return false;

            bool supportWglArbMultisamples = wglExtensions.Contains("WGL_ARB_multisample");
            if (!supportWglArbMultisamples)
                return false;
            return true;
        }

        protected void InitializeMultisampling()
        {
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.ResizeRedraw, true);

            noneToolStripMenuItem.Enabled = false;
            x2ToolStripMenuItem.Enabled = false;
            x4ToolStripMenuItem.Enabled = false;
            x8ToolStripMenuItem.Enabled = false;

            Control dummyControl = new Control();
            IntPtr dummyHDC = User.GetDC(dummyControl.Handle);
            IntPtr dummyHRC = InitializeDummyContext(dummyHDC);
            Wgl.wglMakeCurrent(dummyHDC, dummyHRC);

            if (!IsAllAboutMsaaSupported(dummyHDC, dummyHRC))
            {
                Wgl.wglDeleteContext(dummyHRC);
                dummyControl.Dispose();
                NormalyInitialize();
                return;
            }

            int[] iAttribs = new int[]{ Wgl.WGL_SUPPORT_OPENGL_EXT, 1,  //0, 1,
                                        Wgl.WGL_DRAW_TO_WINDOW_EXT, 1,  //2, 3,
                                        Wgl.WGL_ACCELERATION_EXT, Wgl.WGL_FULL_ACCELERATION_EXT, //4, 5,
                                        Wgl.WGL_PIXEL_TYPE_EXT, Wgl.WGL_TYPE_RGBA_EXT, //6, 7,
                                        Wgl.WGL_COLOR_BITS_EXT, 24,     //8, 9,
                                        Wgl.WGL_ALPHA_BITS_EXT, 8,      //10, 11,
                                        Wgl.WGL_DEPTH_BITS_EXT, 16,     //12, 13,
                                        Wgl.WGL_DOUBLE_BUFFER_EXT, 1,   //14, 15,
                                        Wgl.WGL_SAMPLE_BUFFERS_EXT, 1,  //16, 17, // マルチサンプリングで使用するバッファを作成するかどうかの指定
                                        Wgl.WGL_SAMPLES_EXT, 8, //18, 19, // マルチサンプリングのサンプル数。2, 4, 8, ... を指定可。
                                        0,0 };                          //20, 21  // 最後は 0, 0 としておかなければならないのは、仕様。

            int[] formats = new int[1];
            int[] numFormats = new int[1];
            noneToolStripMenuItem.Enabled = true;
            x2ToolStripMenuItem.Enabled = true;
            x4ToolStripMenuItem.Enabled = true;
            x8ToolStripMenuItem.Enabled = true;
            // zkusím 8 msaa
            bool valid = Wgl.wglChoosePixelFormatARB(dummyHDC, iAttribs, null, formats.Length, formats, numFormats);
            if (!valid || (numFormats[0] <= 0))
                x8ToolStripMenuItem.Enabled = false;

            iAttribs[19] = 4;
            valid = Wgl.wglChoosePixelFormatARB(dummyHDC, iAttribs, null, formats.Length, formats, numFormats);
            if (!valid || (numFormats[0] <= 0))
                x4ToolStripMenuItem.Enabled = false;

            iAttribs[19] = 2;
            valid = Wgl.wglChoosePixelFormatARB(dummyHDC, iAttribs, null, formats.Length, formats, numFormats);
            if (!valid || (numFormats[0] <= 0))
                x2ToolStripMenuItem.Enabled = false;

            Wgl.wglDeleteContext(dummyHRC);
            dummyControl.Dispose();
            SetTheBestAntialiasing();
        }

        protected void SetTheBestAntialiasing() 
        {
            if (x8ToolStripMenuItem.Enabled)
            {
                buttonx8_Click(this, EventArgs.Empty);
                return;
            }
            if (x4ToolStripMenuItem.Enabled)
            {
                buttonx4_Click(this, EventArgs.Empty);
                return;
            }
            if (x2ToolStripMenuItem.Enabled)
            {
                buttonx2_Click(this, EventArgs.Empty);
                return;
            }
            buttonNone_Click(this, EventArgs.Empty);
        }

        private void SwapPanelsDeviceContext()
        {
            _manipulationManager.DeleteAllDisplayLists();// nutno smazat ještě ve starém kontextu, jinak budou nedostupné.
            // navíc může colector přidat mazaný objekt a do seznamu objektů  ke smazání a pak by se uvolnil displayList,
            // který už v novém kontextu může někdo mít. 
            Wgl.wglMakeCurrent(this.hDC, IntPtr.Zero);
            Wgl.wglDeleteContext(this.hRC);
            //this.splitContainer1.Panel1.Controls.Remove(this.panel1);
            //this.panel1.Dispose();
            this.panel1 = new DrawingPanel();
            this.panel1.SetScene(_scene);
            this.panel1.MouseDown += new MouseEventHandler(simpleOpenGlControl1_MouseDown);
            this.panel1.MouseUp += new MouseEventHandler(simpleOpenGlControl1_MouseUp);
            this.panel1.MouseMove += new MouseEventHandler(simpleOpenGlControl1_MouseMove);            
            //this.splitContainer1.Panel1.Controls.Add(this.panel1);
            this.panel1.Dock = DockStyle.Fill;
        }

        /// <summary>
        /// Int.MinValue značí, že něco není podporováno a muselo být zavoláno NormalyInitialize
        /// </summary>
        protected int ChoosePixelFormatInDummyContext(int msaaValue) 
        {
            Control dummyControl = new Control();
            IntPtr dummyHDC = User.GetDC(dummyControl.Handle);
            IntPtr dummyHRC = InitializeDummyContext(dummyHDC);
            if (!IsAllAboutMsaaSupported(dummyHDC, dummyHRC))
            {
                Wgl.wglDeleteContext(dummyHRC);
                dummyControl.Dispose();
                MessageBox.Show("Vaše grafická karta nebo její ovladač nepodporují funkce potřebné pro změnu úrovně vzorkování.");
                return int.MinValue;
            }

            int[] iAttribs = new int[]{ Wgl.WGL_SUPPORT_OPENGL_EXT, 1,  //0, 1,
                                        Wgl.WGL_DRAW_TO_WINDOW_EXT, 1,  //2, 3,
                                        Wgl.WGL_ACCELERATION_EXT, Wgl.WGL_FULL_ACCELERATION_EXT, //4, 5,
                                        Wgl.WGL_PIXEL_TYPE_EXT, Wgl.WGL_TYPE_RGBA_EXT, //6, 7,
                                        Wgl.WGL_COLOR_BITS_EXT, 24,     //8, 9,
                                        Wgl.WGL_ALPHA_BITS_EXT, 8,      //10, 11,
                                        Wgl.WGL_DEPTH_BITS_EXT, 16,     //12, 13,
                                        Wgl.WGL_DOUBLE_BUFFER_EXT, 1,   //14, 15,
                                        Wgl.WGL_SAMPLE_BUFFERS_EXT, 1,  //16, 17, // マルチサンプリングで使用するバッファを作成するかどうかの指定
                                        Wgl.WGL_SAMPLES_EXT, msaaValue, //18, 19, // マルチサンプリングのサンプル数。2, 4, 8, ... を指定可。
                                        0,0 };                          //20, 21  // 最後は 0, 0 としておかなければならないのは、仕様。

            int[] formats = new int[1];
            int[] numFormats = new int[1];

            // zkusím zvolený msaa
            bool valid = Wgl.wglChoosePixelFormatARB(dummyHDC, iAttribs, null, formats.Length, formats, numFormats);

            Wgl.wglDeleteContext(dummyHRC);
            dummyControl.Dispose();

            if (!valid || (numFormats[0] <= 0))
            {
                // pokud není podporován 
                MessageBox.Show("Vaše grafická karta nebo její ovladač nepodporují zvolenou úroveň vzorkování.");
                return int.MinValue;
            }
            return formats[0];
        }
        protected void InitializeContext(int msaaValue)
        {
            pixelFormatID = ChoosePixelFormatInDummyContext(msaaValue);
            if (pixelFormatID == int.MinValue) // něco není podporováno
            {
                NormalyInitialize();
                return;
            }

            Panel previousPanel = null;
            if (this.hRC != IntPtr.Zero)
            {
                previousPanel = this.panel1;
                SwapPanelsDeviceContext();
            }

            this.hDC = User.GetDC(this.panel1.Handle);
            this.pfd = new Gdi.PIXELFORMATDESCRIPTOR();

            Wgl.wglDescribePixelFormat(this.hDC, pixelFormatID, 40, ref this.pfd);
            Wgl.wglSwapBuffers(this.hDC);
            bool valid = Gdi.SetPixelFormat(this.hDC, pixelFormatID, ref this.pfd);

            if (!valid)
            {
                MessageBox.Show("SetPixelFormat failed.");
                throw new Exception("SetPixelFormat failed.");
            }

            this.hRC = Wgl.wglCreateContext(this.hDC);
            if (this.hRC == IntPtr.Zero)
            {
                MessageBox.Show("CreateContext failed.");
                throw new Exception("CreateContext failed.");
            }
            Wgl.wglMakeCurrent(this.hDC, this.hRC);

            //Tao.OpenGl.Gl, Tao.Platform.Windows.Wglを使うときのお約束。
            Gl.ReloadFunctions();
            Wgl.ReloadFunctions();

            Gl.glEnable(Gl.GL_MULTISAMPLE);
            this.doubleBuffer = ((this.pfd.dwFlags & Gdi.PFD_DOUBLEBUFFER) == Gdi.PFD_DOUBLEBUFFER);
            if (previousPanel != null)
                this.panel1.Size = previousPanel.Size;
            _scene.GLInit(GetOpenglControlWidth(), GetOpenglControlHeight(), this.hDC, this.doubleBuffer);
            if (_manipulationManager != null)
                _manipulationManager.SetMSAAInvoked();
            if (previousPanel != null)
            {
                this.splitContainer1.Panel1.Controls.Add(this.panel1);
                this.splitContainer1.Panel1.Controls.Remove(previousPanel);
                previousPanel.Dispose();
            }
            SetProgressBarText("Úrověň vzorkování úspěšně změněna.", 3);
            RenderScene();
        }
        /// <summary>
        /// マルチサンプリング無しで、普通に初期化。
        /// </summary>
        private void NormalyInitialize()
        {
            Panel previousPanel = null;
            if (this.hRC != IntPtr.Zero)
            {
                previousPanel = this.panel1;
                SwapPanelsDeviceContext();
            }

            this.pfd = new Gdi.PIXELFORMATDESCRIPTOR();
            this.pfd.dwFlags = Gdi.PFD_SUPPORT_OPENGL | Gdi.PFD_DRAW_TO_WINDOW | Gdi.PFD_DOUBLEBUFFER;
            this.pfd.iPixelType = Gdi.PFD_TYPE_RGBA;
            this.pfd.cColorBits = 32;
            this.pfd.cAlphaBits = 8;

            this.hDC = User.GetDC(this.panel1.Handle);

            pixelFormatID = Gdi.ChoosePixelFormat(this.hDC, ref this.pfd);
            if (pixelFormatID <= 0) 
            {
                MessageBox.Show("ChoosePixelFormat failed.");
                throw new Exception("ChoosePixelFormat failed.");
            }

            Wgl.wglDescribePixelFormat(this.hDC, pixelFormatID, 40, ref this.pfd);
            Wgl.wglSwapBuffers(this.hDC);

            Gdi.SetPixelFormat(this.hDC, pixelFormatID, ref this.pfd);
            this.hRC = Wgl.wglCreateContext(this.hDC);

            if (this.hRC == IntPtr.Zero)
            {
                MessageBox.Show("Creating dummy context failed.");
                throw new Exception("Creating dummy context failed.");
            }
            Wgl.wglMakeCurrent(this.hDC, this.hRC);

            Gl.ReloadFunctions();
            Wgl.ReloadFunctions();

            int err = Gl.glGetError();

            if (err != Gl.GL_NO_ERROR)
            {
                MessageBox.Show("Error code = " + err.ToString());
                throw new Exception("Error code = " + err.ToString());
            }
            noneToolStripMenuItem.Checked = true;
            this.doubleBuffer = ((this.pfd.dwFlags & Gdi.PFD_DOUBLEBUFFER) == Gdi.PFD_DOUBLEBUFFER);

            if (previousPanel != null)
                this.panel1.Size = previousPanel.Size;
            _scene.GLInit(GetOpenglControlWidth(), GetOpenglControlHeight(), this.hDC, this.doubleBuffer);
            if (_manipulationManager != null)
                _manipulationManager.SetMSAAInvoked();
            if (previousPanel != null)
            {
                this.splitContainer1.Panel1.Controls.Add(this.panel1);
                this.splitContainer1.Panel1.Controls.Remove(previousPanel);
                previousPanel.Dispose();
            }
            RenderScene();
        }

        private void buttonNone_Click(object sender, EventArgs e)
        {
            this.noneToolStripMenuItem.Checked = true;
            this.x2ToolStripMenuItem.Checked = false;
            this.x4ToolStripMenuItem.Checked = false;
            this.x8ToolStripMenuItem.Checked = false;
            InitializeContext(1);
        }
        private void buttonx2_Click(object sender, EventArgs e)
        {
            this.noneToolStripMenuItem.Checked = false;
            this.x2ToolStripMenuItem.Checked = true;
            this.x4ToolStripMenuItem.Checked = false;
            this.x8ToolStripMenuItem.Checked = false;
            InitializeContext(2);
        }
        private void buttonx4_Click(object sender, EventArgs e)
        {
            this.noneToolStripMenuItem.Checked = false;
            this.x2ToolStripMenuItem.Checked = false;
            this.x4ToolStripMenuItem.Checked = true;
            this.x8ToolStripMenuItem.Checked = false;
            InitializeContext(4);
        }
        private void buttonx8_Click(object sender, EventArgs e)
        {
            this.noneToolStripMenuItem.Checked = false;
            this.x2ToolStripMenuItem.Checked = false;
            this.x4ToolStripMenuItem.Checked = false;
            this.x8ToolStripMenuItem.Checked = true;
            InitializeContext(8);
        }

        internal IntPtr GetCurrenthRC()
        {
            return this.hRC;
        }
        internal IntPtr GetCurrentDC()
        {
            return this.hDC;
        }
        internal Gdi.PIXELFORMATDESCRIPTOR GetCurrentPixelFormat()
        {
            return this.pfd;
        }
        internal int GetPixelFormatID() 
        {
            return pixelFormatID;
        }
    }
}
