﻿using System;
using System.Collections.Generic;
using System.Linq;
using Tao.OpenGl;
using Geometry.GraphicObjects;
using Geometry.BaseClasses;
using GMath;
using Geometry.GGraphic;
using Tao.Platform.Windows;

namespace Geometry
{
    public class Scene
    {
        private static int clippingCubeWidth = 30;

        private double[] clippingBackPlane = new double[] { 0, 1, 0, 0 };
        private double[] clippingFrontPlane = new double[] { 0, -1, 0, 0 };
        private double[] clippingTopPlane = new double[] { 0, 0, -1, 0 };
        private double[] clippingBottomPlane = new double[] { 0, 0, 1, 0 };
        private double[] clippingLeftPlane = new double[] { 1, 0, 0, 0 };
        private double[] clippingRightPlane = new double[] { -1, 0, 0, 0 };

        protected List<GraphicObject> list = new List<GraphicObject>();
        protected double _zoom = 50;
        protected RotationAngles _angles = new RotationAngles(0, 0, 0);

        private float[] materialAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };  // ambientni slozka barvy materialu
        private float[] materialDifuse = { 1.0f, 1.0f, 1.0f, 1.0f };  // difuzni slozka barvy materialu
        private float[] materialOdlesky = { 0.2f, 0.2f, 0.2f, 1.0f }; // barva odlesku
        private float[] materialShininess = { 20.0f };                 // faktor odlesku

        private float[] light_position = { 0.0f, 0.0f, 1.0f, 1.0f};          // pozice svetla
        private float[] light_color = { 0.8f, 0.8f, 0.8f };           // barva prvniho svetla

        private GColor backColor = new GColor(0.8, 0.8, 0.8);
        private double width;
        private double height;

        public static BaseQuality QualityBaseLow = new BaseQuality(BaseQuality.BaseEnum.LOW);
        public static BaseQuality QualityBaseMedium = new BaseQuality(BaseQuality.BaseEnum.MEDIUM);
        public static BaseQuality QualityBaseHigh = new BaseQuality(BaseQuality.BaseEnum.HIGH);
        public static BaseQuality QualityBaseVeryHigh = new BaseQuality(BaseQuality.BaseEnum.VERYHIGH);
        public static RelativeQuality QualityRelativeLow = new RelativeQuality(RelativeQuality.RelativeEnum.LOW);
        public static RelativeQuality QualityRelativeNormal = new RelativeQuality(RelativeQuality.RelativeEnum.NORMAL);

        private bool _fillPolygons = true;
        private BaseQuality _baseQuality = QualityBaseMedium;
        protected bool _doubleBuffered;
        protected IntPtr _hDC;

        protected CenterOfScene _centerOfScene;

        public Scene() 
        {
            _centerOfScene = new CenterOfScene(new Vector(0, 0, 0));
        }

        internal void GLInit(int width, int height, IntPtr hDC, bool doubleBuffered)
        {
            _hDC = hDC;
            _doubleBuffered = doubleBuffered;
            Gl.glClearColor(backColor.GetRedF(), backColor.GetGreenF(), backColor.GetBlueF(), 1.0f);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();            
            Gl.glShadeModel(Gl.GL_SMOOTH);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            //Gl.glEnable(Gl.GL_LINE_SMOOTH);
            //Gl.glEnable(Gl.GL_POINT_SMOOTH);
            //Gl.glHint(Gl.GL_POINT_SMOOTH_HINT, Gl.GL_NICEST);
            //Gl.glHint(Gl.GL_LINE_SMOOTH_HINT, Gl.GL_NICEST);//antialiasing
            //Gl.glEnable(Gl.GL_CULL_FACE);
            Gl.glDepthFunc(Gl.GL_LESS);
            Gl.glEnable(Gl.GL_COLOR_MATERIAL);      // barva objektu je pouzita z glColor..

            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_AMBIENT, materialAmbient);    // nastaveni ambientni slozky barvy materialu
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE, materialDifuse);    // nastaveni difuzni slozky barvy materialu
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SPECULAR, materialOdlesky);  // nastaveni barvy odlesku
            Gl.glMaterialfv(Gl.GL_FRONT_AND_BACK, Gl.GL_SHININESS, materialShininess);// nastaveni faktoru odlesku

            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, light_position);         // nastaveni pozice svetla
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, light_color);          // nastaveni barvy svetla
            Gl.glLightModeli(Gl.GL_LIGHT_MODEL_TWO_SIDE, Gl.GL_TRUE);       // povoli pocitani osvetleni pro obe strany
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);
            //Gl.glEnable(Gl.GL_LINE_WIDTH);
            SetProjectiveView(width, height);
            //Gl.glClearAccum(0.0f, 0.0f, 0.0f, 0.0f);
            //Gl.glEnable(Gl.GL_MULTISAMPLE_ARB);
            //Gl.glDisable(Gl.GL_DEPTH_TEST);
            // //Gl.glDepthMask(Gl.GL_FALSE);                      // Z-buffer v rezimu read-only
            //Gl.glEnable(Gl.GL_BLEND);                         // povoleni michani
            //Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);// michaci funkce        
        }
        internal void SetOrthoView(int width, int height)
        {
            this.width = width;
            this.height = height;
            // Set viewport to window dimensions.
            Gl.glViewport(0, 0, width, height);
            // Reset projection matrix stack
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            const float nRange = 80.0f;
            // Prevent a divide by zero
            if (height == 0)
            {
                height = 1;
            }
            // Establish clipping volume (left, right, bottom,
            // top, near, far)
            if (width <= height)
            {
                Gl.glOrtho(-nRange, nRange, -nRange * height / width,
                nRange * height / width, -nRange, nRange);
            }
            else
            {
                Gl.glOrtho(-nRange * width / height,
                nRange * width / height,
                -nRange, nRange, -nRange, nRange);
            }
            // reset modelview matrix stack
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
        }
        internal void SetProjectiveView(int width, int height)
        {
            height = Math.Max(height, 1);
            width = Math.Max(width, 1);
            this.width = width;
            this.height = height;

            Gl.glMatrixMode(Gl.GL_PROJECTION);// Zvolí projekční matici

            Gl.glViewport(0, 0, width, height);

            Gl.glLoadIdentity();// Reset matice

            float aspect = (float)width / (float)height;

            Glu.gluPerspective(45.0f, aspect, 0.1f, 1000.0f);// Výpočet perspektivy

            Gl.glMatrixMode(Gl.GL_MODELVIEW);// Zvolí matici Modelview

            Gl.glLoadIdentity();// Reset matice
        }


        protected void CallClippingPlanes() 
        {
            Gl.glPushMatrix();
            Gl.glTranslated(0, -clippingCubeWidth, 0);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE0, clippingBackPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE0);
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            Gl.glTranslated(0, clippingCubeWidth, 0);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE1, clippingFrontPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE1);
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            Gl.glTranslated(-clippingCubeWidth, 0, 0);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE2, clippingLeftPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE2);
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            Gl.glTranslated(clippingCubeWidth, 0, 0);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE3, clippingRightPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE3);
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            Gl.glTranslated(0, 0, clippingCubeWidth);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE4, clippingTopPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE4);
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            Gl.glDisable(Gl.GL_CLIP_PLANE5);
            Gl.glTranslated(0, 0, -clippingCubeWidth);
            Gl.glClipPlane(Gl.GL_CLIP_PLANE5, clippingBottomPlane);
            Gl.glEnable(Gl.GL_CLIP_PLANE5);
            Gl.glPopMatrix();
        }

        public void RenderScene()
        {
            try
            {
                Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
                //Gl.glAlphaFunc(Gl.GL_EQUAL, Gl.GL_ONE);
                //Gl.glDepthMask(Gl.GL_TRUE);
                Gl.glMatrixMode(Gl.GL_MODELVIEW);
                Gl.glLoadIdentity();
                Vector center = _centerOfScene.GetCenterInStandardBasis().GetReprezentant();
                Gl.glTranslated(0, 0, -_zoom);

                Gl.glRotated(-_angles.GetXAngle(), 1, 0, 0);
                Gl.glRotated(-_angles.GetYAngle(), 0, 1, 0);

                Gl.glTranslated(-center.GetX(), -center.GetY(), -center.GetZ());

                CallClippingPlanes();
                int polygonMode;
                if (_fillPolygons)
                    polygonMode = Gl.GL_FILL;
                else
                    polygonMode = Gl.GL_LINE;

                foreach (GraphicObject graphicObject in list)
                {
                    Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, polygonMode);
                    graphicObject.Draw(this);
                }

                //_centerOfScene.Draw(this);
                if (_doubleBuffered)
                {
                    Wgl.wglSwapBuffers(_hDC);
                }
                else
                {
                    Gl.glFinish();
                }
            }
            catch (Exception) 
            {
            }
        }
        public void RenderSceneInvertedY()
        {
            try
            {
                Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
                //Gl.glAlphaFunc(Gl.GL_EQUAL, Gl.GL_ONE);
                //Gl.glDepthMask(Gl.GL_TRUE);
                Gl.glMatrixMode(Gl.GL_MODELVIEW);
                Gl.glLoadIdentity();
                Gl.glScaled(1, -1, 1);
                Vector center = _centerOfScene.GetCenterInStandardBasis().GetReprezentant();
                Gl.glTranslated(0, 0, -_zoom);

                Gl.glRotated(-_angles.GetXAngle(), 1, 0, 0);
                Gl.glRotated(-_angles.GetYAngle(), 0, 1, 0);

                Gl.glTranslated(-center.GetX(), -center.GetY(), -center.GetZ());

                CallClippingPlanes();
                int polygonMode;
                if (_fillPolygons)
                    polygonMode = Gl.GL_FILL;
                else
                    polygonMode = Gl.GL_LINE;

                foreach (GraphicObject graphicObject in list)
                {
                    Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, polygonMode);
                    graphicObject.Draw(this);
                }

                //_centerOfScene.Draw(this);
                if (_doubleBuffered)
                {
                    Wgl.wglSwapBuffers(_hDC);
                }
                else
                {
                    Gl.glFinish();
                }
            }
            catch (Exception e)
            {
                ManipulationManager.FailOccured(e);
            }
        }



        /// <summary>
        /// Prida objekt do sezmanu objektu sceny.
        /// Nastavi objektu priznak ze je ve scene.
        /// </summary>
        /// <param name="drawable"></param>
        public void AddObject(GraphicObject drawable)
        {
            list.Add(drawable);
            drawable.SetInScene(true);
        }
        /// <summary>
        /// Ze seznamu objektu ve scene vrati ten, ktery je na indexu index.
        /// </summary>
        public GraphicObject ObjectAt(int index)
        {
            if (index >= 0 && index < list.Count)
            {
                return list.ElementAt(index);
            }
            return null;
        }
        /// <summary>
        /// Smaze puvodni object a na konec prida novy objekt.
        /// Staremu objektu nastavi priznak ze neni ve scene.
        /// Novemu objektu nnastavi priznak ze je ve scene.
        /// </summary>
        public void ChangeObject(GraphicObject oldGraphicObject, GraphicObject newGraphicObject)
        {
            if (DeleteObject(oldGraphicObject))
                AddObject(newGraphicObject);
        }
        /// <summary>
        /// Smaze ze seznamu objektu ve scene objekt na indexu index.
        /// Nastavi objektu priznak ze neni ve scene.
        /// </summary>
        /// <param name="index"></param>
        public bool DeleteObject(GraphicObject oldGraphicObject)
        {
            int i = 0;
            foreach (GraphicObject item in this.list)
            {
                if (item.GetID() == oldGraphicObject.GetID())
                {
                    list.RemoveAt(i);
                    oldGraphicObject.SetInScene(false);
                    return true;
                }
                i++;
            }
            return false;
        }
        public void DeleteObjects() 
        {
            list = new List<GraphicObject>(); 
        }
        public int GetObjectsCount() 
        {
            return list.Count;
        }


        public static int GetClippingCubeWidth() { return clippingCubeWidth; }

        public BaseQuality GetBaseQuality() 
        {
            return _baseQuality;
        }
        public void SetBaseQuality(BaseQuality quality) 
        {
            _baseQuality = quality;
        }


        public void SetBackColor(GColor color) 
        {
            this.backColor = color;
            Gl.glClearColor(backColor.GetRedF(), backColor.GetGreenF(), backColor.GetBlueF(), 1.0f);        
        }
        public GColor GetBackColor() 
        {
            return backColor;
        }

        public void SetFillPolygons(bool value) { _fillPolygons = value; }
        public bool GetFillPolygons() { return _fillPolygons; }

        public void MoveSceneCenter(Vector delta)
        {
            double[] values = new double[16];
            Gl.glGetDoublev(Gl.GL_MODELVIEW_MATRIX, values);
            TransformationMatrix rotation = new TransformationMatrix(4, 4, values);

            delta = delta * _zoom / 74;

            _centerOfScene.SetCenterInStandardBasis(
                new CenterParametrization(_centerOfScene.GetCenterInStandardBasis().GetReprezentant() +
                    delta * rotation.TransponMatrix()));
        }
        public void ResetSceneCenter()
        {
            _centerOfScene.SetCenterInStandardBasis(new CenterParametrization());
        }
        public void IncSceneZoom(double zoom)
        {
            _zoom += zoom;
        }
        public double GetSceneZoom()
        {
            return _zoom;
        }
        public void IncSceneAngles(double xDelta, double yDelta)
        {
            _angles.SetXAngle(_angles.GetXAngle() + xDelta);
            _angles.SetYAngle(_angles.GetYAngle() + yDelta);
        }
        public RotationAngles GetSceneAngles()
        {
            return _angles;
        }
    }
}
