﻿using System;
using System.Collections.Generic;
using Geometry;

namespace GMath
{
    public class Complex
    {
        public double _real;
        public double _i;
        public double _phi;
        public double _z;

        public Complex()
        {
            _real = 0;
            _i = 0;
        }

        public Complex(double real, double i)
        {
            _real = real;
            _i = i;
            _z = Norm();
            ToPolar();
        }

        public Complex(Complex c)
        {
            _real = c._real;
            _i = c._i;
            _z = Norm();
            ToPolar();
        }

        public Complex(double real)
        {
            _real = real;
            _i = 0;
            _z = Norm();
            ToPolar();
        }

        public double Norm()
        {
            return Math.Sqrt(_real * _real + _i * _i);
        }

        public void ToPolar()
        {
            if (_z == 0)
            {
                this._phi = 0;
                return;
            }
            this._phi = Math.Acos(_real / _z);// * 180) / Math.PI;
        }

        public static List<Complex> ThirdSqrt(Complex c)
        {
            List<Complex> thirdSqrt = new List<Complex>();
            double sqrt = Program.Cbroot(c._z);
            double angle1 = c._phi / 3.0;
            double angle2 = (c._phi + 2.0 * Math.PI) / 3.0;
            double angle3 = (c._phi + 4.0 * Math.PI) / 3.0;
            Complex c1 = new Complex(sqrt * Math.Cos(angle1), sqrt * Math.Sin(angle1));
            Complex c2 = new Complex(sqrt * Math.Cos(angle2), sqrt * Math.Sin(angle2));
            Complex c3 = new Complex(sqrt * Math.Cos(angle3), sqrt * Math.Sin(angle3));
            thirdSqrt.Add(c1);
            thirdSqrt.Add(c2);
            thirdSqrt.Add(c3);
            return thirdSqrt;
        }
        public static List<Complex> SecondSqrt(Complex c)
        {
            List<Complex> secondSqrt = new List<Complex>();
            double sqrt = Math.Sqrt(c._z);
            double angle1 = c._phi / 2.0;
            double angle2 = (c._phi + 2.0 * Math.PI) / 2.0;
            Complex c1 = new Complex(sqrt * Math.Cos(angle1), sqrt * Math.Sin(angle1));
            Complex c2 = new Complex(sqrt * Math.Cos(angle2), sqrt * Math.Sin(angle2));
            secondSqrt.Add(c1);
            secondSqrt.Add(c2);
            return secondSqrt;
        }
        public static List<Complex> SecondSqrt1728(Complex c) 
        {
            double r = Math.Sqrt((c._real + c._z) / 2.0);
            double s = Math.Sqrt((c._z - c._real) / 2.0);
            if (c._i > 0)
                return new List<Complex>() { new Complex(r, s), new Complex(-r, -s) };
            else
                return new List<Complex>() { new Complex(-r, s), new Complex(r, -s) };
            
            //if (c._i == 0 && c._real >= 0)
            //    return new List<Complex>() {new Complex(Math.Sqrt(c._real)), new Complex(-Math.Sqrt(c._real)) };
            //double y = Math.Sqrt((r - c._real) / 2.0);
            //double x = c._i / (2 * y);
            //return new List<Complex>() 
            //{
            //    new Complex(x, y),
            //    new Complex(-x, -y)
            //};
        }
        public static List<Complex> ThirdSqrt(double d)
        {
            Complex c = new Complex(d);
            return ThirdSqrt(c);
        }
        public static List<Complex> SecondSqrt(double d)
        {
            Complex c = new Complex(d);
            return SecondSqrt(c);

        }

        public static Complex operator -(Complex c)
        {
            return new Complex(-c._real, -c._i);
        }
        public static Complex operator -(double d, Complex c)
        {
            return (-c + d);
        }
        public static Complex operator -(Complex c, double d)
        {
            return c + (-d);
        }
        public static Complex operator +(Complex c, double d)
        {
            return new Complex(c._real + d, c._i);
        }
        public static Complex operator +(double d, Complex c)
        {
            return new Complex(c._real + d, c._i);
        }
        public static Complex operator +(Complex c1, Complex c2)
        {
            return new Complex(c1._real + c2._real, c1._i + c2._i);
        }

        public static Complex operator -(Complex c1, Complex c2)
        {
            return new Complex(c1._real - c2._real, c1._i - c2._i);
        }
        public static Complex operator *(double d, Complex c)
        {
            return new Complex(c._real * d, c._i * d);
        }
        public static Complex operator *(Complex c, double d)
        {
            return d * c;
        }

        public static Complex operator *(Complex c1, Complex c2)
        {
            double a = c1._real * c2._real;
            double ab = c1._real * c2._i + c1._i * c2._real;
            double b = c1._i * c2._i;

            return new Complex(a - b, ab);
        }

        public static Complex operator /(double d, Complex c)
        {
            Complex c1 = new Complex(d);
            double r = c._real / (c._real * c._real + c._i * c._i);
            double i = -(c._i) / (c._real * c._real + c._i * c._i);
            Complex c2 = new Complex(r, i);
            return c1 * c2;
        }
        public static Complex operator /(Complex a, Complex b)
        {
            Complex conjugative = new Complex(b._real, -b._i);
            return (a * conjugative) / (b * conjugative)._real; 
        }

        public static Complex operator /(Complex c, double d)
        {
            return new Complex(c._real / d, c._i / d);
        }
    }
}
