﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GMath
{
    public class Interval
    {
        private double _a;
        private double _b;
        private bool _isEmpty;

        public Interval(double a, double b) 
        {
            if (a < b)
            {
                _a = a;
                _b = b;
            }
            else 
            {
                _a = b;
                _b = a;
            }
            if (double.IsNaN(a) || double.IsNaN(b))
                _isEmpty = true;
            else _isEmpty = false;
        }

        public Interval(double a, double b, bool c)
        {
            _a = a;
            _b = b;
            if (double.IsNaN(a) || double.IsNaN(b))
                _isEmpty = true;
            else _isEmpty = false;
        }

        public static Interval GetBiggestInterval(List<Interval> intervals) 
        {
            Interval retval = null;
            if (intervals.Count > 0)
                retval = intervals[0];
            else return null;
            foreach (Interval interval in intervals) 
            {
                if (interval.GetSize() > retval.GetSize())
                    retval = interval;
            }
            return retval;
        }

        public double GetSize() 
        {
            if (this._isEmpty)
                return 0;
            if (this._a > this._b)
                return 0;
            return (this._b - this._a);
        }

        public void SetIsEmpty(bool value) 
        {
            _isEmpty = value;
        }

        //public static bool operator ==(Interval a, Interval b)
        //{
        //    try
        //    {
        //        return a._a == b._a && a._b == b._b; problem pokud je nektery null.
        //    }
        //    catch (Exception) { return false; }
        //}

        //public static bool operator !=(Interval a, Interval b) 
        //{
        //    return !(a == b);
        //}

        public static Interval operator *(double d, Interval interval) 
        {
            return new Interval(d * interval.GetA(), d * interval.GetB());
        }
        public static Interval operator /(Interval interval, double d)
        {
            return new Interval(interval.GetA() / d, interval.GetB() / d);
        }
 

        public bool Belongs(double x) { return x >= _a && x <= _b; }

        public double GetA() { return _a; }

        public double GetB() { return _b; }

        public void SetA(double a) { _a = a; }

        public void SetB(double b) { _b = b; }

        public bool IsEmpty() { return _isEmpty; }

        public static Interval Intersect(Interval a, Interval b) 
        {
            double min = Math.Max(a._a, b._a);
            double max = Math.Min(a._b, b._b);
            Interval retval = new Interval(min, max);
            if (max < min || a._isEmpty || b._isEmpty)
                retval._isEmpty = true;
            return retval;
        }

        public static Interval Intersect(List<Interval> list) 
        {
            List<double> lowers = new List<double>();
            List<double> uppers = new List<double>();
            bool isEmpty = false;
            foreach (Interval interval in list) 
            {
                lowers.Add(interval._a);
                uppers.Add(interval._b);
                if (interval._isEmpty)
                {
                    isEmpty = true;
                }
            }
            Interval retval = new Interval(lowers.Max(), uppers.Min());
            retval._isEmpty = isEmpty;
            if (retval._a > retval._b)
                retval._isEmpty = true;
            return retval;
        }

        public static List<Interval> Intersect(List<Interval> list, Interval interval) 
        {
            List<Interval> retval = new List<Interval>();
            Interval temp;
            for (int i = 0; i < list.Count; i++) 
            {
                temp = Intersect(list[i], interval);
                if (!temp.IsEmpty())
                    retval.Add(temp);
            }
            return retval;
        }

        public static List<Interval> Intersect(List<Interval> a, List<Interval> b)
        {
            List<Interval> retval = new List<Interval>();
            Interval temp;
            for (int i = 0; i < a.Count; i++)
            {
                for (int j = 0; j < b.Count; j++)
                {
                    temp = Intersect(a[i], b[j]);
                    if (!temp.IsEmpty())
                        retval.Add(temp);
                }
            }
            return retval;
        }


        public static Interval UnionOfIntersectingIntervals(Interval a, Interval b) 
        {
            double min = Math.Min(a._a, b._a);
            if (double.IsNaN(a._a))
                min = b._a;
            if (double.IsNaN(b._a))
                min = a._a;
            double max = Math.Max(a._b, b._b);
            if (double.IsNaN(a._b))
                max = b._b;
            if (double.IsNaN(b._b))
                min = b._b;
            Interval retval = new Interval(min, max);
            if (max < min || a._isEmpty || b._isEmpty)
                retval._isEmpty = true;
            return retval;
        }

        public static Interval UnionSpecial(Interval a, Interval b)
        {
            List<double> list = new List<double>();
            if (!double.IsNaN(a._a))
                list.Add(a._a);
            if (!double.IsNaN(a._b))
                list.Add(a._b);
            if (!double.IsNaN(b._a))
                list.Add(b._a);
            if (!double.IsNaN(b._b))
                list.Add(b._b);
            //if (list.Count == 0)
            //    return new Interval(0,0);
            double min = list.Min();
            double max = list.Max();
            Interval retval = new Interval(min, max);
            if (max < min)
                retval._isEmpty = true;
            return retval;
        }

        public static List<Interval> Union(Interval a, Interval b) 
        {
            if (Intersecting(a, b))
                return new List<Interval>() { UnionOfIntersectingIntervals(a, b) };
            return new List<Interval>() { a, b };
        }


        public static List<Interval> Union(List<Interval> intervals, Interval interval) 
        {
            if (interval.IsEmpty())
                return intervals;
            intervals = SortIntervals(intervals);
            bool found = false;
            for (int i = 0; i < intervals.Count; i++) 
            {
                if (Intersecting(intervals[i], interval))
                {
                    intervals[i]._a = Math.Min(intervals[i]._a, interval._a);
                    intervals[i]._b = Math.Max(intervals[i]._b, interval._b);
                    found = true;
                    break;
                }
            }
            if (!found)
                intervals.Add(interval);
            for (int i = 0; i < intervals.Count -1; i++) 
            {
                if (Intersecting(intervals[i], intervals[i + 1]))
                {
                    intervals.Insert(i, UnionOfIntersectingIntervals(intervals[i], intervals[i + 1]));
                    intervals.RemoveAt(i + 1);
                    intervals.RemoveAt(i + 1);
                    i -= 1;
                }
            }
            return intervals;
        }

        public static List<Interval> SortIntervals(List<Interval> input) 
        {
            input.Sort(new Comparison<Interval>(IntervalCompareFunction));
            return input;
        }
        private static int IntervalCompareFunction(Interval a, Interval b) 
        {
            if (a._a < b._a)
                return -1;
            if (b._a < a._a)
                return 1;
            return 0;
        }

        private static bool Intersecting(Interval a, Interval  b) 
        {
            return a.Belongs(b._a) || a.Belongs(b._b) ||
                b.Belongs(a._a) || b.Belongs(a._b); 
        } 
 
        /// <summary>
        /// Predpoklad a je podmnozinou b nebo b je podnozinou a.
        /// Jinak vraci prazdny seznam.
        /// </summary>
        public static List<Interval> MinusSmallerFromBigger(Interval a, Interval b) 
        {
            List<Interval> retval = new List<Interval>();
            if (!Intersecting(a, b))
                return retval;
            if (a.Size() < b.Size())// b je podmnozinou a 
            {
                retval.Add(new Interval(a._a, b._a));
                retval.Add(new Interval(b._b, a._b));
            }
            if (b.Size() < a.Size())// a je podmnozinou b 
            {
                retval.Add(new Interval(b._a, a._a));
                retval.Add(new Interval(a._b, b._b));
            }
            return retval;
        }

        public double Size() 
        {
            return _b - _a;
        }



        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }
    }
}
