﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    public class Matrix
    {
        protected int _width;
        protected int _height;
        protected double[] _values;


        protected Matrix() { }

        public Matrix(int size)
        {
            _width = size;
            _height = size;
            _values = new double[size * size];
            for (int i = 0; i < size; i++)
            {
                for (int j = 0; j < size; j++)
                {
                    _values[i * size + j] = 0;
                }
                _values[i * size + i] = 1;
            }
        }

        public Matrix(int height, int width, double[] values) 
        {
            _width = width;
            _height = height;
            _values = values;
        }

        public Matrix(Matrix matrix) 
        {
            _width = matrix._width;
            _height = matrix._height;
            _values = new double[_width * _height];
            int i = 0;
            foreach (double d in matrix._values)
                _values[i++] = d;
        }

        /// <summary>
        /// Pouze pro zdedene tridy
        /// </summary>
        /// <returns></returns>
        public double[] GetValues() { return _values; }


        public int GetWidth() { return _width; }

        public int GetHeight() { return _height; }

        public double[] ToDoubleArray()
        {
            return _values;
        }

        public double Get(int row, int column) 
        {
            if (row > -1 && row < _height && column > -1 && column < _width)
                return _values[row * _width + column];
            else throw new Exception("Snazite se ziskat prvek matice na pozici, ktera je mimo rozmery matice.");
        }

        public double Get(int i)
        {
            if (i > -1 && i < _width * _height)
                return _values[i];
            else throw new Exception("Index je mimo hranice rozmeru matice.");
        }

        private void Set(int i, double value) { _values[i] = value; }

        /// <summary>
        /// testovano
        /// </summary>
        /// <returns></returns>
        public Matrix TransponMatrix() 
        {
            double[] values = new double[_width * _height];
            for (int i = 0; i < _height; i++)
            {
                for (int j = 0; j < _width; j++) 
                {
                    values[j * _height + i] = _values[i * _width + j];
                }
            }
            return new Matrix(_width, _height, values);
        }

        public Matrix SwitchIandJrow(int i, int j) 
        {
            double[] values = new double[_height*_width];
            for (int k = 0; k < _height*_width; k++) 
            {
                values[k] = _values[k];
            }
            double temp;
            for (int k = 0; k < _width; k++) 
            {
                temp = _values[i * _height + k];
                _values[i * _height + k] = _values[j * _height + k];
                _values[j * _height + k] = temp;
            }
            return new Matrix(_height, _width, values);
        }

        /// <summary>
        /// testovano
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Matrix operator*(Matrix a, Matrix b) 
        {
            if (a._width != b._height) 
            {
                throw new Exception("Snazite se nasobit matice spatnych rozmeru.");
            }
            int m = a._height;
            int n = a._width;
            int z = b._width;
            double[] values = new double[a._height * b._width];
            double sum;
            for (int i = 0; i < m; i++) 
            {
                for (int j = 0; j < z; j++) 
                {
                    sum = 0;
                    for(int k = 0; k < n; k++) 
                    {
                        sum += a._values[i*n + k]* b._values[z * k + j];    
                    }
                    values[i * z + j] = sum;
                }
            }
            //for (int i = 0; i < a._height * b._width; i++)
            //{
            //    if (Program.IsEqualInTolerance(values[i], 0))
            //        values[i] = 0;
            //}
            return new Matrix(m, z, values);
        }



        /// <summary>
        /// Vytvori submatici A_(i,j), kde i=row a j=column. 
        /// </summary>
        /// <param name="column"></param>
        /// <returns></returns>
        public Matrix GetSubMatrix(int row,int column) 
        {
            double[] values = new double[(this._height-1)*(this._width-1)];
            int index = 0;
            for (int i = 0; i < this._height; i++) 
            {
                if (i != row)
                {
                    for (int j = 0; j < this._width; j++)
                    {
                        if (j != column)
                        {
                            values[index] = this._values[i * this._height + j];
                            index++;
                        }
                    }
                }
            }
            return new Matrix(this._height-1,this._width-1,values);   
        }

        /// <summary>
        /// Pouze pro matice 4x4.
        /// </summary>
        /// <returns></returns>
        public double Determinant4() 
        {
            double sum = 0;
            int sign = 1;
            for (int i = 0; i < 4; i++)
            {
                sum = sum + sign * this._values[i] * this.GetSubMatrix(0,i).Determinant3();
                sign = sign * -1;
            }
            return sum;
        }

        /// <summary>
        /// Vypocita determinant matice 3x3
        /// </summary>
        /// <returns></returns>
        public double Determinant3() 
        {
            double sum = 0;
            int sign = 1;
            for (int i = 0; i < 3; i++)
            {
                sum = sum + sign * this._values[i] * this.GetSubMatrix(0, i).Determinant2();
                sign = sign * -1;
            }
            return sum;
        }

        /// <summary>
        /// Vypocita determinant matice 2x2
        /// </summary>
        /// <returns></returns>
        public double Determinant2() 
        {
            return (this._values[0] * this._values[3]) - (this._values[1] * this._values[2]);
        }


        #region<Deprecated>

        /// <summary>
        /// Vrací double[16] prevedeny na double[9].
        /// Posledni radek a sloupec jsou ignorovany.
        /// </summary>
        private  double[] RotationToThreeXthree(double[] rotation)
        {
            double[] retval = new double[9];
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 3; j++)
                {
                    retval[i * 3 + j] = rotation[i * 4 + j];
                }
            }
            return retval;
        }

        /// <summary>
        /// Posune vector(prvni arg) o MINUS !vektor! (druhy arg).
        /// Size udava pocet prvku vektoru(vetsinou 3);
        /// </summary>
        private double[] ShiftVectorByVector(double[] vector, Vector shift, int size)
        {
            for (int i = 0; i < size; i++)
            {
                vector[i] += -shift.Get(i);
            }
            return vector;
        }

                /// <summary>
        /// Prevede double[] na double[][]
        /// </summary>
        private double[][] ToTwoDimensional(double[] matrix, int size)
        {
            double[][] retval = new double[size][];
            for (int i = 0; i < size; i++)
            {
                double[] retvali = new double[size];
                for (int j = 0; j < size; j++)
                {
                    retvali[j] = matrix[i * size + j];
                }
                retval[i] = retvali;
            }
            return retval;
        }

        /// <summary>
        /// Prevede double[][] na double[]
        /// </summary>
        private double[] ToOneDimensional(double[][] matrix, int size)
        {
            double[] retval = new double[size * size];
            for (int i = 0; i < size; i++)
            {
                for (int j = 0; j < size; j++)
                {
                    retval[i * size + j] = matrix[i][j];
                }
            }
            return retval;
        }
        
        #endregion

        /// <summary>
        /// Předané matice upraví
        /// Upravi matici na trojuhelnikovou, zároven upravuje matici jednotkovou.
        /// </summary>
        /// <returns></returns>
        private void MakeTriangleSystem(Matrix matrix, Matrix eMatrix, int size)
        {
            List<int> usedRows = new List<int>();
            for (int i = 0; i < size; i++)
            {
                // zarucim aby i.clen i.radku nebyl nulovy, jinak prohodit radky
                int rowIndex = FindNonZeroValueIndexAtI(matrix, i, usedRows);
                if (rowIndex == -1)
                    continue;
                usedRows.Add(i);
                // Prohodím vektory tak, aby se nedělilo nulou
                SwapRows(matrix, eMatrix, i, rowIndex);
                MakeZeroAtPosition(matrix, eMatrix, i, usedRows);
            }
        }

        /// <summary>
        /// Upraví předané matice !!
        /// Upraví soustavu lin. rovnic tak, aby i-tý prvek každého vektoru pod diagonálou obsahoval 0. 
        /// Předpokládá soustavu 3 rovnic o 3. neznámích.(3 vektory o x,y,z);
        /// </summary>
        /// <returns></returns>
        private void MakeZeroAtPosition(Matrix matrix, Matrix eMatrix, int i, List<int> usedRows)
        {
            double delta;
            int size = matrix._height;
            for (int j = 0; j < size; j++)// j-ty radek 
            {
                if (i == j) continue;
                delta = -matrix._values[j * size + i] / matrix._values[i * size + i];
                for (int k = 0; k < size; k++) 
                {
                    matrix._values[j * size + k] = matrix._values[i * size + k] * delta + matrix._values[j * size + k];
                    eMatrix._values[j * size + k] = eMatrix._values[i * size + k] * delta + eMatrix._values[j * size + k];
                }
            }
        }

        /// <summary>
        /// Vrací index vektoru, jež má na pozici i nenulový prvek,
        /// Tento hledá od 0-teho radku, ale preskakuje jiz pouzite radky.
        /// Pokud takový není vrací -1.
        /// </summary>
        /// <returns></returns>
        private int FindNonZeroValueIndexAtI(Matrix matrix, int i, List<int> usedRows)
        {
            int index = 0;
            for (int j = 0; j < matrix._height; j++)
            {
                if (!Program.IsEqualInTolerance(matrix._values[j*_height + i], 0) &&
                    !IsInUsedRowsList(usedRows, j))
                    return index;
                index++;
            }
            return -1;
        }

        /// <summary>
        /// Vrací pravdu pokud se celociselna hodnota index nachazi v seznamu celociselnych hodnot UsedRows.
        /// Jinak false;
        /// </summary>
        private bool IsInUsedRowsList(List<int> usedRows, int index)
        {
            foreach (int i in usedRows)
            {
                if (i == index)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Upraví předanou matici
        /// Vymeni radky matice
        /// </summary>
        /// <returns></returns>
        private void SwapRows(Matrix matrix, Matrix eMatrix, int i, int j)
        {
            double temp;
            int size = matrix._height;
            for (int k = 0; k < size; k++)
            {
                temp = matrix._values[i * size + k];
                matrix._values[i * size + k] = matrix._values[j * size + k];
                matrix._values[j * size + k] = temp;

                temp = eMatrix._values[i * size + k];
                eMatrix._values[i * size + k] = eMatrix._values[j * size + k];
                eMatrix._values[j * size + k] = temp;
            }
        }


        public static Vector operator*(Vector vector, Matrix matrix)
        {
            double temp = 0;
            int size = matrix._height;
            Vector retval = new Vector(0, 0, 0);

            for (int i = 0; i < size; i++)
            {
                temp = 0;
                for (int j = 0; j < size; j++)
                {
                    temp += matrix.Get(j, i) * vector.Get(j);
                }
                retval.Set(i, temp);
            }
            return retval;
        }




        /// <summary>
        /// Upraví předané matice
        /// Krok potrebny pri ziskavani inverzni matice.
        /// Po gaus-jordanove eliminaci vynasobi prvky diagonaly tak, aby tam byli jednicky.
        /// Plus koeficientem kterym nasobi i-ty prkek i-teho radku vynasobi cely i-ty radek eMatrix. 
        /// Osetruje deleni 0;
        /// </summary>
        private void SetDiagonalToOnes(Matrix matrix, Matrix eMatrix, int size)
        {
            double delta;
            for (int i = 0; i < size; i++)
            {
                if (matrix._values[i*size + i] != 0)
                {
                    delta = 1 / matrix._values[i * size + i];
                    for (int j = 0; j < size; j++)
                    {
                        eMatrix._values[i * size + j] *= delta;
                    }
                }
            }
        }

        public Matrix ComputeInverseMatrix()
        {
            double[] values = new double[this._width*this._height];
            for (int i = 0; i < this._width*this._height; i++)
            {
                values[i] = this._values[i];
            }
            Matrix retval = new Matrix(this._height, this._width, values);
            int size = this._height;
            Matrix e = new Matrix(size);
            MakeTriangleSystem(retval, e, size);
            SetDiagonalToOnes(retval, e, size);
            return e;
        }
    
        public static Matrix operator-(Matrix a, Matrix b)
        {
            if (a._height != b._height && a._width != b._width)
                throw new Exception("Nelze odecitat matice ruznych rozmeru");
            double[] values = new double[a._height*a._width];
            for(int i = 0; i < a._height*a._width; i++)
            {
                values[i] = a._values[i]-b._values[i];
            }
            return new Matrix(a._height,a._width, values);
        }

        public static Matrix operator *(double koef, Matrix matrix) 
        {
            double[] values = new double[matrix._height * matrix._width];
            for (int i = 0; i < matrix._height * matrix._width; i++)
                values[i] = matrix._values[i] * koef;
            return new Matrix(matrix._height, matrix._width, values);
        }

        public override string ToString()
        {
            string retval = "";
            for (int i = 0; i < this._height; i++)
            {
                for (int j = 0; j < this._width; j++)
                {
                    retval += _values[i * _width + j].ToString() + " ";
                }
                retval += "\n";
            }
            return retval;
        }



        public static bool AreMatrixesEqualInTolerance(Matrix a, Matrix b)
        {
            if (a._height != b._height || a._width != b._width)
                throw new Exception("Nelze srovnavat matice ruznych rozmeru.");
            bool same = true;
            for (int i = 0; i < a._width * a._height; i++)
            {
                same = same && Math.Abs((a._values[i]) - (b._values[i])) < 0.001;//Program.IsEqualInTolerance(a._values[i], b._values[i]);
            }
            return same;
        }
    }
}
