﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GMath
{
    public class ComplicatedEquation
    {
        private List<ComplicatedEquationMember> _members;


        public ComplicatedEquation(List<ComplicatedEquationMember> members)
        {
            _members = members;
        }

        public ComplicatedEquation(ParametrizationRST parametrization) 
        {
            _members = new List<ComplicatedEquationMember>();
            if (parametrization.ContainsRParameter())
                _members.Add(new ComplicatedEquationMember(parametrization.GetR(), "r"));
            if (parametrization.ContainsSParameter())
                _members.Add(new ComplicatedEquationMember(parametrization.GetS(), "s"));
            if (parametrization.ContainsTParameter())
                _members.Add(new ComplicatedEquationMember(parametrization.GetT(), "t"));
            if (parametrization.ContainsConstant())
                _members.Add(new ComplicatedEquationMember(parametrization.GetConstant(), ""));
        }

        public int GetMembersCount() { return _members.Count; }

        public static ComplicatedEquation operator *(ComplicatedEquation a, ComplicatedEquation b) 
        {
            List<ComplicatedEquationMember> members = new List<ComplicatedEquationMember>();
            for (int i = 0; i < a.GetMembersCount(); i++) 
            {
                for (int j = 0; j < b.GetMembersCount(); j++) 
                {
                    members.Add(a._members[i] * b._members[j]);
                }
            }
            return new ComplicatedEquation(members);
        }

        public static ComplicatedEquation operator +(ComplicatedEquation a, ComplicatedEquation b) 
        {
            int j;
            for (int i = 0; i < b.GetMembersCount(); i++) 
            {
                for (j = 0; j < a.GetMembersCount(); j++) 
                {
                    if (a._members[i].GetVariables() == b._members[j].GetVariables())
                    {
                        a._members[i] += b._members[j];
                        break;
                    }
                }
                if (j == a.GetMembersCount())
                    a._members.Add(b._members[i]);
            }
            return a;
        }
    }
}
