﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GMath
{
    public class ParametrizationMatrix
    {
        private const int WIDTH = 4;
        private const int HEIGHT = 4;
        protected ComplicatedEquation[] _values;


        public ParametrizationMatrix(ComplicatedEquation[] values) 
        {
            _values = values;
        }

        public ParametrizationMatrix(Matrix matrix) 
        {
            _values = new ComplicatedEquation[WIDTH * HEIGHT];
            int i = 0;
            foreach (double d in matrix.ToDoubleArray())
                _values[i++] = new ComplicatedEquation(
                    new List<ComplicatedEquationMember>() {new ComplicatedEquationMember(d, "") });
        }



        public ComplicatedEquation Get(int row, int column) 
        {
            if (row > -1 && row < HEIGHT && column > -1 && column < WIDTH)
                return _values[row * WIDTH + column];
            else throw new Exception("Snazite se ziskat prvek matice na pozici, ktera je mimo rozmery matice.");
        }

        public ComplicatedEquation Get(int i)
        {
            if (i > -1 && i < WIDTH * HEIGHT)
                return _values[i];
            else throw new Exception("Index je mimo hranice rozmeru matice.");
        }

        private void Set(int i, ComplicatedEquation value) { _values[i] = value; }

        /// <summary>
        /// testovano
        /// </summary>
        public static ParametrizationMatrix operator *(ParametrizationMatrix a, ParametrizationMatrix b)
        {
            int m = HEIGHT;
            int n = WIDTH;
            int z = WIDTH;
            ComplicatedEquation[] values = new ComplicatedEquation[WIDTH * HEIGHT];
            ComplicatedEquation sum;
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < z; j++)
                {
                    sum = new ComplicatedEquation(
                        new List<ComplicatedEquationMember>() { new ComplicatedEquationMember(0, "") });
                    for (int k = 0; k < n; k++)
                    {
                        sum += a._values[i * n + k] * b._values[z * k + j];
                    }
                    values[i * z + j] = sum;
                }
            }
            return new ParametrizationMatrix(values);
        }
    }
}
