﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;


namespace GMath
{
    public class Vector
    {
        private double _x;
        private double _y;
        private double _z;
        private double _one;

        public Vector(Vector src) 
        {
            _x = src._x;
            _y = src._y;
            _z = src._z;
            _one = src._one;
        }

        public Vector(double x, double y, double z)
        {
            _x = x;
            _y = y;
            _z = z;
            _one = 1;
        }

        public Vector(double x, double y, double z, double one) 
        {
            _x = x;
            _y = y;
            _z = z;
            _one = one;
        }

        /// <summary>
        /// Makes null Vector (0,0,0);
        /// </summary>
        public Vector()
        {
            _x = 0;
            _y = 0;
            _z = 0;
            _one = 1;
        }

        public bool IsZero()
        {
            return Program.IsEqualInTolerance(Size(this), 0);
        }

        public static Vector operator +(Vector u, Vector v)
        {
            return new Vector(u._x + v._x, u._y + v._y, u._z + v._z);
        }

        public static Vector operator -(Vector u, Vector v)
        {
            return new Vector(u._x - v._x, u._y - v._y, u._z - v._z);
        }

        public static double operator *(Vector u, Vector v)
        {
            return u._x * v._x + u._y * v._y + u._z * v._z;
        }

        public static Vector operator *(double d, Vector v)
        {
            return new Vector(d * v.GetX(), d * v._y, d * v._z);
        }

        public static Vector operator *(Vector v, double d)
        {
            return d * v;
        }

        public static Vector operator /(Vector v, double d)
        {
            return new Vector(v._x / d, v._y / d, v._z / d);
        }



        public double Get(int i)
        {
            switch (i)
            {
                case 0: return _x;
                case 1: return _y;
                case 2: return _z;
                case 3: return 1;
                default: throw new ArgumentException("Vector get(i), bad arg. +{i}");
            }
        }

        public void Set(int i, double value)
        {
            switch (i)
            {
                case 0: _x = value;
                    break;
                case 1: _y = value;
                    break;
                case 2: _z = value;
                    break;
                case 3: return;
                default: throw new ArgumentException("Vector set(i,value), bad arg. +{i}");
            }
        }

        public double GetX() { return _x; }

        public double GetY() { return _y; }

        public double GetZ() { return _z; }

        public double GetOne() { return _one; }

        public void SetX(double x)
        {
            this._x = x;
        }

        public void SetY(double y)
        {
            this._y = y;
        }

        public void SetZ(double z) 
        {
            this._z = z;
        }

        public Vector SetValues(double x, double y, double z) 
        {
            _x = x;
            _y = y;
            _z = z;
            return this;
        }



        /// <summary>
        /// Upraví vstup, neměnit!!!
        /// </summary>
        /// <returns></returns>
        public Vector NornVector()
        {
            double lenght = Math.Sqrt(_x * _x + _y * _y + _z * _z);
            if (lenght == 0)
            {
                _x = 0;
                _y = 0;
                _z = 0;
                return this;
            }
            _x = _x / lenght;
            _y = _y / lenght;
            _z = _z / lenght;
            return this;
        }

        public static Vector CrossProduct(Vector u, Vector v)
        {
            double c1 = u._y * v._z - u._z * v._y;
            double c2 = u._z * v._x - u._x * v._z;
            double c3 = u._x * v._y - u._y * v._x;

            return new Vector(c1, c2, c3);
        }

        public static double Distance(Vector u, Vector v)
        {
            double a = u._x - v._x;
            double b = u._y - v._y;
            double c = u._z - v._z;

            return Math.Sqrt(a * a + b * b + c * c);
        }

        /// <summary>
        /// Vraci velikost vektoru
        /// </summary>
        public static double Size(Vector a) 
        {
            return Math.Sqrt(a._x * a._x + a._y * a._y + a._z * a._z);
        }

        /// <summary>
        /// Vraci uhel svirany vektory "a" a "b";
        /// Uhel vraci v radianech.
        /// </summary>
        /// <returns></returns>
        public static double Angle(Vector a, Vector b) 
        {
            double skalarniSoucin = a * b;
            double sizeOfA = Size(a);
            double sizeOfB = Size(b);
            double value = skalarniSoucin / (sizeOfA * sizeOfB);
            if (value < -1)
                value = -1;
            if (value > 1)
                value = 1;
            return Math.Acos(value);
        }

        /// <summary>
        /// Vrací úhel o ktery se musí otočit první vektor, tak aby byl roven druhému vektoru.
        /// Je třeba dodat osu, podle ktere má být první vektor otočen, osa která vznikne vektorovým součinem,
        /// je taková, že vždy získáme kladný úhel.
        /// </summary>
        public static double OrientedAngle(Vector a, Vector b, Vector axis) 
        {
            double angle = (Angle(a, b) * 180.0) / Math.PI;
            TransformationMatrix m1 = TransformationMatrix.GetRotationMatrixAroundAAxe(axis, -angle);
            Angle(a, b);
            TransformationMatrix m2 = TransformationMatrix.GetRotationMatrixAroundAAxe(axis, angle);
            Vector temp1 = a * m1;
            Vector temp2 = a * m2;
            double deviation1 = Vector.Size(b - temp1);
            double deviation2 = Vector.Size(b - temp2);
            if (deviation1 < deviation2)
                return angle;
            return -angle;
        }



        /// <summary>
        /// Pouziva IntersectionTolerance porovnavani z tridy Program. 
        /// </summary>
        public static bool AreTwoVectorsInIntersectionToleranceLinearyBounded(Vector a, Vector b) 
        {
            double koeficient = 1;
            if (!Program.IsEqualInTolerance(b._x, 0))
            {
                koeficient = a._x / b._x;
                return Vector.AreEqualInIntersectionTolerance(a, b * koeficient);
            }
            if (!Program.IsEqualInTolerance(b._y, 0))
            {
                koeficient = a._y / b._y;
                return Vector.AreEqualInIntersectionTolerance(a, b * koeficient);
            }
            if (!Program.IsEqualInTolerance(b._z, 0))
            {
                koeficient = a._z / b._z;
                return Vector.AreEqualInIntersectionTolerance(a, b * koeficient);
            }
            return true;
        }
        /// <summary>
        /// Pouziva IntersectionTolerance porovnavani z tridy Program. 
        /// </summary>
        public static bool AreEqualInIntersectionTolerance(Vector a, Vector b) 
        {
            return Program.IsEqualInToleranceIntersectionVector(a._x, b._x) &&
               Program.IsEqualInToleranceIntersectionVector(a._y, b._y) &&
               Program.IsEqualInToleranceIntersectionVector(a._z, b._z);
        }




        public static bool AreLinearyBounded(Vector a, List<Vector> list)
        {
            if (a == null || list == null || list.Count == 0)
                return false;
            foreach (Vector v in list)
                if (v == null)
                    return true;
            if (list.Count == 1)
            {
                return Vector.AreTwoVectorsLinearyBounded(a, list[0]);
            }
            if (list.Count == 2) 
                return AreThreeVectorsLinearyBounded(a,list);
            return true;
        }

        private static bool AreTwoVectorsLinearyBounded(Vector a, Vector b) 
        {
            double koeficient = 1;
            if (!Program.IsEqualInTolerance(b._x, 0))
            {
                koeficient = a._x / b._x;
                return Vector.AreEqualInTolerance(a, b * koeficient);
            }
            if (!Program.IsEqualInTolerance(b._y, 0))
            {
                koeficient = a._y / b._y;
                return Vector.AreEqualInTolerance(a, b * koeficient);
            }
            if (!Program.IsEqualInTolerance(b._z, 0))
            {
                koeficient = a._z / b._z;
                return Vector.AreEqualInTolerance(a, b * koeficient);
            }
            return true;
        }

        private static bool AreThreeVectorsLinearyBounded(Vector a, List<Vector> list)
        {
            Equation x = new Equation(a._x, list[0]._x, list[1]._x);
            Equation y = new Equation(a._y, list[0]._y, list[1]._y);
            Equation z = new Equation(a._z, list[0]._z, list[1]._z);
            SimpleEquationSystem oves = new SimpleEquationSystem(x, y, z);
            OwnVectorParametrization solution = oves.Solve();
            if (solution == null || solution.IsZero())
                return false;
            return true;
        }

        public static bool RightOrientationOfBasis(Vector v, List<Vector> vectors) 
        {
            Vector v1 = vectors[0];
            Vector v2 = vectors[1];
            double[] values = new double[]
            {
                v1._x, v1._y, v1._z, 0,
                v2._x, v2._y, v2._z, 0,
                v._x, v._y, v._z, 0,
                0, 0, 0, 1
            };
            return new Matrix(4, 4, values).Determinant4() > 0;
        }

        public static bool AreEqualInTolerance(Vector a, Vector b) 
        {
            return Program.IsEqualInTolerance(a._x, b._x) &&
                Program.IsEqualInTolerance(a._y, b._y) &&
                Program.IsEqualInTolerance(a._z, b._z);
        }

        public bool ContainsNaN() 
        {
            return
                double.IsNaN(_x) ||
                double.IsNaN(_y) ||
                double.IsNaN(_z);
        }




        /// <summary>
        /// Vrati seznam Orotnormalnich vektoru, ktere tvori ortonormalni bazi predanych vektoru.
        /// Predpokladam "PRESNE 2" vektory.
        /// </summary>
        /// <param name="vectors"></param>
        /// <returns></returns>
        public static List<Vector> OrthoNormalize(List<Vector> vectors)
        {
            foreach (Vector v in vectors)
                if (v == null)
                    return vectors;
            if (vectors.Count == 3)
                return OrthoNormalizeThreeVectors(vectors);
            if (vectors == null || vectors.Count != 2)
            {
                if (vectors.Count != 0)
                    foreach (Vector v in vectors)
                    {
                        v.NornVector();
                    }
                return vectors;
            }
            return OrthoNormalizeTwoVectors(vectors);
        }

        private static List<Vector> OrthoNormalizeTwoVectors(List<Vector> vectors) 
        {
            List<Vector> retval = new List<Vector>();
            Vector v1 = vectors[0];
            v1.NornVector();
            retval.Add(vectors[0]);
            double size = Vector.Size(vectors[0]);
            // vi = ui - (vj * ui)/|vj|2*vj; 
            Vector v2 = (vectors[1] - (vectors[0] * (vectors[0] * vectors[1]))) /
                Vector.Size(vectors[1] - (vectors[0] * (vectors[0] * vectors[1])));
            retval.Add(v2);
            return retval;
        }

        private static List<Vector> OrthoNormalizeThreeVectors(List<Vector> vectors)
        {
            List<Vector> retval = new List<Vector>();
            Vector v1 = vectors[0];
            Vector v2 = vectors[1];
            Vector v3 = vectors[2];
            Vector u1 = v1.NornVector();
            retval.Add(u1);
            Vector u2 = (v2 - (u1 * (u1 * v2))) /
                Vector.Size(v2 - (u1 * (u1 * v2)));
            retval.Add(u2);
            Vector u3 = (v3 - (u2 * (u2 * v3)) - u1 * (u1 * v3)) /
                Vector.Size(v3 - (u2 * (u2 * v3)) - u1 * (u1 * v3));
            retval.Add(u3);
            return retval;
        }

        public static Vector Avarage(Vector a, Vector b) 
        {
            double x = (a._x + b._x) / 2;
            double y = (a._y + b._y) / 2;
            double z = (a._z + b._z) / 2;
            return new Vector(x,y,z);
        }


        public override string ToString()
        {
            return "(" + _x.ToString() + ", " + _y.ToString() + ", " + _z.ToString() + ")";
        }
    }
}
