﻿
using Geometry;
namespace GMath
{
    public class CenterEquationSystem:EquationSystem
    {
        public CenterEquationSystem(Equation a, Equation b, Equation c):base(a,b,c) 
        { }

        ////////////////////////////////////////////////////////////////////////
        //////////////////////  Reseni nehomogenni soustavy rovnic /////////////
        ////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// Resi soustavu 3 rovnic o trech neznamych a jedne konstante.
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public CenterParametrization Solve()
        {
            MakeTriangleSystem();
            CenterParametrization retval = SolveTriangleMatrix();
            return retval;
        }
        /// <summary>
        /// Vraci reseni soustavy 3 rovnic o trech neznamich.
        /// </summary>
        /// <param name="matrix"></param>
        /// <returns></returns>
        private CenterParametrization SolveTriangleMatrix()
        {
            if (Program.IsEqualInTolerance(_rows[0].GetX(), 0))
            {
                if (Program.IsEqualInTolerance(_rows[1].GetY(), 0))
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CaseNNN();
                    else return CaseNNP();
                }
                else
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CaseNPN();
                    else return CaseNPP();
                }
            }
            else
            {
                if (Program.IsEqualInTolerance(_rows[1].GetY(), 0))
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CasePNN();
                    else return CasePNP();
                }
                else
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CasePPN();
                    else return CasePPP();
                }
            }
        }


        /// <summary>
        /// 0 0 0 m
        /// 0 0 0 n
        /// 0 0 0 p
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CaseNNN()
        {
            if (!Program.IsEqualInTolerance(_rows[0].GetConstant(), 0) ||
                !Program.IsEqualInTolerance(_rows[1].GetConstant(), 0) ||
                !Program.IsEqualInTolerance(_rows[2].GetConstant(), 0))
                return null;
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 0 0 0 m
        /// 0 0 0 n
        /// 0 0 1 p
        /// -----
        /// (p,q,x)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CaseNNP()
        {
            if (!Program.IsEqualInTolerance(_rows[0].GetConstant(), 0) ||
                !Program.IsEqualInTolerance(_rows[1].GetConstant(), 0))
                return null;
            double xx = -_rows[2].GetConstant() / _rows[2].GetZ();
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(xx, 0, 0, 0);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 0 0 0 m
        /// 0 1 x n 
        /// 0 0 0 p
        /// -----
        /// (p,x,r)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CaseNPN()
        {
            if (!Program.IsEqualInTolerance(_rows[0].GetConstant(), 0) ||
                !Program.IsEqualInTolerance(_rows[2].GetConstant(), 0))
                return null;
            double xx = -(_rows[1].GetZ()) / _rows[1].GetY();
            double cc = -(_rows[1].GetConstant()) / _rows[1].GetY();
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(cc, 0, 0, xx);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 0 0 0 m
        /// 0 1 0 n
        /// 0 0 1 p
        /// -----
        /// (p,x,x)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CaseNPP()
        {
            if (!Program.IsEqualInTolerance(_rows[0].GetConstant(), 0))
                return null;
            double yy = -_rows[1].GetConstant() / _rows[1].GetY();
            double zz = -_rows[2].GetConstant() / _rows[2].GetZ();
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(yy, 0, 0, 0);
            ParametrizationRST z = new ParametrizationRST(zz, 0, 0, 0);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 1 x x m
        /// 0 0 0 n 
        /// 0 0 0 p
        /// -----
        /// (x,q,r)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CasePNN()
        {
            if (!Program.IsEqualInTolerance(_rows[1].GetConstant(), 0) ||
                !Program.IsEqualInTolerance(_rows[2].GetConstant(), 0))
                return null;
            double cc = -(_rows[0].GetConstant()) / _rows[0].GetX();
            double x1 = -(_rows[0].GetY()) / _rows[0].GetX();
            double x2 = -(_rows[0].GetZ()) / _rows[0].GetX();
            ParametrizationRST x = new ParametrizationRST(cc, 0, x1, x2);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 1 x 0 m
        /// 0 0 0 n 
        /// 0 0 1 p
        /// -----
        /// (x,q,0)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CasePNP()
        {
            if (!Program.IsEqualInTolerance(_rows[1].GetConstant(), 0))
                return null;
            double xx = -(_rows[0].GetY()) / _rows[0].GetX();
            double cc = -(_rows[0].GetConstant()) / _rows[0].GetX();
            double zz = -(_rows[2].GetConstant()) / _rows[2].GetZ();
            ParametrizationRST x = new ParametrizationRST(cc, 0, xx, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(zz, 0, 0, 0);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 1 0 x m
        /// 0 1 x n
        /// 0 0 0 p
        /// -----
        /// (x,x,r)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CasePPN()
        {
            if (!Program.IsEqualInTolerance(_rows[2].GetConstant(), 0))
                return null;
            double xx = -(_rows[0].GetZ()) / _rows[0].GetX();
            double c1 = -(_rows[0].GetConstant()) / _rows[0].GetX();
            double yy = -(_rows[1].GetZ()) / _rows[1].GetY();
            double c2 = -(_rows[1].GetConstant()) / _rows[1].GetY();
            ParametrizationRST x = new ParametrizationRST(c1, 0, 0, xx);
            ParametrizationRST y = new ParametrizationRST(c2, 0, 0, yy);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new CenterParametrization(x, y, z);
        }
        /// <summary>
        /// 1 0 0 m
        /// 0 1 0 n 
        /// 0 0 1 p
        /// -----
        /// (x,x,x)
        /// </summary>
        /// <returns></returns>
        private CenterParametrization CasePPP()
        {
            double xx = -(_rows[0].GetConstant()) / _rows[0].GetX();
            double yy = -(_rows[1].GetConstant()) / _rows[1].GetY();
            double zz = -(_rows[2].GetConstant()) / _rows[2].GetZ();            
            ParametrizationRST x = new ParametrizationRST(xx, 0, 0, 0);
            ParametrizationRST y = new ParametrizationRST(yy, 0, 0, 0);
            ParametrizationRST z = new ParametrizationRST(zz, 0, 0, 0);
            return new CenterParametrization(x, y, z);
        }

        ////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////

        ///// <summary>
        ///// 0 0 0 m
        ///// 0 1 0 n
        ///// 0 0 1 p
        ///// -------
        ///// (p,x,x);
        ///// </summary>
        ///// <returns></returns>
        //private CenterParametrization CaseNegativeA()
        //{
        //    if (!Program.IsEqualInTolerance(_rows[0].GetConstant(), 0))
        //        return null;
        //    //throw new Exception("Nehomogenni soustava rovnic nema reseni 0x = c");
        //    double yConstant = -_rows[1].GetConstant() / _rows[1].GetY();
        //    double yParameter = 0;
        //    double zConstant = -_rows[2].GetConstant() / _rows[2].GetZ();
        //    double zParameter = 0;
        //    return new CenterParametrization(0, 1, yConstant, yParameter, zConstant, zParameter);
        //}

        ///// <summary>
        ///// 1 x 0 m
        ///// 0 0 0 n
        ///// 0 0 1 p
        ///// -------
        ///// (x,q,x);
        ///// </summary>
        ///// <returns></returns>
        //private CenterParametrization CaseNegativeB()
        //{
        //    if (!Program.IsEqualInTolerance(_rows[1].GetConstant(), 0))
        //        return null;
        //    //throw new Exception("Nehomogenni soustava rovnic nema reseni 0y = c");
        //    double xConstant = -_rows[0].GetConstant() / _rows[0].GetX();
        //    double xParameter = -_rows[0].GetY() / _rows[0].GetX(); ;
        //    double zConstant = -_rows[2].GetConstant() / _rows[2].GetZ();
        //    double zParameter = 0;
        //    return new CenterParametrization(xConstant, xParameter, 0, 1, zConstant, zParameter);
        //}

        ///// <summary>
        ///// 1 0 x m
        ///// 0 1 x n
        ///// 0 0 0 p
        ///// -------
        ///// (x,x,r);
        ///// </summary>
        ///// <returns></returns>
        //private CenterParametrization CaseNegativeC()
        //{
        //    if (!Program.IsEqualInTolerance(_rows[2].GetConstant(), 0))
        //        return null;
        //    //throw new Exception("Nehomogenni soustava rovnic nema reseni 0z = c");
        //    double yConstant = -_rows[1].GetConstant() / _rows[1].GetY();
        //    double yParameter = -_rows[1].GetZ() / _rows[1].GetY();
        //    double xConstant = -_rows[0].GetConstant() / _rows[0].GetX();
        //    double xParameter = -_rows[0].GetZ() / _rows[0].GetX();
        //    return new CenterParametrization(xConstant, xParameter, yConstant, yParameter, 0, 1);
        //}

        ///// <summary>
        ///// 1 0 0 m
        ///// 0 1 0 n
        ///// 0 0 1 p
        ///// -------
        ///// (x,x,x);
        ///// </summary>
        ///// <returns></returns>
        //private CenterParametrization CasePositiveAll()
        //{
        //    double xConstant = -_rows[0].GetConstant() / _rows[0].GetX();
        //    double yConstant = -_rows[1].GetConstant() / _rows[1].GetY();
        //    double zConstant = -_rows[2].GetConstant() / _rows[2].GetZ();
        //    return new CenterParametrization(xConstant, 0, yConstant, 0, zConstant, 0);
        //}

    }
}
