﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    public class EquationSystem
    {
        protected Equation[] _rows;
        protected int _rowsCount;
        public int _version;

        public EquationSystem(Equation a, Equation b, Equation c) 
        {
            if ((a.GetVersion() == b.GetVersion()) && (b.GetVersion() == c.GetVersion()))
            {
                _rows = new Equation[3];
                _rows[0] = a;
                _rows[1] = b;
                _rows[2] = c;
                _version = a.GetVersion();
                _rowsCount = 3;
            }
            else 
            {
                throw new Exception("Rovnice predane konstruktoru systemu rovnic nejsou stejneho typu.");
            }
        }

        /// <summary>
        /// Upravi matici soustvy 3 rovnic o 3 neznamich na trojuhelnikovou.
        /// </summary>
        /// <returns></returns>
        public void MakeTriangleSystem()
        {
            List<int> usedRows = new List<int>();
            for (int i = 0; i < _rowsCount; i++)
            {
                // zarucim aby i.clen i.radku nebyl nulovy, jinak prohodit radky
                int rowIndex = FindNonZeroValueIndexAtI(i, usedRows);
                if (rowIndex == -1)
                    continue;
                usedRows.Add(i);
                // Prohodím vektory tak, aby se nedělilo nulou
                SwapRows(i, rowIndex);
                MakeZeroAtPosition(i, usedRows);
            }
        }

        /// <summary>
        /// Upraví soustavu lin. rovnic tak, aby i-tý prvek každého vektoru pod diagonálou obsahoval 0. 
        /// Předpokládá soustavu 3 rovnic o 3. neznámích.(3 vektory o x,y,z);
        /// </summary>
        /// <returns></returns>
        protected void MakeZeroAtPosition(int i, List<int> usedRows)
        {
            double delta;
            for (int j = 0; j < _rowsCount; j++)// j-ty radek 
            {
                if (i == j) continue;
                delta = -_rows[j].Get(i) / _rows[i].Get(i);
                _rows[j] = _rows[i] * delta + _rows[j];
            }
        }

        /// <summary>
        /// Vrací index vektoru, jež má na pozici i nenulový prvek,
        /// Tento hledá od 0-teho radku, ale preskakuje jiz pouzite radky.
        /// Pokud takový není vrací -1.
        /// </summary>
        /// <returns></returns>
        protected int FindNonZeroValueIndexAtI(int i, List<int> usedRows)
        {
            int index = 0;
            int from = 0;
            for (int j = from; j < _rowsCount; j++)
            {
                if (!Program.IsEqualInTolerance(_rows[j].Get(i), 0) &&
                    !IsInUsedRowsList(usedRows, j))
                    return index;
                index++;
            }
            return -1;
        }

        /// <summary>
        /// Vrací pravdu pokud se celociselna hodnota index nachazi v seznamu celociselnych hodnot UsedRows.
        /// Jinak false;
        /// </summary>
        protected bool IsInUsedRowsList(List<int> usedRows, int index)
        {
            foreach (int i in usedRows)
            {
                if (i == index)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Vymeni radky soustavy 3 rovnic o 4 sloupcich.
        /// </summary>
        /// <returns></returns>
        protected void SwapRows(int i, int j)
        {
            Equation temp = new Equation(_rows[i]);
            _rows[i] = _rows[j];
            _rows[j] = temp;
        }
    }
}
