﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    class SimpleEquationSystem:EquationSystem
    {
        public SimpleEquationSystem(Equation a, Equation b, Equation c)
            : base(a, b, c) 
        {
        }

        ////////////////////////////////////////////////////////////////////////////////////
        /////////////////  Reseni homogenni soustavy rovnic  ///////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// Z trojuhelnikove matice soustavy 3 rovnic o 3 neznamych vyextrahuje řešení.
        /// (navzajem kolme vektory)
        /// </summary>
        /// <param name="eqations"></param>
        /// <returns></returns>
        public OwnVectorParametrization Solve()
        {
            MakeTriangleSystem();
            OwnVectorParametrization retval = SolveTriangleMatrix();
            return retval;
        }

        /// <summary>
        /// Vraci reseni soustavy 3 rovnic o trech neznamich.
        /// </summary>
        /// <param name="matrix"></param>
        /// <returns></returns>
        private OwnVectorParametrization SolveTriangleMatrix() 
        {
            if (Program.IsEqualInTolerance(_rows[0].GetX(), 0))
            {
                if (Program.IsEqualInTolerance(_rows[1].GetY(), 0))
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))                    
                        return CaseNegativeAA();                    
                    else return CaseNegaticeAB();
                }
                else 
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))                    
                        return CaseNegativeAC();
                    else return CaseNegativeAD();
                }
            }
            else
            {
                if (Program.IsEqualInTolerance(_rows[1].GetY(), 0))
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CasePositiveAA();
                    else return CasePositiveAB();
                }
                else 
                {
                    if (Program.IsEqualInTolerance(_rows[2].GetZ(), 0))
                        return CasePositiveAC();
                    else return CasePositiveAD();
                }
            }
        }

        /// <summary>
        /// 0 0 0
        /// 0 0 0
        /// 0 0 0
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CaseNegativeAA() 
        {
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 0 0 0
        /// 0 0 0
        /// 0 0 1
        /// -----
        /// (p,q,0)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CaseNegaticeAB() 
        {
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 0);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 0 0 0
        /// 0 1 x
        /// 0 0 0
        /// -----
        /// (p,x,r)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CaseNegativeAC() 
        {
            double xx = -_rows[1].GetZ() / _rows[1].GetY();
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 0, xx);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 0 0 0
        /// 0 1 0
        /// 0 0 1
        /// -----
        /// (p,0,0)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CaseNegativeAD()
        {
            ParametrizationRST x = new ParametrizationRST(0, 1, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 0, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 0);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 1 x x
        /// 0 0 0
        /// 0 0 0
        /// -----
        /// (x,q,r)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CasePositiveAA()
        {
            double x1 = -_rows[0].GetY() / _rows[0].GetX();
            double x2 = -_rows[0].GetZ() / _rows[0].GetX();
            ParametrizationRST x = new ParametrizationRST(0, 0, x1, x2);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 1 x 0
        /// 0 0 0
        /// 0 0 1
        /// -----
        /// (x,q,0)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CasePositiveAB()
        {
            double xx = -_rows[0].GetY() / _rows[0].GetX();
            ParametrizationRST x = new ParametrizationRST(0, 0, xx, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 1, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 0);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 1 0 x
        /// 0 1 x
        /// 0 0 0
        /// -----
        /// (x,x,r)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CasePositiveAC()
        {
            double x1 = -_rows[1].GetZ() / _rows[1].GetY();
            double x2 = -_rows[0].GetZ() / _rows[0].GetX();
            ParametrizationRST x = new ParametrizationRST(0, 0, 0, x2);
            ParametrizationRST y = new ParametrizationRST(0, 0, 0, x1);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 1);
            return new OwnVectorParametrization(x,y,z);
        }

        /// <summary>
        /// 1 0 0
        /// 0 1 0
        /// 0 0 1
        /// -----
        /// (0,0,0)
        /// </summary>
        /// <returns></returns>
        private OwnVectorParametrization CasePositiveAD()
        {
            ParametrizationRST x = new ParametrizationRST(0, 0, 0, 0);
            ParametrizationRST y = new ParametrizationRST(0, 0, 0, 0);
            ParametrizationRST z = new ParametrizationRST(0, 0, 0, 0);
            return new OwnVectorParametrization(x,y,z);
        }
    }
}
