﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GMath;

namespace GMath
{
    class ComplexQuadraticEquation:IComplexEquation
    {
        private Complex _a;
        private Complex _b;
        private Complex _c;

        /// <summary>
        /// Používat pouze metody pro vyřešení a substituci, jiné metoda tato třída nepřepisuje.
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <param name="c"></param>
        public ComplexQuadraticEquation(Complex a, Complex b, Complex c) 
        {
            _a = a;
            _b = b;
            _c = c;
        }

        public override Complex Substitute(Complex x)
        {
            return _a * x * x + _b * x + _c;
        }
        public override double Substitute(double x)
        {
            return (_a * x * x + _b * x + _c)._z;
        }

        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////

        public override List<List<Complex>> SolveGetAllSetsOfRoots()
        {
            if (_a._z == 0)
                return new ComplexLinearEquation(_b, _c).SolveGetAllSetsOfRoots();

            Complex discriminant = (_b * _b) - (4 * _a * _c);
            List<List<Complex>> linearSolutions = new ComplexLinearEquation(_b, _c).SolveGetAllSetsOfRoots();
            List<List<Complex>> alternativeSolutions = CheckForSpecialCases();
            List<Complex> roots1 = Solve1();

            List<List<Complex>> retval = Append(linearSolutions, alternativeSolutions);
            retval.Add(roots1);
            return retval;
        }

        protected List<List<Complex>> CheckForSpecialCases()
        {
            List<List<Complex>> alternativeSolutions = new List<List<Complex>>();
            if (_c._z == 0)
            {
                if (_b._z == 0)
                {
                    alternativeSolutions.Add(new List<Complex>() { new Complex(0), new Complex(0) });
                    return alternativeSolutions;
                }
                alternativeSolutions = new ComplexLinearEquation(_a, _b).SolveGetAllSetsOfRoots();
                alternativeSolutions[0].Add(new Complex(0));
                return alternativeSolutions;
            }
            return alternativeSolutions;
        }

        protected List<Complex> Solve1()
        {
            Complex discriminant = (_b * _b) - (4 * _a * _c);
            Complex x1 = (-_b - Complex.SecondSqrt1728(discriminant)[0]) / (2 * _a);
            Complex x2 = (-_b + Complex.SecondSqrt1728(discriminant)[0]) / (2 * _a);
            return new List<Complex>() { x1, x2 };
        }

        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////

        protected override List<Complex> GetListOfKoeficientsComplex()
        {
            return new List<Complex>() { new Complex(_a), new Complex(_b), new Complex(_c) };
        }
    }
}
