﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    public class IComplexEquation:SuperEquation
    {

        protected virtual IComplexEquation MakeComplex(List<Complex> koeficients) 
        {
            switch (koeficients.Count)
            {
                case 1: return new ComplexLinearEquation(new Complex(0), koeficients[0]); 
                case 2: return new ComplexLinearEquation(koeficients[0], koeficients[1]);
                case 3: return new ComplexQuadraticEquation(koeficients[0], koeficients[1], koeficients[2]); 
            }
            throw new Exception("Snažíte se vytvořit polynom neznámého stupně.");
        }

        protected virtual List<Complex> GetListOfKoeficientsComplex()
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }

        protected override List<Complex> GetTheRootsFromSetOfRoots(List<List<Complex>> setsOfRoots)
        {
            try
            {
                if (setsOfRoots.Count == 0)
                    return new List<Complex>();

                List<double> testValues = new List<double>() { -100, -10, -1, 0, 1, 10, 100 };
                double minWorseSubstitutionDifference = double.MaxValue;
                List<Complex> koeficients = GetListOfKoeficientsComplex();
                int koefsCount = koeficients.Count;
                int indexOfRoots = 0;
                for (int i = 0; i < koefsCount; i++)
                {
                    if (koeficients[i]._z == 0)
                        continue;
                    IComplexEquation normalizedEquation = Normalize(this, koeficients[i]);
                    double localWorseSubstitutionDifference;
                    for (int j = 0; j < setsOfRoots.Count; j++)
                    {
                        localWorseSubstitutionDifference = GetWorseSubstitutionDifference(normalizedEquation, setsOfRoots[j], testValues);
                        if (localWorseSubstitutionDifference < minWorseSubstitutionDifference)
                        {
                            minWorseSubstitutionDifference = localWorseSubstitutionDifference;
                            indexOfRoots = j;
                        }
                    }
                }
                return setsOfRoots[indexOfRoots];
            }
            catch (Exception)
            {
                return new List<Complex>();
            }
        }
        private double GetWorseSubstitutionDifference(IComplexEquation originalEquationNormalized, List<Complex> roots, List<double> testValues)
        {
            double worseSubstitutionDifference = 0;
            double rootsSubstitutuon;
            double originalEquationSubstitution;
            foreach (double x in testValues)
            {
                originalEquationSubstitution = Math.Abs(originalEquationNormalized.Substitute(x));
                rootsSubstitutuon = Math.Abs(Substitute(roots, x));
                worseSubstitutionDifference = Math.Max(worseSubstitutionDifference,
                    Math.Abs(originalEquationSubstitution - rootsSubstitutuon));
            }
            return worseSubstitutionDifference;
        }
        private double Substitute(List<Complex> roots, double x)
        {
            if (roots.Count == 0)
                return 0;
            Complex retval = new Complex(1);
            foreach (Complex root in roots)
            {
                retval = retval * (x - root);
            }
            return retval._z;
        }

        protected IComplexEquation NormalizeComplex(IComplexEquation equation)
        {
            List<Complex> koeficients = GetListOfKoeficientsComplex();
            List<double> zKoeficients = new List<double>();
            foreach (Complex c in koeficients)
                zKoeficients.Add(c._z);
            double d = Program.GetNormalizingKoeficient(zKoeficients);
            List<Complex> newKoeficients = new List<Complex>();
            foreach (Complex koeficient in koeficients)
                newKoeficients.Add(d * koeficient);
            return MakeComplex(newKoeficients);
        }
        protected IComplexEquation Normalize(IComplexEquation equation, Complex koeficient)
        {
            List<Complex> koeficients = GetListOfKoeficientsComplex();
            List<Complex> newKoeficients = new List<Complex>();
            foreach (Complex k in koeficients)
                newKoeficients.Add(k / koeficient);
            return MakeComplex(newKoeficients);
        }
    }
}
