﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    public class LinearEquation:SuperEquation 
    {
        private double _a;
        private double _b;

        public LinearEquation(double a, double b) 
        {
            _a = a;
            _b = b;
        }

        public double GetA() { return _a; }
        public double GetB() { return _b; }


        public override List<List<Complex>> SolveGetAllSetsOfRoots()
        {
            if (_a == 0)
            {
                return new List<List<Complex>>();
            }
            return new List<List<Complex>>() { new List<Complex>() { new Complex(-_b / _a, 0) } };
        }

        public override bool ContainsVariable()
        {
            return !Program.IsEqualInTolerance(_a, 0);
        }


        public static QuadraticEquation operator*(LinearEquation a, LinearEquation b)
        {
            return new QuadraticEquation(a._a * b._a, a._a * b._b + a._b * b._a, b._b * b._b);
        }

        public override Complex Substitute(Complex root)
        {
            return _a * root + _b;
        }
        public override double Substitute(double root)
        {
            return _a * root + _b;
        }

        public LinearEquation Derivate() 
        {
            return new LinearEquation(0, _a);
        }

        protected override List<double> GetListOfKoeficients()
        {
            return new List<double>() { _a, _b };
        }
    }
}
