﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;

namespace GMath
{
    public class QuadraticEquation:SuperEquation
    {
        private double _a;
        private double _b;
        private double _c;

        public QuadraticEquation(double a, double b, double c) 
        {
            _a = a;
            _b = b;
            _c = c;
        }

        public override bool ContainsVariable()
        {
            return !Program.IsEqualInTolerance(_a, 0) || !Program.IsEqualInTolerance(_b, 0);
        }

        public static QuadraticEquation operator-(QuadraticEquation a, QuadraticEquation b)
        {
            return new QuadraticEquation(
                a._a - b._a,
                a._b - b._b,
                a._c - b._c);
        }
        public static CubicEquation operator *(QuadraticEquation a, LinearEquation b)
        {
            return new CubicEquation(
                a._a * b.GetA(),
                a._b * b.GetA() + a._a * b.GetB(),
                a._c * b.GetA() + a._b * b.GetB(),
                a._c * b.GetB());
        }


        public double GetA() { return _a; }
        public double GetB() { return _b; }
        public double GetC() { return _c; }

        public override Complex Substitute(Complex x)
        {
            return _a * x * x + _b * x + _c;
        }
        public override double Substitute(double x)
        {
            return _a * x * x + _b * x + _c;
        }

        public LinearEquation Derivate() 
        {
            return new LinearEquation(2 * _a, _b);
        }

        protected override List<double> GetListOfKoeficients()
        {
            return new List<double>() { _a, _b, _c };
        }

        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////

        public override List<List<Complex>> SolveGetAllSetsOfRoots()
        {
            if (_a == 0)
                return new LinearEquation(_b, _c).SolveGetAllSetsOfRoots();

            Complex discriminant = new Complex((_b * _b) - (4 * _a * _c), 0);
            List<List<Complex>> linearSolutions = new LinearEquation(_b, _c).SolveGetAllSetsOfRoots();
            List<List<Complex>> alternativeSolutions = CheckForSpecialCases();
            List<Complex> roots1 = Solve1();

            List<List<Complex>> retval = Append(linearSolutions, alternativeSolutions);
            retval.Add(roots1);
            return retval;
        }

        protected List<List<Complex>> CheckForSpecialCases()
        {
            List<List<Complex>> alternativeSolutions = new List<List<Complex>>();
            if (_c == 0)
            {
                if (_b == 0)
                {
                    alternativeSolutions.Add(new List<Complex>() { new Complex(0), new Complex(0) });
                    return alternativeSolutions;
                }
                alternativeSolutions = new LinearEquation(_a, _b).SolveGetAllSetsOfRoots();
                alternativeSolutions[0].Add(new Complex(0));
                return alternativeSolutions;
            }
            return alternativeSolutions;
        }

        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////

        protected List<Complex> Solve1()
        {
            Complex discriminant = new Complex((_b * _b) - (4 * _a * _c), 0);
            Complex x1 = (-_b - Complex.SecondSqrt1728(discriminant)[0]) / (2 * _a);
            Complex x2 = (-_b + Complex.SecondSqrt1728(discriminant)[0]) / (2 * _a);
            return new List<Complex>() { x1, x2 };
        }
    }
}
