﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;
using System.Drawing;

namespace GMath
{
    public class SuperEquation
    {
        protected double _tolerance = 0.0000000000000001;
        protected int _iterations = 30;

        protected struct HornerDivisionPair
        {
            public SuperEquation divisionResult;
            public double reminder;

            public HornerDivisionPair(SuperEquation divisionResultArg, double reminderArg) 
            {
                divisionResult = divisionResultArg;
                reminder = reminderArg;
            }
        }

        protected struct RootSubstitionPair 
        {
            public double root;
            public double substitution;

            public RootSubstitionPair(double rootArg, double substitutionArg) 
            {
                root = rootArg;
                substitution = substitutionArg;
            }
        }


        public SuperEquation Make(List<double> koeficients) 
        {
            switch (koeficients.Count)
            {
                case 0: return new LinearEquation(0, 0);
                case 1: return new LinearEquation(0, koeficients[0]); 
                case 2: return new LinearEquation(koeficients[0], koeficients[1]);
                case 3: return new QuadraticEquation(koeficients[0], koeficients[1], koeficients[2]);
                case 4: return new CubicEquation(koeficients[0], koeficients[1], koeficients[2], koeficients[3]);
                case 5: return new QuarticEquation(koeficients[0], koeficients[1], koeficients[2], koeficients[3], koeficients[4]); 
            }
            throw new Exception("Snažíte se vytvořit polynom neznámého stupně.");
        }

        public virtual bool ContainsVariable() { return true; }


        public virtual List<List<Complex>> SolveGetAllSetsOfRoots()
        {
            throw new Exception("tato metoda musi byt prepsana v potomkovi.");
        }
        /// <summary>
        /// Normalizace nema smysl, vetsina rovnic se deli A. 
        /// </summary>
        /// <returns></returns>
        public virtual List<double> SolveGetAllSetsOfRootsReal()
        {
            List<double> realRoots = new List<double>();
            List<List<Complex>> complexRoots = SolveGetAllSetsOfRoots();
            foreach (List<Complex> list in complexRoots)
                foreach (Complex c in list)
                    realRoots.Add(c._real);
            return realRoots;
        }
        
        public virtual List<Complex> SolveComplex()
        {
            List<Complex> complexRoots = GetTheRootsFromSetOfRoots(SolveGetAllSetsOfRoots());
            return complexRoots;
        }
        public virtual List<double> Solve()
        {
            SuperEquation normalizedEquation = Normalize(this);
            List<List<Complex>> allRoots = normalizedEquation.SolveGetAllSetsOfRoots();
            List<Complex> complexRoots = normalizedEquation.GetTheRootsFromSetOfRoots(allRoots);
            List<double> realRoots = new List<double>();
            foreach (Complex c in complexRoots)
                if (Program.IsEqualInToleranceLow(c._i, 0))
                    realRoots.Add(c._real);
            return realRoots;
        }

        protected virtual List<Complex> GetTheRootsFromSetOfRoots(List<List<Complex>> setsOfRoots) 
        {
            try
            {
                if (setsOfRoots.Count == 0)
                    return new List<Complex>();

                List<double> testValues = new List<double>() { -100, -10, -1, 0, 1, 10, 100 };
                double minWorseSubstitutionDifference = double.MaxValue;
                List<double> koeficients = GetListOfKoeficients();
                int koefsCount = koeficients.Count;
                int indexOfRoots = 0;
                for (int i = 0; i < koefsCount; i++)
                {
                    if (koeficients[i] == 0)
                        continue;
                    SuperEquation normalizedEquation = Normalize(this, koeficients[i]);
                    double localWorseSubstitutionDifference;
                    for (int j = 0; j < setsOfRoots.Count; j++)
                    {
                        localWorseSubstitutionDifference = GetWorseSubstitutionDifference(normalizedEquation, setsOfRoots[j], testValues);
                        if (localWorseSubstitutionDifference < minWorseSubstitutionDifference)
                        {
                            minWorseSubstitutionDifference = localWorseSubstitutionDifference;
                            indexOfRoots = j;
                        }
                    }
                }
                return setsOfRoots[indexOfRoots];
            }
            catch (Exception) 
            {
                return new List<Complex>();
            }
        }
        private double GetWorseSubstitutionDifference(SuperEquation originalEquationNormalized, List<Complex> roots, List<double> testValues) 
        {
            double worseSubstitutionDifference = 0;
            double rootsSubstitutuon;
            double originalEquationSubstitution;
            foreach (double x in testValues) 
            {
                originalEquationSubstitution = Math.Abs(originalEquationNormalized.Substitute(x));
                rootsSubstitutuon = Math.Abs(Substitute(roots, x));
                worseSubstitutionDifference = Math.Max(worseSubstitutionDifference, 
                    Math.Abs(originalEquationSubstitution - rootsSubstitutuon));
            }
            return worseSubstitutionDifference;
        }
        private double Substitute(List<Complex> roots, double x)
        {
            if (roots.Count == 0)
                return 0;
            Complex retval = new Complex(1);
            foreach (Complex root in roots)
            {
                retval = retval * (x - root);
            }
            return retval._z;
        }
        
        protected SuperEquation Normalize(SuperEquation equation) 
        {
            List<double> koeficients = GetListOfKoeficients();
            double d = Program.GetNormalizingKoeficient(koeficients);
            List<double> newKoeficients = new List<double>();
            foreach (double koeficient in koeficients)
                newKoeficients.Add(d * koeficient);
            return Make(newKoeficients);
        }
        protected SuperEquation Normalize(SuperEquation equation, double koeficient) 
        {
            List<double> koeficients = GetListOfKoeficients();
            List<double> newKoeficients = new List<double>();
            foreach (double k in koeficients)
                newKoeficients.Add(k / koeficient);
            return Make(newKoeficients);
        }

        public virtual Complex Substitute(Complex x)
        {
            throw new NotImplementedException();
        }
        public virtual double Substitute(double root) 
        {
            throw new NotImplementedException();
        }
        public virtual List<double> Substitute(List<double> roots) 
        {
            List<double> retval = new List<double>();
            foreach (double x in roots)
                retval.Add(Substitute(x));
            return retval;
        }

        ////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////


        protected virtual List<double> GetListOfKoeficients() 
        {
            throw new Exception("Tato metoda musí být přepsána v potomcích.");
        }

        protected List<List<Complex>> Append(List<List<Complex>> a, List<List<Complex>> b) 
        {
            if (a == null && b == null)
                return new List<List<Complex>>();
            if (a == null)
                return b;
            if (b == null)
                return a;
            List<List<Complex>> result = new List<List<Complex>>();
            foreach (List<Complex> c in a)
                result.Add(c);
            foreach (List<Complex> c in b)
                result.Add(c);
            return result;
        }
    }
}
