﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GMath
{
    public class CenterParametrization
    {
        private ParametrizationRST _x;
        private ParametrizationRST _y;
        private ParametrizationRST _z;


        public CenterParametrization(double xConstant, double yConstant, double zConstant) 
        {
            _x = new ParametrizationRST(xConstant, 0, 0, 0);
            _y = new ParametrizationRST(yConstant, 0, 0, 0);
            _z = new ParametrizationRST(zConstant, 0, 0, 0);
        }
        public CenterParametrization(Vector c) 
        {
            _x = new ParametrizationRST(c.GetX(), 0, 0, 0);
            _y = new ParametrizationRST(c.GetY(), 0, 0, 0);
            _z = new ParametrizationRST(c.GetZ(), 0, 0, 0);
        }
        public CenterParametrization(CenterParametrization src) 
        {
            _x = new ParametrizationRST(src._x);
            _y = new ParametrizationRST(src._y);
            _z = new ParametrizationRST(src._z);        
        }
        public CenterParametrization() 
        {
            _x = new ParametrizationRST(0, 0, 0, 0);
            _y = new ParametrizationRST(0, 0, 0, 0);
            _z = new ParametrizationRST(0, 0, 0, 0);
        }
        public CenterParametrization(ParametrizationRST x, ParametrizationRST y, ParametrizationRST z)
        {
            _x = x;
            _y = y;
            _z = z;
        }

        public Vector GetReprezentant() 
        {
            return GetReprezentant(0, 0, 0);
        }
        private Vector GetReprezentant(double r, double s, double t)
        {
            double x = _x.GetConstant() + r * _x.GetR() + s * _x.GetS() + t * _x.GetT();
            double y = _y.GetConstant() + r * _y.GetR() + s * _y.GetS() + t * _y.GetT();
            double z = _z.GetConstant() + r * _z.GetR() + s * _z.GetS() + t * _z.GetT();
            return new Vector(x, y, z);
        }

        public ParametrizationRST GetX() { return _x; }
        public ParametrizationRST GetY() { return _y; }
        public ParametrizationRST GetZ() { return _z; }

        public static CenterParametrization operator *(CenterParametrization parametrization, Matrix matrix) 
        {
            Vector column1 = new Vector(matrix.Get(0), matrix.Get(4), matrix.Get(8), matrix.Get(12));
            Vector column2 = new Vector(matrix.Get(1), matrix.Get(5), matrix.Get(9), matrix.Get(13));
            Vector column3 = new Vector(matrix.Get(2), matrix.Get(6), matrix.Get(10), matrix.Get(14));
            //Vector column4 = new Vector(matrix.Get(3), matrix.Get(7), matrix.Get(11), matrix.Get(15));
            return new CenterParametrization
                (parametrization * column1, parametrization * column2, parametrization * column3);
        }
        private CenterParametrization Shit(Matrix matrix, CenterParametrization parametrization)
        {
            Vector row1 = new Vector(matrix.Get(0), matrix.Get(1), matrix.Get(2), matrix.Get(3));
            Vector row2 = new Vector(matrix.Get(4), matrix.Get(5), matrix.Get(6), matrix.Get(7));
            Vector row3 = new Vector(matrix.Get(8), matrix.Get(9), matrix.Get(10), matrix.Get(11));
            //Vector column4 = new Vector(matrix.Get(3), matrix.Get(7), matrix.Get(11), matrix.Get(15));
            return new CenterParametrization
                (parametrization * row1, parametrization * row2, parametrization * row3);
        }
        public static ParametrizationRST operator *(CenterParametrization parametrization, Vector vector) 
        {
            ParametrizationRST a = parametrization._x * vector.GetX();
            ParametrizationRST b = parametrization._y * vector.GetY();
            ParametrizationRST c = parametrization._z * vector.GetZ();
            return a + b + c;
        }

        public static bool AreEqualInTolerance(CenterParametrization a, CenterParametrization b) 
        {
            return 
                a._x == b._x && 
                a._y == b._y && 
                a._z == b._z;
        }

        public int GetParametersCount()
        {
            int xCount = 0;
            if (_x.GetR() != 0 || _y.GetR() != 0 || _z.GetR() != 0)
                xCount = 1;

            int yCount = 0;
            if (_x.GetS() != 0 || _y.GetS() != 0 || _z.GetS() != 0)
                yCount = 1;

            int zCount = 0;
            if (_x.GetT() != 0 || _y.GetT() != 0 || _z.GetT() != 0)
                zCount = 1;

            return xCount + yCount + zCount;
        }

        public bool IsZero()
        {
            return _x.IsZero() && _y.IsZero() && _z.IsZero();
        }

        public override string ToString()
        {
            string retval = "";
            retval += "x:=" + _x.ToString() + "\n";
            retval += "y:=" + _y.ToString() + "\n";
            retval += "z:=" + _z.ToString() + "\n";
            return retval;
        }
    }
}
