﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GMath
{
    public class OwnVectorParametrization
    {
        private ParametrizationRST _x; // parametrizace slozky x.
        private ParametrizationRST _y; // parametrizace slozky y.
        private ParametrizationRST _z; // parametrizace slozky z.

        public OwnVectorParametrization(OwnVectorParametrization src) 
        {
            _x = new ParametrizationRST(src._x);
            _y = new ParametrizationRST(src._y);
            _z = new ParametrizationRST(src._z);
        }
        public OwnVectorParametrization(ParametrizationRST x, ParametrizationRST y, ParametrizationRST z) 
        {
            _x = x;
            _y = y;
            _z = z;
        }

        public Vector GetReprezentant(double r, double s, double t)
        {
            double x = _x.GetConstant() + r * _x.GetR() + s * _x.GetS() + t * _x.GetT();
            double y = _y.GetConstant() + r * _y.GetR() + s * _y.GetS() + t * _y.GetT();
            double z = _z.GetConstant() + r * _z.GetR() + s * _z.GetS() + t * _z.GetT();
            return new Vector(x, y, z);
        }

        public ParametrizationRST GetX() { return _x; }
        public ParametrizationRST GetY() { return _y; }
        public ParametrizationRST GetZ() { return _z; }

        public bool ContainsRParameter() 
        {
            return _x.ContainsRParameter() || _y.ContainsRParameter() || _z.ContainsRParameter();
        }
        public bool ContainsSParameter()
        {
            return _x.ContainsSParameter() || _y.ContainsSParameter() || _z.ContainsSParameter();
        }
        public bool ContainsTParameter()
        {
            return _x.ContainsTParameter() || _y.ContainsTParameter() || _z.ContainsTParameter();
        }
        public int GetParametersCount()
        {
            int xCount = 0;
            if (_x.GetR() != 0 || _y.GetR() != 0 || _z.GetR() != 0)
                xCount = 1;

            int yCount = 0;
            if (_x.GetS() != 0 || _y.GetS() != 0 || _z.GetS() != 0)
                yCount = 1;

            int zCount = 0;
            if (_x.GetT() != 0 || _y.GetT() != 0 || _z.GetT() != 0)
                zCount = 1;

            return xCount + yCount + zCount;
        }

        public List<double> GetRList() 
        {
            return new List<double>() { _x.GetR(), _y.GetR(), _z.GetR() };
        }
        public List<double> GetSList()
        {
            return new List<double>(){_x.GetS(), _y.GetS(), _z.GetS()};
        }
        public List<double> GetTList()
        {
            return new List<double>(){_x.GetT(), _y.GetT(), _z.GetT()};
        }
        public List<double> GetConstantsList()
        {
            return new List<double>(){_x.GetConstant(), _y.GetConstant(), _z.GetConstant()};
        }

        public void SubstituteForR(ParametrizationRST r) 
        {
            this._x.SubstituteForR(r);
            this._y.SubstituteForR(r);
            this._z.SubstituteForR(r);
        }
        public void SubstituteForS(ParametrizationRST s)
        {
            this._x.SubstituteForS(s);
            this._y.SubstituteForS(s);
            this._z.SubstituteForS(s);
        }
        public void SubstituteForT(ParametrizationRST t)
        {
            this._x.SubstituteForT(t);
            this._y.SubstituteForT(t);
            this._z.SubstituteForT(t);
        }

        public void SubstituteForR(double r)
        {
            this._x.ChooseForR(r);
            this._y.ChooseForR(r);
            this._z.ChooseForR(r);
        }
        public void SubstituteForS(double s)
        {
            this._x.ChooseForS(s);
            this._y.ChooseForS(s);
            this._z.ChooseForS(s);
        }
        public void SubstituteForT(double t)
        {
            this._x.ChooseForT(t);
            this._y.ChooseForT(t);
            this._z.ChooseForT(t);
        }

        public static OwnVectorParametrization operator +(OwnVectorParametrization a, OwnVectorParametrization b)
        {
            return new OwnVectorParametrization(a._x + b._x, a._y + b._y, a._z + b._z);
        }

        public static bool AreEqual(OwnVectorParametrization a, OwnVectorParametrization b)
        {
            return a._x == b._x && a._y == b._y && a._z == b._z;
        }
        public override string ToString()
        {
            string retval = "";
            retval += "x:=" + _x.ToString() + "\n";
            retval += "y:=" + _y.ToString() + "\n";
            retval += "z:=" + _z.ToString() + "\n";
            return retval;
        }
        public bool IsZero()
        {
            return _x.IsZero() && _y.IsZero() && _z.IsZero();
        }
    }
}
