﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GMath;

namespace Geometry.GraphicObjects
{
    public class ClippingCube
    {
        private double _size = Scene.GetClippingCubeWidth();

        private List<Vector> _defaultVertexes;
        private List<Vector> _actualVertexes;

        public ClippingCube() 
        {
            double size = _size;
            _defaultVertexes = new List<Vector>() 
            {
                new Vector(-size,-size,-size),
                new Vector(-size,-size, size),
                new Vector(-size, size,-size),
                new Vector(-size, size, size),
                new Vector( size,-size,-size),
                new Vector( size,-size, size),
                new Vector( size, size,-size),
                new Vector( size, size, size)
            };
            _actualVertexes = _defaultVertexes;
        }

        /// <summary>
        /// Zavolat pro přebuildování displayListu objektu.
        /// </summary>
        public void Actualize(TransformationMatrix transformationMatrix) 
        {
            _actualVertexes = GetVertexes(transformationMatrix);
        }

        private List<Vector> GetVertexes(TransformationMatrix transformationMatrix) 
        {
            List<Vector> vertexes = new List<Vector>();
            for (int i = 0; i < _defaultVertexes.Count; i++)
            {
                vertexes.Add(_defaultVertexes[i] * transformationMatrix);
            }
            return vertexes;
        }

        /// <summary>
        ///Zjistí, zda se velmi neposunula pozice ořezávací krychle oproti minulé pozici, kdy byl object buildován.
        /// </summary>
        public bool IsRebuildNeeded(TransformationMatrix objectsNewTransformation)
        {
            List<Vector> currentVertexes = GetVertexes(objectsNewTransformation);

            double left1 = GetLeftMostVertex(_actualVertexes).GetX();
            double left2 = GetLeftMostVertex(currentVertexes).GetX();
            double right1 = GetRightMostVertex(_actualVertexes).GetX();
            double right2 = GetRightMostVertex(currentVertexes).GetX();
            double bottom1 = GetBottomtMostVertex(_actualVertexes).GetY();
            double bottom2 = GetBottomtMostVertex(currentVertexes).GetY();
            double top1 = GetTopMostVertex(_actualVertexes).GetY();
            double top2 = GetTopMostVertex(currentVertexes).GetY();
            double back1 = GetBackMostVertex(_actualVertexes).GetZ();
            double back2 = GetBackMostVertex(currentVertexes).GetZ();
            double front1 = GetFrontMostVertex(_actualVertexes).GetZ();
            double front2 = GetFrontMostVertex(currentVertexes).GetZ();

            double tolerance = _size / 3;
            return
                Distance(left1, left2) > tolerance ||
                Distance(right1, right2) > tolerance ||
                Distance(back1, back2) > tolerance ||
                Distance(front1, front2) > tolerance ||
                Distance(bottom1, bottom2) > tolerance ||
                Distance(back1, back2) > tolerance;
        }

        private double Distance(double a, double b) 
        {
            return Math.Abs(a - b);
        }

        private Vector GetLeftMostVertex(List<Vector> vertexes) 
        {
            int index = 0;
            double coord = vertexes[0].GetX();
            for (int i = 1; i < vertexes.Count; i++) 
            {
                if (vertexes[i].GetX() < coord)
                {
                    coord = vertexes[i].GetX();
                    index = i;
                }
            }
            return vertexes[index];
        }
        private Vector GetRightMostVertex(List<Vector> vertexes)
        {
            int index = 0;
            double coord = vertexes[0].GetX();
            for (int i = 1; i < vertexes.Count; i++)
            {
                if (vertexes[i].GetX() > coord)
                {
                    coord = vertexes[i].GetX();
                    index = i;
                }
            }
            return vertexes[index];
        }
        private Vector GetBottomtMostVertex(List<Vector> vertexes)
        {
            int index = 0;
            double coord = vertexes[0].GetY();
            for (int i = 1; i < vertexes.Count; i++)
            {
                if (vertexes[i].GetY() < coord)
                {
                    coord = vertexes[i].GetY();
                    index = i;
                }
            }
            return vertexes[index];
        }
        private Vector GetTopMostVertex(List<Vector> vertexes)
        {
            int index = 0;
            double coord = vertexes[0].GetY();
            for (int i = 1; i < vertexes.Count; i++)
            {
                if (vertexes[i].GetY() > coord)
                {
                    coord = vertexes[i].GetY();
                    index = i;
                }
            }
            return vertexes[index];
        }
        private Vector GetBackMostVertex(List<Vector> vertexes)
        {
            int index = 0;
            double coord = vertexes[0].GetZ();
            for (int i = 1; i < vertexes.Count; i++)
            {
                if (vertexes[i].GetZ() < coord)
                {
                    coord = vertexes[i].GetZ();
                    index = i;
                }
            }
            return vertexes[index];
        }
        private Vector GetFrontMostVertex(List<Vector> vertexes)
        {
            int index = 0;
            double coord = vertexes[0].GetZ();
            for (int i = 1; i < vertexes.Count; i++)
            {
                if (vertexes[i].GetZ() > coord)
                {
                    coord = vertexes[i].GetZ();
                    index = i;
                }
            }
            return vertexes[index];
        }
    }
}
