﻿using Tao.OpenGl;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.ConicSections
{
    /// <summary>
    /// x2/a2  = 0;
    /// </summary>
    public class DoubleLine : SimpleRuledConic
    {
        private double _a;

        public static string canonicFormatString = "E[x-!m;!a]L[ = 0]";
        public static string parametricXFormatString = "L[x = !m]";
        public static string parametricYFormatString = "L[y = v]";

        public static string _wordDescriptionFirstCase = "Dvojnásobná přímka";
        public static string _wordDescriptionSecondCase = "Dvojnásobné přímky";
        public static string _wordDescriptionFourthCase = "Dvojnásobnou přímku";

        public DoubleLine(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        {
            _a = parametricEquation.GetA();
        }

        /// x = m.
        public double ComputeXCoordinateAt(double v)
        {
            return 0;
        }

        /// y = n + v.
        public double ComputeYCoordinateAt(double v)
        {
            return v;
        }


        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            double uMin = GetClippingBottom(scene);
            double uMax = GetClippingTop(scene);

            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            BuildAsLineStrip(uMin, uMax);
            Gl.glEndList();
        }

        public override void BuildAsIntersection(Scene scene, RelativeQuality relativeQuality)
        {
            double uMin = GetClippingBottom(scene);
            double uMax = GetClippingTop(scene);

            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            BuildAsCylinder(uMin, uMax);
            Gl.glEndList();
        }

        protected void BuildAsCylinder(double uMin, double uMax)
        {
            Vector a, b;
            a = new Vector(ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsCylinder(a, b);
        }
        protected void BuildAsLineStrip(double uMin, double uMax)
        {
            Vector a, b;
            a = new Vector(ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsLineStrip(a, b);
        }



        protected override ConicSection CallConstructor(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new DoubleLine(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override ConicMatrix GetCanonicMatrix()
        {
            double A = 1 / (_parametricEquation.GetA() * _parametricEquation.GetA());
            ConicEquation newCanonicEquation = new ConicEquation(A, 0, 0, 0, 0, 0);
            return new ConicMatrix(newCanonicEquation);
        }
    }
}
