﻿using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.ConicSections
{
    public class EmptySet : ConicSection
    {

        public static string canonicFormatString = "L[" + U.EmptySet + "]";
        public static string parametricXFormatString = "L[x = " + U.EmptySet + "]";
        public static string parametricYFormatString = "L[y = " + U.EmptySet + "]";

        public static string _wordDescriptionFirstCase = "Prázdná množina";
        public static string _wordDescriptionSecondCase = "Prázdné množiny";
        public static string _wordDescriptionFourthCase = "Prázdnou množinu";

        public EmptySet(GColor color, string name, int ID)
            : base(new ConicEquation(1,0,0,0,0,1), new ConicParametricEquation(0, 0, 0),
            new TransformationMatrix(4), color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        { }

        public EmptySet(ConicEquation commonEquation, GColor color, string name, int ID)
            : base(commonEquation, new ConicParametricEquation(0, 0, 0),
            new TransformationMatrix(4), color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        { }

        public EmptySet(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name, _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString,
            parametricYFormatString, ID)
        {
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
        }


        protected override ConicSection CallConstructor(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new EmptySet(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override ConicMatrix GetCanonicMatrix()
        {
            ConicEquation newCanonicEquation = new ConicEquation(0, 0, 0, 0, 0, -1);
            return new ConicMatrix(newCanonicEquation);
        }
    }
}
