﻿using System;
using Tao.OpenGl;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.ConicSections
{
    public class ImaginarCrossingLines:ConicSection
    {
        private double _a;
        private double _b;
        private double _baseQuality = 30;

        public static string canonicFormatString = "E[x-!m;!a]L[ + ]E[y-!n;!b]L[ = 0]";
        public static string parametricXFormatString = "L[x = !m]";
        public static string parametricYFormatString = "L[y = !n]";

        public static string _wordDescriptionFirstCase = "Imaginární různoběžné přímky";
        public static string _wordDescriptionSecondCase = "Imaginárních různoběžných přímek";
        public static string _wordDescriptionFourthCase = "Imaginární různoběžné přímky";


        public ImaginarCrossingLines(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        {
            _a = parametricEquation.GetA();
            _b = parametricEquation.GetB();
        }
        /// x = m.
        public double ComputeXCoordinateAt(double v)
        {
            return 0;
        }

        /// y = n.
        public double ComputeYCoordinateAt(double v)
        {
            return 0;
        }

        /// <summary>
        /// Dosadi do parametricke rovnice v a vraci souradnice bodu.
        /// </summary>
        /// <param name="u"></param>
        /// <param name="v"></param>
        /// <returns></returns>
        public Vector ComputeCoordinatesAt(double v) 
        {
            double x = ComputeXCoordinateAt(v);
            double y = ComputeYCoordinateAt(v);

            return new Vector(x, y, 0);
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);

            double step = Math.PI / (_baseQuality * (scene.GetBaseQuality() + relativeQuality));

            Gl.glDisable(Gl.GL_LIGHTING); 
            Gl.glBegin(Gl.GL_LINE_STRIP);
            Gl.glVertex3d(ComputeXCoordinateAt(0), ComputeYCoordinateAt(0),0);
            Gl.glEnd();
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEndList();
        }



        protected override ConicSection CallConstructor(ConicEquation commonEquation, ParametricEquation parametricEquation, 
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ImaginarCrossingLines(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override ConicMatrix GetCanonicMatrix()
        {
            double A = 1 / (_parametricEquation.GetA() * _parametricEquation.GetA());
            double B = 1 / (_parametricEquation.GetB() * _parametricEquation.GetB());
            ConicEquation newCanonicEquation = new ConicEquation(A, B, 0, 0, 0, 0);
            return new ConicMatrix(newCanonicEquation);
        }
    
    }
}
