﻿using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.ConicSections
{
    /// <summary>
    /// x2/a2 + y2/b2 = 1;
    /// </summary>
    public class ImaginarElipse : ConicSection
    {

        private double _a;
        private double _b;

        public static string canonicFormatString = "E[x-!m;!a]L[ + ]E[y-!n;!b]L[ = -1]";
        public static string parametricXFormatString = "L[x = " + U.EmptySet + "]";
        public static string parametricYFormatString = "L[y = " + U.EmptySet + "]";

        public static string _wordDescriptionFirstCase = "Imaginární elipsa";
        public static string _wordDescriptionSecondCase = "Imaginární elipsy";
        public static string _wordDescriptionFourthCase = "Imaginární elipsu";

        public ImaginarElipse(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        {
            _a = parametricEquation.GetA();
            _b = parametricEquation.GetB();
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
        }


        protected override ConicSection CallConstructor(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ImaginarElipse(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override ConicMatrix GetCanonicMatrix()
        {
            double A = -1 / (_parametricEquation.GetA() * _parametricEquation.GetA());
            double B = -1 / (_parametricEquation.GetB() * _parametricEquation.GetB());
            ConicEquation newCanonicEquation = new ConicEquation(A, B, 0, 0, 0, -1);
            return new ConicMatrix(newCanonicEquation);
        }
    }
}
