﻿using Tao.OpenGl;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.ConicSections
{
    public class ParallelLines : SimpleRuledConic
    {
        private double _a;

        public static string canonicFormatString = "E[x-!m;!a]L[= 1]";
        public static string parametricXFormatString = "L[x = !m " + U.PM + "!a]";
        public static string parametricYFormatString = "L[y = u]";

        public static string _wordDescriptionFirstCase = "Rovnoběžné přímky";
        public static string _wordDescriptionSecondCase = "Rovnoběžných přímek";
        public static string _wordDescriptionFourthCase = "Rovnoběžné přímky";

        public ParallelLines(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString, ID)
        {
            _a = parametricEquation.GetA();
        }

        /// x = m +- a.
        public double ComputeXCoordinateAt(double v)
        {
            return _a;
        }

        /// y = v 
        public double ComputeYCoordinateAt(double v)
        {
            return v;
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            double ctop = GetClippingTop(scene);
            double cbottom = GetClippingBottom(scene);

            double uMin = cbottom;
            double uMax = ctop;

            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            BuildAsLineStrip(uMin, uMax);
            Gl.glEndList();
        }

        public override void BuildAsIntersection(Scene scene, RelativeQuality relativeQuality)
        {
            double ctop = GetClippingTop(scene);
            double cbottom = GetClippingBottom(scene);

            double uMin = cbottom;
            double uMax = ctop;

            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            BuildAsCylinder(uMin, uMax);
            Gl.glEndList();
        }

        protected void BuildAsCylinder(double uMin, double uMax)
        {
            Vector a, b;
            a = new Vector(ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsCylinder(a, b);
            a = new Vector(-ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(-ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsCylinder(a, b);            
        }
        protected void BuildAsLineStrip(double uMin, double uMax)
        {
            Vector a, b;
            a = new Vector(ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsLineStrip(a, b);
            a = new Vector(-ComputeXCoordinateAt(uMin), ComputeYCoordinateAt(uMin), 0);
            b = new Vector(-ComputeXCoordinateAt(uMax), ComputeYCoordinateAt(uMax), 0);
            Planar.Line.BuildLineAsLineStrip(a, b); 
        }


        protected override ConicSection CallConstructor(ConicEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ParallelLines(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override ConicMatrix GetCanonicMatrix()
        {
            double A = 1 / (_parametricEquation.GetA() * _parametricEquation.GetA());
            ConicEquation newCanonicEquation = new ConicEquation(A, 0, 0, 0, 0, -1);
            return new ConicMatrix(newCanonicEquation);
        }
    }
}
