﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GMath;
using Tao.OpenGl;

namespace Geometry.GraphicObjects
{
    /// <summary>
    /// Válec s poloměrem _radius, středem základny v bodě (0,0,0) směřující ve směru osy x. 
    /// </summary>
    public class CurveCylinder
    {
        private int _angles = Configuration.CurveCylinderAngles;
        private double _radius = Configuration.CurveCylinderRadius;
        private Vector _baseCenter = new Vector(0,0,0);
        private Vector _heightVector = new Vector(1,0,0);
        private List<Vector> _basePoints;
        private List<Vector> _baseNormals;
        protected CurveSphere _curveSphere;


        public CurveCylinder()
        {
            _basePoints = new List<Vector>();
            _baseNormals = new List<Vector>();
            double x = 0;
            double y = 0;
            double z = 0;
            double angleDelta = Math.PI * 2 / _angles;
            for (int i = 0; i < _angles; i++) 
            {
                x = 0;
                y = Math.Cos(angleDelta * i);
                z = Math.Sin(angleDelta * i);
                _baseNormals.Add(new Vector(x,y,z));
                _basePoints.Add(_radius * new Vector(x, y, z));
            }
            _curveSphere = new CurveSphere();
        }


        public void MakeCylinder(Vector baseCenter, Vector b) 
        {
            _heightVector = Vector.Size(b - baseCenter) * new Vector(1, 0, 0);
            _baseCenter = baseCenter;

            Gl.glBegin(Gl.GL_TRIANGLE_STRIP);
            Vector temp;
            Vector normal;
            int i = 0;
            foreach (Vector v in _basePoints) 
            {
                temp = v;
                normal = -1*_baseNormals[i].NornVector();
                Gl.glNormal3d(normal.GetX(), normal.GetY(), normal.GetZ());
                Gl.glVertex3d(temp.GetX(), temp.GetY(), temp.GetZ());
                temp = v + _heightVector;                
                Gl.glVertex3d(temp.GetX(), temp.GetY(), temp.GetZ());
                i++;
            }
            temp = _basePoints[0];
            normal = -1*_baseNormals[0].NornVector();
            Gl.glNormal3d(normal.GetX(), normal.GetY(), normal.GetZ());
            Gl.glVertex3d(temp.GetX(), temp.GetY(), temp.GetZ());
            temp = temp + _heightVector;
            Gl.glVertex3d(temp.GetX(), temp.GetY(), temp.GetZ());

            Gl.glEnd();
        }

        public void MakeCylinderComplete(Vector a, Vector b) 
        {
            Vector direction = b - a;
            Vector x = new Vector(1, 0, 0);
            Vector axe = Vector.CrossProduct(direction, x).NornVector();
            double angle = Vector.Angle(direction, x) * 180 / Math.PI;
            TransformationMatrix rotation = TransformationMatrix.GetRotationMatrixAroundAAxe(axe, angle);
            Gl.glPushMatrix();
            Gl.glTranslated(a.GetX(), a.GetY(), a.GetZ());
            _curveSphere.MakeDisplayList();
            Gl.glMultMatrixd(rotation.ToDoubleArray());
            MakeCylinder(a, b);
            Gl.glPopMatrix();
        }
    }
}
