﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GMath;
using Tao.OpenGl;

namespace Geometry.GraphicObjects
{
    public class CurveSphere
    {
        private double _radius = Configuration.CurveCylinderRadius;
        private int _angles = Configuration.CurveCylinderSphereAngles;
        private List<Vector> _points;
        private List<Vector> _normals;

        public CurveSphere() 
        {
            _points = new List<Vector>();
            _normals = new List<Vector>();
            ComputePoints();
        }
        public CurveSphere(double radius)
        {
            _radius = radius;
            _points = new List<Vector>();
            _normals = new List<Vector>();
            ComputePoints();
        }

        private void ComputePoints()
        {
            double vStep = (Math.PI / _angles);
            double uStep = vStep;

            for (double v = -Math.PI / 2; v < Math.PI; v += vStep)
            {
                if (v > Math.PI / 2 - vStep)
                    v = Math.PI / 2 - vStep;
                for (double u = 0; u < Math.PI * 4 + uStep; u += uStep)
                {
                    if (u > Math.PI * 2)
                        u = Math.PI * 2;
                    SetVextexAndHisNormal(u, v);
                    SetVextexAndHisNormal(u, v + vStep);
                    if (u == Math.PI * 2)
                        break;
                }
                if (v == Math.PI / 2 - vStep)
                    break;
            }
        }

        private double ComputeXCoordinateAt(double u, double v)
        {
            return Math.Cos(v) * Math.Cos(u);
        }
        private double ComputeYCoordinateAt(double u, double v)
        {
            return Math.Cos(v) * Math.Sin(u);
        }
        private double ComputeZCoordinateAt(double u, double v)
        {
            return Math.Sin(v);
        }

        private Vector ComputeCoordinatesAt(double u, double v)
        {
            double x = ComputeXCoordinateAt(u, v);
            double y = ComputeYCoordinateAt(u, v);
            double z = ComputeZCoordinateAt(u, v);

            return new Vector(x, y, z);
        }

        private void SetVextexAndHisNormal(double u, double v)
        {
            Vector normal = ComputeCoordinatesAt(u, v);
            _normals.Add(normal);
            Vector coordinate = normal * _radius;
            _points.Add(coordinate);
        }

        public void MakeDisplayList()
        {
            Gl.glBegin(Gl.GL_TRIANGLE_STRIP);
            for (int i = 0; i < _points.Count; i++) 
            {
                Gl.glNormal3d(-_normals[i].GetX(), -_normals[i].GetY(), -_normals[i].GetZ());
                Gl.glVertex3d(_points[i].GetX(), _points[i].GetY(), _points[i].GetZ());
            }
            Gl.glEnd();
        }
    }
}
