﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Tao.OpenGl;
using Geometry.BaseClasses;

namespace Geometry.GraphicObjects
{
    public class GraphicObject
    {
        private bool _isInScene;
        private bool _deleted = false;
        protected RelativeQuality _relativeQuality;

        protected int _ID;
        protected GColor _color;
        protected string _name;
        protected int _displayListIdentificator = -1;
        protected bool _visible = true;
        private string _wordDescriptionFirstCase;
        private string _wordDescriptionSecondCase;
        private string _wordDescriptionFourthCase;

        public GraphicObject() 
        {
            _relativeQuality = Scene.QualityRelativeNormal;
        }

        /// <summary>
        /// DisplayListy se maží ručně.
        /// Každý objekt je mazán, jen pokud se mění úroveň MSAA nebo kvalita, nebo se ukončuje program nebo AddIntersection.
        /// Dočasné průniky se smažou jakmile jsou nahrazeny dalším průnikem. Tyto průniky vznikají jen při rotaci.
        /// Prvním otočením se smaže i displayList průniku, který je uložen mezi grafickými objetky, protože se ale po uzavření dialogu rotace, 
        /// kvůli možným přidaným průnikům během rotace aktializují původní objekty průniku, tak to nevadí.
        /// </summary>
        internal virtual void DeleteDisplayList()
        {
            if (_displayListIdentificator != -1)
                Gl.glDeleteLists(_displayListIdentificator, 1);
            _displayListIdentificator = -1;
        }

        /// <summary>
        /// Zkopiruje objektu: Jmeno, Barvu, Viditelnost, Viditelnost os.
        /// </summary>
        public virtual void CopyDrawableSlots(GraphicObject newObject, GraphicObject parent)
        {
            //newObject._deleted = parent._deleted;
            //newObject._isInScene = parent._isInScene;
            newObject._visible = parent._visible;
            newObject._color = parent._color;
            newObject._name = parent._name;
        }


        public virtual void Draw(Scene scene)
        { 
            throw new Exception("Tato metoda musí být přepsána v potomkovi."); 
        }
        public virtual void DrawAsIntersection(Scene scene) 
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi."); 
        }
        public virtual void DrawIgnoreVisibilityAndAxes(Scene scene)
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }
        public virtual void BuildDisplayListIfNecessary(Scene scene, RelativeQuality relativeQuality)
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }
        public virtual void BuildDisplayList(Scene scene, RelativeQuality relativeQuality) 
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }



        public bool GetIsInScene()
        {
            return _isInScene;
        }
        public void SetInScene(bool isInScene)
        {
            _isInScene = isInScene;
        }

        public bool GetDeleted() { return _deleted; }
        public void SetDeleted(bool deleted) { _deleted = deleted; }

        public RelativeQuality GetRelativeQuality() { return _relativeQuality; }

        public int GetID() { return _ID; }
        public void SetID(int ID) { _ID = ID; } 

        public string GetName()
        {
            return _name;
        }
        public void ChangeName(string name)
        {
            this._name = name;
        }

        public GColor GetColor()
        {
            return _color;
        }
        public void ChangeColor(GColor color)
        {
            _color = color;
        }

        public void SetVisible(bool v) { _visible = v; }
        public bool GetVisible() { return _visible; }

        public virtual bool AreEqual(GraphicObject secondObject) 
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }


        public virtual string GetWordDescriptionFirstCase() 
        {
            return _wordDescriptionFirstCase + " " + _name; 
        }
        public virtual string GetWordDescriptionSecondCase()
        { 
            return _wordDescriptionSecondCase + " " + _name; 
        }
        public virtual string GetWordDescriptionFourthCase() 
        {
            return _wordDescriptionFourthCase + " " + _name; 
        }


        protected void SetWordDescriptionFirstCase(string description) 
        {
            _wordDescriptionFirstCase = description;
        }
        protected void SetWordDescriptionSecondCase(string description)
        {
            _wordDescriptionSecondCase = description; 
        }
        protected void SetWordDescriptionFourthCase(string description) 
        {
            _wordDescriptionFourthCase = description;
        }

        internal int GetDisplayListID()
        {
            return _displayListIdentificator;
        }
    }
}
