﻿using System.Collections.Generic;
using Geometry.GraphicObjects.ConicSections;
using GMath;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Matrixes;
using System;


namespace Geometry.GraphicObjects.Intersections
{
    class ConicAndConicIntersectionRecognizer
    {
        private ConicSection _conicA;
        private ConicSection _conicB;
        private ConicSection _simpleRuled;
        private GColor _color;
        private string _name;
        private int _ID;
        TransformationMatrix trans; // transformace jedne z kuzelosecek
        ConicSection conicACopy;
        ConicSection conicBCopy;

        protected struct IntersectionPoint 
        {
            public GPoint point;
            public double substitution;
            public double pvSubstitution;

            public IntersectionPoint(GPoint pointArg, double substitutionArg, double pvSubstitutionArg) 
            {
                point = pointArg;
                substitution = substitutionArg;
                pvSubstitution = pvSubstitutionArg;
            }
        }

        public ConicAndConicIntersectionRecognizer(
            ConicSection conicA, ConicSection conicB, GColor color, string name, int ID) 
        {
            _conicA = conicA;
            _conicB = conicB;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public ConicAndConicIntersection GetIntersection()
        {
            try
            {
                Plan conicAPlan = (Plan)_conicA.GetConicPlan().MakeCopyOfObject(_conicA.GetConicPlan().GetID());
                Plan conicBPlan = (Plan)_conicB.GetConicPlan().MakeCopyOfObject(_conicB.GetConicPlan().GetID());
                conicAPlan.SetTransformationMatrixAlgebraic(_conicA.GetTransformationMatrixAlgebraic());
                conicBPlan.SetTransformationMatrixAlgebraic(_conicB.GetTransformationMatrixAlgebraic());

                PlanAndPlanIntersectionRecognizer ir =
                    new PlanAndPlanIntersectionRecognizer(
                        conicAPlan, conicBPlan, _color, _name, _ID);
                PlanAndPlanIntersection semiIntersection = ir.GetIntersection();

                List<GraphicObject> intersections = new List<GraphicObject>();
                if (semiIntersection.GetTheOnlyIntersection() is Plan)
                    intersections = GetConicAndConicIntersection();
                else if (semiIntersection.GetTheOnlyIntersection() is Line)
                    intersections = GetConicsAndLineIntersection((Line)semiIntersection.GetTheOnlyIntersection());

                for (int i = intersections.Count - 1; i > -1; i--) // odstraním možné prázdné průniky
                    if (intersections[i] is EmptyIntersection)
                        intersections.RemoveAt(i);
                if (intersections.Count == 0) // pokud nemám zádný průnik, přidám prázdný průnik
                    intersections.Add(new EmptyIntersection());
                return new ConicAndConicIntersection(_conicA, _conicB, intersections, _color, _name, _ID);
            }
            catch (Exception) 
            {
                return new ConicAndConicIntersection(_conicA, _conicB, new List<GraphicObject>() { new EmptyIntersection() },
                    _color, _name, _ID);
            }
        }

        private List<GraphicObject> GetConicsAndLineIntersection(Line line) 
        {
            ConicAndLineIntersectionRecognizer ir = 
                new ConicAndLineIntersectionRecognizer(_conicA, line, _color, _name, _ID);
            ConicAndLineIntersection semiIntersection = ir.GetIntersection();
            if (semiIntersection.GetIntersections().Count == 1 && semiIntersection.GetTheOnlyIntersection() is Line)
            {
                ConicAndLineIntersectionRecognizer ir2 =
                    new ConicAndLineIntersectionRecognizer(_conicB, line, _color, _name, _ID);
                ConicAndLineIntersection semiIntersection2 = ir2.GetIntersection();
                return semiIntersection2.GetIntersections(); // prunik kuželosečky a primky
            }
            if (semiIntersection.GetTheOnlyIntersection() is GPoint)
                return GetConicBAndPointsIntersections(line, semiIntersection.GetIntersections());
            return new List<GraphicObject>(); // prazdny prunik
        }

        private List<GraphicObject> GetConicBAndPointsIntersections(Line line, List<GraphicObject> maybePointsA)
        {
            ConicAndLineIntersectionRecognizer ir =
                new ConicAndLineIntersectionRecognizer(_conicB, line, _color, _name, _ID);
            ConicAndLineIntersection semiIntersection = ir.GetIntersection();

            List<GraphicObject> intersections = new List<GraphicObject>();

            List<GraphicObject> maybePointsB = semiIntersection.GetIntersections();
            for (int i = 0; i < maybePointsA.Count; i++)
            {
                if (maybePointsA[i] is GPoint)
                {
                    for (int j = 0; j < maybePointsB.Count; j++) 
                    {
                        if (maybePointsB[j] is GPoint) 
                        {
                            Vector coordinatesA = ((GPoint)maybePointsA[i]).GetCenterInStandardBasis().GetReprezentant();
                            Vector coordinatesB = ((GPoint)maybePointsB[j]).GetCenterInStandardBasis().GetReprezentant();
                            double distance = Vector.Distance(coordinatesA, coordinatesB);
                            if (distance < Program.TwoPointsIntersectionCompareDistance)
                            {
                                intersections.Add(maybePointsA[i]);
                            }
                        }
                    }
                }
            }
            return intersections;
        }

        private List<GraphicObject> GetConicAndConicIntersection()
        {
            if (_conicA.AreEqual(_conicB))
                return GetConicAsIntersection();
            conicACopy = (ConicSection)_conicA.MakeCopyOfObject(_conicA.GetID());
            conicBCopy = (ConicSection)_conicB.MakeCopyOfObject(_conicB.GetID());
            trans = _conicA.GetTransformationMatrixAlgebraic().ComputeInverseMatrix();
            // test potvrdil, že kuželosečky nemusí být uplne 
            // ve stejne rovine, proto prevod do kanonic
            conicACopy.SetTransformationMatrixAlgebraic(
                trans *
                _conicA.GetTransformationMatrixAlgebraic());
            conicBCopy.SetTransformationMatrixAlgebraic(
                trans *
                _conicB.GetTransformationMatrixAlgebraic());
            ConicSection P = conicACopy;
            ConicSection Q = conicBCopy;
            if (P is SimpleRuledConic)
                _simpleRuled = (SimpleRuledConic)P.MakeCopyOfObject(P.GetID());
            else if (Q is SimpleRuledConic)
                _simpleRuled = (SimpleRuledConic)Q.MakeCopyOfObject(Q.GetID());
            else
            {
                _simpleRuled = GetCanonizedSimpleRuledConic(P, Q);
                if (_simpleRuled == null)// potreba kvuli zjednoduseni P-xQ=R, spravne ma byt Px-Qy=R.
                    _simpleRuled = GetCanonizedSimpleRuledConic(Q, P);
            }
            if (_simpleRuled == null)
                return new List<GraphicObject>(); // Může být prázdný GetIntersection si to pohlídá
            ConicMatrix Rmatrix = new ConicMatrix(_simpleRuled.GetCommonEquation());
            ConicMatrix workMatrix; // one from Pmatrix or Qmatrix
            ConicSection workConic; //
            workMatrix = new ConicMatrix(P.GetCommonEquation());
            workConic = conicACopy;
            if (_simpleRuled.AreEqual(P))
            {
                workConic = conicBCopy;
                workMatrix = new ConicMatrix(Q.GetCommonEquation());
            }
            workMatrix = workMatrix.ApplyTransformation(_simpleRuled.GetToCanonicTransformationMatrix());
            List<ParametrizationVector> parametrizationVectors = ConicsSimpleRuledParametrizations.GetParametrization(_simpleRuled);
            List<GraphicObject> semiIntersections = new List<GraphicObject>();
            List<GraphicObject> temp;

            foreach (ParametrizationVector parametrizationVector in parametrizationVectors)
            {
                temp = GetIntersectionSection(parametrizationVector, workMatrix);
                for (int i = 0; i < temp.Count; i++)
                    semiIntersections.Add(temp[i]);
            }
            List<GraphicObject> intersections = new List<GraphicObject>();
            int pointsCount = 0;

            for (int i = 0; i < semiIntersections.Count; i++)
                if (semiIntersections[i] is GPoint && pointsCount < 4)
                {
                    intersections.Add(semiIntersections[i]);
                    pointsCount++;
                }
                else if (!(semiIntersections[i] is GPoint))
                    intersections.Add(semiIntersections[i]);
            return intersections;
        }       
        private List<GraphicObject> GetIntersectionSection(ParametrizationVector pVector, ConicMatrix Pmatrix)
        {
            Expression equation = (pVector * Pmatrix) * pVector;
            equation = equation.Normalize();
            return GetNoVIS(pVector, equation);
        }
        private List<GraphicObject> GetNoVIS(ParametrizationVector pVector, Expression equation)
        {
            QuarticEquation quarticEquation = equation.GetQuarticEquationInU();
            if (!quarticEquation.ContainsVariable())
                if (!Program.IsEqualInTolerance(quarticEquation.GetE(), 0))
                    return new List<GraphicObject>() { };
                else return GetPartOfSimpleRuledAsIntersection(_simpleRuled, pVector);
            List<double> solutions = quarticEquation.Solve();
            return GetIntersectionPoints(pVector, solutions);
        }
        private List<GraphicObject> GetIntersectionPoints(ParametrizationVector pVector, List<double> uSolutions)
        {
            List<GraphicObject> intersections = new List<GraphicObject>();
            uSolutions = Program.DistinctInTolerance(uSolutions);
            // do definicniho oboru hyperboly nepatri nula, proto v tomto "u" nemohu hledat bod
            if (_simpleRuled is Hyperbola)
                for (int i = uSolutions.Count - 1; i >= 0; i--)
                    if (Program.IsEqualInTolerance(uSolutions[i], 0))
                        uSolutions.RemoveAt(i);
            Vector coordinates;
            foreach (double u in uSolutions)
            {
                GPoint point = GetPoint(pVector, u);
                coordinates = point.GetCenterInStandardBasis().GetReprezentant();
                intersections.Add(point);
            }
            return intersections;
        }
        private GPoint GetPoint(ParametrizationVector pVector, double u) 
        {
            Vector coordinates = pVector.GetValueAt(u, 1);
            coordinates = coordinates * _simpleRuled.GetTransformationMatrixAlgebraic().ComputeInverseMatrix();
            coordinates = coordinates * trans;
            return new GPoint(coordinates, _color, _name, _ID);
        }
        private DoubleLine GetDoubleLineFromSimpleRuledParametrization(ParametrizationVector parametrizationVector)
        {
            DoubleLine doubleLine = ConicsSimpleRuledParametrizations.GetDoubleLineFromParametrizationOfCompoundConicsection(parametrizationVector, _simpleRuled);
            if (doubleLine == null)
                throw new Exception("Neznama kužeosečka... Nelze ji rozložit na přímky.");
            return doubleLine;
        }
        /// <summary>
        /// prunikem je část simpleRuled kuželosečky, ne celá, 
        /// protože jinak by byly totožné vstupní kuželosečky a to je jejich porovnáním ošetřené.
        /// </summary>
        private List<GraphicObject> GetPartOfSimpleRuledAsIntersection(ConicSection simpleRulled, ParametrizationVector pVector)
        {
            // prunikem je část simpleRuled kuželosečky, ne celá, 
            // protože jinak by byly totožné vstupní kuželosečky a to je jejich porovnáním ošetřené.
            DoubleLine line = GetDoubleLineFromSimpleRuledParametrization(pVector);
            //line.ApplyTransformation(_simpleRuled.GetTransformationMatrixAlgebraic());
            line.ApplyTransformation(_conicA.GetTransformationMatrixAlgebraic());
            return new List<GraphicObject>() { line };
        }
        private List<GraphicObject> GetConicAsIntersection()
        {
            ConicSection copy = (ConicSection)_conicA.MakeCopyOfObject(_ID);
            return new List<GraphicObject>() { copy };
        }


        private List<GraphicObject> GetConicAndConicIntersectionSecondMethod()
        {
            if (_conicA.AreEqual(_conicB))
                return GetConicAsIntersection();
            ConicSection conicACopy = (ConicSection)_conicA.MakeCopyOfObject(_conicA.GetID());
            ConicSection conicBCopy = (ConicSection)_conicB.MakeCopyOfObject(_conicB.GetID());
            trans = _conicA.GetTransformationMatrixAlgebraic().ComputeInverseMatrix();
            // test potvrdil, že kuželosečky nemusí být uplne 
            // ve stejne rovine, proto prevod do kanonic
            conicACopy.SetTransformationMatrixAlgebraic(
                trans *
                _conicA.GetTransformationMatrixAlgebraic());
            conicBCopy.SetTransformationMatrixAlgebraic(
                trans *
                _conicB.GetTransformationMatrixAlgebraic());

            return GetIntersectionPointsSecondMethod(conicACopy, conicBCopy);
        }        /// <summary>
        /// Obě kuželosečky jsou ve stejné rovině. A kuželosečka conicACopy je v kanonickém tvaru.
        /// Vyjádřím z její rovnice "x" a dosadím do rovnice conicBCopy. 
        /// </summary>
        private List<GraphicObject> GetIntersectionPointsSecondMethod(ConicSection conicACopy, ConicSection conicBCopy)
        {
            ConicEquation eA = conicACopy.GetCommonEquation(); // je v kanonickem tvaru
            List<UV> valuesA = new List<UV>();
            valuesA.Add(new UV(-eA.GetB(), 2, 0));
            valuesA.Add(new UV(-eA.GetH(), 1, 0));
            valuesA.Add(new UV(-eA.GetL(), 0, 0));
            Expression x = new Expression(valuesA) / eA.GetA();

            ConicEquation eB = conicBCopy.GetCommonEquation();
            Expression withoutSqrt = eB.GetA() * x;
            withoutSqrt += new Expression(new UV(eB.GetB(), 2, 0));
            withoutSqrt += new Expression(new UV(eB.GetH(), 1, 0));
            withoutSqrt += new Expression(new UV(eB.GetL(), 0, 0));

            Expression underSqrt = new Expression(0);
            underSqrt += new Expression(new UV(eB.GetD(), 1, 0));
            underSqrt += new Expression(new UV(eB.GetG(), 0, 0));

            Expression squared = (Expression)(underSqrt * underSqrt * x) - (Expression)(withoutSqrt * withoutSqrt);

            QuarticEquation quarticEquation = squared.GetQuarticEquationInU();
            List<double> ySolutions = quarticEquation.Solve();

            List<GraphicObject> intersections = new List<GraphicObject>();
            Vector coordinates;
            ySolutions = Program.DistinctInTolerance(ySolutions);
            double substitution;
            foreach (double y in ySolutions)
            {
                QuadraticEquation qe = new QuadraticEquation(eB.GetA(), eB.GetG() + eB.GetD() * y, eB.GetB() * y * y + eB.GetH() * y + eB.GetL());
                List<double> solss = qe.Solve();
                solss = Program.DistinctInTolerance(solss);
                foreach (double xLocal in solss)
                {
                    coordinates = new Vector(xLocal, y, 0);
                    double substitutionToA = Math.Abs(conicACopy.GetCommonEquation().Substitute(coordinates));
                    double substitutionToB = Math.Abs(conicBCopy.GetCommonEquation().Substitute(coordinates));
                    coordinates = coordinates * trans;
                    if (substitutionToA < Program.TwoPointsIntersectionCompareDistance &&
                        substitutionToB < Program.TwoPointsIntersectionCompareDistance)
                    {
                        substitution = Math.Max(substitutionToA, substitutionToB);
                        GPoint intersection = new GPoint(coordinates, _color, _name, _ID);
                        IntersectionPoint point = new IntersectionPoint(intersection, substitution, 0);
                        intersections.Add(intersection);
                    }
                }
            }
            return intersections;
        }


        private ConicSection GetCanonizedSimpleRuledConic(ConicSection p, ConicSection q)
        {
            ConicMatrix Pmatrix = new ConicMatrix(p.GetCommonEquation());
            ConicMatrix Qmatrix = new ConicMatrix(q.GetCommonEquation());
            Matrix upperP = Pmatrix.GetSubMatrix(2, 2);
            Matrix upperQ = Qmatrix.GetSubMatrix(2, 2);
            QuadraticEquation lambdaQuadratic = GetConicsPencilDeterminant(upperP, upperQ);
            List<double> lambda = lambdaQuadratic.Solve();
            //List<double> subs = lambdaQuadratic.Substitute(lambda);
            ConicSection temp = null;
            for (int i = 0; i < lambda.Count; i++)
            {
                temp = GetSimpleRuledConic(Pmatrix, Qmatrix, lambda[i]);
                if (temp != null && IsSimpleRuled(temp))
                    return temp;              
            }
            CubicEquation lambdaCubic = GetConicsPencilDeterminant(Pmatrix, Qmatrix);
            lambda = lambdaCubic.Solve();
            for (int i = 0; i < lambda.Count; i++)
            {
                temp = GetSimpleRuledConic(Pmatrix, Qmatrix, lambda[i]);
                if (temp != null && IsSimpleRuled(temp))                
                    return temp;                
            }
            if (IsSimpleRuled(p))
                return p;
            if (IsSimpleRuled(q))
                return q;
            return null;
        }
        private ConicSection GetSimpleRuledConic(ConicMatrix a, ConicMatrix b, double lambda)
        {
            ConicMatrix simpleRuled = a - (lambda * b);
            string matrix = simpleRuled.ToString();
            ConicEquation equation = new ConicEquation(simpleRuled);
            double cislo = simpleRuled.GetSubMatrix(2, 2).Determinant2();
            ConicRecognizer conicRecognizer = 
                new ConicRecognizer(equation, new GColor(1, 1, 1), "", _ID);
            return conicRecognizer.Recognize();
        }
        private bool IsSimpleRuled(ConicSection conicSection)
        {
            return (
                conicSection is ParallelLines ||
                conicSection is DoubleLine ||
                conicSection is Hyperbola ||
                conicSection is Parabola ||
                conicSection is CrossingLines);
        }

        private CubicEquation GetConicsPencilDeterminant(ConicMatrix p, ConicMatrix q)
        {
            // udelam rozvoj podle prvniho sloupce
            LinearEquation a = new LinearEquation(-q.GetA(), p.GetA());
            LinearEquation d = new LinearEquation(-q.GetD(), p.GetD());
            LinearEquation g = new LinearEquation(-q.GetG(), p.GetG());
            CubicEquation e1 = GetConicsPencilDeterminant(p.GetSubMatrix(0, 0), q.GetSubMatrix(0, 0)) * a;
            CubicEquation e2 = GetConicsPencilDeterminant(p.GetSubMatrix(1, 0), q.GetSubMatrix(1, 0)) * d;
            CubicEquation e3 = GetConicsPencilDeterminant(p.GetSubMatrix(2, 0), q.GetSubMatrix(2, 0)) * g;
            return e1 - e2 + e3; // z vlatnosti rozvoje determinantu.
        }
        private QuadraticEquation GetConicsPencilDeterminant(Matrix upMatrixP, Matrix upMatrixQ)
        {
            double[] p = upMatrixP.ToDoubleArray();
            double[] q = upMatrixQ.ToDoubleArray();
            QuadraticEquation m03 = DeterminantSubPart(p[0], p[3], q[0], q[3]);
            QuadraticEquation m12 = DeterminantSubPart(p[1], p[2], q[1], q[2]);
            return m03 - m12;
        }
        /// <summary>
        /// Vraci koeficienty kvadraticke rovnice
        /// (a1 - a2x)(b1 - b2x)
        /// </summary>
        /// <returns></returns>
        private QuadraticEquation DeterminantSubPart(double a1, double b1, double a2, double b2)
        {
            double abs = (a1 * b1);
            double lambda = -(a1 * b2 + a2 * b1);
            double lambda2 = (a2 * b2);
            return new QuadraticEquation(lambda2, lambda, abs);
        }



        private void InsertOnSortedPosition(IntersectionPoint point, List<IntersectionPoint> points) 
        {
            int i = 0;
            for (i = 0; i < points.Count; i++)
            {
                if (points[i].substitution > point.substitution)
                    break;
            }
            points.Insert(i, point);
        }
    }
}
