﻿using System.Collections.Generic;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Intersections;
using GMath;
using System;

namespace Geometry.GraphicObjects.Intersections
{
    class ConicAndPlanIntersectionRecognizer
    {
        private ConicSection _conicSection;
        private Plan _plan;
        private GColor _color;
        private string _name;
        private int _ID;


        public ConicAndPlanIntersectionRecognizer(ConicSection conicSection, Plan plan, GColor color, string name, int ID)
        {
            _conicSection = conicSection;
            _plan = plan;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public ConicAndPlanIntersection GetIntersection()
        {
            try
            {
                Plan conicPlan = _conicSection.GetConicPlan();
                PlanAndPlanIntersection planAndPlanIntersection =
                    new PlanAndPlanIntersectionRecognizer(conicPlan, _plan, _color, _name, _ID).GetIntersection();
                List<GraphicObject> intersections = new List<GraphicObject>();
                if (planAndPlanIntersection.GetTheOnlyIntersection() is Plan)
                    intersections = GetConicAsIntersection();
                if (planAndPlanIntersection.GetTheOnlyIntersection() is Line)
                    intersections = GetConicAndLineIntersection((Line)planAndPlanIntersection.GetTheOnlyIntersection());

                for (int i = intersections.Count - 1; i > -1; i--) // odstraním možné prázdné průniky
                {
                    if (intersections[i] is EmptyIntersection)
                        intersections.RemoveAt(i);
                }
                if (intersections.Count == 0) // pokud nemám zádný průnik, přidám prázdný průnik
                    intersections.Add(new EmptyIntersection());
                return new ConicAndPlanIntersection(_conicSection, _plan, intersections, _color, _name, _ID);
            }
            catch (Exception) 
            {
                return new ConicAndPlanIntersection(_conicSection, _plan, new List<GraphicObject>() {new EmptyIntersection() },
                    _color, _name, _ID);
            }
        }

        private List<GraphicObject> GetConicAsIntersection()
        {
            ConicSection copy = (ConicSection)_conicSection.MakeCopyOfObject(_ID);
            return new List<GraphicObject>() { copy };
        }

        private List<GraphicObject> GetConicAndLineIntersection(Line line) 
        {
            List<GraphicObject> intersections = new  ConicAndLineIntersectionRecognizer(
                _conicSection, line, _color, _name, _ID).GetIntersection().GetIntersections();
            return intersections;
        }
    }
}
