﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.ConicSections;
using GMath;
using Geometry.GraphicObjects.Equations;

namespace Geometry.GraphicObjects.Intersections
{
    class ConicAndPointIntersectionRecognizer
    {
        private ConicSection _conicSection;
        private GPoint _point;
        private GColor _color;
        private string _name;
        private int _ID;

        public ConicAndPointIntersectionRecognizer(ConicSection conicSection, GPoint point, GColor color, string name, int ID)
        {
            _conicSection = conicSection;
            _point = point;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public ConicAndPointIntersection GetIntersection()
        {
            ConicEquation conicCanonicEquation = new ConicEquation(_conicSection.GetCanonicMatrix());
            Vector oldPointCoordinates = _point.GetCenterInStandardBasis().GetReprezentant();
            //Vector newPointsCoordinates = oldPointCoordinates * _conicSection.GetToCanonicTransformationMatrix();
            Vector newPointsCoordinates = oldPointCoordinates * _conicSection.GetTransformationMatrixAlgebraic();

            if (BelongsPointToConicSection(newPointsCoordinates, conicCanonicEquation))
                return GetPointAsIntersection();
            else return GetEmptyIntersection();
        }
        private bool BelongsPointToConicSection(Vector pointInConicCoordinates, ConicEquation conicCanonicEquation)
        {
            double substitution = conicCanonicEquation.Substitute(pointInConicCoordinates);
            if ((Math.Abs(substitution) <= Program.TwoPointsIntersectionCompareDistance)
                && Program.IsEqualInToleranceIntersection(pointInConicCoordinates.GetZ(), 0))
                return true;
            return false;
        }

        private ConicAndPointIntersection GetPointAsIntersection()
        {
            Vector coordinates = _point.GetCenterInStandardBasis().GetReprezentant();
            GPoint intersection =
                new GPoint(new Vector(coordinates.GetX(), coordinates.GetY(), coordinates.GetZ()), _color, _name, _ID);

            return new ConicAndPointIntersection(
                _conicSection, _point, new List<GraphicObject>() { intersection }, _color, _name, _ID);
        }


        private ConicAndPointIntersection GetEmptyIntersection()
        {
            return new ConicAndPointIntersection(
                _conicSection, _point, new List<GraphicObject>() { new EmptyIntersection() }, _color, _name, _ID);
        }
    }
}
