﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.BaseClasses;
using Tao.OpenGl;

namespace Geometry.GraphicObjects.Intersections
{
    public class IIntersection:GraphicObject
    {
        protected GraphicObject _objectA; 
        protected GraphicObject _objectB;
        protected List<GraphicObject> _intersections;

        public IIntersection(GraphicObject objectA, GraphicObject objectB, 
            List<GraphicObject> intersections, GColor color, string name, int ID) 
        {
            _objectA = objectA;
            _objectB = objectB;
            _intersections = intersections;
            base._color = color;
            base._name = name;
            base._ID = ID;
        }

        public GraphicObject GetObjectA() { return _objectA; }
        public GraphicObject GetObjectB() { return _objectB; }

        public GraphicObject GetTheOnlyIntersection() { return _intersections[0]; }
        public List<GraphicObject> GetIntersections() { return _intersections; }


        public virtual IIntersection RecomputeIntersection
            (Scene scene, GraphicObject objectA, GraphicObject objectB, int newID) 
        { 
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }

        public override void BuildDisplayListIfNecessary(Scene scene, RelativeQuality relativeQuality)
        {
            for (int i = 0; i < _intersections.Count; i++)
            {
                if (_intersections[i] is ConicSections.ConicSection)
                    ((ConicSections.ConicSection)_intersections[i]).BuildAsIntersection(scene, relativeQuality);
                else if (_intersections[i] is Planar.Line)
                    ((Planar.Line)_intersections[i]).BuildAsIntersection(scene, relativeQuality);
                else _intersections[i].BuildDisplayListIfNecessary(scene, relativeQuality);
            }
        }

        public override void BuildDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
           BuildDisplayListIfNecessary(scene, relativeQuality);
        }

        public override void Draw(Scene scene)
        {
            Gl.glPushMatrix();
            if (base._visible)
            {
                Gl.glColor3dv(_color);
                for (int i = 0; i < _intersections.Count; i++)
                {
                    _intersections[i].DrawAsIntersection(scene);
                }
            }
            Gl.glPopMatrix();
        }

        internal override void DeleteDisplayList()
        {
            for (int i = 0; i < _intersections.Count; i++)
                _intersections[i].DeleteDisplayList();
        }


        public override string GetWordDescriptionFirstCase()
        {
            return
                "Průnik " + _name + " " + 
                _objectA.GetWordDescriptionSecondCase() + " a " + 
                _objectB.GetWordDescriptionSecondCase();
        }
        public override string GetWordDescriptionSecondCase()
        {
            return
                "Průniku " + _name + " " + 
                _objectA.GetWordDescriptionSecondCase() + " a " +
                _objectB.GetWordDescriptionSecondCase();
        }
        public override string GetWordDescriptionFourthCase()
        {
            return
                "Průnik " + _name + " " + 
                _objectA.GetWordDescriptionSecondCase() + " a " +
                _objectB.GetWordDescriptionSecondCase();
        }
    }
}
