﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Intersections;

namespace Geometry.GraphicObjects.Intersections
{
    public class IntersectionRecognizer
    {
        private GraphicObject _objectA;
        private GraphicObject _objectB;
        private GColor _color;
        private string _name;
        private int _ID;

        public IntersectionRecognizer(GraphicObject objectA, GraphicObject objectB, GColor color, string name, int ID) 
        {
            _objectA = objectA;
            _objectB = objectB;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public IIntersection Recognize() 
        {
            if (_objectA is Quadric || _objectB is Quadric)
                return RecognizeIntersectionWithQuadric();
            else if (_objectA is ConicSection || _objectB is ConicSection)
                return RecognizeIntersectionWithConic();
            else if (_objectA is Plan || _objectB is Plan)
                return RecognizeIntersectionWithPlan();
            else if (_objectA is Line || _objectB is Line)
                return RecognizeIntersectionWithLine();
            else if (_objectA is GPoint || _objectB is GPoint)
                return RecognizeIntersectionWithPoint();
            else throw new Exception("Nelze určit průnik zadaných objektů.");
        }

        private IIntersection RecognizeIntersectionWithQuadric() 
        {
            Quadric quadric;
            GraphicObject graphicObject;
            if (_objectA is Quadric)
            {
                quadric = (Quadric)_objectA;
                graphicObject = _objectB;
            }
            else
            {
                quadric = (Quadric)_objectB;
                graphicObject = _objectA;
            }

            if (graphicObject is Quadric)
                return new QuadricAndQuadricIntersectionRecognizer(quadric, (Quadric)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is ConicSection)
                return new QuadricAndConicIntersectioRecognizer(quadric, (ConicSection)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is Plan)
                return new QuadricAndPlanItersectionRecognizer(quadric, (Plan)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is Line)
                return new QuadricAndLineIntersectionRecognizer(quadric, (Line)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is GPoint)
                return new QuadricAndPointIntersectionRecognizer(quadric, (GPoint)graphicObject, _color, _name, _ID).GetIntersection();
            throw new Exception("Nelze určit průnik zadaných objektů.");
        }
        private IIntersection RecognizeIntersectionWithConic() 
        {
            ConicSection conicSection;
            GraphicObject graphicObject;
            if (_objectA is ConicSection)
            {
                conicSection = (ConicSection)_objectA;
                graphicObject = _objectB;
            }
            else
            {
                conicSection = (ConicSection)_objectB;
                graphicObject = _objectA;
            }

            if (graphicObject is ConicSection)
                return new ConicAndConicIntersectionRecognizer(conicSection, (ConicSection)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is Plan)
                return new ConicAndPlanIntersectionRecognizer(conicSection, (Plan)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is Line)
                return new ConicAndLineIntersectionRecognizer(conicSection, (Line)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is GPoint)
                return new ConicAndPointIntersectionRecognizer(conicSection, (GPoint)graphicObject, _color, _name, _ID).GetIntersection();
            throw new Exception("Nelze určit průnik zadaných objektů.");
        }
        private IIntersection RecognizeIntersectionWithPlan()
        {
            Plan plan;
            GraphicObject graphicObject;
            if (_objectA is Plan)
            {
                plan = (Plan)_objectA;
                graphicObject = _objectB;
            }
            else
            {
                plan = (Plan)_objectB;
                graphicObject = _objectA;
            }

            if (graphicObject is Plan)
                return new PlanAndPlanIntersectionRecognizer(plan, (Plan)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is Line)
                return new PlanAndLineIntersectionRecognizer(plan, (Line)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is GPoint)
                return new PlanAndPointIntersectionRecognizer(plan, (GPoint)graphicObject, _color, _name, _ID).GetIntersection();
            throw new Exception("Nelze určit průnik zadaných objektů.");
        }
        private IIntersection RecognizeIntersectionWithLine()
        {
            Line line;
            GraphicObject graphicObject;
            if (_objectA is Line)
            {
                line = (Line)_objectA;
                graphicObject = _objectB;
            }
            else
            {
                line = (Line)_objectB;
                graphicObject = _objectA;
            }

            if (graphicObject is Line)
                return new LineAndLineIntersectionRecognizer(line, (Line)graphicObject, _color, _name, _ID).GetIntersection();
            if (graphicObject is GPoint)
                return new LineAndPointIntersectionRecognizer(line, (GPoint)graphicObject, _color, _name, _ID).GetIntersection();
            throw new Exception("Nelze určit průnik zadaných objektů.");
        }
        private IIntersection RecognizeIntersectionWithPoint() 
        {
            if (_objectA is GPoint && _objectB is GPoint)
            {
                return new PointAndPointIntersectionRecognizer((GPoint)_objectA, (GPoint)_objectB, _color, _name, _ID).GetIntersection();
            }
            throw new Exception("Nelze určit průnik zadaných objektů.");
        }
    }
}
