﻿using System;
using System.Collections.Generic;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using GMath;

namespace Geometry.GraphicObjects.Intersections
{
    public class LineAndPointIntersectionRecognizer
    {
        private Line _line;
        private GPoint _point;
        private GColor _color;
        private string _name;
        private int _ID;

        public LineAndPointIntersectionRecognizer(Line line, GPoint point, GColor color, string name, int ID) 
        {
            _line = line;
            _point = point;
            _color = color;
            _name = name;
            _ID = ID;
        }


        public LineAndPointIntersection GetIntersection()
        {
            if (IsPointOnLine())
                return GetPointAsIntersection();
            return GetEmptyIntersection();
        }

        /// <summary>
        /// Najdu rovinu jejiz normalovy vektor je zkoumana primka.
        /// Potom rovinu posunu tak, aby prochazela zkoumanym bodem.
        /// Najdu prunik roviny a primky.
        /// Podle vzdálenosti nalezeneho bodu a zkoumaneho bodu rozhodnu, zda náležní bod přímce.
        /// </summary>
        private bool IsPointOnLine()
        {
            LineParametricEquation equation = (LineParametricEquation)_line.GetParametricEquation();
            Vector coordinates = _point.GetCenterInStandardBasis().GetReprezentant();
            Plan plan = new Plan(new PlanEquation(equation.GetUx(), equation.GetUy(), equation.GetUz(), 0), _color, _name, _ID);
            plan.SetCenterInStandardBasis(new CenterParametrization(coordinates));

            PlanAndLineIntersection planAndLineIntersection = new PlanAndLineIntersectionRecognizer(plan, _line, _color, _name, _ID).GetIntersection();
            GPoint pointOnLine;
            try
            {   // toto by nemelo nikdy nastat, prunik roviny a jeji normaly musi byt vzdy bod.
                pointOnLine = (GPoint)planAndLineIntersection.GetTheOnlyIntersection();
            }
            catch (Exception)
            {
                return false;
            }
            Vector pointOnLineCoordinates = pointOnLine.GetCenterInStandardBasis().GetReprezentant();
            double distance = Vector.Distance(coordinates, pointOnLineCoordinates);
            return Math.Abs(distance) <= Program.TwoPointsIntersectionCompareDistance;
        }


        private LineAndPointIntersection GetPointAsIntersection()
        {
            Vector coordinates = _point.GetCenterInStandardBasis().GetReprezentant();
            GPoint intersection =
                new GPoint(new Vector(coordinates.GetX(), coordinates.GetY(), coordinates.GetZ()), _color, _name, _ID);
            return new LineAndPointIntersection(_line, _point, new List<GraphicObject>() { intersection }, _color, _name, _ID);
        }


        private LineAndPointIntersection GetEmptyIntersection()
        {
            return new LineAndPointIntersection(
                _line, _point, new List<GraphicObject>() { new EmptyIntersection() }, _color, _name, _ID);
        }
    }
}
