﻿using System.Collections.Generic;
using Geometry.GraphicObjects.Planar;
using GMath;

namespace Geometry.GraphicObjects.Intersections
{
    public class PointAndWithCommonEquationObjectIntersectionRecognizer
    {
        private WithCommonEquationObject _withCommonEquationObject;
        private GPoint _point;
        private GColor _color;
        private string _name;
        private int _ID;

        public PointAndWithCommonEquationObjectIntersectionRecognizer
            (WithCommonEquationObject withCommonEquationObject, GPoint point, GColor color, string name, int ID) 
        {
            _withCommonEquationObject = withCommonEquationObject;
            _point = point;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public PointAndWithCommonEquationObjectIntersection GetIntersection() 
        {
            double substitution = _withCommonEquationObject.GetCommonEquation()
                .Substitute(_point.GetCenterInStandardBasis().GetReprezentant());
            if (Program.IsEqualInToleranceIntersection(substitution, 0))
                return GetPointAsIntersection();
            return GetEmptyIntersection();
        }

        private PointAndWithCommonEquationObjectIntersection GetPointAsIntersection()
        {
            Vector coordinates = _point.GetCenterInStandardBasis().GetReprezentant();
            GPoint intersection = 
                new GPoint(new Vector(coordinates.GetX(), coordinates.GetY(), coordinates.GetZ()), _color, _name, _ID);
            return new PointAndWithCommonEquationObjectIntersection(
                _withCommonEquationObject, _point, new List<GraphicObject>() { intersection }, _color, _name, _ID);
        }


        private PointAndWithCommonEquationObjectIntersection GetEmptyIntersection()
        {
            return new PointAndWithCommonEquationObjectIntersection(
                _withCommonEquationObject, _point, new List<GraphicObject>() { new EmptyIntersection() }, _color, _name, _ID);
        }
    }
}
