﻿using System.Collections.Generic;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Intersections;
using GMath;
using System;

namespace Geometry.GraphicObjects.Intersections
{
    class QuadricAndConicIntersectioRecognizer
    {

        private Quadric _quadric;
        private ConicSection _conicSection;
        private GColor _color;
        private string _name;
        private int _ID;

        public QuadricAndConicIntersectioRecognizer(
            Quadric quadric, ConicSection conicSection, GColor color, string name, int ID)
        {
            _quadric = quadric;
            _conicSection = conicSection;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public QuadricAndConicIntersection GetIntersection() 
        {
            try
            {
                Plan conicPlan = _conicSection.GetConicPlan();

                QuadricAndPlanItersectionRecognizer ir = new QuadricAndPlanItersectionRecognizer(
                    _quadric, conicPlan, _color, _name, _ID);
                QuadricAndPlanIntersection semiIntersection = ir.GetIntersection();

                GraphicObject graphicObject = semiIntersection.GetTheOnlyIntersection();
                List<GraphicObject> intersections = new List<GraphicObject>();
                if (graphicObject is Plan)
                    intersections = GetConicAsIntersection();
                else if (graphicObject is ConicSection)
                    intersections = GetConicAndConicIntersection((ConicSection)graphicObject);
                else if (graphicObject is Line)
                    intersections = GetConicAndLineIntersection((Line)graphicObject);

                for (int i = intersections.Count - 1; i > -1; i--) // odstraním možné prázdné průniky
                {
                    if (intersections[i] is EmptyIntersection)
                        intersections.RemoveAt(i);
                }
                //intersections.Add(graphicObject);
                if (intersections.Count == 0) // pokud nemám zádný průnik, přidám prázdný průnik
                    intersections.Add(new EmptyIntersection());
                return new QuadricAndConicIntersection(_quadric, _conicSection, intersections, _color, _name, _ID);
            }
            catch (Exception) 
            {
                return new QuadricAndConicIntersection(_quadric, _conicSection, new List<GraphicObject>() { new EmptyIntersection() },
                    _color, _name, _ID);
            }
        }


        private List<GraphicObject> GetConicAsIntersection() 
        {
            ConicSection copyConic = (ConicSection)_conicSection.MakeCopyOfObject(_ID);
            return new List<GraphicObject>() { copyConic };
        }

        private List<GraphicObject> GetConicAndConicIntersection(ConicSection conicB) 
        {
            ConicAndConicIntersectionRecognizer ir = new ConicAndConicIntersectionRecognizer(_conicSection, conicB, _color, _name, _ID);
            ConicAndConicIntersection semiIntersection = ir.GetIntersection();
            return semiIntersection.GetIntersections();
        }

        private List<GraphicObject> GetConicAndLineIntersection(Line line)
        {
            ConicAndLineIntersectionRecognizer ir = new ConicAndLineIntersectionRecognizer(
                _conicSection, line, _color, _name, _ID);
            ConicAndLineIntersection semiIntersection = ir.GetIntersection();
            foreach (GraphicObject intersection in semiIntersection.GetIntersections())
                if (intersection is WithParametricEquationObject)
                {
                    WithParametricEquationObject withParametric = (WithParametricEquationObject)intersection;
                    withParametric.SetTransformationMatrixAlgebraic(
                        _conicSection.GetConicPlan().GetTransformationMatrixAlgebraic() * 
                        withParametric.GetTransformationMatrixAlgebraic());
                }
            return semiIntersection.GetIntersections();
        }
    }
}
