﻿using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.ConicSections;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using GMath;
using System;

namespace Geometry.GraphicObjects.Intersections
{
    class QuadricAndPlanItersectionRecognizer
    {
        private Quadric _quadric;
        private Plan _plan;
        private GColor _color;
        private string _name;
        private int _ID;


        public QuadricAndPlanItersectionRecognizer(Quadric quadric, Plan plan, GColor color, string name, int ID) 
        {
            _quadric = quadric;
            _plan = plan;
            _color = color;
            _name = name;
            _ID = ID;
        }

        public QuadricAndPlanIntersection GetIntersection()
        {
            try
            {
                Matrix transformation = _plan.GetTransformationMatrixAlgebraic();
                Matrix inverseTransformation = transformation.ComputeInverseMatrix();
                QuadricMatrix q =
                    new QuadricMatrix(_quadric.GetCommonEquation()).ApplyTransformation(inverseTransformation);

                ConicEquation conicEquation =
                    new ConicEquation(q.GetA(), q.GetB(), q.GetD() * 2, q.GetG() * 2, q.GetH() * 2, q.GetL());

                GraphicObject intersection = null;

                if (!Program.IsEqualInTolerance(q.GetA(), 0) ||
                    !Program.IsEqualInTolerance(q.GetB(), 0) ||
                    !Program.IsEqualInTolerance(q.GetD(), 0))
                    intersection = GetConicSection(conicEquation);
                else if (!Program.IsEqualInToleranceIntersection(q.GetG(), 0) ||
                         !Program.IsEqualInToleranceIntersection(q.GetH(), 0))
                    intersection = GetLine(conicEquation);
                else if (Program.IsEqualInToleranceIntersection(q.GetL(), 0))
                    intersection = GetPlan();

                if (intersection == null)
                    intersection = new EmptyIntersection();
                return new QuadricAndPlanIntersection(_quadric, _plan, intersection, _color, _name, _ID);
            }
            catch (Exception) 
            {
                return new QuadricAndPlanIntersection(_quadric, _plan, new EmptyIntersection(), _color, _name, _ID);
            }
        }

        private ConicSection GetConicSection(ConicEquation conicEquation) 
        {
            ConicRecognizer cs = new ConicRecognizer(conicEquation, _color, _name, _ID);
            ConicSection conicSection = cs.Recognize();
            //RotationAngles angles3 = _plan.GetTransformationMatrixAlgebraic().DecomposeRotationMatrixOnAngles();            
            //RotationAngles angles2 = conicSection.GetTransformationMatrixAlgebraic().DecomposeRotationMatrixOnAngles();
            TransformationMatrix transformationMatrix = _plan.GetTransformationMatrixAlgebraic() * conicSection.GetTransformationMatrixAlgebraic();
            conicSection.SetTransformationMatrixAlgebraic(transformationMatrix);
            //RotationAngles angles = conicSection.GetConicPlan().GetTransformationMatrixAlgebraic().DecomposeRotationMatrixOnAngles();
            //RotationAngles decCo = ConicSection.DecomposeTransformationMatrix(transformationMatrix).conicTransformation.DecomposeRotationMatrixOnAngles();
            //RotationAngles decPl = ConicSection.DecomposeTransformationMatrix(transformationMatrix).planTransformation.DecomposeRotationMatrixOnAngles();
            //RotationAngles u = transformationMatrix.DecomposeRotationMatrixOnAngles();
            //RotationAngles uu = conicSection.GetTransformationMatrixAlgebraic().DecomposeRotationMatrixOnAngles();
            return conicSection;
        }

        private Line GetLine(ConicEquation conicEquation)
        {
            double a = conicEquation.GetG();
            double b = conicEquation.GetH();
            double c = conicEquation.GetL();

            return new Line(new LineEquation(a, b, c), _color, _name, _ID);
        }

        private Plan GetPlan() 
        {
            return new Plan((PlanEquation)_plan.GetCommonEquation(), _color, _name , _ID);
        }
    }
}
