﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Geometry.GraphicObjects.Intersections
{
    public class UV
    {
        private int _uExponent;
        private int _vExponent;
        private double _value;

        public UV(UV src)
        {
            _value = src._value;
            _uExponent = src._uExponent;
            _vExponent = src._vExponent;
        }
        public UV(double value, int uExponent, int vExponent)
        {
            _value = value;
            _uExponent = uExponent;
            _vExponent = vExponent;
        }

        public int GetUexponent() { return _uExponent; }
        public int GetVexponent() { return _vExponent; }

        public void SetUexponent(int uExponent) { _uExponent = uExponent; }
        public void SetVexponent(int vExponent) { _vExponent = vExponent; }

        public double GetValue() { return _value; }
        public void SetValue(double value) { _value = value; }

        public UV SubstituteForU(double u) 
        {
            return new UV(Math.Pow(u, _uExponent) * _value, 0, _vExponent);
        }
        public UV SubstituteForV(double v)
        {
            return new UV(Math.Pow(v, _vExponent) * _value, _uExponent, 0);
        }


        public static UV operator *(double a, UV b) 
        {
            return new UV(a*b.GetValue(), b.GetUexponent(), b.GetVexponent());
        }
        public static UV operator *(UV a, double b)
        {
            return new UV(b * a.GetValue(), a.GetUexponent(), a.GetVexponent());
        }
        public static UV operator *(UV a, UV b)
        {
            return new UV(a.GetValue() * b.GetValue(),
                a.GetUexponent() + b.GetUexponent(),
                a.GetVexponent() + b.GetVexponent());
        }
        public static UV operator /(UV a, double b)
        {
            return new UV(a.GetValue() / b, a.GetUexponent(), a.GetVexponent());
        }
        public static UV operator -(UV a)
        {
            return new UV(-a.GetValue(), a.GetUexponent(), a.GetVexponent());
        }

        public double GetValueAt(double u, double v) 
        {
            double retval = Math.Pow(u, GetUexponent()) * Math.Pow(v, GetVexponent()) * GetValue();
            return retval;
        }
        public double GetValueAt(double u) 
        {
            return Math.Pow(u, _uExponent) * _value;
        }

        public bool IsZero() 
        {
            return Program.IsEqualInTolerance(_value, 0);
        }

        public override string ToString()
        {
            if (Program.IsEqualInTolerance(GetValue(), 0))
                return "";
            string retval = "+" + Program.GetOutputFormatedForCleverLabel(GetValue());
            if ((GetValue() == 1) && (GetVexponent() != 0 || GetUexponent() != 0))
                retval = "";
            if (GetUexponent() != 0)
                retval += "u" + GetExponentUnicode(GetUexponent());
            if (GetVexponent() != 0)
                retval += "v" + GetExponentUnicode(GetVexponent());
            return retval;
        }
        public string ToStringIgnoreV()
        {
            if (Program.IsEqualInTolerance(GetValue(), 0))
                return "";
            string retval = "+" + Program.GetOutputFormatedForCleverLabel(GetValue());
            if ((GetValue() == 1) && GetUexponent() != 0)
                retval = "";
            if (GetUexponent() != 0)
                retval += "u" + GetExponentUnicode(GetUexponent());
            return retval;
        }

        private string GetExponentUnicode(int exponent) 
        {
            switch (exponent) 
            {
                case -2: return U.ExpMinus2;
                case -1: return U.ExpMinus1;                
                case 0: return "1";
                case 1: return "";
                case 2: return U.Exp2;
                case 3: return U.Exp3;
                case 4: return U.Exp4;
                default: throw new Exception("Je volána funkce pro získání unicode textu exponentu, s nepředpokládaným exponentem.");
            }
        }

        public bool ContainsUWithExponent(int exponent) 
        {
            return (this.GetUexponent() == exponent) && !Program.IsEqualInTolerance(this.GetValue(), 0);
        }
        public bool ContainsVWithExponent(int exponent)
        {
            return (this.GetVexponent() == exponent) && !Program.IsEqualInTolerance(this.GetValue(), 0);
        }
    }
}
