﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Geometry.GraphicObjects.Intersections
{
    public class LinearV : IParameterVParametrization
    {
        private Expression _numerator;
        private Expression _denominator;

        public LinearV(Expression numerator, Expression denominator) 
        {
            _numerator = numerator;
            _denominator = denominator;
        }

        public Expression GetNumerator() 
        { 
            return _numerator;
        }
        public Expression GetDenominator() 
        {
            return _denominator;
        }

        public override IParameterVParametrization EliminateZeros()
        {
            return new LinearV(_numerator.EliminateZeros(), _denominator.EliminateZeros());
        }

        public override double GetVvalue(double u, int sign)
        {
            double numerator = _numerator.GetValueAt(u);
            double denominator = _denominator.GetValueAt(u);
            return numerator / denominator;
        }

        public override string ToFormatStringIgnoreV()
        {
            if (!_numerator.IsZero())
            {
                return "L[v=]" +
                    "F["
                        + _numerator.ToFormatStringIgnoreV()
                        + ";"
                        + _denominator.ToFormatStringIgnoreV()
                   + "]";
            }
            else return "L[v = 0]";
        }
    }
}
