﻿using Geometry.GraphicObjects.Matrixes;
using GMath;
using System;

namespace Geometry.GraphicObjects.Intersections
{
    public class ParametrizationVector
    {
        private Expression _xParametrization;
        private Expression _yParametrization;
        private Expression _zParametrization;
        private Expression _oneParametrization;

        public ParametrizationVector(Expression xParametrization, Expression yParametrization,
            Expression zParametrization, Expression oneParametrization)
        {
            _xParametrization = xParametrization;
            _yParametrization = yParametrization;
            _zParametrization = zParametrization;
            _oneParametrization = oneParametrization;
        }

        public Expression GetXParametrization() { return _xParametrization; }
        public Expression GetYParametrization() { return _yParametrization; }
        public Expression GetZParametrization() { return _zParametrization; }
        public Expression GetOneParametrization() { return _oneParametrization; }

        public Vector GetValueAt(double u, double v)
        {
            double x = _xParametrization.GetValueAt(u,v);
            double y = _yParametrization.GetValueAt(u,v);
            double z = _zParametrization.GetValueAt(u,v);
            return new Vector(x, y, z);
        }

        public static Expression operator *(ParametrizationVector a, ParametrizationVector b)
        {
            Expression retval =
            a.GetXParametrization() * b.GetXParametrization() +
            a.GetYParametrization() * b.GetYParametrization() +
            a.GetZParametrization()*  b.GetZParametrization() +
            a.GetOneParametrization()*  b.GetOneParametrization();
            return retval;
        }
        public static Expression operator *(ParametrizationVector a, Vector b)
        {
            Expression retval =
            b.GetX() * a._xParametrization +
            b.GetY() * a._yParametrization +
            b.GetZ() * a._zParametrization +
            b.GetOne() * a._oneParametrization;
            return retval;
        }

        /// <summary>
        /// z-ová složka parametrizacniho vektoru se ingnoruje.
        /// </summary>
        public static ParametrizationVector operator *(ParametrizationVector vector, ConicMatrix matrix)
        {
            Expression xParametrization, yParametrization, zParametrization, oneParametrization;
            xParametrization = vector * new Vector(matrix.GetA(), matrix.GetD(), 0, matrix.GetG());
            yParametrization = vector * new Vector(matrix.GetD(), matrix.GetB(), 0, matrix.GetH());
            zParametrization = new Expression(0);
            oneParametrization = vector * new Vector(matrix.GetG(), matrix.GetH(), 0, matrix.GetL());
            return new ParametrizationVector(xParametrization, yParametrization, zParametrization, oneParametrization);
        }
        public static ParametrizationVector operator *(ParametrizationVector vector, QuadricMatrix matrix)
        {
            Expression xParametrization, yParametrization, zParametrization, oneParametrization;
            xParametrization = vector * new Vector(matrix.GetA(), matrix.GetD(), matrix.GetE(), matrix.GetG());
            yParametrization = vector * new Vector(matrix.GetD(), matrix.GetB(), matrix.GetF(), matrix.GetH());
            zParametrization = vector * new Vector(matrix.GetE(), matrix.GetF(), matrix.GetC(), matrix.GetK());
            oneParametrization = vector * new Vector(matrix.GetG(), matrix.GetH(), matrix.GetK(), matrix.GetL());
            return new ParametrizationVector(xParametrization, yParametrization, zParametrization, oneParametrization);
        }
        public static ParametrizationVector operator *(ParametrizationVector vector, Matrix matrix)
        {
            Expression xParametrization, yParametrization, zParametrization, oneParametrization;
            xParametrization = vector * new Vector(matrix.Get(0), matrix.Get(4), matrix.Get(8), matrix.Get(12));
            yParametrization = vector * new Vector(matrix.Get(1), matrix.Get(5), matrix.Get(9), matrix.Get(13));
            zParametrization = vector * new Vector(matrix.Get(2), matrix.Get(6), matrix.Get(10), matrix.Get(14));
            oneParametrization = vector * new Vector(matrix.Get(3), matrix.Get(7), matrix.Get(11), matrix.Get(15));
            return new ParametrizationVector(xParametrization, yParametrization, zParametrization, oneParametrization);
        }

        /// <summary>
        /// Předpoklám, že všechny složky parametrizace jsou stejného typu
        /// </summary>
        public ParametrizationVector SubstituteForU(double u) 
        {
            return new ParametrizationVector(
                _xParametrization.SubstituteForU(u), 
                _yParametrization.SubstituteForU(u),
                _zParametrization.SubstituteForU(u),
                _oneParametrization);
        }

        public ParametrizationVector SubstituteForV(int v)
        {
            return new ParametrizationVector(
                _xParametrization.SubstituteForV(v),
                _yParametrization.SubstituteForV(v),
                _zParametrization.SubstituteForV(v),
                _oneParametrization);
        }
    }
}

