﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Geometry.GraphicObjects.Intersections
{
    public class QuadraticV : IParameterVParametrization
    {
        private Expression _a;
        private Expression _b;
        private Expression _c;
        private Expression _discriminant;


        public QuadraticV(Expression a, Expression b, Expression c) 
        {
            _a = a;
            _b = b;
            _c = c;
            _discriminant = (b * b - (4 * a * c));
        }

        public Expression GetA()
        {
            return _a;
        }
        public Expression GetB()
        {
            return _b;
        }
        public Expression GetC()
        {
            return _c;
        }

        public override IParameterVParametrization EliminateZeros()
        {
            QuadraticV retval = new QuadraticV(_a.EliminateZeros(), _b.EliminateZeros(), _c.EliminateZeros());
            retval._discriminant = _discriminant.EliminateZeros();
            return retval;
        }

        public override double GetVvalue(double u, int sign)
        {
            double d = _discriminant.GetValueAt(u);             
            double localA = _a.GetValueAt(u);
            double localB = _b.GetValueAt(u);
            double localC = _c.GetValueAt(u);
            double v =  (-localB + sign * Math.Sqrt(d)) / (2 * localA);
            return v;
        }

        /// <summary>
        /// Diskriminat se položí nule a vyřeší se vzniklá rovnice. 
        /// Takto získáme definiční obor..
        /// Pokud vykreslujeme křivku chceme abychom s ohledem na nepřesnosti výpočtu 
        /// pokryli maximální možný rozsah parametru křivky.
        /// Proto, pokud je hodnota diskriminantu menší jak nula předpokládám, že je přesně nula.
        /// </summary>
        public double GetVvalueConvertDiskriminantNegativity(double u, int sign) 
        {
            double d = _discriminant.GetValueAt(u);
            if (d < 0)
                d = 0;
            double localA = _a.GetValueAt(u);
            double localB = _b.GetValueAt(u);
            double localC = _c.GetValueAt(u);
            return (-localB + sign * Math.Sqrt(d)) / (2 * localA);
        }

        public override string ToFormatStringIgnoreV()
        {
            if (!_discriminant.IsZero() && !_b.IsZero())
            {
                return "L[v=]" +
                "F["
                        + (-1 * _b).ToFormatStringIgnoreV()
                        + "L[" + U.PM + "]"
                        + "S[" + _discriminant.ToFormatStringIgnoreV() + "]"
                   + ";"
                        + (2 * _a).ToFormatStringIgnoreV()
               + "]";
            }
            else if (_discriminant.IsZero() && !_b.IsZero())
            {
                return "L[v=]" +
                    "F["
                            + (-1 * _b).ToFormatStringIgnoreV()
                       + ";"
                            + (2 * _a).ToFormatStringIgnoreV()
                   + "]";
            }
            else if (_discriminant.IsZero() && _b.IsZero())
                return "L[v = 0]";
            else return "L[v=]" +
                "F["
                        + (-1 * _b).ToFormatStringIgnoreV()
                        + "L[" + U.PM + "]"
                        + "S[" + _discriminant.ToFormatStringIgnoreV() + "]"
                   + ";"
                        + (2 * _a).ToFormatStringIgnoreV()
               + "]";
        }
    }
}
