﻿using System;
using System.Collections.Generic;
using GMath;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Planar;

namespace Geometry.GraphicObjects.Intersections
{
    public class QuadricAndQuadricIntersectionRecognizer
    {
        Quadric _P;
        Quadric _Q;
        SimpleRulledQuadric _simpleRuled;
        GColor _color;
        string _name;
        int _ID;


        public QuadricAndQuadricIntersectionRecognizer(Quadric P, Quadric Q, GColor color, string name, int ID)
        {
            _ID = ID;
            _P = P;
            _Q = Q;
            _color = color;
            _name = name;
        }


        public QuadricAndQuadricIntersection GetIntersection() 
        {
            try
            {
                if (_P.AreEqual(_Q))
                    return GetQuadricAsIntersection();
                if (IsSimpleRuled(_P))
                    _simpleRuled = (SimpleRulledQuadric)_P.MakeCopyOfObject(_P.GetID());
                else if (IsSimpleRuled(_Q))
                    _simpleRuled = (SimpleRulledQuadric)_Q.MakeCopyOfObject(_Q.GetID());
                else
                {
                    _simpleRuled = GetSimpleRuledQuadric(_P, _Q);
                    if (_simpleRuled == null)// potreba kvuli zjednoduseni P-xQ=R, spravne ma byt Px-Qy=R.
                        _simpleRuled = GetSimpleRuledQuadric(_Q, _P);
                    if (_simpleRuled == null)
                    {
                        _simpleRuled = GetSimpleRuledQuadricWhenVanishing(_P, _Q);
                        if (_simpleRuled == null)// potreba kvuli zjednoduseni P-xQ=R, spravne ma byt Px-Qy=R.
                            _simpleRuled = GetSimpleRuledQuadricWhenVanishing(_Q, _P);
                        if (_simpleRuled == null)
                        {
                            return new QuadricAndQuadricIntersection(_P, _Q, new EmptySet(_color, _name, _ID), new List<GraphicObject>() { new EmptyIntersection() }, _color, _name, _ID);
                        }
                    }
                }
                QuadricMatrix workMatrix; // one from Pmatrix or Qmatrix
                workMatrix = new QuadricMatrix(_P.GetCommonEquation());
                if (_simpleRuled.AreEqual(_P))
                {
                    workMatrix = new QuadricMatrix(_Q.GetCommonEquation());
                }
                workMatrix = workMatrix.ApplyTransformation(_simpleRuled.GetToCanonicTransformationMatrix());
                QuadricsSimpleRuledParametrizations simpleRuledParametrizations = new QuadricsSimpleRuledParametrizations();
                List<ParametrizationVector> parametrizationVectors = simpleRuledParametrizations.GetParametrization(_simpleRuled);
                List<GraphicObject> sections = new List<GraphicObject>();
                List<GraphicObject> temp;
                foreach (ParametrizationVector parametrizationVector in parametrizationVectors)
                {
                    temp = GetIntersectionSection(parametrizationVector, workMatrix);
                    for (int i = 0; i < temp.Count; i++)
                        sections.Add(temp[i]);
                }
                if (sections.Count == 0)
                    sections.Add(new EmptyIntersection());
                //sections.Add(_simpleRuled);
                return new QuadricAndQuadricIntersection(_P, _Q, _simpleRuled, sections, _color, _name, _ID);
            }
            catch (Exception) 
            {
                return new QuadricAndQuadricIntersection(_P, _Q, null, new List<GraphicObject>() { new EmptyIntersection() },
                    _color, _name, _ID);
            }
        }

        private List<GraphicObject> GetIntersectionSection(ParametrizationVector pVector, QuadricMatrix workMatrix)
        {
            Expression equation = ((pVector * workMatrix) * pVector).Normalize();

            if (equation.IsZero())
                return GetSimpleRuledAsIntersection(pVector);
            if (equation.ContainsVWithExponent(2))
                return GetQuadraticVIS(pVector, equation);
            if (equation.ContainsVWithExponent(1) && !equation.ContainsUTimesVOnly())
                return GetLinearVIS(pVector, equation);
            if (equation.ContainsUTimesVOnly())
                return GetUAndVAreZero(pVector);
            if (equation.ContainsUDivideVOnly())
                return GetVIsZero(pVector);
            return GetNoVIS(pVector, equation);
        }

        private List<GraphicObject> GetQuadraticVIS(ParametrizationVector pVector, Expression equation)
        {
            List<Expression> quadraticEquation = equation.GetQuadraticEquationInV();
            Expression a = quadraticEquation[0];
            Expression b = quadraticEquation[1];
            Expression c = quadraticEquation[2];
            Expression discriminant = (b * b - 4 * a * c);
            QuadraticV vParametrization = new QuadraticV(a, b, c);
            QuarticEquation quarticEquation = discriminant.GetQuarticEquationInU();
            List<double> discriminantRoots = quarticEquation.SolveGetAllSetsOfRootsReal();
            List<double> denominatorRoots = Program.DistinctInLowTolerance(a.GetQuarticEquationInU().Solve());
            DefinitionInterval definitionInterval = GetDefinitionInterval(discriminantRoots, discriminant, denominatorRoots);
            //for (int i = 0; i < definitionInterval.GetIntervals().Count; i++)
            //    if (definitionInterval.GetIntervals()[i].Belongs(17000) && definitionInterval.GetIntervals().Count == 2)
            //        GetDefinitionInterval(discriminantRoots, discriminant, denominatorRoots);
            //GetDefinitionInterval(discriminantRoots, discriminant, denominatorRoots);            
            List<GraphicObject> retvalIntersections = new List<GraphicObject>();
            retvalIntersections = GetIntersectionPartsWhenDenominatorIsZero(pVector, denominatorRoots, equation);
            retvalIntersections.Add(new CurveAsQuadricsIntersection(pVector, vParametrization, definitionInterval, _simpleRuled));
            return retvalIntersections;
        }
        private List<GraphicObject> GetLinearVIS(ParametrizationVector pVector, Expression equation)
        {
            List<Expression> linearEquation = equation.GetLinearEquationInV();
            Expression numerator = linearEquation[1];
            Expression denominator = linearEquation[0];
            List<double> denominatorRoots = denominator.GetQuarticEquationInU().Solve();
            List<double> minusValues = Program.DistinctInTolerance(denominatorRoots);
            // pokud je simple ruled hyperbolicky cylinder, pak do jeho denicniho oboru nikdy nepatri nula.
            if (_simpleRuled is HyperbolicCylinder)
                minusValues.Add(0);
            DefinitionInterval definitionInterval = new DefinitionInterval(
                new List<Interval>() { new Interval(double.NegativeInfinity, double.PositiveInfinity) },
                minusValues);
            LinearV vParametrization = new LinearV(-numerator, denominator);

            List<GraphicObject> retvalIntersections = new List<GraphicObject>();
            retvalIntersections = GetIntersectionPartsWhenDenominatorIsZero(pVector, denominatorRoots, equation);
            retvalIntersections.Add(new CurveAsQuadricsIntersection(pVector, vParametrization, definitionInterval, _simpleRuled));
            return retvalIntersections;

        }
        private List<GraphicObject> GetNoVIS(ParametrizationVector pVector, Expression equation)
        {
            QuarticEquation quarticEquation = equation.GetQuarticEquationInU();
            if (!quarticEquation.ContainsVariable())
                if (!Program.IsEqualInTolerance(quarticEquation.GetE(), 0))
                    return new List<GraphicObject>() { };
                else GetSimpleRuledAsIntersection(pVector);
            List<double> equationSolutions = quarticEquation.Solve();
            return GetVParamIS(pVector, equationSolutions);
        }
        private List<GraphicObject> GetUAndVAreZero(ParametrizationVector pVector)
        {
            if (_simpleRuled is HyperbolicCylinder)
                return GetVIsZero(pVector);
            DefinitionInterval definitionInterval =
                new DefinitionInterval(new Interval(double.NegativeInfinity, double.PositiveInfinity));
            return new List<GraphicObject>()
            {
                new CurveAsQuadricsIntersection(pVector.SubstituteForV(0), new ConstantV(), definitionInterval, _simpleRuled),
                new CurveAsQuadricsIntersection(pVector.SubstituteForU(0), new ConstantU(), definitionInterval, _simpleRuled)
            };
        }
        private List<GraphicObject> GetVIsZero(ParametrizationVector pVector)
        {
            DefinitionInterval definitionInterval =
                new DefinitionInterval(new Interval(double.NegativeInfinity, double.PositiveInfinity), new List<double>() { 0 });
            return new List<GraphicObject>()
            {
                new CurveAsQuadricsIntersection(pVector.SubstituteForV(0), new ConstantV(), definitionInterval, _simpleRuled)
            };
        }
        private List<GraphicObject> GetSimpleRuledAsIntersection(ParametrizationVector pVector) 
        {
            DoublePlan plan = GetPlanFromSimpleRuledParametrization(pVector);
            return new List<GraphicObject>() { plan };
        }
        private List<GraphicObject> GetEmptyIntersection()
        {
            return new List<GraphicObject> { new EmptyIntersection() };
        }
        private List<GraphicObject> GetVParamIS(ParametrizationVector pVector, List<double> uSolutions) 
        {
            List<GraphicObject> retval = new List<GraphicObject>();
            uSolutions = Program.DistinctInTolerance(uSolutions);
            // pokud je simple ruled hyperbolicky cylinder, pak do jeho denicniho oboru nikdy nepatri nula.
            // takze, v tom bode nemuzu hledat nejaky bod
            if (_simpleRuled is HyperbolicCylinder)
                for (int i = uSolutions.Count - 1; i >= 0; i--)
                    if (Program.IsEqualInTolerance(uSolutions[i], 0))
                        uSolutions.RemoveAt(i);
            foreach (double u in uSolutions)
            {
                retval.Add(new CurveAsQuadricsIntersection(pVector.SubstituteForU(u),
                    new ConstantU(),
                    new DefinitionInterval(new Interval(double.NegativeInfinity, double.PositiveInfinity)), _simpleRuled));
            }
            return retval;
        }
        private QuadricAndQuadricIntersection GetQuadricAsIntersection()
        {
            Quadric copy = (Quadric)_P.MakeCopyOfObject(_ID);
            return new QuadricAndQuadricIntersection(_P, _Q, new EmptySet(_color, _name, _ID),
                new List<GraphicObject>() { copy }, _color, _name, _ID);
        }
        private DoublePlan GetPlanFromSimpleRuledParametrization(ParametrizationVector parametrizationVector)
        {
            DoublePlan doublePlan = QuadricsSimpleRuledParametrizations.GetPlanFromParametrizationOfCompoundQuadric(parametrizationVector, _simpleRuled);
            if (doublePlan == null)
                throw new Exception("Neznama kvadrika... Nelze ji rozložit na roviny.");
            return doublePlan;
        }

        /// <summary>
        /// Když výraz v * M * v po vyjádření v, obsahuje zlomek jehož čitatel 
        /// může být nulový pro jisté hodnoty U, pak pro tuto situaci, 
        /// dosadím do výrazu v * M * v, tyto hodnoty U.
        /// 
        /// Pokud vzniklá rovnice má netriviální řešení, vyřeším a z honot U a V po dosazení získám body.
        /// Pokud má triviální řešení, může parametr V nabývat libovolných hodnot.
        /// DenominatorRoots musí být distinct.
        /// 
        /// pokud je simple ruled hyperbolicky cylinder, pak do jeho denicniho oboru nikdy nepatri nula.
        /// </summary>
        private List<GraphicObject> GetIntersectionPartsWhenDenominatorIsZero(ParametrizationVector pVector, List<double> denominatorRoots, Expression equation)
        {
            // pokud je simple ruled hyperbolicky cylinder, pak do jeho denicniho oboru nikdy nepatri nula.
            // takze, v tom bode nemuzu hledat nejaky bod
            if (_simpleRuled is HyperbolicCylinder)
                for (int i = denominatorRoots.Count - 1; i >= 0; i--)
                    if (Program.IsEqualInTolerance(denominatorRoots[i], 0))
                        denominatorRoots.RemoveAt(i);

            List<GraphicObject> retvalIntersections = new List<GraphicObject>();
            double denominatorRoot;
            for (int i = 0; i < denominatorRoots.Count; i++)
            {
                denominatorRoot = denominatorRoots[i];
                Expression e = (Expression)equation.SubstituteForU(denominatorRoot);
                if (e.IsZero())
                {
                    retvalIntersections.Add(new CurveAsQuadricsIntersection(pVector.SubstituteForU(denominatorRoot),
                        new ConstantU(), new DefinitionInterval(new Interval(double.NegativeInfinity, double.PositiveInfinity)), _simpleRuled));
                }
                else
                {
                    List<GraphicObject> temp = GetPointsWhenDenominatorIsZero(pVector.SubstituteForU(denominatorRoot), e);
                    foreach (GraphicObject o in temp)
                        retvalIntersections.Add(o);
                }
            }
            return retvalIntersections;
        }
        /// <summary>
        /// Když výraz v * M * v po vyjádření v, obsahuje zlomek jehož čitatel 
        /// může být nulový pro jisté hodnoty U, pak pro tuto situaci, 
        /// dosadím do výrazu v * M * v, tyto hodnoty U.
        /// 
        /// Pokud vzniklá rovnice má netriviální řešení, vyřeším a z honot U a V po dosazení získám body.
        /// </summary>
        private List<GraphicObject> GetPointsWhenDenominatorIsZero(ParametrizationVector pVector, Expression substitutedEquation)
        {
            List<GraphicObject> retvalPoints = new List<GraphicObject>();
            List<double> vSolutions;
            if (substitutedEquation.ContainsVWithExponent(2))
            {
                List<Expression> qEquation = substitutedEquation.GetQuadraticEquationInV();
                QuadraticEquation quadraticEquation =
                    new QuadraticEquation(qEquation[0].GetValueAt(1, 1), qEquation[1].GetValueAt(1, 1), qEquation[2].GetValueAt(1, 1));
                vSolutions = quadraticEquation.Solve();
            }
            if (substitutedEquation.ContainsVWithExponent(1))
            {
                List<Expression> lEquation = substitutedEquation.GetLinearEquationInV();
                LinearEquation linearEquation =
                    new LinearEquation(lEquation[0].GetValueAt(1, 1), lEquation[1].GetValueAt(1, 1));
                vSolutions = linearEquation.Solve();
            }
            else vSolutions = new List<double>();
            Vector coordinates;
            foreach (double v in vSolutions)
            {
                coordinates = pVector.GetValueAt(1, v) * _simpleRuled.GetTransformationMatrixAlgebraic().ComputeInverseMatrix();
                double substitutionToA = Math.Abs(_P.GetCommonEquation().Substitute(coordinates));
                double substitutionToB = Math.Abs(_Q.GetCommonEquation().Substitute(coordinates));

                GPoint intersection = new GPoint(coordinates, _color, _name, _ID);
                retvalPoints.Add(intersection);
            }
            return retvalPoints;
        }
        private DefinitionInterval GetDefinitionInterval(List<double> discriminantSolutions, Expression discriminant, List<double> denominatorRoots)
        {
            List<double> discriminantSolutionsCopy = new List<double>();
            // pokud je simple ruled hyperbolicky cylinder, pak do jeho denicniho oboru nikdy nepatri nula.
            List<double> minusValues = new List<double>();
            foreach (double d in denominatorRoots)
                minusValues.Add(d);
            if (_simpleRuled is HyperbolicCylinder)
                minusValues.Add(0);

            foreach (double d in discriminantSolutions)
            {
                discriminantSolutionsCopy.Add(d);
                discriminantSolutionsCopy.Add(-d);
            }
            foreach (double d in minusValues)
                discriminantSolutionsCopy.Add(d);
            discriminantSolutionsCopy.Add(double.NegativeInfinity);
            discriminantSolutionsCopy.Add(double.PositiveInfinity);
            discriminantSolutionsCopy.Sort();
            Interval temp;
            List<Interval> intervals = new List<Interval>();
            for (int i = 0; i < discriminantSolutionsCopy.Count - 1; i++)
            {
                temp = TestDiscriminantRoot(discriminantSolutionsCopy[i], discriminantSolutionsCopy[i + 1], discriminant);
                if (temp != null)
                    intervals = Interval.Union(intervals, temp);
            }
            List<double> isolatedPoints = new List<double>();
            bool isolated = true;
            discriminantSolutionsCopy = Program.DistinctInLowTolerance(discriminantSolutions);
            for (int j = 0; j < discriminantSolutionsCopy.Count; j++)
            {
                isolated = true;
                if (!Program.IsEqualInToleranceIntersection(discriminant.GetValueAt(discriminantSolutionsCopy[j], 1), 0))
                    continue;
                for (int i = 0; i < intervals.Count; i++)
                {
                    if (intervals[i].Belongs(discriminantSolutionsCopy[j]) ||
                        Program.IsEqualInToleranceIntersection(intervals[i].GetA(), discriminantSolutionsCopy[j]) ||
                        Program.IsEqualInToleranceIntersection(intervals[i].GetB(), discriminantSolutionsCopy[j]))
                        isolated = false;
                }
                if (isolated)
                    isolatedPoints.Add(discriminantSolutionsCopy[j]);
            }
            return new DefinitionInterval(intervals, isolatedPoints, minusValues);
        }
        private Interval TestDiscriminantRoot(double left, double right, Expression discriminant)
        {
            if (discriminant.IsZero())
                return new Interval(left, right);

            double middleValue = (left + right) / 2.0;

            if (left == double.NegativeInfinity)
                middleValue = right - 1000;
            if (right == double.PositiveInfinity)
                middleValue = left + 1000;
            if (left == double.NegativeInfinity && right == double.PositiveInfinity)
                middleValue = 0;

            if (discriminant.GetValueAt(middleValue) >= 0)
                return new Interval(left, right);
            else return null;
        }
        private List<double> Linearize(List<List<double>> list)
        {
            List<double> retval = new List<double>();
            for (int i = 0; i < list.Count; i++)
                for (int j = 0; j < list[i].Count; j++)
                    retval.Add(list[i][j]);
            return retval;
        }



        private SimpleRulledQuadric GetSimpleRuledQuadric(Quadric p, Quadric q) 
        {
            QuadricMatrix Pmatrix = new QuadricMatrix(p.GetCommonEquation());
            QuadricMatrix Qmatrix = new QuadricMatrix(q.GetCommonEquation());
            Matrix upperP = Pmatrix.GetSubMatrix(3, 3);
            Matrix upperQ = Qmatrix.GetSubMatrix(3, 3);
            CubicEquation lambdaCubic = GetQuadricsPencilDeterminant(upperP, upperQ);
            List<double> lambda = lambdaCubic.Solve();
            Quadric temp = null;
            for (int i = 0; i < lambda.Count; i++)
            {
                temp = GetSimpleRuledQuadric(Pmatrix, Qmatrix, lambda[i]);
                if (temp != null && IsSimpleRuled(temp))
                    return (SimpleRulledQuadric)temp;
            }
            return null;
        }
        private SimpleRulledQuadric GetSimpleRuledQuadricWhenVanishing(Quadric p, Quadric q) 
        {
            QuadricMatrix Pmatrix = new QuadricMatrix(p.GetCommonEquation());
            QuadricMatrix Qmatrix = new QuadricMatrix(q.GetCommonEquation());
            Quadric temp = null;
            QuarticEquation lambdaQuartic = GetQuadricsPencilDeterminant(Pmatrix, Qmatrix);
            List<double> lambda = lambdaQuartic.Solve();
            for (int i = 0; i < lambda.Count; i++)
            {
                temp = GetSimpleRuledQuadric(Pmatrix, Qmatrix, lambda[i]);
                if (temp != null && IsSimpleRuled(temp))
                    return (SimpleRulledQuadric)temp;
            }
            return null;
        }
        private bool IsSimpleRuled(Quadric quadric) 
        {
            return (quadric is ImaginarCrossingPlans || quadric is DoublePlan || quadric is CrossingPlans ||
                quadric is ParallelPlans || quadric is HyperbolicCylinder || quadric is ParabolicCylinder ||
                quadric is HyperbolicParaboloid);
        }
        private Quadric GetSimpleRuledQuadric(QuadricMatrix a, QuadricMatrix b, double lambda) 
        {
            QuadricMatrix simpleRuled = a - (lambda * b);
            string matrix = simpleRuled.ToString();
            QuadricEquation equation = new QuadricEquation(simpleRuled);
            if (equation.ISQuadricsEquationInTolerance())
            {
                QuadricRecognizer quadricRecognizer = new QuadricRecognizer(equation, new GColor(1, 1, 1), "", _ID);
                Quadric quadric = quadricRecognizer.RecognizeQuadric();
                return quadric;
            }else return TryPlanAsSimpleRulled(equation);
        }
        private Quadric TryPlanAsSimpleRulled(QuadricEquation equation) 
        {
            double a = equation.GetA();
            double b = equation.GetB();
            double c = equation.GetC();
            double d = equation.GetD();
            double e = equation.GetE();
            double f = equation.GetF();
            double g = equation.GetG();
            double h = equation.GetH();
            double k = equation.GetK();
            double l = equation.GetL();
            bool isPlan = true;
            isPlan = isPlan && Program.IsEqualInTolerance(a, 0);
            isPlan = isPlan && Program.IsEqualInTolerance(b, 0);
            isPlan = isPlan && Program.IsEqualInTolerance(c, 0);
            isPlan = isPlan && Program.IsEqualInTolerance(d, 0);
            isPlan = isPlan && Program.IsEqualInTolerance(e, 0);
            isPlan = isPlan && Program.IsEqualInTolerance(f, 0);
            if (!isPlan)
                return null;
            isPlan = false;
            isPlan = isPlan || !Program.IsEqualInTolerance(g, 0);
            isPlan = isPlan || !Program.IsEqualInTolerance(h, 0);
            isPlan = isPlan || !Program.IsEqualInTolerance(k, 0);
            if (!isPlan)
                return null;
            
            Planar.Plan plan =
                new Planar.Plan(new PlanEquation(g, h, k, l),new GColor(0,0,0.5),"", _ID);

            QuadricMatrix planCanonicMatrix = 
                new QuadricMatrix(new double[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0 });
            QuadricEquation newEquation = 
                new QuadricEquation(planCanonicMatrix.ApplyTransformation(plan.GetTransformationMatrixAlgebraic()));

            return new QuadricRecognizer(newEquation, new GColor(1,0,0), "ugh", _ID).RecognizeQuadric();
        }


        private QuarticEquation GetQuadricsPencilDeterminant(QuadricMatrix p, QuadricMatrix q)
        {
            // udelam rozvoj podle prvniho sloupce
            LinearEquation a = new LinearEquation(-q.GetA(), p.GetA());
            LinearEquation d = new LinearEquation(-q.GetD(), p.GetD());
            LinearEquation e = new LinearEquation(-q.GetE(), p.GetE());
            LinearEquation g = new LinearEquation(-q.GetG(), p.GetG());
            QuarticEquation e1 = GetQuadricsPencilDeterminant(p.GetSubMatrix(0, 0), q.GetSubMatrix(0, 0)) * a;
            QuarticEquation e2 = GetQuadricsPencilDeterminant(p.GetSubMatrix(1, 0), q.GetSubMatrix(1, 0)) * d;
            QuarticEquation e3 = GetQuadricsPencilDeterminant(p.GetSubMatrix(2, 0), q.GetSubMatrix(2, 0)) * e;
            QuarticEquation e4 = GetQuadricsPencilDeterminant(p.GetSubMatrix(3, 0), q.GetSubMatrix(3, 0)) * g;
            return e1 - e2 + e3 - e4; // z vlatnosti rozvoje determinantu.
        }
        private CubicEquation GetQuadricsPencilDeterminant(Matrix upMatrixP, Matrix upMatrixQ) 
        {
            double[] p = upMatrixP.ToDoubleArray();
            double[] q = upMatrixQ.ToDoubleArray();
            CubicEquation m048 = DeterminantSubPart(p[0], p[4], p[8], q[0], q[4], q[8]);
            CubicEquation m156 = DeterminantSubPart(p[1], p[5], p[6], q[1], q[5], q[6]);
            CubicEquation m372 = DeterminantSubPart(p[3], p[7], p[2], q[3], q[7], q[2]);
            CubicEquation m642 = DeterminantSubPart(p[6], p[4], p[2], q[6], q[4], q[2]);
            CubicEquation m750 = DeterminantSubPart(p[7], p[5], p[0], q[7], q[5], q[0]);
            CubicEquation m318 = DeterminantSubPart(p[3], p[1], p[8], q[3], q[1], q[8]);
            CubicEquation x = m048 + m156 + m372 - m642 - m750 - m318;
            //List<double> solutions = x.Solve();
            //double temp,temp2;
            //foreach (double d in solutions)
            //{
            //    temp = x.Substitute(d);
            //    temp2 = (upMatrixP - d * upMatrixQ).Determinant3();
            //    temp = temp;
            //}

            return m048 + m156 + m372 - m642 - m750 - m318;
        }
        /// <summary>
        /// Vraci koeficienty kubicke rovnice
        /// (a1 - a2x)(b1 - b2x)(c1 - c2x)
        /// </summary>
        /// <returns></returns>
        private CubicEquation DeterminantSubPart(double a1, double b1, double c1, double a2, double b2, double c2) 
        {
            double abs = (a1 * b1 * c1);
            double lambda = -(a1 * b2 * c1 + a2 * b1 * c1 + a1 * b1 * c2);
            double lambda2 = (a1 * b2 * c2 + a2 * b2 * c1 + a2 * b1 * c2);
            double lambda3 = -(a2*b2*c2);
            return new CubicEquation(lambda3, lambda2, lambda, abs);
        }
    }
}
