﻿using Geometry.GraphicObjects.Equations;
using GMath;

namespace Geometry.GraphicObjects.Matrixes
{
    /// <summary>
    /// a d g
    /// d b h
    /// g h l
    /// </summary>
    public class ConicMatrix:Matrix
    {
        public const int Height = 3;
        public const int Width = 3;

        public ConicMatrix(double[] values) 
        {
            _height = Height;
            _width = Width;
            _values = values;
        }

        public ConicMatrix(Matrix matrix)
        {
            _height = Height;
            _width = Width;
            _values = matrix.ToDoubleArray();
        }

        public ConicMatrix(ConicEquation conicEquation) 
        {
            _height = Height;
            _width = Width;
            _values = new double[_width*_height];
            _values[0] = conicEquation.GetA();
            _values[1] = conicEquation.GetD()/2;
            _values[2] = conicEquation.GetG()/2;

            _values[3] = conicEquation.GetD()/2;
            _values[4] = conicEquation.GetB();
            _values[5] = conicEquation.GetH()/2;

            _values[6] = conicEquation.GetG()/2;
            _values[7] = conicEquation.GetH()/2;
            _values[8] = conicEquation.GetL();
        }

        public ConicMatrix ApplyTransformation(TransformationMatrix transformation)
        {
            return new ConicMatrix(
                (transformation.GetSubMatrix(2,2) * this) * transformation.GetSubMatrix(2,2).TransponMatrix());
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public double GetA() { return _values[0]; }

        public double GetB() { return _values[4]; }

        public double GetD() { return _values[1]; }

        public double GetG() { return _values[2]; }

        public double GetH() { return _values[5]; }

        public double GetL() { return _values[8]; }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static ConicMatrix operator -(ConicMatrix a, ConicMatrix b)
        {
            return new ConicMatrix(new Matrix(Height, Width, a._values) - new Matrix(Height, Width, b._values));
        }

        public static ConicMatrix operator *(double koef, ConicMatrix matrix)
        {
            return new ConicMatrix(koef * new Matrix(Height, Width, matrix._values));
        }
    }
}
