﻿using Geometry.GraphicObjects.Equations;
using GMath;

namespace Geometry.GraphicObjects.Matrixes
{
    public class QuadricMatrix:Matrix
    {
        public const int Width = 4;
        public const int Height = 4;

        public QuadricMatrix(double[] values) 
        {
            _height = Height;
            _width = Width;
            _values = values;
        }

        public QuadricMatrix(Matrix matrix)
        {
            _height = Height;
            _width = Width;
            _values = matrix.ToDoubleArray();
        }

        public QuadricMatrix(QuadricEquation equation) 
        {
            _height = Height;
            _width = Width;
            _values = new double[]
            {
                equation.Get(0),equation.Get(3)/2.0,equation.Get(4)/2.0,equation.Get(6)/2.0,
                equation.Get(3)/2.0,equation.Get(1),equation.Get(5)/2.0,equation.Get(7)/2.0,
                equation.Get(4)/2.0,equation.Get(5)/2.0,equation.Get(2),equation.Get(8)/2.0,
                equation.Get(6)/2.0,equation.Get(7)/2.0,equation.Get(8)/2.0,equation.Get(9)
            };
        }

        public QuadricMatrix ApplyTransformation(Matrix transformation)
        {
            return new QuadricMatrix((transformation * this) * transformation.TransponMatrix());
        }

        public double GetA()
        {
            return _values[0];
        }
        public double GetB()
        {
            return _values[5];
        }
        public double GetC()
        {
            return _values[10];
        }
        public double GetD()
        {
            return _values[1];
        }
        public double GetE()
        {
            return _values[2];
        }
        public double GetF()
        {
            return _values[6];
        }
        public double GetG()
        {
            return _values[3];
        }
        public double GetH()
        {
            return _values[7];
        }
        public double GetK()
        {
            return _values[11];
        }
        public double GetL()
        {
            return _values[15];
        }

        public static QuadricMatrix operator -(QuadricMatrix a, QuadricMatrix b) 
        {
            return new QuadricMatrix(new Matrix(Height, Width, a._values) - new Matrix(Height, Width, b._values));
        }

        public static QuadricMatrix operator *(double koef, QuadricMatrix matrix) 
        {
            QuadricMatrix retval = new QuadricMatrix((koef) * new Matrix(Height, Width, matrix._values));
            //retval.Set(0, retval.Get(0) * 2.0);
            //retval.Set(5, retval.Get(5) * 2.0);
            //retval.Set(10, retval.Get(10) * 2.0);
            //retval.Set(15, retval.Get(15) * 2.0);
            return retval;
        }
    }
}
