﻿using System;
using Tao.OpenGl;
using Geometry.GraphicObjects.Equations;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Planar
{
    public class GPoint:WithParametricEquationObject
    {


        public static string _parametricXFormatString = "L[x = !m]";
        public static string _parametricYFormatString = "L[y = !n]";
        public static string _parametricZFormatString = "L[z = !o]";

        public static string _canonicParametricXFormatString = "L[x = 0]";
        public static string _canonicParametricYFormatString = "L[y = 0]";
        public static string _canonicParametricZFormatString = "L[z = 0]";

        public static string _wordDescriptionFirstCase = "Bod";
        public static string _wordDescriptionSecondCase = "Bodu";
        public static string _wordDescriptionFourthCase = "Bod";

        private double _radius = 0.2;

        public GPoint(Vector coordinates, GColor color, string name, int ID) 
        {
            CenterParametrization center = 
                new CenterParametrization(coordinates.GetX(), coordinates.GetY(), coordinates.GetZ());
            _parametricEquation = new PointParametricEquation();
            base._color = color;
            base._name = name;
            base._ID = ID;
            base.SetWordDescriptionFirstCase(_wordDescriptionFirstCase);
            base.SetWordDescriptionSecondCase(_wordDescriptionSecondCase);
            base.SetWordDescriptionFourthCase(_wordDescriptionFourthCase);
            base._parametricEquationFormatString = new ParametricEquationsFormatString(_parametricXFormatString,
                _parametricYFormatString, _parametricZFormatString);
            TransformationMatrix rotationMatrix = new TransformationMatrix(4);
            SetTransformationMatrixAlgebraic(GetTransformationMatrixAlgebraic(rotationMatrix, center));
        }



        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            //Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
            double quality = 20;
            double x, y, z;
            double step = Math.PI / quality;
            Gl.glBegin(Gl.GL_TRIANGLE_STRIP);

            for (double i = -Math.PI / 2; i < Math.PI / 2; i += step)
            {
                for (double j = -Math.PI; j <= Math.PI; j += step)
                {
                    x = (Math.Cos(i) * Math.Sin(j));
                    y = (Math.Sin(i));//*sin((double)j);
                    z = (Math.Cos(i) * Math.Cos(j));
                    Gl.glNormal3d(-x, -y, -z);
                    x = _radius * x;
                    y = _radius * y;
                    z = _radius * z;
                    Gl.glVertex3d(x, y, z);
                    x = (Math.Cos(i + step) * Math.Sin(j));
                    y = (Math.Sin(i + step));//*sin((double)j);
                    z = (Math.Cos(i + step) * Math.Cos(j));
                    Gl.glNormal3d(-x, -y, -z);
                    x = _radius * x;
                    y = _radius * y;
                    z = _radius * z;
                    Gl.glVertex3d(x, y, z);
                }
            }
            ///Gl.glNormal3d(0, 0, 0);
            Gl.glEnd();
            Gl.glEndList();
        }



        public override WithParametricEquationObject MakeCopyOfObject(KoefsAndCenterPair newKoefsAndCenter, int ID)
        {
            CenterParametrization center = newKoefsAndCenter.GetCenterParametrization();
            GPoint point = new GPoint(center.GetReprezentant(), this.GetColor(), this.GetName(), ID);
            point.CopyDrawableSlots(point, this);
            return point;
        }

        public override WithParametricEquationObject MakeCopyOfObject(int ID)
        {
            CenterParametrization centerCopy =
                new CenterParametrization(GetCenterInStandardBasis());
            return MakeCopyOfObject(new KoefsAndCenterPair(null, centerCopy), ID);
        }

        protected override void ActualizeEquations()
        {
            _parametricEquationFormatString.SubstituteToFormatStrings(null, GetCenterInStandardBasis());
        }
    }
}
