﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Tao.OpenGl;
using GMath;

namespace Geometry.GraphicObjects.Planar
{
    public static class SuperPlan
    {
        private static int step = 5;
        private static int steps = 12;

        /// <summary>
        /// Vytvori grafickou reprezentaci roviny s rovnici z = 0;
        /// x = u;
        /// y = v;
        /// </summary>
        /// <param name="scene"></param>
        public static void BuildPlanDisplayList(Scene scene, double uMin, double uMax, double vMin, double vMax)
        {
            Gl.glEnable(Gl.GL_LIGHTING);
            for (double u = uMin; u < uMax; u += step)
            {
                if (u > uMax - step)
                    u = uMax - step;
                Gl.glBegin(Gl.GL_TRIANGLE_STRIP);
                for (double v = vMin; v <= vMax; v += step)
                {
                    Gl.glVertex3d(u, v, 0);
                    Gl.glVertex3d(u + step, v, 0);
                }
                Gl.glEnd();
                if (u == uMax - step)
                    break;
            }
            //double uMiddle = (uMin + uMax)/2.0;
            //double vMiddle = (vMin + vMax)/2.0;
            //Gl.glBegin(Gl.GL_LINE_LOOP);
            //Gl.glVertex3d(uMin, vMin, 0);
            //Gl.glVertex3d(uMin, vMax, 0);
            //Gl.glVertex3d(uMax, vMax, 0);
            //Gl.glVertex3d(uMax, vMin, 0);
            //Gl.glEnd();
        }

        /// <summary>
        /// Vytvori grafickou reprezentaci roviny ktera vznikne rotaci roviny x = 0,
        /// Okolo osy Z o libovolny uhel.
        /// Tedy predpoklad, ze rovina prochazi pocatkem.
        /// <param name="scene"></param>
        public static void BuildPlanDisplayList(Scene scene, Vector a, Vector b, double vMin, double vMax)
        {
            if (a.GetX() > b.GetX())
            {
                Vector temp = a;
                a = b;
                b = temp;
            }
            Vector delta = (b - a) / steps;
            Gl.glEnable(Gl.GL_LIGHTING);
            Vector v1 = a;
            Vector v2;
            for (int i = 0; i < 1; i += 0)
            {
                v2 = v1 + delta;
                if (v2.GetX() >= b.GetX())
                {
                    v2 = b;
                    i = 1;
                }
                Gl.glBegin(Gl.GL_TRIANGLE_STRIP);
                for (double v = vMin; v <= vMax + step; v += step)
                {
                    Gl.glVertex3d(v1.GetX(), v1.GetY(), Math.Min(v, vMax));
                    Gl.glVertex3d(v2.GetX(), v2.GetY(), Math.Min(v, vMax));
                }
                v1 = v2;
                Gl.glEnd();
            }
        }
    }
}
