﻿using System;
using System.Collections.Generic;
using GMath;

namespace Geometry.GraphicObjects.Equations
{
    public class CommonEquation
    {  
        protected List<double> _values;

        protected CommonEquation() { }
        public CommonEquation(List<double> values)
        {
            _values = values;
        }

        public virtual CommonEquation Normalize() 
        {
            double d = Program.GetNormalizingKoeficient(_values);
            return d * this;
        }


        /// <summary>
        /// Předpokládá porovnávání Obecných rovnic stejného typu.
        /// Jinak může řící že equationy o stejném počtu prvků ale různých typů jsou stejné.
        /// Neprovadim typovou kontrolu, protoze se pouziva i na porovnani zdedeneho CommonEquationu 
        /// s jeho kopii, ktera ale neni specificka, je cisty CommonEquation  
        /// </summary>
        public static bool AreEqualInTolerance(CommonEquation a, CommonEquation b) 
        {
            if (a.GetMembersCount() != b.GetMembersCount())
                return false;
            bool equal = true;
            for (int i = 0; i < a._values.Count; i++) 
            {
                equal = equal && Program.IsEqualInToleranceTextBoxLoterance(a._values[i], b._values[i]);
            }
            return equal;
        }



        public List<double> CopyValues(CommonEquation src)
        {
            List<double> values = new List<double>();
            foreach (double d in src._values)
                values.Add(d);
            return values;
        }

        /// <summary>
        /// Modification modifies original equation.
        /// </summary>
        public List<double> GetValues() { return _values; }

        /// <summary>
        /// Return count of members of equation
        /// </summary>
        public int GetMembersCount() { return _values.Count; }

        public double Get(int i)
        {
            if (i < 0 || i > _values.Count - 1)
                throw new Exception("Index je mimo rozsah indexu clenu rovnice.");
            return _values[i];
        }

        public void Set(int i, double value)
        {
            if (i < 0 || i > _values.Count - 1)
                throw new Exception("Index je mimo rozsah indexu clenu rovnice.");
            _values[i] = value;
        }

        public CommonEquation MultiplyByMinusOne()
        {
            List<double> values = new List<double>();
            for (int i = 0; i < _values.Count; i++)
            {
                values.Add(-_values[i]);
            }
            return new CommonEquation(values);
        }

        public static CommonEquation operator*(double koeficient, CommonEquation equation)
        {
            List<double> values = new List<double>();
            foreach (double d in equation._values)
                values.Add(koeficient * d);
            return new CommonEquation(values);
        }

        public int GetIndexOfMinValue()
        {
            double min = Int16.MaxValue;
            int index = 0;
            for (int i = 0; i < _values.Count; i++)
            {
                if (!Program.IsEqualInTolerance(_values[i], 0) && Math.Abs(_values[i]) < min)
                {
                    min = _values[i];
                    index = i;
                }
            }
            return index;
        }

        public CommonEquation DivideByMinKoeficient()
        {
            int index = GetIndexOfMinValue();
            double numerator = _values[index];
            if (numerator != 0)
            {
                List<double> values = new List<double>();
                for (int i = 0; i < _values.Count; i++)
                {
                    values.Add(_values[i] / numerator);
                }
                return new CommonEquation(values);
            }
            return this;
        }

        public virtual double Substitute(Vector values) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }

        public virtual double GetA() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetB() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetC() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetD() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetE() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetF() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetG() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetH() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetK() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetL() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }

        public virtual void SetA(double a) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetB(double b) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetC(double c) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetD(double d) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetE(double e) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetF(double f) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetG(double g) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetH(double h) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetK(double k) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetL(double l) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
    }
}
