﻿using System;
using System.Collections.Generic;
using Geometry.GraphicObjects.Matrixes;
using GMath;

namespace Geometry.GraphicObjects.Equations
{   
    /// <summary>
    /// Ax2 + By2 + Cxy + Dx + Ey + F = 0;
    /// </summary>
    public class ConicEquation:CommonEquation
    {
        public const int MembersCount = 6;

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public ConicEquation(CommonEquation src) 
        {
            if (src.GetValues().Count == MembersCount)
                _values = src.GetValues();
            else throw new ArgumentException();
        }
        
        public ConicEquation(ConicEquation src) 
        {
            _values = base.CopyValues(src);
        }

        public ConicEquation(double A, double B, double D, double G, double H, double L) 
        {
            _values = new List<double>() { A, B, D, G, H, L };
        }

        public ConicEquation(List<double> values)
        {
            if (values.Count != MembersCount)
                throw new Exception("Snazite se vytvorit equation kuzelosecky, ktery nema sest clenu.");
            _values = values;
        }

        public ConicEquation(ConicMatrix conicMatrix) 
        {
            _values = new List<double>{
            conicMatrix.GetA(),
            conicMatrix.GetB(),
            conicMatrix.GetD()*2,
            conicMatrix.GetG()*2,
            conicMatrix.GetH()*2,
            conicMatrix.GetL()};
        }

        public override CommonEquation Normalize()
        {
            return new ConicEquation(base.Normalize().GetValues());
        }



        
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////


        public ConicEquation DivideBy(double d)
        {
            List<double> values = new List<double>();
            for (int i = 0; i < _values.Count; i++)
                values.Add(_values[i] / d);
            return new ConicEquation(values);
        }

        public new ConicEquation MultiplyByMinusOne()
        {
            CommonEquation newEquation = base.MultiplyByMinusOne();
            return new ConicEquation(newEquation.GetValues());
        }

        public new ConicEquation DivideByMinKoeficient()
        {
            CommonEquation newEquation = base.DivideByMinKoeficient();
            return new ConicEquation(base._values);
        }



        public override double Substitute(Vector values)
        {
            return
                GetA() * values.GetX() * values.GetX() +
                GetB() * values.GetY() * values.GetY() +
                GetD() * values.GetX() * values.GetY() +
                GetG() * values.GetX() +
                GetH() * values.GetY() +
                GetL();
        }



        public override double GetA() { return _values[0]; }

        public override double GetB() { return _values[1]; }

        public override double GetD() { return _values[2]; }

        public override double GetG() { return _values[3]; }

        public override double GetH() { return _values[4]; }

        public override double GetL() { return _values[5]; }

        public override double GetC()
        {
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetE()
        {
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetF()
        {
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetK()
        {
            throw new Exception("Invalid coeficient wanted.");
        }



        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public override void SetA(double a) { _values[0] = a; }

        public override void SetB(double b) { _values[1] = b; }

        public override void SetD(double d) { _values[2] = d; }

        public override void SetG(double g) { _values[3] = g; }

        public override void SetH(double h) { _values[4] = h; }

        public override void SetL(double l) { _values[5] = l; }

        public override void SetC(double c)
        {
            throw new Exception("Invalid coeficient setting.");
        }
        public override void SetE(double e)
        {
            throw new Exception("Invalid coeficient setting.");
        }
        public override void SetF(double f)
        {
            throw new Exception("Invalid coeficient setting.");
        }
        public override void SetK(double k)
        {
            throw new Exception("Invalid coeficient setting.");
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

    }
}
