﻿using System;
using System.Collections.Generic;
using GMath;

namespace Geometry.GraphicObjects.Equations
{
    public class LineEquation:CommonEquation
    {
        public const int MembersCount = 3;

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public LineEquation(LineEquation src) 
        {
            _values = base.CopyValues(src);
        }

        public LineEquation(double A, double B, double C) 
        {
            _values = new List<double>() { A, B, C};
        }

        public LineEquation(List<double> values)
        {
            if (values.Count != MembersCount)
                throw new Exception("Snazite se vytvorit equation primky, ktery nema ctyri cleny.");
            _values = values;
        }

        public override CommonEquation Normalize()
        {
            return new LineEquation(base.Normalize().GetValues());
        }



        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////



        public new LineEquation MultiplyByMinusOne()
        {
            CommonEquation newEquation = base.MultiplyByMinusOne();
            return new LineEquation(base._values);
        }

        public new LineEquation DivideByMinKoeficient()
        {
            CommonEquation newEquation = base.DivideByMinKoeficient();
            return new LineEquation(base._values);
        }


        public override double Substitute(Vector values)
        {
            return
                GetA() * values.GetX() +
                GetB() * values.GetY() +
                GetC();
        }


        public override double GetA() { return _values[0]; }
        public override double GetB() { return _values[1]; }
        public override double GetC() { return _values[2]; }

        public override double GetD(){
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetG(){
            throw new Exception("Invalid coeficient wanted.");
        }

        public override double GetH(){
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetL(){
            throw new Exception("Invalid coeficient wanted.");
        }


        public override double GetE()
        {
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetF()
        {
            throw new Exception("Invalid coeficient wanted.");
        }
        public override double GetK()
        {
            throw new Exception("Invalid coeficient wanted.");
        }



        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public override void SetA(double a) { _values[0] = a; }
        public override void SetB(double b) { _values[1] = b; }
        public override void SetC(double c) { _values[2] = c; }

        public override void SetD(double d) 
        {
            throw new Exception("Invalid coeficient setting."); 
        }

        public override void SetG(double g) 
        { throw new Exception("Invalid coeficient setting.");
        }

        public override void SetH(double h)
        { throw new Exception("Invalid coeficient setting.");
        }

        public override void SetL(double l)
        { 
            throw new Exception("Invalid coeficient setting.");
        }


        public override void SetE(double e)
        {
            throw new Exception("Invalid coeficient setting.");
        }
        public override void SetF(double f)
        {
            throw new Exception("Invalid coeficient setting.");
        }
        public override void SetK(double k)
        {
            throw new Exception("Invalid coeficient setting.");
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
    }
}
