﻿using System;
using System.Collections.Generic;
using GMath;

namespace Geometry.GraphicObjects.Equations
{
    public class PlanEquation : CommonEquation
    {
        public const int MembersCount = 4;

        public PlanEquation(PlanEquation src) 
        {
            _values = base.CopyValues(src);
        }

        public PlanEquation(double A, double B, double C, double D)
        {
            _values = new List<double>() { A, B, C, D};
        }

        public PlanEquation(List<double> values)
        {
            if (values.Count != MembersCount)
            {
                throw new Exception("Snazite se vytvorit equation roviny, ktery nema ctyri cleny.");
            }
            else _values = values;
        }

        public override CommonEquation Normalize()
        {
            return new PlanEquation(base.Normalize().GetValues());
        }




        public new PlanEquation MultiplyByMinusOne()
        {
            CommonEquation newEquation = base.MultiplyByMinusOne();
            return new PlanEquation(base._values);
        }

        public new PlanEquation DivideByMinKoeficient()
        {
            CommonEquation newEquation = base.DivideByMinKoeficient();
            return new PlanEquation(base._values);
        }

        public override double Substitute(Vector values)
        {
            return
                GetA() * values.GetX() +
                GetB() * values.GetY() +
                GetC() * values.GetZ() +
                GetD();
        }


        public override double GetA()
        {
            return _values[0];
        }
        public override double GetB()
        {
            return _values[1];
        }
        public override double GetC()
        {
            return _values[2];
        }
        public override double GetD()
        {
            return _values[3];
        }

        public override void SetA(double a)
        {
            _values[0] = a;
        }
        public override void SetB(double b)
        {
            _values[1] = b;
        }
        public override void SetC(double c)
        {
            _values[2] = c;
        }
        public override void SetD(double d)
        {
            _values[3] = d;
        }
    }
}
