﻿using System;
using System.Collections.Generic;
using Geometry.GraphicObjects.Matrixes;
using GMath;

namespace Geometry.GraphicObjects.Equations
{
    public class QuadricEquation:CommonEquation
    {
        public const int MembersCount = 10;

        public QuadricEquation(CommonEquation src) 
        {
            if (src.GetValues().Count == MembersCount)
                _values = src.GetValues();
            else throw new ArgumentException();
        }

        public QuadricEquation(QuadricEquation src) 
        {
            _values = base.CopyValues(src);
        }

        public QuadricEquation(double A, double B, double C,
                                double D, double E, double F,
                                    double G, double H, double K, double L) 
        {
            _values = new List<double>(){ A, B, C, D, E, F, G, H, K, L };
        }

        public QuadricEquation(List<double> values)
        {
            if (values.Count != MembersCount)
            {
                throw new Exception("Snazite se vytvorit equation kvadriky, ktery nema deset clenu."); 
            }
            else _values = values;
        }

        public QuadricEquation(QuadricMatrix matrix) 
        {
            _values = new List<double>()
            {
                matrix.GetA(),matrix.GetB(),matrix.GetC(),
                matrix.GetD()*2.0,matrix.GetE()*2.0,matrix.GetF()*2.0,
                matrix.GetG()*2.0, matrix.GetH()*2.0,matrix.GetK()*2.0,
                matrix.GetL()
            };
        }

        public override CommonEquation Normalize()
        {
            return new QuadricEquation(base.Normalize().GetValues());
        }

        public bool ISQuadricsEquationInTolerance() 
        {
            if (!Program.IsEqualInTolerance(_values[0], 0))
                return true;
            if (!Program.IsEqualInTolerance(_values[1], 0))
                return true;
            if (!Program.IsEqualInTolerance(_values[2], 0))
                return true;
            if (!Program.IsEqualInTolerance(_values[3], 0))
                return true;
            if (!Program.IsEqualInTolerance(_values[4], 0))
                return true;
            if (!Program.IsEqualInTolerance(_values[5], 0))
                return true;
            return false;
        }

        public QuadricEquation DivideBy(double d) 
        {
            List<double> values = new List<double>();
            for (int i = 0; i < _values.Count; i++)
                values.Add(_values[i]/d);
            return new QuadricEquation(values);
        }  



        public new QuadricEquation MultiplyByMinusOne()
        {
            CommonEquation newEquation = base.MultiplyByMinusOne();
            return new QuadricEquation(newEquation.GetValues());
        }

        public new QuadricEquation DivideByMinKoeficient()
        {
            CommonEquation newEquation = base.DivideByMinKoeficient();
            return new QuadricEquation(base._values);
        }


        public override double Substitute(Vector values)
        {
            return
                GetA() * values.GetX() * values.GetX() +
                GetB() * values.GetY() * values.GetY() + 
                GetC() * values.GetZ() * values.GetZ() +
                GetD() * values.GetX() * values.GetY() +
                GetE() * values.GetX() * values.GetZ() +
                GetF() * values.GetY() * values.GetZ() +
                GetG() * values.GetX() +
                GetH() * values.GetY() +
                GetK() * values.GetZ() +
                GetL();
        }


        public override double GetA()
        {
            return _values[0];
        }
        public override double GetB()
        {
            return _values[1];
        }
        public override double GetC()
        {
            return _values[2];
        }
        public override double GetD()
        {
            return _values[3];
        }
        public override double GetE()
        {
            return _values[4];
        }
        public override double GetF()
        {
            return _values[5];
        }
        public override double GetG()
        {
            return _values[6];
        }
        public override double GetH()
        {
            return _values[7];
        }
        public override double GetK()
        {
            return _values[8];
        }
        public override double GetL()
        {
            return _values[9];
        }

        public override void SetA(double a)
        {
            _values[0] = a;
        }
        public override void SetB(double b)
        {
            _values[1] = b;
        }
        public override void SetC(double c)
        {
            _values[2] = c;
        }
        public override void SetD(double d)
        {
            _values[3] = d;
        }
        public override void SetE(double e)
        {
            _values[4] = e;
        }
        public override void SetF(double f)
        {
            _values[5] = f;
        }
        public override void SetG(double g)
        {
            _values[6] = g;
        }
        public override void SetH(double h)
        {
            _values[7] = h;
        }
        public override void SetK(double k)
        {
            _values[8] = k;
        }
        public override void SetL(double l)
        {
            _values[9] = l;
        }
    }
}
