﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Geometry.GraphicObjects.Equations
{
    /// <summary>
    /// Nese Informace o koeficientech parametricke/kanonicke rovnice objektu.
    /// </summary>
    public class ParametricEquation
    {
        
        protected List<double> _values;


        public ParametricEquation() { }

        public ParametricEquation(List<double> values) 
        {
            _values = values;
        }

        /// <summary>
        /// Předpokládá porovnávání rovnic stejného typu.
        /// Jinak může řící že equationy o stejném počtu prvků ale různých typů jsou stejné.
        /// Neprovadim typovou kontrolu, protoze se pouziva i na porovnani zdedeneho Equationu 
        /// s jeho kopii, ktera ale neni specificka, je cisty ParametricEquation  
        /// </summary>
        public static bool AreEqualInTolerance(ParametricEquation a, ParametricEquation b) 
        {
            if (a.GetMembersCount() != b.GetMembersCount())
                return false;
            bool equal = true;
            for (int i = 0; i < a._values.Count; i++)
            {
                equal = equal && Program.IsEqualInTolerance(a._values[i], b._values[i]);
            }
            return equal;
        }

        public double Get(int i)
        {
            if (i < 0 || i > _values.Count - 1)
                throw new Exception("Index je mimo rozsah indexu clenu rovnice.");
            return _values[i];
        }


        public List<double> CopyValues(ParametricEquation src) 
        {
            List<double> values = new List<double>();
            foreach (double d in src._values)
                values.Add(d);
            return values;
        }

        public int GetMembersCount() { return _values.Count; }


        public virtual double GetA() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetB() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); ; }
        public virtual double GetC() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); ; }
        public virtual double GetP() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }


        public virtual double GetUx() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetUy() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetUz() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetVx() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetVy() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual double GetVz() { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }

        public virtual void SetA(double value) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetB(double value) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetC(double value) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }
        public virtual void SetP(double value) { throw new Exception("Tato metoda musí být přepsána v potomkovi."); }


        public virtual void SetUx(double value) { }
        public virtual void SetUy(double value) { }
        public virtual void SetUz(double value) { }
        public virtual void SetVx(double value) { }
        public virtual void SetVy(double value) { }
        public virtual void SetVz(double value) { }
    }
}
