﻿using System.Collections.Generic;
using GMath;
using Geometry.GraphicObjects.Matrixes;

namespace Geometry.GraphicObjects.Quadrics
{
    class PeakComputer
    {

        public PeakComputer() { }

        public Vector GetPeak(QuadricMatrix matrix, List<Vector> ownVectors)
        {
            decimal a = (decimal)matrix.GetA();
            decimal b = (decimal)matrix.GetB();
            decimal c = (decimal)matrix.GetC();
            decimal d = (decimal)matrix.GetD();
            decimal e = (decimal)matrix.GetE();
            decimal f = (decimal)matrix.GetF();
            decimal g = (decimal)matrix.GetG();
            decimal h = (decimal)matrix.GetH();
            decimal k = (decimal)matrix.GetK();
            decimal l = (decimal)matrix.GetL();
            Vector u = ownVectors[0];
            Vector v = ownVectors[1];
            Vector w = ownVectors[2];
            decimal m1 = (decimal)u.GetX() * a + (decimal)u.GetY() * d + (decimal)u.GetZ() * e;
            decimal n1 = (decimal)u.GetX() * d + (decimal)u.GetY() * b + (decimal)u.GetZ() * f;
            decimal p1 = (decimal)u.GetX() * e + (decimal)u.GetY() * f + (decimal)u.GetZ() * c;
            decimal s1 = (decimal)u.GetX() * g + (decimal)u.GetY() * h + (decimal)u.GetZ() * k;

            decimal m2 = (decimal)v.GetX() * a + (decimal)v.GetY() * d + (decimal)v.GetZ() * e;
            decimal n2 = (decimal)v.GetX() * d + (decimal)v.GetY() * b + (decimal)v.GetZ() * f;
            decimal p2 = (decimal)v.GetX() * e + (decimal)v.GetY() * f + (decimal)v.GetZ() * c;
            decimal s2 = (decimal)v.GetX() * g + (decimal)v.GetY() * h + (decimal)v.GetZ() * k;

            PeakEquation equation1 = new PeakEquation(m1, n1, p1, 0, 0, 0, 0, 0, 0, s1);
            PeakEquation equation2 = new PeakEquation(m2, n2, p2, 0, 0, 0, 0, 0, 0, s2);
            PeakEquation equation4 = new PeakEquation(2 * g, 2 * h, 2 * k, 2 * d, 2 * e, 2 * f, a, b, c, l);

            return Method(equation1, equation2, equation4);
        }

        private Vector Method(PeakEquation equation1, PeakEquation equation2, PeakEquation equation4)
        {
            if (!Program.IsEqualInToleranceDecimal(equation1.GetM(), 0))
            {
                PeakEquation MSubstituent = equation1.GetMAsSubstituent();
                equation2 = equation2.SubstituteToPeakEquationForM(MSubstituent);
                equation4 = equation4.SubstituteToPeakEquationForM(MSubstituent);
                return MethodM(equation2, equation4, MSubstituent);
            }
            if (!Program.IsEqualInToleranceDecimal(equation1.GetN(), 0))
            {
                PeakEquation NSubstituent = equation1.GetNAsSubstituent();
                equation2 = equation2.SubstituteToPeakEquationForN(NSubstituent);
                equation4 = equation4.SubstituteToPeakEquationForN(NSubstituent);
                return MethodN(equation2, equation4, NSubstituent);
            }
            if (!Program.IsEqualInToleranceDecimal(equation1.GetP(), 0))
            {
                PeakEquation PSubstituent = equation1.GetPAsSubstituent();
                equation2 = equation2.SubstituteToPeakEquationForP(PSubstituent);
                equation4 = equation4.SubstituteToPeakEquationForP(PSubstituent);
                return MethodP(equation2, equation4, PSubstituent);
            }
            equation2 = equation2.ZeroM();
            equation4 = equation4.ZeroM();
            return MethodM(equation2, equation4, new PeakEquation(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
        }

        private Vector MethodM(PeakEquation equation2, PeakEquation equation4, PeakEquation MSubstituent)
        {
            if (!Program.IsEqualInToleranceDecimal(equation2.GetN(), 0))
            {
                PeakEquation NSubstituent = equation2.GetNAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForN(NSubstituent);
                return MethodMN(equation4, MSubstituent, NSubstituent);
            }
            if (!Program.IsEqualInToleranceDecimal(equation2.GetP(), 0))
            {
                PeakEquation PSubstituent = equation2.GetPAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForP(PSubstituent);
                return MethodMP(equation4, MSubstituent, PSubstituent);
            }
            equation4 = equation4.ZeroN();
            return MethodMN(equation4, MSubstituent, new PeakEquation(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));

        }

        private Vector MethodN(PeakEquation equation2, PeakEquation equation4, PeakEquation NSubstituent)
        {
            if (!Program.IsEqualInToleranceDecimal(equation2.GetM(), 0))
            {
                PeakEquation MSubstituent = equation2.GetMAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForM(MSubstituent);
                return MethodNM(equation4, NSubstituent, MSubstituent);
            }
            if (!Program.IsEqualInToleranceDecimal(equation2.GetP(), 0))
            {
                PeakEquation PSubstituent = equation2.GetPAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForP(PSubstituent);
                return MethodNP(equation4, NSubstituent, PSubstituent);
            }
            equation4 = equation4.ZeroM();
            return MethodNM(equation4, NSubstituent, new PeakEquation(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
        }

        private Vector MethodP(PeakEquation equation2, PeakEquation equation4, PeakEquation PSubstituent)
        {
            if (!Program.IsEqualInToleranceDecimal(equation2.GetM(), 0))
            {
                PeakEquation MSubstituent = equation2.GetMAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForM(MSubstituent);
                return MethodPM(equation4, PSubstituent, MSubstituent);
            }
            if (!Program.IsEqualInToleranceDecimal(equation2.GetN(), 0))
            {
                PeakEquation NSubstituent = equation2.GetNAsSubstituent();
                equation4 = equation4.SubstituteToPeakEquationForN(NSubstituent);
                return MethodPN(equation4, PSubstituent, NSubstituent);
            }
            equation4 = equation4.ZeroM();
            return MethodPM(equation4, PSubstituent, new PeakEquation(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
        }

        private Vector MethodMN(PeakEquation equation4, PeakEquation MSubstituent, PeakEquation NSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForP();
            List<double> Psolution = quadraticEquation.Solve();
            if (Psolution == null || Psolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal P = (decimal)Psolution[0];
            decimal N = NSubstituent.SubstituteForP(P).GetConst();
            decimal M = MSubstituent.SubstituteForP(P).SubstituteForN(N).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }

        private Vector MethodMP(PeakEquation equation4, PeakEquation MSubstituent, PeakEquation PSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForN();
            List<double> Nsolution = quadraticEquation.Solve();
            if (Nsolution == null || Nsolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal N = (decimal)Nsolution[0];
            decimal P = PSubstituent.SubstituteForN(N).GetConst();
            decimal M = MSubstituent.SubstituteForP(P).SubstituteForN(N).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }

        private Vector MethodNP(PeakEquation equation4, PeakEquation NSubstituent, PeakEquation PSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForM();
            List<double> Msolution = quadraticEquation.Solve();
            if (Msolution == null || Msolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal M = (decimal)Msolution[0];
            decimal P = PSubstituent.SubstituteForM(M).GetConst();
            decimal N = NSubstituent.SubstituteForM(M).SubstituteForP(P).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }

        private Vector MethodNM(PeakEquation equation4, PeakEquation NSubstituent, PeakEquation MSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForP();
            List<double> Psolution = quadraticEquation.Solve();
            if (Psolution == null || Psolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal P = (decimal)Psolution[0];
            decimal M = MSubstituent.SubstituteForP(P).GetConst();
            decimal N = NSubstituent.SubstituteForP(P).SubstituteForM(M).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }

        private Vector MethodPM(PeakEquation equation4, PeakEquation PSubstituent, PeakEquation MSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForN();
            List<double> Nsolution = quadraticEquation.Solve();
            if (Nsolution == null || Nsolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal N = (decimal)Nsolution[0];
            decimal M = MSubstituent.SubstituteForN(N).GetConst();
            decimal P = PSubstituent.SubstituteForN(N).SubstituteForM(M).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }

        private Vector MethodPN(PeakEquation equation4, PeakEquation PSubstituent, PeakEquation NSubstituent)
        {
            QuadraticEquation quadraticEquation = equation4.GetQuadratiicEquationForM();
            List<double> Msolution = quadraticEquation.Solve();
            if (Msolution == null || Msolution.Count == 0)
                return new Vector(0, 0, 0);
            decimal M = (decimal)Msolution[0];
            decimal N = NSubstituent.SubstituteForM(M).GetConst();
            decimal P = PSubstituent.SubstituteForM(M).SubstituteForN(N).GetConst();
            return new Vector((double)M, (double)N, (double)P);
        }
    }
}
