﻿using System.Collections.Generic;
using GMath; 

namespace Geometry.GraphicObjects.Quadrics
{
    class PeakEquation
    {
        List<decimal> _equation;

        public PeakEquation(decimal M, decimal N, decimal P, decimal MN, decimal MP, decimal NP,
            decimal M2, decimal N2, decimal P2, decimal constant)
        {
            this._equation = new List<decimal>() { M, N, P, MN, MP, NP, M2, N2, P2, constant };
        }

        private PeakEquation(List<decimal> values)
        {
            _equation = values;
        }

        private PeakEquation(PeakEquation copier)
        {
            List<decimal> retvalValues = new List<decimal>();
            foreach (decimal d in copier._equation)
                retvalValues.Add(d);
            _equation = retvalValues;
        }

        /// <summary>
        /// Násobení substituovaneho prvku!!!
        /// </summary>
        /// <returns></returns>
        private PeakEquation MultByM()
        {
            List<decimal> retvalValues = new List<decimal>() { GetConst(), 0, 0, GetN(), GetP(), 0, GetM(), 0, 0, 0 };
            return new PeakEquation(retvalValues);
        }

        /// <summary>
        /// Násobení substituovaneho prvku!!!
        /// </summary>
        /// <returns></returns>
        private PeakEquation MultByN()
        {
            List<decimal> retvalValues = new List<decimal>() { 0, GetConst(), 0, GetM(), 0, GetP(), 0, GetN(), 0, 0 };
            return new PeakEquation(retvalValues);
        }

        /// <summary>
        /// Násobení substituovaneho prvku!!!
        /// </summary>
        /// <returns></returns>
        private PeakEquation MultByP()
        {
            List<decimal> retvalValues = new List<decimal>() { 0, 0, GetConst(), 0, GetM(), GetN(), 0, 0, GetP(), 0 };
            return new PeakEquation(retvalValues);
        }

        public static PeakEquation operator *(decimal d, PeakEquation equation)
        {
            PeakEquation retvalEquation = new PeakEquation(equation);
            for (int i = 0; i < equation._equation.Count; i++)
                retvalEquation._equation[i] = equation._equation[i] * d;
            return retvalEquation;
        }

        public static PeakEquation operator *(PeakEquation equation, decimal d)
        {
            return d * equation;
        }


        public static PeakEquation operator *(PeakEquation a, PeakEquation b)
        {
            decimal M = a.GetM() * b.GetConst() + a.GetConst() * b.GetM();
            decimal N = a.GetN() * b.GetConst() + a.GetConst() * b.GetN();
            decimal P = a.GetP() * b.GetConst() + a.GetConst() * b.GetP();
            decimal MN = a.GetM() * b.GetN() + a.GetN() * b.GetM();
            decimal MP = a.GetM() * b.GetP() + a.GetP() * b.GetM();
            decimal NP = a.GetN() * b.GetP() + a.GetP() * b.GetN();
            decimal M2 = a.GetM() * b.GetM();
            decimal N2 = a.GetN() * b.GetN();
            decimal P2 = a.GetP() * b.GetP();
            decimal constant = a.GetConst() * b.GetConst();
            return new PeakEquation(M, N, P, MN, MP, NP, M2, N2, P2, constant);
        }

        public static PeakEquation operator +(PeakEquation a, PeakEquation b)
        {
            List<decimal> resultValues = new List<decimal>();
            for (int i = 0; i < a._equation.Count; i++)
            {
                resultValues.Add(a._equation[i] + b._equation[i]);
            }
            return new PeakEquation(resultValues);
        }

        public PeakEquation GetMAsSubstituent()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                0, -GetN() / GetM(), -GetP() / GetM(), 
                0, 0, 0, 
                0, 0, 0, 
                -GetConst() / GetM() 
            };
            return new PeakEquation(retvalValues);
        }

        public PeakEquation GetNAsSubstituent()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                -GetM()/GetN(), 0, -GetP() / GetN(), 
                0, 0, 0, 
                0, 0, 0, 
                -GetConst() / GetN() 
            };
            return new PeakEquation(retvalValues);
        }

        public PeakEquation GetPAsSubstituent()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                -GetM()/GetP(), -GetN() / GetP(), 0, 
                0, 0, 0, 
                0, 0, 0, 
                -GetConst() / GetP() 
            };
            return new PeakEquation(retvalValues);
        }

        public PeakEquation SubstituteToPeakEquationForM(PeakEquation substituent)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            retvalEquation = retvalEquation + substituent * GetM();
            retvalEquation.SetM(0);
            retvalEquation = retvalEquation + substituent.MultByN() * GetMN();
            retvalEquation.SetMN(0);
            retvalEquation = retvalEquation + substituent.MultByP() * GetMP();
            retvalEquation.SetMP(0);
            retvalEquation = retvalEquation + (substituent * substituent) * GetM2();
            retvalEquation.SetM2(0);
            return retvalEquation;
        }

        public PeakEquation SubstituteToPeakEquationForN(PeakEquation substituent)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            retvalEquation = retvalEquation + substituent * GetN();
            retvalEquation.SetN(0);
            retvalEquation = retvalEquation + substituent.MultByM() * GetMN();
            retvalEquation.SetMN(0);
            retvalEquation = retvalEquation + substituent.MultByP() * GetNP();
            retvalEquation.SetNP(0);
            retvalEquation = retvalEquation + (substituent * substituent) * GetN2();
            retvalEquation.SetN2(0);
            return retvalEquation;
        }

        public PeakEquation SubstituteToPeakEquationForP(PeakEquation substituent)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            retvalEquation = retvalEquation + substituent * GetP();
            retvalEquation.SetP(0);
            retvalEquation = retvalEquation + substituent.MultByM() * GetMP();
            retvalEquation.SetMP(0);
            retvalEquation = retvalEquation + substituent.MultByN() * GetNP();
            retvalEquation.SetNP(0);
            retvalEquation = retvalEquation + (substituent * substituent) * GetP2();
            retvalEquation.SetP2(0);
            return retvalEquation;
        }

        public PeakEquation ZeroM()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                0, GetN(), GetP(), 0, 0, GetNP(), 0, GetN2(), GetP2(), GetConst()
            };
            return new PeakEquation(retvalValues);
        }

        public PeakEquation ZeroN()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                GetM(), 0, GetP(), 0, GetMP(), 0, GetM2(), 0, GetP2(), GetConst()
            };
            return new PeakEquation(retvalValues);
        }

        public PeakEquation ZeroP()
        {
            List<decimal> retvalValues = new List<decimal>()
            {
                GetM(), GetN(), 0, GetMN(), 0, 0, GetM2(), GetN2(), 0, GetConst()
            };
            return new PeakEquation(retvalValues);
        }

        public QuadraticEquation GetQuadratiicEquationForM()
        {
            return new QuadraticEquation((double)GetM2(), (double)GetM(), (double)GetConst());
        }

        public QuadraticEquation GetQuadratiicEquationForN()
        {
            return new QuadraticEquation((double)GetN2(), (double)GetN(), (double)GetConst());
        }

        public QuadraticEquation GetQuadratiicEquationForP()
        {
            return new QuadraticEquation((double)GetP2(), (double)GetP(), (double)GetConst());
        }

        public PeakEquation SubstituteForM(decimal mValue)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            decimal constant = GetConst() + GetM() * mValue + GetM2() * mValue * mValue;
            decimal P = GetP() + GetMP() * mValue;
            decimal N = GetN() + GetMN() * mValue;
            retvalEquation.SetConst(constant);
            retvalEquation.SetN(N);
            retvalEquation.SetP(P);
            retvalEquation.SetM(0);
            retvalEquation.SetMN(0);
            retvalEquation.SetMP(0);
            retvalEquation.SetM2(0);
            return retvalEquation;
        }

        public PeakEquation SubstituteForN(decimal nValue)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            decimal constant = GetConst() + GetN() * nValue + GetN2() * nValue * nValue;
            decimal P = GetP() + GetNP() * nValue;
            decimal M = GetM() + GetMN() * nValue;
            retvalEquation.SetConst(constant);
            retvalEquation.SetM(M);
            retvalEquation.SetP(P);
            retvalEquation.SetN(0);
            retvalEquation.SetMN(0);
            retvalEquation.SetNP(0);
            retvalEquation.SetN2(0);
            return retvalEquation;
        }

        public PeakEquation SubstituteForP(decimal pValue)
        {
            PeakEquation retvalEquation = new PeakEquation(this);
            decimal constant = GetConst() + GetP() * pValue + GetP2() * pValue * pValue;
            decimal M = GetM() + GetMP() * pValue;
            decimal N = GetN() + GetNP() * pValue;
            retvalEquation.SetConst(constant);
            retvalEquation.SetM(M);
            retvalEquation.SetN(N);
            retvalEquation.SetP(0);
            retvalEquation.SetMP(0);
            retvalEquation.SetNP(0);
            retvalEquation.SetP2(0);
            return retvalEquation;
        }

        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////

        public decimal GetM() { return _equation[0]; }

        public decimal GetN() { return _equation[1]; }

        public decimal GetP() { return _equation[2]; }

        private decimal GetMN() { return _equation[3]; }

        private decimal GetMP() { return _equation[4]; }

        private decimal GetNP() { return _equation[5]; }

        private decimal GetM2() { return _equation[6]; }

        private decimal GetN2() { return _equation[7]; }

        private decimal GetP2() { return _equation[8]; }

        public decimal GetConst() { return _equation[9]; }

        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////

        private void SetM(decimal x) { _equation[0] = x; }

        private void SetN(decimal x) { _equation[1] = x; }

        private void SetP(decimal x) { _equation[2] = x; }

        private void SetMN(decimal x) { _equation[3] = x; }

        private void SetMP(decimal x) { _equation[4] = x; }

        private void SetNP(decimal x) { _equation[5] = x; }

        private void SetM2(decimal x) { _equation[6] = x; }

        private void SetN2(decimal x) { _equation[7] = x; }

        private void SetP2(decimal x) { _equation[8] = x; }

        private void SetConst(decimal x) { _equation[9] = x; }

        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////

    }
}
