﻿using System;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    public class Quadric:WithCanonicEquationObject
    {     
        public static string _commonEquationInitialFormatString = 
            "L[!Ax" + U.Exp2 + " + !By" + U.Exp2 + " + !Cz" + U.Exp2 + 
            " + !Dxy + !Exz + !Fyz + !Gx + !Hy + !Kz + !L = 0]";

        public Quadric(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, 
            string wordDescriptionFirstCase, string wordDescriptionSecondCase, string wordDescriptionFourthCase, 
            string canonicFormatString, 
            string parametricXFormatString, string parametricYFormatString,string parametricZFormatString, 
            int ID)
        {
            base._ID = ID;
            base._commonEquation = commonEquation;
            base._parametricEquation = parametricEquation;
            base._color = color;
            base._name = name;
            base.SetWordDescriptionFirstCase(wordDescriptionFirstCase);
            base.SetWordDescriptionSecondCase(wordDescriptionSecondCase);
            base.SetWordDescriptionFourthCase(wordDescriptionFourthCase);
            base._commonEquationFormatSring = new CommonEquationFormatString(_commonEquationInitialFormatString);
            base._canonicEquationFormatString = new CanonicEquationFormatString(canonicFormatString);
            base._parametricEquationFormatString = new ParametricEquationsFormatString(parametricXFormatString,
                parametricYFormatString, parametricZFormatString);
            SetTransformationMatrixAlgebraic(toCanonicTransformationMatrix.ComputeInverseMatrix());
                //ConvertOnRigthOrientation(toCanonicTransformationMatrix).ComputeInverseMatrix());
        }


        public override WithCommonEquationObject MakeCopyOfObject(CommonEquation commonEquation, int ID)
        {
            QuadricRecognizer recognizer = new QuadricRecognizer((QuadricEquation)commonEquation, _color, _name, ID);
            Quadric newQuadric = recognizer.RecognizeQuadric();
            newQuadric.CopyDrawableSlots(newQuadric, this);
            return newQuadric;
        }
        /// <summary>
        /// Kanonicka nebo parametricka rovnice editovana.
        /// Vytvori se novy object kvadriky.
        /// Zkopiruji se ji drawable slots.
        /// </summary>
        public override WithParametricEquationObject MakeCopyOfObject(KoefsAndCenterPair newKoeficientsAndCenter, int ID)
        {
            ParametricEquation newParametricEquation = newKoeficientsAndCenter.GetParametricEquation();
            CenterParametrization newCenterInCanonicBasis = newKoeficientsAndCenter.GetCenterParametrization();
            CenterParametrization newCenterParametrization = ConvertCanonicToStandardBasis(newCenterInCanonicBasis);
            TransformationMatrix newToCanonicMatrix =
                GetToCanonicTransformationMatrix(GetToCanonicTransformationMatrix(), newCenterParametrization);
            QuadricMatrix canonicMatrix = GetCanonicMatrix((QuadricParametricEquation)newKoeficientsAndCenter.GetParametricEquation());
            QuadricMatrix commonMatrix = canonicMatrix.ApplyTransformation(newToCanonicMatrix.ComputeInverseMatrix());
            QuadricEquation newCommonEquation = (QuadricEquation)NormalizeEquation(new QuadricEquation(commonMatrix));

            Quadric quadric = CallConstructor(newCommonEquation, newParametricEquation, newToCanonicMatrix, _color, _name, ID);
            quadric.CopyDrawableSlots(quadric, this);
            return quadric;
        }

        public override WithParametricEquationObject MakeCopyOfObject(int ID)
        {
            QuadricParametricEquation equationCopy =
                new QuadricParametricEquation((QuadricParametricEquation)_parametricEquation);
            CenterParametrization centerCopy =
                new CenterParametrization(GetCenterInCanonicBasis());
            return MakeCopyOfObject(new KoefsAndCenterPair(equationCopy, centerCopy), ID);
        }



        protected virtual Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }




        public new QuadricEquation GetCommonEquation() 
        { 
            return (QuadricEquation)_commonEquation; 
        }
        public QuadricParametricEquation GetCanonicEquation() 
        { 
            return (QuadricParametricEquation)_parametricEquation; 
        }


        protected override void ActualizeEquations()
        {
            Matrix transformation = GetTransformationMatrixAlgebraic();
            QuadricMatrix canonicMatrix = GetCanonicMatrix((QuadricParametricEquation)_parametricEquation);
            QuadricMatrix commonMatrix = canonicMatrix.ApplyTransformation(transformation);
            QuadricEquation equation = new QuadricEquation(commonMatrix);
            CenterParametrization canonicCenter = GetCenterInCanonicBasis();
            _commonEquation = equation;
            _commonEquation = NormalizeEquation(equation);
            _commonEquationFormatSring.SubstituteToFormatString(_commonEquation);
            _canonicEquationFormatString.SubstituteToFormatString(_parametricEquation, canonicCenter);
            _parametricEquationFormatString.SubstituteToFormatStrings(_parametricEquation, canonicCenter);
            //Program.Drawtest((QuadricEquation)_commonEquation, 1);
        }
        public virtual QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation) 
        {
            throw new Exception("Tato metoda musí být přepsána v potomkovi.");
        }
        public Matrix GetCommonMatrixWithTransformation(Matrix transformation)
        {
            return GetCanonicMatrix((QuadricParametricEquation)_parametricEquation).ApplyTransformation(transformation);
        }
    }
}
