﻿using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    /// <summary>
    /// Eliptická válcová plocha.
    /// Kanonická rovnice:       x*x/a*a + y*y/b*b -1 = 0; 
    /// Parametricka Rovnice:    x = m + a*cos(u).
    ///                          y = n + b*sin(u).
    ///                          z = p + v. (u = (0,2PI), v=R).
    /// </summary>
    public class ImaginarElipticCylinder : Quadric
    {
        private double _a;
        private double _b;

        public static string canonicFormatString = "E[x-!m;!a]L[+]E[y-!n;!b]L[=-1]";
        public static string parametricXFormatString = "L[x=" + U.EmptySet + "]";
        public static string parametricYFormatString = "L[y=" + U.EmptySet + "]";
        public static string parametricZFormatString = "L[z=" + U.EmptySet + "]";
        public static string _wordDescriptionFirstCase = "Imaginární eliptická válcová plocha";
        public static string _wordDescriptionSecondCase = "Imaginární eliptické válcové plochy";
        public static string _wordDescriptionFourthCase = "Imaginární eliptickou válcovou plochu";

        public ImaginarElipticCylinder(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
                        color, name,
                        _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
                        canonicFormatString,
                        parametricXFormatString, parametricYFormatString, parametricZFormatString, ID)
        {
            _a = parametricEquation.GetA();
            _b = parametricEquation.GetB();
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {

        }

        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ImaginarElipticCylinder(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            double A = -1 / (parametricEquation.GetA() * parametricEquation.GetA());
            double B = -1 / (parametricEquation.GetB() * parametricEquation.GetB());
            QuadricEquation newCanonicEquation = new QuadricEquation(A, B, 0, 0, 0, 0, 0, 0, 0, -1);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}

