﻿using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    /// <summary>
    /// Rotacni Elipsoid.
    /// Rovnice:    x = m + a*cos(v)*cos(u).
    ///             y = n + a*cos(v)*sin(u).
    ///             z = p + b*sin(v). (u = (0,2PI), t=(-PI/2,PI/2)).
    /// </summary>
    public class ImaginarElipsoid : Quadric
    {
        private double _a;
        private double _b;
        private double _c;

        public static string canonicFormatString = "E[x-!m;!a]L[+]E[y-!n;!b]L[+]E[z-!o;!c]L[ = -1]";
        public static string parametricXFormatString = "L[x = " + U.EmptySet + "]";
        public static string parametricYFormatString = "L[y = " + U.EmptySet + "]";
        public static string parametricZFormatString = "L[z = " + U.EmptySet + "]";
        public static string _wordDescriptionFirstCase = "Imaginární elipsoid";
        public static string _wordDescriptionSecondCase = "Imaginárního elipsoidu";
        public static string _wordDescriptionFourthCase = "Imaginární elipsoid";

        public ImaginarElipsoid(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
                        color, name,
                        _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
                        canonicFormatString,
                        parametricXFormatString, parametricYFormatString, parametricZFormatString, ID) 
        {
            _a = parametricEquation.GetA();
            _b = parametricEquation.GetB();
            _c = parametricEquation.GetC();
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality) 
        {
            
        }

        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ImaginarElipsoid(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }


        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            double A = -1 / (parametricEquation.GetA() * parametricEquation.GetA());
            double B = -1 / (parametricEquation.GetB() * parametricEquation.GetB());
            double C = -1 / (parametricEquation.GetC() * parametricEquation.GetC());
            QuadricEquation newCanonicEquation = new QuadricEquation(A, B, C, 0, 0, 0, 0, 0, 0, -1);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}
