﻿using System.Collections.Generic;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    public class EmptySet:SimpleRulledQuadric
    {
        public static string canonicFormatString = "L[" + U.EmptySet + "]";
        public static string parametricXFormatString = "L[x = " + U.EmptySet + "]";
        public static string parametricYFormatString = "L[y = " + U.EmptySet + "]";
        public static string parametricZFormatString = "L[z = " + U.EmptySet + "]";
        public static string _wordDescriptionFirstCase = "Prázdná množina";
        public static string _wordDescriptionSecondCase = "Prázdné množiny";
        public static string _wordDescriptionFourthCase = "Prázdnou množinu";

        public EmptySet(GColor color, string name, int ID)
            : base(new QuadricEquation(-1, 0, 0, 0, 0, 0, 0, 0, 0, -1),
            new QuadricParametricEquation(0, 0,0, 0),
            new TransformationMatrix(4), color, name,
            _wordDescriptionFirstCase,_wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString,
            parametricZFormatString, ID)
        { }

        public EmptySet(QuadricEquation commonEquation, GColor color, string name, int ID)
            : base(commonEquation, new QuadricParametricEquation(0, 0, 0, 0),
            new TransformationMatrix(4), color, name,
            _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
            canonicFormatString, parametricXFormatString, parametricYFormatString,
            parametricZFormatString, ID)
        { }


        public EmptySet(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
            color, name, 
            _wordDescriptionFirstCase,_wordDescriptionSecondCase, _wordDescriptionFourthCase, 
            canonicFormatString,
            parametricXFormatString, parametricYFormatString, parametricZFormatString, ID)
        {
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality) 
        {
        }

        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new EmptySet(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }

        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            QuadricEquation newCanonicEquation = new QuadricEquation(0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}
