﻿using System.Collections.Generic;
using Tao.OpenGl;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Matrixes;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    /// <summary>
    /// kanonicka rovnice:
    ///     x*x/a*a =0;
    /// parametricka rovnice:
    ///     x = 0;
    ///     y = u;
    ///     z = v;
    /// </summary>
    public class DoublePlan:SimpleRulledQuadric
    {
        private double _a;
        public static string canonicFormatString = "E[x-!m;!a]L[=0]";
        public static string parametricXFormatString = "L[x=!m]";
        public static string parametricYFormatString = "L[y=u]";
        public static string parametricZFormatString = "L[z=v]";
        public static string _wordDescriptionFirstCase = "Dvojnásobná rovina";
        public static string _wordDescriptionSecondCase = "Dvojnásobné roviny";
        public static string _wordDescriptionFourthCase = "Dvojnásobnou rovinu";

        public DoublePlan(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
                        color, name,
                        _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
                        canonicFormatString,
                        parametricXFormatString, parametricYFormatString, parametricZFormatString, ID) 
        {
            _a = parametricEquation.GetA();
        }

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality)
        {
            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            Gl.glPushMatrix();
            BuildPlan(scene);
            Gl.glPopMatrix();
            Gl.glEndList();
        }

        public void BuildPlan(Scene scene) 
        {
            Gl.glRotated(90, 0, 1, 0); // rotace roviny do spravne pozice, puvodne z=0,ted x=0
            double uMin = -GetClippingFront(scene);
            double uMax = -GetClippingBack(scene);
            double vMin = GetClippingBottom(scene);
            double vMax = GetClippingTop(scene);
            Gl.glNormal3d(0, 0, _normalSign);
            SuperPlan.BuildPlanDisplayList(scene, uMin, uMax, vMin, vMax);
        }

        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new DoublePlan(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }

        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            double A = 1 / (parametricEquation.GetA() * parametricEquation.GetA());
            QuadricEquation newCanonicEquation = new QuadricEquation(A, 0, 0, 0, 0, 0, 0, 0, 0, 0);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}
