﻿using System;
using System.Collections.Generic;
using Geometry.BaseClasses;
using Geometry.GraphicObjects.Matrixes;
using Geometry.GraphicObjects.Equations;
using GMath;
using Tao.OpenGl;

namespace Geometry.GraphicObjects.Quadrics
{
    /// <summary>
    /// x*x/a*a + y*y/b*b = 0;
    /// </summary>
    public class ImaginarCrossingPlans:SimpleRulledQuadric
    {
        private double _a;
        private double _b;
        public static string canonicFormatString = "E[x-!m;!a]L[+]E[y-!n;!b]L[=0]";
        public static string parametricXFormatString = "L[x =!m]";
        public static string parametricYFormatString = "L[y =!n]";
        public static string parametricZFormatString = "L[z =u]";
        public static string _wordDescriptionFirstCase = "Imaginární různoběžné roviny";
        public static string _wordDescriptionSecondCase = "Imaginárních různoběžných rovin";
        public static string _wordDescriptionFourthCase = "Imaginární různoběžné roviny";

        public ImaginarCrossingPlans(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
                        color, name,
                        _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
                        canonicFormatString,
                        parametricXFormatString, parametricYFormatString, parametricZFormatString, ID)
        {
            _a = parametricEquation.GetA();
            _b = parametricEquation.GetB();
        }

        /// <summary>
        /// Jelikoz predpokladam kanonicky tvar ve tvaru x*x/a*a - y*y/b*b - 1 = 0,
        /// pak vim ze primka lezi v kanonicke bazi v rovine x,y.
        /// Vyjadřím z kanonicke rovnice x a dosadím dvě hodnoty y.
        /// Z vypočítaných funkčních hodnot určím směrový vektor.
        /// </summary>
        public void ComputeAndSetDirectionVector() 
        {
            double y1 = 0;
            double y2 = 100;

            double x1 = Math.Sqrt((_b * _b + y1 * y1) / (_a * _a * _b * _b));
            double x2 = Math.Sqrt((_b * _b + y2 * y2) / (_a * _a * _b * _b));

            Vector directionVector = new Vector(x2, y2, 0) - new Vector(x1, y1, 0);
        }


        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality) 
        {
            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            BuildAsLineStrip(scene);
            Gl.glEndList();
        }

        private void BuildAsLineStrip(Scene scene) 
        {
            Vector a = new Vector(0, 0, GetClippingBack(scene));
            Vector b = new Vector(0, 0, GetClippingFront(scene));
            Planar.Line.BuildLineAsLineStrip(a, b);
        }

        private void BuildAsCylinder(Scene scene) 
        {
            Vector a = new Vector(0, 0, GetClippingBack(scene));
            Vector b = new Vector(0, 0, GetClippingFront(scene));
            Planar.Line.BuildLineAsCylinder(a,b);
        }

        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ImaginarCrossingPlans(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }

        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            double A = 1 / (parametricEquation.GetA() * parametricEquation.GetA());
            double B = 1 / (parametricEquation.GetB() * parametricEquation.GetB());
            QuadricEquation newCanonicEquation = new QuadricEquation(A, B, 0, 0, 0, 0, 0, 0, 0, 0);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}
