﻿using System.Collections.Generic;
using Tao.OpenGl;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Matrixes;
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects.Quadrics
{
    /// <summary>
    /// x*x/a*a -1 =0;
    /// </summary>
    public class ParallelPlans:SimpleRulledQuadric
    {
        private double _a;

        public static string canonicFormatString = "E[x-!m;!a]L[=1]";
        public static string parametricXFormatString = "L[x = " + U.PM + "!m]";
        public static string parametricYFormatString = "L[y = u]";
        public static string parametricZFormatString = "L[z = v]";
        public static string _wordDescriptionFirstCase = "Rovnoběžné roviny";
        public static string _wordDescriptionSecondCase = "Rovnoběžných rovin";
        public static string _wordDescriptionFourthCase = "Rovnoběžné roviny";

        public ParallelPlans(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix,
            GColor color, string name, int ID)
            : base(commonEquation, parametricEquation, toCanonicTransformationMatrix,
                        color, name,
                        _wordDescriptionFirstCase, _wordDescriptionSecondCase, _wordDescriptionFourthCase,
                        canonicFormatString,
                        parametricXFormatString, parametricYFormatString, parametricZFormatString, ID)
        {
            _a = parametricEquation.GetA();
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        protected override void BuildParametricDisplayList(Scene scene, RelativeQuality relativeQuality) 
        {
            base._relativeQuality = relativeQuality;
            Gl.glPushMatrix();
            _displayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_displayListIdentificator, Gl.GL_COMPILE);
            Gl.glPushMatrix();
            BuildPlan(_a,scene);
            Gl.glPopMatrix();
            BuildPlan(-_a, scene);
            Gl.glEndList();
            Gl.glPopMatrix();
            //Program.Drawtest(GetOriginalEquation().ToDoubleArray(), 0);
        }

        public void BuildPlan(double m, Scene scene) 
        {
            Gl.glTranslated(m, 0, 0);
            Gl.glRotated(90, 0, 1, 0);
            Gl.glNormal3d(0, 0, _normalSign);

            double uMin = -GetClippingFront(scene);
            double uMax = -GetClippingBack(scene);

            double vMin = GetClippingBottom(scene);
            double vMax = GetClippingTop(scene);

            SuperPlan.BuildPlanDisplayList(scene, uMin, uMax, vMin, vMax);
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////


        protected override Quadric CallConstructor(CommonEquation commonEquation, ParametricEquation parametricEquation,
            TransformationMatrix toCanonicTransformationMatrix, GColor color, string name, int ID)
        {
            return new ParallelPlans(commonEquation, parametricEquation, toCanonicTransformationMatrix, color, name, ID);
        }

        public override QuadricMatrix GetCanonicMatrix(QuadricParametricEquation parametricEquation)
        {
            double A = 1 / (parametricEquation.GetA() * parametricEquation.GetA());
            QuadricEquation newCanonicEquation = new QuadricEquation(A, 0, 0, 0, 0, 0, 0, 0, 0, -1);
            return new QuadricMatrix(newCanonicEquation);
        }
    }
}
