﻿using System;
using Geometry.GGraphic.Forms.Detail.FormatString;
using Geometry.GraphicObjects.Equations;
using Tao.OpenGl;
using Geometry.GraphicObjects.Planar; 
using Geometry.BaseClasses;
using GMath;

namespace Geometry.GraphicObjects
{
    public class WithParametricEquationObject:GraphicObject
    {
        protected ParametricEquationsFormatString _parametricEquationFormatString;
        protected ParametricEquation _parametricEquation;
        protected Vector _clippingCenter = new Vector();
        protected int _axesDisplayListIdentificator;
        protected bool _axesVisible = false;
        protected ClippingCube _clippingCube = new ClippingCube();

        /// <summary>
        /// Rotace prevadejici z kanonickeho tvaru na obecny.
        /// </summary>
        protected TransformationMatrix _transformationAlgebraic;
        /// <summary>
        /// Tuto matici stanovím jako matici (_transformationAlgebraic.ComputeInverseMatrix().TransponMatrix());
        /// Matice pro urychleni kresleni, pri zmene libovolne transformace aktualizovat.
        /// </summary>
        protected TransformationMatrix _transformationOpenGL;


        public WithParametricEquationObject()
        {

        }




        public bool GetAxesVisible() { return _axesVisible; }
        public void SetAxesVisible(bool visible) { _axesVisible = visible; }



        public override void Draw(Scene scene)
        {
            if (!_visible)
            {
                return;
            }
            Gl.glPushMatrix();
            Gl.glEnable(Gl.GL_NORMALIZE);
            Gl.glMultMatrixd(_transformationOpenGL.ToDoubleArray());
            Gl.glColor4dv(_color);
            Gl.glCallList(_displayListIdentificator);
            if (_axesVisible)
                Gl.glCallList(_axesDisplayListIdentificator);
            Gl.glPopMatrix();
        }
        public override void DrawAsIntersection(Scene scene)
        {
            Gl.glPushMatrix();
            Gl.glEnable(Gl.GL_NORMALIZE);
            Gl.glMultMatrixd(_transformationOpenGL.ToDoubleArray());
            Gl.glCallList(_displayListIdentificator);
            Gl.glPopMatrix();
        }
        public override void DrawIgnoreVisibilityAndAxes(Scene scene)
        {
            Gl.glPushMatrix();
            Gl.glEnable(Gl.GL_NORMALIZE);
            Gl.glColor3dv(_color);
            Gl.glMultMatrixd(_transformationOpenGL.ToDoubleArray());
            Gl.glCallList(_displayListIdentificator);
            Gl.glPopMatrix();
        }


        /// <summary>
        /// Pokud má object displayListIdentifikator roven -1, pak vždy buildujeme, jinak jen pokud je to potřeba.
        /// </summary>
        /// <param name="scene"></param>
        /// <param name="relativeQuality"></param>
        public override void BuildDisplayListIfNecessary
            (Scene scene, RelativeQuality relativeQuality)
        {
            if (_displayListIdentificator == -1 || _clippingCube.IsRebuildNeeded(_transformationAlgebraic))
            {
                if (_displayListIdentificator != -1)
                    Gl.glDeleteLists(_displayListIdentificator, 1);
                BuildDisplayList(scene, relativeQuality);
            }
        }
        public override void BuildDisplayList
            (Scene scene, RelativeQuality relativeQuality)
        {
            BuildParametricDisplayList(scene, relativeQuality);
            BuildObjectsAxes(scene);
            _clippingCube.Actualize(_transformationAlgebraic);
        }
        protected virtual void BuildParametricDisplayList
            (Scene scene, RelativeQuality relativeQuality) { }


        protected void BuildObjectsAxes(Scene scene)
        {
            _axesDisplayListIdentificator = Gl.glGenLists(1);
            Gl.glNewList(_axesDisplayListIdentificator, Gl.GL_COMPILE);

            int lenght = Scene.GetClippingCubeWidth() * 2;
            Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glBegin(Gl.GL_LINES);
            Gl.glColor3dv(Axes._xColor);
            Gl.glVertex3d(-lenght, 0, 0);
            Gl.glVertex3d(lenght, 0, 0);
            Gl.glColor3dv(Axes._yColor);
            Gl.glVertex3d(0, -lenght, 0);
            Gl.glVertex3d(0, lenght, 0);
            Gl.glColor3dv(Axes._zColor);
            Gl.glVertex3d(0, 0, -lenght);
            Gl.glVertex3d(0, 0, lenght);
            Gl.glEnd();
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEndList();
        }


        /// <summary>
        /// Poté co se změnila transformace, 
        /// je třeba upravit obecnou rovnici kvadrik a kuželoseček.
        /// A obecnou a parametrickou rovnici roviny.
        /// A parametrickou rovnici přímky.
        /// </summary>
        protected virtual void ActualizeEquations() { throw new Exception("Must be overriden."); }



        public ParametricEquation GetParametricEquation() 
        { 
            return _parametricEquation; 
        }
        public ParametricEquationsFormatString GetParametricEquationsFormatString() 
        {
            return _parametricEquationFormatString; 
        }



        public TransformationMatrix GetRotationInStandardBasis() 
        {
            TransformationMatrix transformation = new TransformationMatrix(_transformationAlgebraic);
            transformation.SetTranslation(new Vector(0,0,0));           
            return transformation;
        }
        public void SetRotationInStandardBasis(TransformationMatrix rotation)
        {
            CenterParametrization centerInStandardBasis = GetCenterInStandardBasis();
            SetTransformationMatrixAlgebraic(GetTransformationMatrixAlgebraic(rotation, centerInStandardBasis));
        }
        /// <summary>
        /// Nastaví rotace a Aktualizuje rovnice, tím se aktualizují i regulace.
        /// </summary>
        public void SetTransformationMatrixAlgebraic(TransformationMatrix transformationMatrix) 
        {
            _transformationAlgebraic = transformationMatrix;
            _transformationOpenGL = _transformationAlgebraic.ComputeInverseMatrix();
            _clippingCenter = _transformationAlgebraic.GetTranslation();
            ActualizeEquations();
        }
        /// <summary>
        /// Zavolá SetTransformationMatrixAlgebraic a ta aktualizuje rovnice.
        /// </summary>
        public TransformationMatrix GetTransformationMatrixAlgebraic() 
        {
            return _transformationAlgebraic; 
        }
        public TransformationMatrix GetTransformationMatrixAlgebraic(TransformationMatrix rotation, CenterParametrization newCenterParametrization)
        {
            TransformationMatrix copy = new TransformationMatrix(rotation).TransponMatrix(); // jakoze OpenGL matice.
            Vector center = newCenterParametrization.GetReprezentant();
            copy.SetTranslation(center);
            return copy.ComputeInverseMatrix(); // pak ji prevedu na algebraickou.
        }
        /// <summary>
        /// Zavolá SetTransformationMatrixAlgebraic a ta aktualizuje rovnice.
        /// </summary>
        public void SetCenterInStandardBasis(CenterParametrization centerInSceneBasis)
        {
            //_centerInStandardBasis = centerInSceneBasis; // podobny kod je v GetTransformationMatrixAlgebraic...
            Vector center = centerInSceneBasis.GetReprezentant();
            TransformationMatrix temp = GetRotationInStandardBasis().TransponMatrix();
            temp.SetTranslation(center);
            SetTransformationMatrixAlgebraic(temp.ComputeInverseMatrix());
        }
        public CenterParametrization GetCenterInStandardBasis()
        {
            TransformationMatrix temp = _transformationOpenGL;
            return new CenterParametrization(temp.GetTranslation());
        }


        public void ApplyRotationInStandardBasis(TransformationMatrix rotation) 
        {
            TransformationMatrix m = rotation * GetRotationInStandardBasis();
            //m = m.NormalizeRows();
            //SetTransformationMatrixAlgebraic(m);
            SetRotationInStandardBasis(m);
        }

        public RotationAngles GetRotationAnglesInStandardBasis()
        {
            return GetRotationInStandardBasis().DecomposeRotationMatrixOnAngles();
        }



        private double _clippingKoeficient = 2;
        public double GetClippingBack(Scene scene)
        {
            return _clippingCenter.GetZ() + (_clippingKoeficient* -Scene.GetClippingCubeWidth());
        }
        public double GetClippingFront(Scene scene)
        {
            return _clippingCenter.GetZ() + (_clippingKoeficient * Scene.GetClippingCubeWidth());
        }
        public double GetClippingLeft(Scene scene)
        {
            return _clippingCenter.GetX() + (_clippingKoeficient * -Scene.GetClippingCubeWidth());
        }
        public double GetClippingRight(Scene scene)
        {
            return _clippingCenter.GetX() + (_clippingKoeficient * Scene.GetClippingCubeWidth());
        }
        public double GetClippingTop(Scene scene)
        {
            return _clippingCenter.GetY() + (_clippingKoeficient * Scene.GetClippingCubeWidth());
        }
        public double GetClippingBottom(Scene scene)
        {
            return _clippingCenter.GetY() + (_clippingKoeficient * -Scene.GetClippingCubeWidth());
        }
        


        // Vrací arcus Sinh, coz je ln(x + Odm(x*x + 1));
        public double ArcSinh(double x)
        {
            return Math.Log(x + Math.Sqrt(x * x + 1), Math.E);
        }
        // Vrací arcus Cosh, coz je ln(x + Odm(x*x - 1));
        public double ArcCosh(double x)
        {
            x = Math.Abs(x);
            return Math.Log(x + Math.Sqrt(x * x - 1), Math.E); //*sign ;
        }




        // Z parent objektu se zkopiruji nektere sloty.
        public virtual WithParametricEquationObject MakeCopyOfObject(KoefsAndCenterPair newKoefsAndCenter, int ID) 
        {
            throw new Exception("Tato metod musí být přepsána v potomkovi."); 
        }
        // Vytvori naprostou kopii parentObjektu
        public virtual WithParametricEquationObject MakeCopyOfObject(int ID) 
        {
            throw new Exception("Tato metod musí být přepsána v potomkovi."); 
        }
        /// <summary>
        /// Zkopiruje objektu: Jmeno, Barvu, Viditelnost, Viditelnost os.
        /// </summary>
        public override void CopyDrawableSlots(GraphicObject newObject, GraphicObject parent) 
        {
            base.CopyDrawableSlots(newObject, parent);
            if (newObject is WithParametricEquationObject && parent is WithParametricEquationObject)
                ((WithParametricEquationObject)newObject)._axesVisible = ((WithParametricEquationObject)parent)._axesVisible;
        }
    }
}
