﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using Tao.OpenGl;
using GMath;
using Geometry.GraphicObjects.Planar;
using Geometry.GraphicObjects.Equations;
using Geometry.GraphicObjects.Intersections;
using Geometry.GraphicObjects.Quadrics;
using Geometry.GraphicObjects.Matrixes;
using System.Drawing;
using Geometry.GraphicObjects.ConicSections;

namespace Geometry
{
    static class Program
    {
        public const double MaxTextBoxValue = 1000;
        public const double MinTextBoxValue = 0.001;
        public const int MaxTextBoxDecimals = 6;
        //public const int MaxLabelDecimals = 2; // zvlast reseno v metode..
        //public const int MaxAngleDecimals = 1; // zvlast reseno v metode..

        public static double TwoPointsIntersectionCompareDistance = 0.1;
        public static double TwoVectorsIntersectionCompareDistance = 0.01;

        public struct DistinctNumber 
        {
            public double number;
            public int multiplicity;
            public DistinctNumber(double numberArg, int multiplicityArg) 
            {
                number = numberArg;
                multiplicity = multiplicityArg;
            }
        }

        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            //ConicMatrixDecompositionTest();
            Application.Run(new Form1());
        }

        private static void ConicMatrixDecompositionTest() 
        {
            Elipse elipse = (Elipse)new ConicRecognizer(new ConicEquation(1, 2, 0, 0, 0, -1), new GColor(1, 1, 1), "", 0).Recognize();
            List<double> angles = new List<double>() { -180, -90, -45, 0, 45, 90, 180 };
            foreach (double angle in angles)
            {
                elipse.SetTransformationMatrixAlgebraic(TransformationMatrix.GetXRotationMatrix(angle));
                //elipse.ApplyTransformation(TransformationMatrix.GetYRotationMatrix(angle));
                elipse.ApplyTransformation(TransformationMatrix.GetZRotationMatrix(angle));
                Vector rotatedNormal = Plan.canonicalNormalVector * elipse.GetTransformationMatrixAlgebraic();
                double angl = Vector.Angle(Plan.canonicalNormalVector, rotatedNormal) * 180 / Math.PI;
                Vector axe = Vector.CrossProduct(Plan.canonicalNormalVector, rotatedNormal);
                TransformationMatrix anglRotation = TransformationMatrix.GetRotationMatrixAroundAAxe(axe, -angl);
                Vector fdvOrig = new Vector(1, 0, 0) * anglRotation;
                Vector sdvOrig = new Vector(0, 1, 0) * anglRotation;

                Vector fdvNew = new Vector(1, 0, 0) * elipse.GetTransformationMatrixAlgebraic();
                Vector sdvNew = new Vector(0, 1, 0) * elipse.GetTransformationMatrixAlgebraic();
                double fdva = Vector.OrientedAngle(fdvOrig, fdvNew, rotatedNormal);// *180 / Math.PI;
                double sdva = Vector.OrientedAngle(sdvOrig, sdvNew, rotatedNormal);// *180 / Math.PI;
            } 
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public static double GetNormalizingKoeficient(List<double> values)
        {
            double minExponent = double.MaxValue;
            double maxExponent = double.MinValue;
            double temp;
            foreach (double koeficient in values)
            {
                temp = Program.GetNumbersDecimalExponent(koeficient);
                minExponent = Math.Min(temp, minExponent);
                maxExponent = Math.Max(temp, maxExponent);
            }
            temp = Math.Pow(10, Math.Floor((minExponent + maxExponent) / -2));
            if (temp != 0)
                return temp;
            else return 1;
        }
        public static double GetNormalizingKoeficient(double value)
        {
            List<double> values = new List<double>() { value };
            return GetNormalizingKoeficient(values);
        }

        public static int GetNumbersDecimalExponent(double value)
        {
            if (Program.IsEqualInTolerance(value, 0))
                return 0;
            double temp = Math.Abs(value);
            return (int)Math.Floor(Math.Log10(temp));
        }

        public static void FormatEquationKoeficients(QuadricEquation equation) 
        {
            double min = double.MaxValue;
            //double max = double.MinValue;
            for (int i = 0; i < 10; i++)
                min = Math.Min(min, equation.Get(i));
            if (min == 0)
                return;
            for (int i = 0; i < 10; i++)
                equation.Set(i, equation.Get(i)*min);
        }

        public static string GetOutputFormatedForAngleDialog(double value)
        {
            string retval = value.ToString("0.#");
            if (value >= 0 || Program.IsEqualInTolerance(value, 0))
                retval = "+" + retval;
            retval = " " + retval;
            retval = retval.Insert(2, " ");
            return retval;
        }
        public static string GetOutputFormatedForCleverLabel(double value) 
        {
            string retval = FormatOnValidDigitsCheckZeroForLabel(value);
            if (value >= 0 || Program.IsEqualInTolerance(value, 0))
                retval = "+" + retval;
            retval = " " + retval;
            retval = retval.Insert(2, " ");
            return retval;
        }
        public static string GetOutputFormatedForTextBox(double value)
        {
            return FormatOnValidDigitsCheckZeroForTextBox(value, MaxTextBoxDecimals);
        }

        private static string FormatOnValidDigitsCheckZeroForTextBox(double value, int decimals) 
        {
            if (value == double.PositiveInfinity)
                return U.Infinity;
            if (value == double.NegativeInfinity)
                return "-" + U.Infinity;
            if (Program.IsEqualInTolerance(value, 0))
                return "0";

            if (Math.Abs(value) < MinTextBoxValue)
                return value.ToString("0.##e+0");

            if (Math.Abs(value) > MaxTextBoxValue)
                return value.ToString("0.##e+0");

            string hashes = "";
            for (int i = 0; i < decimals; i++)
                hashes += "#";
            string temp = value.ToString("0." + hashes);
            return temp;
        }
        private static string FormatOnValidDigitsCheckZeroForLabel(double value)
        {
            if (value == double.PositiveInfinity)
                return U.Infinity;
            if (value == double.NegativeInfinity)
                return "-" + U.Infinity;
            if (Program.IsEqualInTolerance(value, 0))
                return "0";

            if (Math.Abs(value) < MinTextBoxValue)
                return value.ToString("0.#e+0");

            if (Math.Abs(value) > MaxTextBoxValue)
                return value.ToString("0.#e+0");

            if (Math.Abs(value) > 1)
                return value.ToString("0.#");

            if (Math.Abs(value) >= 0.01)
                return value.ToString("0.##");
            else return value.ToString("0.###");
        }


        public static bool CheckInputNumberConstraints(double inputValue, double oldValue) 
        {
            bool ok = true;
            if (oldValue > -1 && oldValue < 1)
            {
                if (oldValue < 0) // |Hodnota| muze byt v mensi nez (MinTextboxValue), pokud je vsak vensi nez puvodni hodnota.
                    ok = ok && oldValue >= inputValue && oldValue != 0;
                else ok = ok && oldValue <= inputValue && oldValue != 0;
            }
            else
            {
                if (oldValue < 0) // |Hodnota| muze byt v vetsi nez (MaxTextBoxValue), pokud je vsak mensi nez puvodni hodnota.
                    ok = ok && oldValue <= inputValue && oldValue != 0;
                else ok = ok && oldValue >= inputValue && oldValue != 0;
            }
            return
                (Math.Abs(inputValue) <= MaxTextBoxValue &&
                (Math.Abs(inputValue) >= MinTextBoxValue || inputValue == 0)) ||
                ok;
        }
      

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////



        /// <summary>
        /// Vraci pravdu(true) pokud je prvni argument blizko druhemu, 
        /// Jak blizko udava druhy argument. (na obe strany) 
        /// </summary>
        /// <returns></returns>
        public static bool IsEqualInToleranceHard(double number, double center)
        {
            double tolerance = 1e-13;
            return (number >= center - tolerance && number <= center + tolerance);
        }

        /// <summary>
        /// Vraci pravdu(true) pokud je prvni argument blizko druhemu, 
        /// Jak blizko udava druhy argument. (na obe strany) 
        /// </summary>
        /// <returns></returns>
        public static bool IsEqualInTolerance(double number, double center)
        {
            double tolerance = 1e-5;
            return (number >= center - tolerance && number <= center + tolerance);
        }

        public static bool IsEqualInToleranceLow(double number, double center) 
        {
            double tolerance = 5e-4;
            return (number >= center - tolerance && number <= center + tolerance);
        }

        public static bool IsEqualInToleranceTextBoxLoterance(double number, double center)
        {
            double tolerance = 0.001;
            return (number >= center - tolerance && number <= center + tolerance);
        }

        public static bool IsEqualInToleranceIntersection(double number, double center)
        {
            double tolerance = 0.01;
            return (number >= center - tolerance && number <= center + tolerance);
        }
        
        public static bool IsEqualInToleranceIntersectionVector(double number, double center)
        {
            double tolerance = TwoVectorsIntersectionCompareDistance;
            return (number >= center - tolerance && number <= center + tolerance);
        }



        /// <summary>
        /// Vraci pravdu(true) pokud je prvni argument blizko druhemu, 
        /// Jak blizko udava druhy argument. (na obe strany) 
        /// </summary>
        /// <returns></returns>
        public static bool IsEqualInToleranceDecimal(decimal number, decimal center)
        {
            decimal tolerance = new decimal(0.0001);
            return (number >= center - tolerance && number <= center + tolerance);
        }

        public static List<DistinctNumber> DistinctInToleranceWithMultiplicity(List<double> values)
        {
            List<DistinctNumber> retval = new List<DistinctNumber>();
            int count = values.Count;
            bool add;
            for (int i = 0; i < count; i++)
            {
                add = true;
                for (int j = 0; j < retval.Count; j++)
                {
                    if (Program.IsEqualInTolerance(values[i], retval[j].number))
                    {
                        retval[j] = new DistinctNumber(retval[j].number, retval[j].multiplicity + 1);
                        add = false;
                    }
                }
                if (add)
                    retval.Add(new DistinctNumber(values[i], 1));
            }
            return retval;
        }
        

        public static List<double> DistinctInTolerance(List<double> values) 
        {
            List<double> retval = new List<double>();
            int count = values.Count;
            bool add;
            for (int i = 0; i < count; i++) 
            {
                add = true;
                for (int j = 0; j < retval.Count; j++)
                {
                    if (Program.IsEqualInTolerance(values[i], retval[j]))
                        add = false;
                }
                if (add)
                    retval.Add(values[i]);
            }
            return retval;
        }

        public static List<double> DistinctInLowTolerance(List<double> values)
        {
            List<double> retval = new List<double>();
            int count = values.Count;
            bool add;
            for (int i = 0; i < count; i++)
            {
                add = true;
                for (int j = 0; j < retval.Count; j++)
                {
                    if (Program.IsEqualInToleranceIntersection(values[i], retval[j]))
                        add = false;
                }
                if (add)
                    retval.Add(values[i]);
            }
            return retval;
        }

        public static Color ColorToSystemColor(GColor color)
        {
            int red = (int)(color.GetRed() * 255);
            int green = (int)(color.GetGreen() * 255);
            int blue = (int)(color.GetBlue() * 255);
            return Color.FromArgb(red, green, blue);
        }

        public static GColor SystemColorToColor(Color color)
        {
            double red = color.R / 255.0;
            double green = color.G / 255.0;
            double blue = color.B / 255.0;
            return new GColor(red, green, blue);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public static double Cbroot(double x)
        {
            if (x < 0)
                return -Math.Pow(-x, 1.0 / 3.0);
            else return Math.Pow(x, 1.0 / 3.0);
        }
    }
}
